<?php

/**
 * JEvents Component for Joomla 1.5.x
 *
 * @version     $Id: icalevent.php 1712 2010-03-04 07:33:11Z geraint $
 * @package     JEvents
 * @copyright   Copyright (C) 2008-2021 GWE Systems Ltd, 2006-2008 JEvents Project Group
 * @license     GNU/GPLv2, see http://www.gnu.org/licenses/gpl-2.0.html
 * @link        https://www.jevents.net
 */
defined('_JEXEC') or die('Restricted Access');

use Joomla\Utilities\ArrayHelper;
use Joomla\Registry\Registry;
use Joomla\CMS\MVC\Controller\BaseController;
use Joomla\CMS\Table\Table;
use Joomla\CMS\Language\Text;
use Joomla\CMS\Component\ComponentHelper;
use Joomla\CMS\Plugin\PluginHelper;
use Joomla\CMS\Pagination\Pagination;
use Joomla\CMS\Router\Route;
use Joomla\CMS\Session\Session;
use Joomla\CMS\Factory;
use Joomla\CMS\HTML\HTMLHelper;
use Joomla\CMS\Filesystem\File;
use Joomla\CMS\Filter\InputFilter;

jimport('joomla.application.component.controller');

class AdminAttendeesController extends BaseController
{

	protected $params;
	protected $jomsocial = false;
	protected $jevrreminders;

	/**
	 * Controler for Sessions
	 * @param array		configuration
	 */
	function __construct($config = array())
	{
		parent::__construct($config);
		$this->registerTask('list', 'overview');
		$this->registerTask('newtransaction', 'edittransaction');
		$this->registerTask('pdfticket', 'ticket');
		$this->registerTask('htmlticket', 'ticket');

		//$this->registerTask( 'record',  'recordAttendance' );
		//$this->registerTask( 'save',  'recordAttendance' );
		$this->registerDefaultTask("overview");

		// Must load admin language files
		$lang = Factory::getLanguage();
		$lang->load("com_jevents", JPATH_ADMINISTRATOR);

		$this->dataModel = new JEventsDataModel();
		$this->queryModel = new JEventsDBModel($this->dataModel);

		$plugin = PluginHelper::getPlugin("jevents", "jevrsvppro");
		$this->params = ComponentHelper::getParams("com_rsvppro");

		include_once(JPATH_ADMINISTRATOR . "/components/com_rsvppro/libraries/attendeehelper.php");
		$this->helper = new RsvpAttendeeHelper($this->params);

		jimport('joomla.filesystem.file');
		if (File::exists(JPATH_SITE . '/components/com_community/community.php'))
		{
			if (ComponentHelper::isEnabled("com_community"))
			{
				$this->jomsocial = true;
			}
		}
		JLoader::register('JevRsvpReminders', JPATH_ADMINISTRATOR . "/components/com_rsvppro/libraries/jevrreminders.php");
		$this->jevrreminders = new JevRsvpReminders($this->params, $this->jomsocial);
	}

	/**
	 * List Atendees
	 *
	 */
	function overview()
	{

		// get the view
		$this->view = $this->getView("attendees", "html");

		$this->view->dataModel = $this->dataModel;
		$mainframe = Factory::getApplication();
		$option = Factory::getApplication()->input->getCmd("option");

		$db = Factory::getDBO();

		$atd_id = Factory::getApplication()->input->get("atd_id", array(), 'ARRAY');
		if (!isset($atd_id[0]) || strpos($atd_id[0], "|") === false)
		{
			throw new Exception( Text::_("RSVP_MISSING_ATDID"), 403);
		}
		list($atd_id, $rp_id) = explode("|", $atd_id[0]);

		$atd_id = intval($atd_id);
		$rp_id = intval($rp_id);

		// first of all clean up any unpaid and unconfirmed attendees
		$this->helper->cleanupUnpaidAttendees($atd_id);

		$sql = "SELECT * FROM #__jev_attendance WHERE id=" . $atd_id;
		$db->setQuery($sql);
		$rsvpdata = $db->loadObject();

		if ($rp_id == 0)
		{
			$vevent = $this->dataModel->queryModel->getEventById($rsvpdata->ev_id, Factory::getApplication()->isClient('administrator'), "icaldb");
			$repeat = $vevent->getFirstRepeat();
		}
		else
		{
			list($year, $month, $day) = JEVHelper::getYMD();
			// include unpublished events
			$repeat = $this->dataModel->queryModel->listEventsById(intval($rp_id), Factory::getApplication()->isClient('administrator'), "icaldb");
			//$repeatdata = $this->dataModel->getEventData(intval($rp_id), "icaldb", $year, $month, $day);
			//if ($repeatdata && isset($repeatdata["row"]))
			//	$repeat = $repeatdata["row"];
		}

		// Add reference to current row and rsvpdata to the registry so that we have access to these in the fields
		$registry = JevRegistry::getinstance("jevents");
		$registry->set("rsvpdata", $rsvpdata);
		$registry->set("event", $repeat);

		$repeating = Factory::getApplication()->input->getInt('repeating', 0);

		$limit = intval($mainframe->getUserStateFromRequest("attendeelistlimit", 'limit', Factory::getApplication()->get("list_limit", 10)));
		$limitstart = intval($mainframe->getUserStateFromRequest("attendeelimitstart", 'limitstart', 0));

		$where = array();
		$join = array();

		$where[] = "ev.ev_id IS NOT NULL";
		$where[] = "atd.id = $atd_id";
		$where[] = "atdees.rp_id = $rp_id";

		$user = Factory::getUser();
		if ($user->id != $repeat->created_by() && !JEVHelper::isAdminUser($user) && !JEVHelper::canDeleteEvent($repeat, $user))
		{
			if (!$this->params->get("showunpaidattendees", 0))
			{
				$where[] = " atdees.attendstate!=4";
			}
			if (!$this->params->get("allowmaybe", 0))
			{
				$where[] = " atdees.attendstate!=2";
			}
			if (!$this->params->get("allowpending", 0))
			{
				$where[] = " atdees.attendstate!=3";
			}
			if (!$this->params->get("shownonattendees", 0))
			{
				$where[] = " atdees.attendstate!=0";
			}
		}

		// non-Admin users should not see pending attendees
		if (!($user->id == $repeat->created_by() || JEVHelper::isAdminUser($user) || JEVHelper::canDeleteEvent($repeat, $user)))
		{
			$where[] = " atdees.attendstate!=3";
		}

		$filter_order = $mainframe->getUserStateFromRequest('rsvppro_attendees.atd_filter_order', 'filter_order', 'atd.id', 'cmd');
		$filter_order_Dir = $mainframe->getUserStateFromRequest( 'rsvppro_attendees.atd_filter_order_Dir', 'filter_order_Dir', ' ASC', 'word');
		$orderby = ' ORDER BY ' . $filter_order . ' ' . $filter_order_Dir;

		$filter_waiting = $mainframe->getUserStateFromRequest('rsvppro_attendees.atd_filter_waiting', 'filter_waiting', -1, 'int');
		if ($filter_waiting >= 0)
		{
			$where[] = "atdees.waiting =" . $filter_waiting;
		}

		$filter_confirmed = $mainframe->getUserStateFromRequest('rsvppro_attendees.atd_filter_confirmed', 'filter_confirmed', -1, 'int');
		if ($filter_confirmed >= 0)
		{
			$where[] = "atdees.confirmed =" . $filter_confirmed;
		}

		$filter_attendstate = $mainframe->getUserStateFromRequest('rsvppro_attendees.atd_filter_attendstate', 'filter_attendstate', -1, 'int');
		if ($filter_attendstate >= 0)
		{
			$where[] = "atdees.attendstate =" . $filter_attendstate;
		}

		$search = $mainframe->getUserStateFromRequest("atsearch{" . RSVP_COM_COMPONENT . "}", 'search', '');
		$search = $db->escape(trim(strtolower($search)));

		if ($search)
		{
			// New parameterised fields may have a name field that we should use that in the search too
			$fieldsearch = "";
			if ($rsvpdata->template != "")
			{
				$xmlfile = JevTemplateHelper::getTemplate($rsvpdata);
				if (is_int($xmlfile) || file_exists($xmlfile))
				{
					$tempparams = new JevRsvpParameter("", $xmlfile, $rsvpdata, $repeat);
					$tempparams = $tempparams->renderToBasicArray('xmlfile', null);
					foreach ($tempparams as $param)
					{
						// is this a name field?  If so then use this in preference to the user's profile name
						if (isset($param["isname"]) && $param["isname"])
						{
							// do we need to search amongst guest values - which would be an array
							if (isset($param["peruser"]) && $param["peruser"] > 0)
							{
								$fieldsearch = " OR atdees.params LIKE '%\"" . $param["name"] . "\":[\"$search%'";
							}
							// or just the primary attendee
							else
							{
								$fieldsearch = " OR atdees.params LIKE '%\"" . $param["name"] . "\":\"$search%'";
							}
							break;
						}
					}
				}
			}
			$where[] = "(atdees.email_address LIKE '%$search%' OR ju.name LIKE '%$search%' OR ju.username LIKE '%$search%' OR ju.email LIKE '%$search%'  OR atdees.id = '$search' $fieldsearch)";
		}

		if ($repeating)
		{
			$where[] = "atd.allrepeats=1 AND atd.allowregistration>0 ";
		}
		else
		{
			$where[] = "atd.allrepeats=0  AND atd.allowregistration>0 ";
		}

		$query = "SELECT count(distinct atdees.id)"
			. "\n FROM #__jevents_vevent AS ev "
			. "\n LEFT JOIN #__jevents_vevdetail as det ON ev.detail_id=det.evdet_id"
			. "\n LEFT JOIN #__jev_attendance AS atd ON atd.ev_id = ev.ev_id"
			. "\n LEFT JOIN #__jev_attendees AS atdees ON atdees.at_id = atd.id"
			. "\n LEFT JOIN #__users AS ju ON ju.id = atdees.user_id"
			. ( count($join) ? "\n LEFT JOIN  " . implode(' LEFT JOIN ', $join) : '' )
			. ( count($where) ? "\n WHERE " . implode(' AND ', $where) : '' );
		$db->setQuery($query);
		//echo $db->_sql;
		try {
			$total = $db->loadResult();
		}
		catch (RuntimeException $e)
		{
			$error = $e->getMessage();
			echo $error;
			$total = 0;
		}
		//echo (string) $db->getQuery(); echo "<br>";
		//echo $total . "<br>";

		if ($limit > $total)
		{
			$limitstart = 0;
		}

		$namefields = array("ju.username", "ju.name", "ju.name, ju.username","ju.name, ju.email","ju.username, ju.email");

		$namefield = $namefields [$this->params->get("userdatatype", 0)];


		$query = "SELECT det.*, atd.* , atd.id as atd_id, atdc.atdcount, atdc.unapproved, atdc.unconfirmed, atdees.*,atdees.id as atdee_id, ju.username, ju.email, "
			. " CASE WHEN atdees.user_id=0 THEN atdees.email_address ELSE CONCAT_WS(' - ',$namefield,ju.email) END as attendee, "
			. " CASE WHEN atdees.user_id=0 THEN atdees.email_address ELSE ju.name END as name "
			. "\n FROM #__jevents_vevent as ev "
			. "\n LEFT JOIN #__jevents_vevdetail as det ON ev.detail_id=det.evdet_id"
			. "\n LEFT JOIN #__jev_attendance AS atd ON atd.ev_id = ev.ev_id"
			. "\n LEFT JOIN #__jev_attendeecount AS atdc ON atd.id = atdc.at_id"
			. "\n LEFT JOIN #__jev_attendees AS atdees ON atdees.at_id = atd.id"
			. "\n LEFT JOIN #__users AS ju ON ju.id = atdees.user_id"
			. ( count($join) ? "\n LEFT JOIN  " . implode(' LEFT JOIN ', $join) : '' )
			. ( count($where) ? "\n WHERE " . implode(' AND ', $where) : '' )
			. "\n GROUP BY atdees.id $orderby"
		;

		if ($limit > 0)
		{
			$query .= "\n LIMIT $limitstart, $limit";
		}
		$db->setQuery($query);

		try {
			$rows = $db->loadObjectList();
		}
		catch (RuntimeException $e)
		{
			$error = $e->getMessage();
			echo $error;
			$rows = array();
		}
		//echo (string) $db->getQuery(); echo "<br>";
		//var_dump($rows);
		//echo "<br>";

		foreach ($rows as $key => $val)
		{
			// attach repeat into row
			$rows[$key]->eventrepeat = $repeat;
			// Make sure guest count is up to date - BEFORE we count the attendees
			$this->analyseGuests($rows[$key], $rsvpdata, $repeat);
		}

		// Now that the guest count and attendee count is up to date we can count the attendees
		$this->countAttendees($atd_id);

		foreach ($rows as $key => $val)
		{
			// attach repeat into row
			$rows[$key]->eventrepeat = $repeat;

			// New parameterised fields may have a name field that we should use
			if ($rsvpdata->template != "")
			{
				$attendee = & $rows[$key];
				$xmlfile = JevTemplateHelper::getTemplate($rsvpdata);
				if ((is_int($xmlfile) || file_exists($xmlfile)) && ($attendee->lockedtemplate == 0 || $attendee->lockedtemplate == $xmlfile))
				{
					if (isset($attendee->params))
					{
						$params = new JevRsvpParameter($attendee->params, $xmlfile, $rsvpdata, $repeat);
					}
					else
					{
						$params = new JevRsvpParameter("", $xmlfile, $rsvpdata, $repeat);
					}

					$params = $params->renderToBasicArray('xmlfile', $attendee);
					foreach ($params as $param)
					{
						// is this a name field?  If so then use this in preference to the user's profile name
						if (isset($param["isname"]) && $param["isname"])
						{
							if (!$attendee->email && $attendee->name)
							{
								$attendee->email = $attendee->name;
							}
							$value = $param["value"];
							if (is_array($value) && $value[0] != "")
							{
								$attendee->name = $value[0];
								$attendee->attendee = $attendee->name . " - " . $attendee->email;
							}
							else if (!is_array($value) && $value != "")
							{
								$attendee->name = $value;
								$attendee->attendee = $attendee->name . " - " . $attendee->email;
							}
						}
					}
				}
			}
		}

		// update the count to reflect the guest counts!
		$this->countAttendees($atd_id);

		$options = array();
		$options[] = HTMLHelper::_('select.option', '-1', Text::_('RSVP_CONFIRMED'));
		$options[] = HTMLHelper::_('select.option', '0', Text::_('RSVP_PENDING'));
		$options[] = HTMLHelper::_('select.option', '1', Text::_('RSVP_IS_CONFIRMED'));
		$confirmed = HTMLHelper::_('select.genericlist', $options, 'filter_confirmed', 'class="inputbox" size="1" onchange="document.adminForm.task.value=\'attendees.list\';document.adminForm.submit();"', 'value', 'text', $filter_confirmed);

		$options = array();
		$options[] = HTMLHelper::_('select.option', '-1', Text::_('RSVP_IS_WAITING'));
		$options[] = HTMLHelper::_('select.option', '0', Text::_('RSVP_NOT_WAITING'));
		$options[] = HTMLHelper::_('select.option', '1', Text::_('RSVP_WAITING'));
		$waiting = HTMLHelper::_('select.genericlist', $options, 'filter_waiting', 'class="inputbox" size="1" onchange="document.adminForm.task.value=\'attendees.list\';document.adminForm.submit();"', 'value', 'text', $filter_waiting);

		$options = array();
		$options[] = HTMLHelper::_('select.option', '-1', Text::_('RSVP_ATTEND_STATUS'));
		$options[] = HTMLHelper::_('select.option', '0', Text::_('RSVP_NOT_ATTENDING'));
		$options[] = HTMLHelper::_('select.option', '1', Text::_('RSVP_ATTENDING'));
		$options[] = HTMLHelper::_('select.option', '2', Text::_('RSVP_MAYBE_ATTENDING'));
		$options[] = HTMLHelper::_('select.option', '3', Text::_('RSVP_PENDING_APPROVAL'));
		$attendstate = HTMLHelper::_('select.genericlist', $options, 'filter_attendstate', 'class="inputbox" size="1" onchange="document.adminForm.task.value=\'attendees.list\';document.adminForm.submit();"', 'value', 'text', $filter_attendstate);

		jimport('joomla.html.pagination');
		$pageNav = new Pagination($total, $limitstart, $limit);

		// Set the layout
		$this->view->setLayout('overview');

		$this->view->atd_id = $atd_id;
		$this->view->rp_id = $rp_id;
		$this->view->items = $rows;
		$this->view->rsvpdata = $rsvpdata;
		$this->view->event =  $repeat;
		$this->view->confirmed =  $confirmed;
		$this->view->attendstate = $attendstate;
		$this->view->repeat = $repeat;
		$this->view->search = $search;
		$this->view->waiting = $waiting ;
		$this->view->order = $filter_order;
		$this->view->orderdir = $filter_order_Dir;
		$this->view->repeating = $repeating;
		$this->view->pageNav = $pageNav;

		$this->view->display();
	}

	function ticket()
	{
		$attendee = Factory::getApplication()->input->getInt("attendee");

		$pdf = Factory::getApplication()->input->getCmd("task") == "attendees.pdfticket";
		$htmlTicket = Factory::getApplication()->input->getCmd("task") == "attendees.htmlticket";

		$user = Factory::getUser();

		// Check for request forgeries
		//Session::checkToken('request') or jexit( 'Invalid Token' );

		if ($user->id == 0 && !$this->params->get("attendemails", 0))
		{
			return false;
		}

		$db = Factory::getDBO();
		$sql = "SELECT * FROM #__jev_attendees WHERE id=" . $attendee;
		$db->setQuery($sql);
		$attendee = $db->loadObject();
		if (!$attendee)
			return false;
		if ($attendee->attendstate != 1 && $attendee->attendstate != 4)
			return;

		$at_id = Factory::getApplication()->input->getInt("at_id", -1);
		$sql = "SELECT * FROM #__jev_attendance WHERE id=" . $attendee->at_id;
		$db->setQuery($sql);
		$rsvpdata = $db->loadObject();
		if (!$rsvpdata)
			return false;

		if ($attendee->user_id != 0 && $attendee->user_id != $user->id)
		{
			$this->authoriseAttendee($rsvpdata);
		}
		else if ($attendee->user_id == 0 && $user->id > 0)
		{
			$this->authoriseAttendee($rsvpdata);
		}

		if ($rsvpdata->allrepeats)
		{
			$ev_id = $rsvpdata->ev_id;
			$datamodel = new JEventsDataModel();
			$row = $datamodel->queryModel->getEventById($ev_id, 1, "icaldb");
		}
		else
		{
			$rp_id = $attendee->rp_id;
			$datamodel = new JEventsDataModel();
			$row = $datamodel->queryModel->listEventsById($rp_id, 1, "icaldb");
		}
		if (!$row)
			return false;

		if ($attendee->user_id == 0 && $user->id == 0)
		{
			$emailaddress = $this->getEmailAddress();
			if ($emailaddress == "" || strtolower($attendee->email_address) != strtolower($emailaddress))
			{
				echo "Invalid security code - please click the link in your confirmation email";
				return false;
			}
		}

		// get the view
		$this->view = $this->getView("attendees", "html");

		if (isset($attendee->params))
		{
			$xmlfile = JevTemplateHelper::getTemplate($rsvpdata);

			if (is_int($xmlfile) || file_exists($xmlfile))
			{
				$params = new JevRsvpParameter($attendee->params, $xmlfile, $rsvpdata, $row);
				$feesAndBalances = $params->outstandingBalance($attendee);
				$this->view->attendeeParams =  $params;
			}
		}
		else
		{
			return;
		}

		if ($attendee->attendstate == 4)
		{
		    if (!in_array("outstandingbalance", RsvpHelper::getTemplateSpecificParameter($rsvpdata, 'whentickets', array())))
            {
                return;
            }
		}


		// Set the layout
		$this->view->setLayout('ticket');
		$this->view->event = $row;
		$this->view->rsvpdata = $rsvpdata;
		$this->view->attendee = $attendee;
		$this->view->pdf = $pdf;
		$this->view->htmlTicket = $htmlTicket;

		Factory::getApplication()->input->set("atd_id", array($rsvpdata->id . "|" . $row->rp_id()));
		$this->view->display();
	}

	/**
	 * Export Attendees
	 *
	 */
	function export()
	{
		// get the view
		$this->view = $this->getView("attendees", "html");

		$mainframe = Factory::getApplication();
		$option = Factory::getApplication()->input->getCmd("option");

		$db = Factory::getDBO();

		$atd_id = Factory::getApplication()->input->get("atd_id", array(), 'ARRAY');
		if (!isset($atd_id[0]) || strpos($atd_id[0], "|") === false)
		{
			throw new Exception(Text::_("RSVP_MISSING_ATDID"), 403);
		}
		list($atd_id, $rp_id) = explode("|", $atd_id[0]);

		$atd_id = intval($atd_id);
		$rp_id = intval($rp_id);

		$repeating = Factory::getApplication()->input->getInt('repeating', 0);

		$where = array();
		$join = array();

		$where[] = "ev.ev_id IS NOT NULL";
		$where[] = "atd.id = $atd_id";
		$where[] = "atdees.rp_id = $rp_id";

		$cid = Factory::getApplication()->input->get("cid", array(0), 'ARRAY');
		$cid = ArrayHelper::toInteger($cid);
		if (count($cid) > 0 && $cid[0] != 0)
		{
			$where[] = "atdees.id in (" . implode(",", $cid) . ")";
		}

		$filter_order = $mainframe->getUserStateFromRequest('rsvppro_attendees.atd_filter_order', 'filter_order', 'atd.id', 'cmd');
		$filter_order_Dir = $mainframe->getUserStateFromRequest('rsvppro_attendees.atd_filter_order_Dir', 'filter_order_Dir', ' ASC', 'word');
		$orderby = ' ORDER BY ' . $filter_order . ' ' . $filter_order_Dir;

		$filter_waiting = $mainframe->getUserStateFromRequest('rsvppro_attendees.atd_filter_waiting', 'filter_waiting', -1, 'int');
		if ($filter_waiting >= 0)
		{
			$where[] = "atdees.waiting =" . $filter_waiting;
		}

		$filter_confirmed = $mainframe->getUserStateFromRequest('rsvppro_attendees.atd_filter_confirmed', 'filter_confirmed', -1, 'int');
		if ($filter_confirmed >= 0)
		{
			$where[] = "atdees.confirmed =" . $filter_confirmed;
		}

		$search = $mainframe->getUserStateFromRequest("search{" . RSVP_COM_COMPONENT . "}", 'search', '');
		$search = $db->escape(trim(strtolower($search)));

		if ($search)
		{
			$where[] = "(atdees.email_address LIKE '%$search%' OR ju.username LIKE '%$search%' OR ju.email LIKE '%$search%' )";
		}

		if ($repeating)
		{
			$where[] = "atd.allrepeats=1 AND atd.allowregistration>0";
		}
		else
		{
			$where[] = "atd.allrepeats=0  AND atd.allowregistration>0 ";
		}

		$query = "SELECT det.*, atd.* , atd.id as atd_id, atdc.atdcount, atdc.unapproved, atdc.unconfirmed, atdees.*,atdees.id as atdee_id,  "
			. " CASE WHEN atdees.user_id=0 THEN atdees.email_address ELSE CONCAT_WS(' - ',ju.username,ju.email) END as attendee, "
			. " CASE WHEN atdees.user_id=0 THEN '' ELSE ju.name END as attendeename, "
			. " CASE WHEN atdees.user_id=0 THEN '' ELSE ju.username END as attendeeusername, "
			. " CASE WHEN atdees.user_id=0 THEN atdees.email_address ELSE ju.email END as attendeemail "
			. "\n FROM #__jevents_vevent as ev "
			. "\n LEFT JOIN #__jevents_vevdetail as det ON ev.detail_id=det.evdet_id"
			. "\n LEFT JOIN #__jev_attendance AS atd ON atd.ev_id = ev.ev_id"
			. "\n LEFT JOIN #__jev_attendeecount AS atdc ON atd.id = atdc.at_id"
			. "\n LEFT JOIN #__jev_attendees AS atdees ON atdees.at_id = atd.id"
			. "\n LEFT JOIN #__users AS ju ON ju.id = atdees.user_id"
			. ( count($join) ? "\n LEFT JOIN  " . implode(' LEFT JOIN ', $join) : '' )
			. ( count($where) ? "\n WHERE " . implode(' AND ', $where) : '' )
			. "\n GROUP BY atdees.id $orderby"
		;
		/*
		  if ($limit>0){
		  $query .= "\n LIMIT $limitstart, $limit";
		  }
		 */
		$db->setQuery($query);

		try {
			$rows = $db->loadObjectList();
		}
		catch (RuntimeException $e)
		{
			$error = $e->getMessage();
			echo $error;
			$rows = array();
		}

		if (count($rows) > 0)
		{

			$row = $rows[0];

			$db = Factory::getDBO();
			$sql = "SELECT * FROM #__jev_attendance WHERE id=" . $row->at_id;
			$db->setQuery($sql);
			$rsvpdata = $db->loadObject();
		}
		else
		{
			$rsvpdata = false;
			return;
		}

		$vevent = $this->dataModel->queryModel->getEventById($rsvpdata->ev_id, true, "icaldb");
		if ($rp_id == 0)
		{
			$repeat = $vevent->getFirstRepeat();
		}
		else
		{
			list($year, $month, $day) = JEVHelper::getYMD();
			// include unpublished events
			$repeat = $this->dataModel->queryModel->listEventsById(intval($rp_id), true, "icaldb");
			//$repeatdata = $this->dataModel->getEventData(intval($rp_id), "icaldb", $year, $month, $day);
			//if ($repeatdata && isset($repeatdata["row"]))
			//	$repeat = $repeatdata["row"];
		}

		$this->authoriseTransactions($repeat, false);

		// Store details in registry - will need them for waiting lists!
		$registry = JevRegistry::getinstance("jevents");
		$registry->set("rsvpdata", $rsvpdata);
		$registry->set("event", $repeat);


		// attach repeat into row
		foreach ($rows as $key => $val)
		{
			$rows[$key]->eventrepeat = $repeat;
			// Make sure guest count is up to date
			//$this->analyseGuests($rows[$key], $rsvpdata, $repeat);  // not for exports
		}
		// update the count to reflect the guest counts!
		//$this->countAttendees($atd_id);  // not for exports
		// Set the layout
		$this->view->setLayout('export');

		$this->view->rsvpdata = $rsvpdata;
		$this->view->atd_id = $atd_id;
		$this->view->rp_id = $rp_id;
		$this->view->items = $rows;
		$this->view->order = $filter_order;
		$this->view->orderdir = $filter_order_Dir;
		$this->view->repeat = $repeat;

		$this->view->displayNoWrapper();
	}

	function message()
	{
		$atd_id = Factory::getApplication()->input->get("atd_id", array(), 'ARRAY');
		if (!isset($atd_id[0]) || strpos($atd_id[0], "|") === false)
		{
			throw new Exception( Text::_("RSVP_MISSING_ATDID"), 403);
		}
		list($atd_id, $rp_id) = explode("|", $atd_id[0]);
		$atd_id = intval($atd_id);
		$rp_id = intval($rp_id);

		$cid = Factory::getApplication()->input->get("cid", array(), 'ARRAY');;
		$cid = ArrayHelper::toInteger($cid);
		if (count($cid) == 0 || !isset($cid[0]) || $cid[0] < 0)
		{
			throw new Exception(Text::_("RSVP_MISSING_ATTENDEE"), 403);
		}
		$message = Factory::getApplication()->input->getRaw("message", "", 'default');
		// If the allow HTML flag is set, apply a safe HTML filter to the variable
		$safeHtmlFilter = JFilterInput::getInstance(null, null, 1, 1);
		$message = $safeHtmlFilter->clean($message, null);

		$subject = Factory::getApplication()->input->getString("subject", "");
		$bcc = Factory::getApplication()->input->getString("messagebcc", "");
		$replyto = Factory::getApplication()->input->getString("replyto", null);
		if ($replyto == "")
		{
			$replyto = null;
		}

		$repeating = Factory::getApplication()->input->getInt("repeating", 0);
		$singlebcc = Factory::getApplication()->input->getInt("singlebcc", 0);

		$this->authoriseTransactions(false, $cid[0]);

		$db = Factory::getDBO();
		$sql = "SELECT * FROM #__jev_attendance WHERE id=" . intval($atd_id);
		$db->setQuery($sql);
		$rsvpdata = $db->loadObject();

		$this->dataModel = new JEventsDataModel();
		$this->queryModel = new JEventsDBModel($this->dataModel);

		// Find the first repeat - including unpublished events
		$vevent = $this->dataModel->queryModel->getEventById($rsvpdata->ev_id, true, "icaldb");
		if ($rp_id == 0)
		{
			$repeat = $vevent->getFirstRepeat();
		}
		else
		{
			list($year, $month, $day) = JEVHelper::getYMD();
			// include unpublished events
			$repeat = $this->dataModel->queryModel->listEventsById(intval($rp_id), 1, "icaldb");
			//$repeatdata = $this->dataModel->getEventData(intval($rp_id), "icaldb", $year, $month, $day);
			//if ($repeatdata && isset($repeatdata["row"]))
			//	$repeat = $repeatdata["row"];
		}

		$user = Factory::getUser();
		if ($user->id != $repeat->created_by() && !JEVHelper::isAdminUser($user) && !JEVHelper::canDeleteEvent($repeat, $user) && !$user->authorise('core.sendmessage', 'com_rsvppro'))
		{
			return;
		}

		if ($message != "" && $subject != "")
		{
			// Override BCC if required

			if ($bcc != "")
			{
				RsvpHelper::setTemplateSpecificParameter($rsvpdata,  "notifybcc", $bcc);
			}

			// send the message
			$db = Factory::getDBO();
			$sql = "SELECT * FROM #__jev_attendees WHERE id IN (" . implode(",", $cid) . ")";
			$db->setQuery($sql);
			$attendees = $db->loadObjectList();

			$row = $repeat;

			$sent = 0;
			foreach ($attendees as $attendee)
			{

				$user = JEVHelper::getUser($attendee->user_id);
				if ($user->id == 0 && $this->params->get("attendemails", 0))
				{
					$name = $attendee->email_address;
					$username = $attendee->email_address;
				}
				else
				{
					$name = $user->name;
					$username = $user->username;
				}
				$onWaitingList = false;
				$success = $this->notifyUser($rsvpdata, $row, $user, $name, $username, $attendee, 'SPECIAL_MESSAGE', $onWaitingList, false, $subject, $message, false, $replyto);
				$sent += $success ? 1 : 0;
				if ($singlebcc == 1) {
					/* Clear the notify bcc param so we don't resend the email to them for each attendee */
					RsvpHelper::setTemplateSpecificParameter($rsvpdata,'notifybcc', null);
				}
			}
			// send the copy to the creatpr
			if (isset($attendee))
			{
				$creator = JEVHelper::getUser($repeat->created_by());
				$onWaitingList = false;
				$success = $this->notifyUser($rsvpdata, $row, $creator, $creator->name, "", $attendee, 'SPECIAL_MESSAGE', $onWaitingList, false, $subject, $message, false, $replyto);
				$sent += $success ? 1 : 0;
			}

			$this->setRedirect("index.php?option=com_rsvppro&task=attendees.overview&atd_id[]=" . $atd_id . "|" . $rp_id . "&repeating=" . $repeating . "&Itemid=" . Factory::getApplication()->input->getInt("Itemid", 0), Text::sprintf("RSVP_MESSAGES_SENT", $sent));
			return;
		}

		// get the view
		$this->view = $this->getView("attendees", "html");
		// Set the layout
		$this->view->setLayout('message');

		$this->view->rsvpdata = $rsvpdata;
		$this->view->cid = $cid;
		$this->view->repeat = $repeat;
		$this->view->repeating = $repeating;
		$this->view->atd_id = $atd_id;
		$this->view->rp_id = $rp_id;
		$this->view->mymessage = $message;
		$this->view->subject = $subject;

		$this->view->params = ComponentHelper::getParams("com_rsvppro");

		$this->view->display();
	}

	function notwaiting()
	{
		$atd_id = Factory::getApplication()->input->get("atd_id", array(), 'ARRAY');
		if (!isset($atd_id[0]) || strpos($atd_id[0], "|") === false)
		{
			throw new Exception(Text::_("RSVP_MISSING_ATDID"), 403);
		}

		$cid = Factory::getApplication()->input->get("cid", array(), 'ARRAY');;
		$cid = ArrayHelper::toInteger($cid);
		if (count($cid) == 0 || !isset($cid[0]) || $cid[0] < 0)
		{
			throw new Exception( Text::_("RSVP_MISSING_ATTENDEE"), 403);
		}

		$repeating = Factory::getApplication()->input->getInt("repeating", 0);

		$atdee = $cid[0];

		$this->authoriseTransactions(false, $cid[0]);

		$db = Factory::getDBO();
		$now = JevDate::getDate("+0 seconds");
		$db->setQuery("UPDATE #__jev_attendees set waiting=0, modified=" . $db->Quote($now->toMySql()) . " where id=" . $atdee);
		$db->execute();

		$params = ComponentHelper::getParams("com_rsvppro");

		// Now make sure the atdcount is in sync - NB only update status of CONFIRMED attendees!
		$db->setQuery("SELECT atdees.id as atdee_id , atdees.*, atd.*, atdcnt.* FROM #__jev_attendance as atd
		LEFT JOIN #__jev_attendees as atdees ON atdees.at_id=atd.id
		LEFT JOIN #__jev_attendeecount as atdcnt ON atdcnt.at_id=atd.id AND atdcnt.rp_id=atdees.rp_id
		WHERE atdees.id=$atdee ");
		$row = $db->loadObject();

		$sql = "SELECT * FROM #__jev_attendance WHERE id=" . $row->at_id;
		$db->setQuery($sql);
		$rsvpdata = $db->loadObject();

		$datamodel = new JEventsDataModel();
		if ($row->rp_id > 0)
		{
			$evt = $datamodel->queryModel->listEventsById($row->rp_id, 1, "icaldb");
		}
		else
		{
			$evt = $datamodel->queryModel->getEventById($rsvpdata->ev_id, 1, "icaldb");
		}

        // I may also need to trigger a plugin to say that the attendee is not waiting
		PluginHelper::importPlugin("rsvppro");
		Factory::getApplication()->triggerEvent( 'onRegistrationStatusChange' , array($row, $evt, $rsvpdata));

		// auto remind attendees
		if ($params->get("autoremind", 0) == 1 && $evt)
		{

			// always allow auto reminders for attendees if forced reminders enabled
			if ($rsvpdata->allowreminders || $this->params->get("forceautoremind", 0) == 1)
			{
				// create reminder
				// NB email address must be in the request object
				$user = JEVHelper::getUser($row->user_id);
				// create reminder
				$this->jevrreminders->remindUser($rsvpdata, $evt, $user, $row->email_address);
				$mainframe = Factory::getApplication();
				$mainframe->enqueueMessage(Text::_("JEV_REMINDER_CONFIRMED"));
			}
		}

		if ($params->get("notifyuser", 0) && $evt)
		{
			// I need the event object to be able to send the message
			if (!isset($this->helper->event) || !$this->helper->event)
			{
				$this->helper->event = $evt;
			}

			$registry = JevRegistry::getinstance("jevents");
			$registry->set("rsvpdata", $rsvpdata);
			$registry->set("event", $evt);

			$this->helper->notifyWaitingUser($row);
		}


		$this->setRedirect("index.php?option=com_rsvppro&task=attendees.overview&atd_id[]=" . $atd_id[0] . "&repeating=" . $repeating . "&Itemid=" . Factory::getApplication()->input->getInt("Itemid", 0), Text::_("RSVP_ATTENDEE_NO_LONGER_ON_WAITING_LIST"));
	}

	function waiting()
	{
		$atd_id = Factory::getApplication()->input->get("atd_id", array(), 'ARRAY');
		if (!isset($atd_id[0]) || strpos($atd_id[0], "|") === false)
		{
			throw new Exception( Text::_("RSVP_MISSING_ATDID"), 403);
		}

		$cid = Factory::getApplication()->input->get("cid", array(), 'ARRAY');;
		$cid = ArrayHelper::toInteger($cid);
		if (count($cid) == 0 || !isset($cid[0]) || $cid[0] < 0)
		{
			throw new Exception( Text::_("RSVP_MISSING_ATTENDEE"), 403);
		}

		$repeating = Factory::getApplication()->input->getInt("repeating", 0);

		$atdee = $cid[0];

		$this->authoriseTransactions(false, $cid[0]);

		$db = Factory::getDBO();
		$now = JevDate::getDate("+0 seconds");
		$db->setQuery("UPDATE #__jev_attendees set waiting=1, modified=" . $db->Quote($now->toMySql()) . " where id=" . $atdee);
		$db->execute();

		$params = ComponentHelper::getParams("com_rsvppro");

		// Now make sure the atdcount is in sync - NB only update status of CONFIRMED attendees!
		$db->setQuery("SELECT atdees.id as atdee_id , atdees.*, atd.*, atdcnt.* FROM #__jev_attendance as atd
		LEFT JOIN #__jev_attendees as atdees ON atdees.at_id=atd.id
		LEFT JOIN #__jev_attendeecount as atdcnt ON atdcnt.at_id=atd.id AND atdcnt.rp_id=atdees.rp_id
		WHERE atdees.id=$atdee ");
		$row = $db->loadObject();

		$sql = "SELECT * FROM #__jev_attendance WHERE id=" . $row->at_id;
		$db->setQuery($sql);
		$rsvpdata = $db->loadObject();

		$datamodel = new JEventsDataModel();
		if ($row->rp_id > 0)
		{
			$evt = $datamodel->queryModel->listEventsById($row->rp_id, 1, "icaldb");
		}
		else
		{
			$evt = $datamodel->queryModel->getEventById($rsvpdata->ev_id, 1, "icaldb");
		}

		// I may also need to trigger a plugin to say that the attendee is  waiting
		PluginHelper::importPlugin("rsvppro");
		Factory::getApplication()->triggerEvent( 'onRegistrationStatusChange' , array($row, $evt, $rsvpdata));

		// auto remind attendees
		if ($params->get("autoremind", 0) == 1 && $evt)
		{

			// always allow auto reminders for attendees if forced reminders enabled
			if ($rsvpdata->allowreminders || $this->params->get("forceautoremind", 0) == 1)
			{
				// create reminder
				// NB email address must be in the request object
				$user = JEVHelper::getUser($row->user_id);
				// create reminder
				$this->jevrreminders->remindUser($rsvpdata, $evt, $user, $row->email_address);
				$mainframe = Factory::getApplication();
				$mainframe->enqueueMessage(Text::_("JEV_REMINDER_CONFIRMED"));
			}
		}

		if ($params->get("notifyuser", 0) && $evt)
		{
			// I need the event object to be able to send the message
			if (!isset($this->helper->event) || !$this->helper->event)
			{
				$this->helper->event = $evt;
			}

			$registry = JevRegistry::getinstance("jevents");
			$registry->set("rsvpdata", $rsvpdata);
			$registry->set("event", $evt);

			$this->helper->notifyWaitingUser($row);
		}


		$this->setRedirect("index.php?option=com_rsvppro&task=attendees.overview&atd_id[]=" . $atd_id[0] . "&repeating=" . $repeating . "&Itemid=" . Factory::getApplication()->input->getInt("Itemid", 0), Text::_("RSVP_ATTENDEE_IS_NOW_ON_WAITING_LIST"));
	}

	function confirm()
	{
		$atd_id = Factory::getApplication()->input->get("atd_id", array(), 'ARRAY');
		if (!isset($atd_id[0]) || strpos($atd_id[0], "|") === false)
		{
			throw new Exception( Text::_("RSVP_MISSING_ATDID"), 403);
		}

		$cid = Factory::getApplication()->input->get("cid", array(), 'ARRAY');;
		$cid = ArrayHelper::toInteger($cid);
		if (count($cid) == 0 || !isset($cid[0]) || $cid[0] < 0)
		{
			throw new Exception( Text::_("RSVP_MISSING_ATTENDEE"), 403);
		}

		$repeating = Factory::getApplication()->input->getInt("repeating", 0);

		$atdee = $cid[0];

		$this->authoriseTransactions(false, $cid[0]);

		$db = Factory::getDBO();
		$now = JevDate::getDate("+0 seconds");
		$db->setQuery("UPDATE #__jev_attendees set confirmed=1, modified=" . $db->Quote($now->toMySql()) . " where id=" . $atdee);
		$db->execute();

		$this->postUpdateActions($this->rsvpdata, $this->event, $this->attendee, 0, $this->getRSVPParmeters($this->rsvpdata, $this->event));

		$this->setRedirect("index.php?option=com_rsvppro&task=attendees.overview&atd_id[]=" . $atd_id[0] . "&repeating=" . $repeating . "&Itemid=" . Factory::getApplication()->input->getInt("Itemid", 0), Text::_("RSVP_ATTENDEE_CONFIRMED"));
	}

	function unconfirm()
	{
		$atd_id = Factory::getApplication()->input->get("atd_id", array(), 'ARRAY');
		if (!isset($atd_id[0]) || strpos($atd_id[0], "|") === false)
		{
			throw new Exception( Text::_("RSVP_MISSING_ATDID"), 403);
		}

		$cid = Factory::getApplication()->input->get("cid", array(), 'ARRAY');;
		$cid = ArrayHelper::toInteger($cid);
		if (count($cid) == 0 || !isset($cid[0]) || $cid[0] < 0)
		{
			throw new Exception( Text::_("RSVP_MISSING_ATTENDEE"), 403);
		}

		$repeating = Factory::getApplication()->input->getInt("repeating", 0);

		$atdee = $cid[0];

		$this->authoriseTransactions(false, $cid[0]);

		$db = Factory::getDBO();
		$now = JevDate::getDate("+0 seconds");
		$db->setQuery("UPDATE #__jev_attendees set confirmed=0, modified=" . $db->Quote($now->toMySql()) . " where id=" . $atdee);
		$db->execute();

		$this->postUpdateActions($this->rsvpdata, $this->event, $this->attendee, 0, $this->getRSVPParmeters($this->rsvpdata, $this->event));

		$this->setRedirect("index.php?option=com_rsvppro&task=attendees.overview&atd_id[]=" . $atd_id[0] . "&repeating=" . $repeating . "&Itemid=" . Factory::getApplication()->input->getInt("Itemid", 0), Text::_("RSVP_ATTENDEE_UNCONFIRMED"));
	}

	function remindconfirm()
	{
		$atd_id = Factory::getApplication()->input->get("atd_id", array(), 'ARRAY');
		if (!isset($atd_id[0]) || strpos($atd_id[0], "|") === false)
		{
			throw new Exception( Text::_("RSVP_MISSING_ATDID"), 403);
		}

		$cid = Factory::getApplication()->input->get("cid", array(), 'ARRAY');;
		$cid = ArrayHelper::toInteger($cid);
		if (count($cid) == 0 || !isset($cid[0]) || $cid[0] < 0)
		{
			throw new Exception( Text::_("RSVP_MISSING_ATTENDEE"), 403);
		}

		list($atd_id, $rp_id) = explode("|", $atd_id[0]);
		$atd_id = intval($atd_id);
		$rp_id = intval($rp_id);

		$repeating = Factory::getApplication()->input->getInt("repeating", 0);

		$atdee = $cid[0];

		$this->authoriseTransactions(false, $cid[0]);

		$db = Factory::getDbo();
		$sql = "SELECT * FROM #__jev_attendance WHERE id=" . $atd_id;
		$db->setQuery($sql);
		$rsvpdata = $db->loadObject();

		$vevent = $this->dataModel->queryModel->getEventById($rsvpdata->ev_id, false, "icaldb");
		if ($rp_id == 0)
		{
			$repeat = $vevent->getFirstRepeat();
		}
		else
		{
			list($year, $month, $day) = JEVHelper::getYMD();
			$repeatdata = $this->dataModel->getEventData($rp_id, "icaldb", $year, $month, $day);
			if ($repeatdata && isset($repeatdata["row"]))
				$repeat = $repeatdata["row"];
		}

		// send email with confirmation details
		$creator = JEVHelper::getUser($repeat->created_by());
		if ($creator)
		{
			$sql = "SELECT * FROM #__jev_attendees WHERE id=" . $cid[0];
			$db->setQuery($sql);
			$attendee = $db->loadObject();
			if ($attendee->email_address == "")
			{
				return;
			}
			$emailaddress = $attendee->email_address;
			$code = base64_encode($emailaddress . ":" . md5($this->params->get("emailkey", "email key") . $emailaddress));
			list($year, $month, $day) = JEVHelper::getYMD();
			$link = Route::_($repeat->viewDetailLink($year, $month, $day, false) . "&em=" . $code, false);

			if (strpos($link, "/") !== 0)
			{
				$link = "/" . $link;
			}

			$uri = JURI::getInstance(JURI::base());
			$root = $uri->toString(array('scheme', 'host', 'port'));

			$link = $root . $link;

			// confirm email message
			$user = JEVHelper::getUser($attendee->user_id);
			$this->notifyUser($rsvpdata, $repeat, $user, $emailaddress, $emailaddress, $attendee, 'cem', $attendee->waiting, "", "", $link);
			/*
			  $subject = Text::sprintf("JEV_CONFIRM_EMAIL_ATTEND_SUBJECT", $repeat->title());
			  $message = Text::sprintf("JEV_CONFIRM_EMAIL_ATTEND", $link, $repeat->title(), $repeat->title());
			  if ($rsvpdata->allowcancellation)
			  {
			  $message .= Text::_("JEV_ATTENDANCE_PENDING");
			  }

			  $this->helper->sendMail($creator->email, $creator->name, $emailaddress, $subject, $message, 1);
			 */
		}

		$this->setRedirect("index.php?option=com_rsvppro&task=attendees.overview&atd_id[]=" . $atd_id . "|" . $rp_id . "&repeating=" . $repeating . "&Itemid=" . Factory::getApplication()->input->getInt("Itemid", 0), Text::_("RSVP_ATTENDEE_REMINDED"));
	}

	function approve()
	{
		$atd_id = Factory::getApplication()->input->get("atd_id", array(), 'ARRAY');
		if (!isset($atd_id[0]) || strpos($atd_id[0], "|") === false)
		{
			throw new Exception( Text::_("RSVP_MISSING_ATDID"), 403);
		}
		list($atd_id, $rp_id) = explode("|", $atd_id[0]);
		$atd_id = intval($atd_id);
		$rp_id = intval($rp_id);

		$cid = Factory::getApplication()->input->get("cid", array(), 'ARRAY');;
		$cid = ArrayHelper::toInteger($cid);
		if (count($cid) == 0 || !isset($cid[0]) || $cid[0] < 0)
		{
			throw new Exception( Text::_("RSVP_MISSING_ATTENDEE"), 403);
		}

		$repeating = Factory::getApplication()->input->getInt("repeating", 0);

		$atdee = $cid[0];

		$this->authoriseTransactions(false, $cid[0]);

		$db = Factory::getDBO();
		$now = JevDate::getDate("+0 seconds");
		$db->setQuery("UPDATE #__jev_attendees set attendstate=1, modified=" . $db->Quote($now->toMySql()) . " where id=" . $atdee);
		$db->execute();

		// notify user?
		if ($this->params->get("notifyapproved", 0))
		{

			$sql = "SELECT * FROM #__jev_attendance WHERE id=" . $atd_id;
			$db->setQuery($sql);
			$rsvpdata = $db->loadObject();

			$vevent = $this->dataModel->queryModel->getEventById($rsvpdata->ev_id, true, "icaldb");
			if ($rp_id == 0)
			{
				$repeat = $vevent->getFirstRepeat();
			}
			else
			{
				list($year, $month, $day) = JEVHelper::getYMD();
				// include unpublished events
				$repeat = $this->dataModel->queryModel->listEventsById(intval($rp_id), Factory::getApplication()->isClient('administrator'), "icaldb");
				//$repeatdata = $this->dataModel->getEventData(intval($rp_id), "icaldb", $year, $month, $day);
				//if ($repeatdata && isset($repeatdata["row"]))
				//	$repeat = $repeatdata["row"];
			}

			Table::addIncludePath(RSVP_TABLES);
			$attendee = Table::getInstance('jev_attendee');

			if ($atdee > 0)
			{
				$attendee->load($atdee);
			}

			$user = JEVHelper::getUser($attendee->user_id);
			if ($user->id == 0 && $this->params->get("attendemails", 0))
			{
				$name = $attendee->email_address;
				$username = $attendee->email_address;
			}
			else
			{
				$name = $user->name;
				$username = $user->username;
			}

			$onWaitingList = false;
			$this->notifyUser($rsvpdata, $repeat, $user, $name, $username, $attendee, 'approve', false);
		}


		$this->setRedirect("index.php?option=com_rsvppro&task=attendees.overview&atd_id[]=" . $atd_id . "|" . $rp_id . "&repeating=" . $repeating . "&Itemid=" . Factory::getApplication()->input->getInt("Itemid", 0), Text::_("JEV_ATTENDANCE_APPROVED"));
	}

	function updatenotes()
	{
		$atd_id = Factory::getApplication()->input->get("atd_id",array(), 'ARRAY');
		if (!isset($atd_id[0]) || strpos($atd_id[0], "|") === false)
		{
			throw new Exception( Text::_("RSVP_MISSING_ATDID"), 403);
		}

		$cid = Factory::getApplication()->input->get("cid", array(0), 'ARRAY');
		$cid = ArrayHelper::toInteger($cid);
		if (count($cid) == 0 || !isset($cid[0]) || $cid[0] < 0)
		{
			throw new Exception( Text::_("RSVP_MISSING_ATTENDEE"), 403);
		}

		$repeating = Factory::getApplication()->input->getInt("repeating", 0);

		$atdee = $cid[0];
		$notes = Factory::getApplication()->input->get("notes",array(), 'ARRAY');
		if (array_key_exists($atdee, $notes))
		{
			$notes = $notes[$atdee];
		}
		else if (array_key_exists(0, $notes))
		{
			$notes = $notes[0];
		}
		else
		{
			$notes = "";
		}

		if (!$this->authoriseTransactions(false, $atdee))
		{
			return;
		}

		// which guests
		Table::addIncludePath(RSVP_TABLES);
		$attendee = Table::getInstance('jev_attendee');
		$attendee->load($atdee);

		$db = Factory::getDBO();
		$now = JevDate::getDate("+0 seconds");
		$db->setQuery("UPDATE #__jev_attendees set modified=" . $db->Quote($now->toMySql()) . ", notes=" . $db->quote($notes) . " where id=" . $atdee);
		$db->execute();

		$this->setRedirect("index.php?option=com_rsvppro&task=attendees.overview&atd_id[]=" . $atd_id[0] . "&repeating=" . $repeating . "&Itemid=" . Factory::getApplication()->input->getInt("Itemid", 0), Text::_("JEV_NOTES_UPDATED"));
	}

	function attend()
	{
		$atd_id = Factory::getApplication()->input->get("atd_id",array(), 'ARRAY');
		if (!isset($atd_id[0]) || strpos($atd_id[0], "|") === false)
		{
			throw new Exception( Text::_("RSVP_MISSING_ATDID"), 403);
		}

		$cid = Factory::getApplication()->input->get("cid", array(0), 'ARRAY');
		$cid = ArrayHelper::toInteger($cid);
		if (count($cid) == 0 || !isset($cid[0]) || $cid[0] < 0)
		{
			throw new Exception( Text::_("RSVP_MISSING_ATTENDEE"), 403);
		}

		$repeating = Factory::getApplication()->input->getInt("repeating", 0);
		$reset = Factory::getApplication()->input->getInt("reset", 0);

		foreach ($cid as $atdee)
		{

			$notes = Factory::getApplication()->input->get("notes",array(), 'ARRAY');
			if (array_key_exists($atdee, $notes))
			{
				$notes = $notes[$atdee];
			}
			else if (array_key_exists(0, $notes))
			{
				$notes = $notes[0];
			}
			else
			{
				$notes = "";
			}

			if (!$this->authoriseTransactions(false, $atdee))
			{
				return;
			}

			// which guests
			Table::addIncludePath(RSVP_TABLES);
			$attendee = Table::getInstance('jev_attendee');
			$attendee->load($atdee);

			if ($reset)
			{
				// If a repeating event without being separate attendance (repeating) and we are resetting the did attend state then do it here
				$db  = Factory::getDBO();
				$sql = "SELECT * FROM #__jev_attendance WHERE id=" . intval($attendee->at_id);
				$db->setQuery($sql);
				$rsvpdata = $db->loadObject();
				if (!$rsvpdata)
				{
					throw new Exception( Text::_("RSVP_MISSING_ATDID"), 403);
				}

				$this->dataModel  = new JEventsDataModel();
				$this->queryModel = new JEventsDBModel($this->dataModel);

				// Find the first repeat - including unpublished events
				$vevent           = $this->dataModel->queryModel->getEventById($rsvpdata->ev_id, true, "icaldb");
				$mostrecentrepeat = $vevent->getMostRecentRepeat();

				if ($reset && $mostrecentrepeat
                    && !$repeating && $attendee->rp_id == 0
                    && $attendee->guestattend !== ""
				    && $attendee->modified < $mostrecentrepeat->_endrepeat
                )
				{
					// Resetting all the guest attend to blank to start again!
					// So if there are any guestattend values already and the modified date is after the last repeat then we have already done this
					$db  = Factory::getDBO();
					$now = JevDate::getDate("+0 seconds");
					$db->setQuery("UPDATE #__jev_attendees set didattend=0 , modified=" . $db->Quote($now->toMySql()) . ", notes=" . $db->quote($notes) . ", guestattend =" . $db->quote('') . " where id=" . $atdee);
					$db->execute();

					$attendee->load($atdee);
				}
			}

			$gc = Factory::getApplication()->input->getInt("gc", 0, "method");
			// This is clicking the icon - all guests attending
			if ($gc == 0)
			{

				$guests = array();
				for ($g = 1; $g <= $attendee->guestcount; $g++)
				{
					$guests[] = $g;
				}
				$guestatttend = implode(",", $guests);
			}
			else
			{
				$guests = $attendee->guestattend != "" ? explode(",", $attendee->guestattend) : array();
				if (!in_array($gc, $guests))
				{
					$guests[] = $gc;
				}
				else
				{
					$lang = Factory::getLanguage();
					$lang->load(RSVP_COM_COMPONENT, JPATH_ADMINISTRATOR);
					echo "<div style='height:100%;width:100%;background-color:red;padding:200px 50px;'><H1 style='padding:100px 20px!important;background-color:white!important;color:red!important;'>" .
					Text::_("RSVP_ATTENDANCE_ALREADY_RECORDED")
					. "</h1></div>";
					Factory::getApplication()->input->set("tmpl", "component");
					return false;
				}
				sort($guests);
				$guestatttend = implode(",", $guests);
			}

			$db = Factory::getDBO();
			$now = JevDate::getDate("+0 seconds");
			$db->setQuery("UPDATE #__jev_attendees set didattend=1 , modified=" . $db->Quote($now->toMySql()) . ", notes=" . $db->quote($notes) . ", guestattend =" . $db->quote($guestatttend) . " where id=" . $atdee);
			$db->execute();
		}
		$notes = Factory::getApplication()->input->getString("notes");
		if (is_array($notes) && isset($notes[0]) && $notes[0] == "qrscanned")
		{
			$msg = "<div style='height:100%;width:100%;background-color:green;padding:100px 50px;'><H1 style='padding:100px 20px!important;background-color:white!important;color;green!important;'>" .
				Text::_("JEV_ATTENDANCE_MARKED")
				. "</h1></div>";
			Factory::getApplication()->input->set("tmpl", "component");
			$this->setRedirect("index.php?option=com_rsvppro&task=attendees.overview&atd_id[]=" . $atd_id[0] . "&repeating=" . $repeating . "&Itemid=" . Factory::getApplication()->input->getInt("Itemid", 0) . "&tmpl=component", $msg);
			return;
		}

		$this->setRedirect("index.php?option=com_rsvppro&task=attendees.overview&atd_id[]=" . $atd_id[0] . "&repeating=" . $repeating . "&Itemid=" . Factory::getApplication()->input->getInt("Itemid", 0), Text::_("JEV_ATTENDANCE_MARKED"));
	}

	function notattend()
	{
		$atd_id = Factory::getApplication()->input->get("atd_id", array(), 'ARRAY');
		if (!isset($atd_id[0]) || strpos($atd_id[0], "|") === false)
		{
			throw new Exception( Text::_("RSVP_MISSING_ATDID"), 403);
		}

		$cid = Factory::getApplication()->input->get("cid", array(), 'ARRAY');;
		$cid = ArrayHelper::toInteger($cid);
		if (count($cid) == 0 || !isset($cid[0]) || $cid[0] < 0)
		{
			throw new Exception( Text::_("RSVP_MISSING_ATTENDEE"), 403);
		}

		$repeating = Factory::getApplication()->input->getInt("repeating", 0);

		foreach ($cid as $atdee)
		{
			$notes = Factory::getApplication()->input->get("notes",array(), 'ARRAY');
			if (array_key_exists($atdee, $notes))
			{
				$notes = $notes[$atdee];
			}
			$this->authoriseTransactions(false, $atdee);

			$db = Factory::getDBO();
			$now = JevDate::getDate("+0 seconds");
			$db->setQuery("UPDATE #__jev_attendees set didattend=0, modified=" . $db->Quote($now->toMySql()) . ", guestattend='', notes=" . $db->quote($notes) . "  where id=" . $atdee);
			$db->execute();
		}
		$this->setRedirect("index.php?option=com_rsvppro&task=attendees.overview&atd_id[]=" . $atd_id[0] . "&repeating=" . $repeating . "&Itemid=" . Factory::getApplication()->input->getInt("Itemid", 0), Text::_("JEV_ATTENDANCE_UNMARKED"));
	}

	function close()
	{
		ob_end_clean();
		?>
		<script type="text/javascript">
			try {
				window.parent.jQuery('.jevmodal').modal('hide');
				//window.parent.jQuery('#translationPopup').modal('hide');
			} catch (e) {
			}
			try {
				window.parent.SqueezeBox.close();
			} catch (e) {
			}
			try {
				window.parent.closedialog();
			} catch (e) {
			}
		</script>
		<?php

		exit();
	}

	function delete()
	{
		// Check for request forgeries
		Session::checkToken('request') or jexit('Invalid Token');

		$atd_id = Factory::getApplication()->input->get("atd_id", array(), 'ARRAY');
		if (!isset($atd_id[0]) || strpos($atd_id[0], "|") === false)
		{
			throw new Exception( Text::_("RSVP_MISSING_ATDID"), 403);
		}
		list($atdid, $rp_id) = explode("|", $atd_id[0]);

		$db = Factory::getDBO();
		$sql = "SELECT * FROM #__jev_attendance WHERE id=" . $atdid;
		$db->setQuery($sql);
		$rsvpdata = $db->loadObject();

		$vevent = $this->dataModel->queryModel->getEventById($rsvpdata->ev_id, false, "icaldb");
		if ($rp_id == 0)
		{
			$repeat = $vevent->getFirstRepeat();
		}
		else
		{
			list($year, $month, $day) = JEVHelper::getYMD();
			$repeatdata = $this->dataModel->getEventData($rp_id, "icaldb", $year, $month, $day);
			if ($repeatdata && isset($repeatdata["row"]))
				$repeat = $repeatdata["row"];
		}

		$cid = Factory::getApplication()->input->get("cid", array(), 'ARRAY');;
		$cid = ArrayHelper::toInteger($cid);
		if (count($cid) == 0 || !isset($cid[0]) || $cid[0] < 0)
		{
			throw new Exception( Text::_("RSVP_MISSING_ATTENDEE"), 403);
		}

		$repeating = Factory::getApplication()->input->getInt("repeating", 0);

		$atdee = implode(",", $cid);

		$this->authoriseTransactions(false, $cid[0]);

		if ($this->params->get("notifyadmincancelled", 0))
		{

			foreach ($cid as $jevattend_id)
			{
				$sql = "SELECT * FROM #__jev_attendees WHERE id=" . $jevattend_id;
				$db = Factory::getDBO();
				$db->setQuery($sql);
				$attendee = $db->loadObject();

				$user = JEVHelper::getUser($attendee->user_id);
				if ($user->id == 0 && $this->params->get("attendemails", 0))
				{
					$name = $attendee->email_address;
					$username = $attendee->email_address;
				}
				else
				{
					$name = $user->name;
					$username = $user->username;
				}

				// attendee has been deleted so set the attendance state to not attending for the message
				$attendee->attendstate = 0;

				$this->notifyUser($rsvpdata, $repeat, $user, $name, $username, $attendee, 'admincancelled', false);
			}
		}

		// cancel all their reminders
		if ($vevent && $rsvpdata->allowreminders)
		{
			foreach ($cid as $jevattend_id)
			{
				$sql = "SELECT * FROM #__jev_attendees WHERE id=" . $jevattend_id;
				$db = Factory::getDBO();
				$db->setQuery($sql);
				$attendee = $db->loadObject();
				$user = JEVHelper::getUser($attendee->user_id);
				$this->jevrreminders->unremindUser($rsvpdata, $vevent, $user, $attendee->email_address);
			}
			Factory::getApplication()->enqueueMessage(Text::_("JEV_REMINDER_CANCELLED"));
		}

		// clean up update actions in plugins
		if ($repeat)
		{
			$rsvpparams = $this->getRSVPParmeters($rsvpdata, $repeat);

			// I may also need to trigger a plugin to say that a registration has been cancelled
			PluginHelper::importPlugin("rsvppro");

			foreach ($cid as $jevattend_id)
			{
				$sql = "SELECT * FROM #__jev_attendees WHERE id=" . $jevattend_id;
				$db = Factory::getDBO();
				$db->setQuery($sql);
				$attendee = $db->loadObject();
				$attendee->attendstate = 0;

				$this->postUpdateActions($rsvpdata, $repeat, $attendee, $attendee->waiting, $rsvpparams);

				// Get the fee data for the state change trigger
				include_once(JPATH_ADMINISTRATOR . "/components/com_rsvppro/models/transactions.php");
				$feemodel = new TransactionsModelTransactions();
				$feemodel->getFeesPaid($attendee, false);

				Factory::getApplication()->triggerEvent( 'onRegistrationStatusChange' , array(&$attendee, $repeat, $rsvpdata));
			}
		}

		$db = Factory::getDBO();
		$db->setQuery("DELETE FROM #__jev_attendees where id IN (" . $atdee . ")");
		$db->execute();

		// Reset the counts
		list($atdid, $rp_id) = explode("|", $atd_id[0]);
		$this->countAttendees(intval($atdid));

		$this->setRedirect("index.php?option=com_rsvppro&task=attendees.overview&atd_id[]=" . $atd_id[0] . "&repeating=" . $repeating . "&Itemid=" . Factory::getApplication()->input->getInt("Itemid", 0), Text::_("RSVP_ATTENDEES_DELETED"));
	}

	function edit()
	{
		// Check for request forgeries
		Session::checkToken('request') or jexit('Invalid Token');

		$atd_id = Factory::getApplication()->input->get("atd_id", array(), 'ARRAY');
		if (!isset($atd_id[0]) || strpos($atd_id[0], "|") === false)
		{
			throw new Exception( Text::_("RSVP_MISSING_ATDID"), 403);
		}

		$cid = Factory::getApplication()->input->get("cid", array(), 'ARRAY');;
		$cid = ArrayHelper::toInteger($cid);
		if (count($cid) == 0 || !isset($cid[0]) || $cid[0] < 0)
		{
			$atdee = 0;
		}
		else
		{
			$atdee = $cid[0];
		}

		$repeating = Factory::getApplication()->input->getInt("repeating", 0);

		// reset counts and wiating list first
		list($atdid, $rp_id) = explode("|", $atd_id[0]);
		$this->countAttendees(intval($atdid));

		Table::addIncludePath(RSVP_TABLES);
		$attendee = Table::getInstance('jev_attendee');

		if ($atdee > 0)
		{
			$attendee->load($atdee);
		}
		else if (Factory::getApplication()->input->getInt('user_id',  0) > 0)
        {
            $attendee->user_id = Factory::getApplication()->input->getInt('user_id',  0);
        }

		$db = Factory::getDBO();
		$sql = "SELECT * FROM #__jev_attendance WHERE id=" . intval($atdid);
		$db->setQuery($sql);
		$rsvpdata = $db->loadObject();

		$this->dataModel = new JEventsDataModel();
		$this->queryModel = new JEventsDBModel($this->dataModel);

		// Find the first repeat - including unpublished events
		$vevent = $this->dataModel->queryModel->getEventById($rsvpdata->ev_id, true, "icaldb");
		if ($rp_id == 0)
		{
			$repeat = $vevent->getFirstRepeat();
		}
		else
		{
			list($year, $month, $day) = JEVHelper::getYMD();
			// include unpublished events
			$repeat = $this->dataModel->queryModel->listEventsById(intval($rp_id), 1, "icaldb");
			//$repeatdata = $this->dataModel->getEventData(intval($rp_id), "icaldb", $year, $month, $day);
			//if ($repeatdata && isset($repeatdata["row"]))
			//	$repeat = $repeatdata["row"];
		}


		$this->authoriseAttendee($rsvpdata);

		// get the view
		$this->view = $this->getView("attendees", "html");
		// Set the layout
		$this->view->setLayout('edit');

		$this->view->attendee = $attendee;
		$this->view->rsvpdata = $rsvpdata;
		$this->view->vevent = $vevent;
		$this->view->atdee = $atdee;
		$this->view->repeat = $repeat;
		$this->view->repeating = $repeating;
		$this->view->atd_id = $atdid;
		$this->view->rp_id = $rp_id;

		$this->view->params = ComponentHelper::getParams("com_rsvppro");

		$this->view->display();
	}

	function save()
	{
		$jinput = Factory::getApplication()->input;
		// Check for request forgeries
		Session::checkToken('request') or jexit('Invalid Token');

		$atd_id = Factory::getApplication()->input->get("atd_id", array(), 'ARRAY');
		if (!isset($atd_id[0]) || strpos($atd_id[0], "|") === false)
		{
			throw new Exception( Text::_("RSVP_MISSING_ATDID"), 403);
		}

		$atdee = Factory::getApplication()->input->getInt("atdee", 0);

		$repeating = Factory::getApplication()->input->getInt("repeating", 0);

		// reset counts and wiating list first
		list($atdid, $rp_id) = explode("|", $atd_id[0]);
		$this->countAttendees(intval($atdid));

		$db = Factory::getDbo();
		$sql = "SELECT * FROM #__jev_attendance WHERE id=" . intval($atdid);
		$db->setQuery($sql);
		$rsvpdata = $db->loadObject();

		$this->authoriseAttendee($rsvpdata);

		$params = "";

		// Need the params for most of these scenarios - bring in line with recordAttendance!
		$params = $this->getRsvpDataParams($rsvpdata);

		// Get the event
		$datamodel = new JEventsDataModel();
		if ($rp_id > 0)
		{
			$row = $datamodel->queryModel->listEventsById($rp_id, 1, "icaldb");
		}
		else
		{
			$row = $datamodel->queryModel->getEventById($rsvpdata->ev_id, 1, "icaldb");
		}

		Table::addIncludePath(RSVP_TABLES);
		$attendee = Table::getInstance('jev_attendee');

		if ($atdee > 0)
		{
			$attendee->load($atdee);
		}

		$sql = "SELECT atdcount FROM #__jev_attendeecount as a WHERE a.at_id=" . $rsvpdata->id;
		if (!$rsvpdata->allrepeats)
		{
			$sql .= " and a.rp_id=" . $rp_id;
		}
		$db->setQuery($sql);
		$attendeeCount = $db->loadResult();

		$attendee->at_id = intval($atdid);
		$attendee->rp_id = $rsvpdata->allrepeats ? 0 : intval($rp_id);
		$attendee->user_id = Factory::getApplication()->input->getInt("user_id", 0);
		$attendee->email_address = Factory::getApplication()->input->getString("jevattend_email");
		// ALWAYS CONFIRMED IS MANUAL ADDITION
		$attendee->confirmed = 1;
		$jevattend = Factory::getApplication()->input->getInt("jevattend", 0);
		$attendee->attendstate = $jevattend;

		// Need the creation date for these records
		if (class_exists("JevDate"))
		{
			$date = JevDate::getDate("+0 seconds");
			$created = $date->toMySql();
		}
		else
		{
			$date = Factory::getDate("+0 seconds");
			$created = $date->toSql();
		}
		if (!isset($attendee->created) || $attendee->created == "" || $attendee->created == "0000-00-00 00:00:00")
		{
			$attendee->created = $created;
		}
		$attendee->modified = $created;

		$attendee->params = $params;

		// set the guest count correctly
		$guestcount = Factory::getApplication()->input->getInt("guestcount", 1);

		// add true to force count for new attendee record
		$oldAttendeesCount = isset($attendee->atdcount) ? $attendee->atdcount : 0;
		if (!$atdee)
		{
			$tempattendee = new stdClass();
			$tempattendee->attendstate = -1;
			$tempattendee->guestcount = 0;
			$this->countAttendeeContributionToAttendance($row, $rsvpdata, $attendee, true);
		}
		else {
			$this->countAttendeeContributionToAttendance($row, $rsvpdata, $attendee);
		}
		$newAttendeesCount = $row->newcnt;

		$attendeeCount += $newAttendeesCount - $oldAttendeesCount;

		$attendeecountchanged = $oldAttendeesCount !== $newAttendeesCount;
		$attendee->atdcount = $row->newcnt;
		$attendee->guestcount = $guestcount;

		$plugin = PluginHelper::getPlugin("jevents", "jevrsvppro");
		//$this->params = new Registry($plugin->params);
		// Should this be on the waiting list
		$attendee->waiting = 0;
		// only make them waiting if attending, pending approval or outstanding balance
		if ($this->params->get("capacity", 0) && $rsvpdata->capacity > 0 && in_array($attendee->attendstate, array(1, 3, 4)))
		{
			if ($attendeeCount >= intval($rsvpdata->capacity) && $attendeecountchanged)
			{
				$attendee->waiting = 1;
			}
		}

		// And the locked template
		$attendee->lockedtemplate = intval($rsvpdata->template);

		// Make sure not already covered!!
		if (intval($attendee->user_id) > 0)
		{
			$db->setQuery("SELECT atdees.id FROM #__jev_attendees as atdees  WHERE atdees.at_id=" . intval($atdid) . " AND atdees.rp_id=" . intval($rp_id) . " AND  atdees.user_id=" . $attendee->user_id);
		}
		else if (trim($attendee->email_address) !== "")
		{
			$db->setQuery("SELECT atdees.id FROM #__jev_attendees as atdees  WHERE atdees.at_id=" . intval($atdid) . " AND atdees.rp_id=" . intval($rp_id) . " AND  LOWER(atdees.email_address)=" . $db->Quote(strtolower($attendee->email_address)));
		}
		else
		{
			$this->setRedirect("index.php?option=com_rsvppro&task=attendees.overview&atd_id[]=" . $atd_id[0] . "&repeating=" . $repeating . "&Itemid=" . Factory::getApplication()->input->getInt("Itemid", 0), Text::_("RSVP_INVALID_INPUTS"));
		}

		$tmprow = $db->loadObject();
		if ($tmprow && $tmprow->id != $atdee)
		{
			$this->setRedirect("index.php?option=com_rsvppro&task=attendees.overview&atd_id[]=" . $atd_id[0] . "&repeating=" . $repeating . "&Itemid=" . Factory::getApplication()->input->getInt("Itemid", 0), Text::_("RSVP_ATTENDEE_ALREADY_EXISTS"));
			return;
		}

		if (!$attendee->store())
		{
			$error = $attendee->getError();
		}

		// attach balance info to the attendee
		if (!isset($attendee->outstandingBalances))
		{
			// This MUST be called before renderToBasicArray to populate the balance fields - so we do it here to be safe
			if (isset($attendee->params))
			{
				$xmlfile = JevTemplateHelper::getTemplate($rsvpdata);

				if (is_int($xmlfile) || file_exists($xmlfile))
				{
					$activePlugin = "";

					$params = new JevRsvpParameter($attendee->params, $xmlfile, $rsvpdata, null);

					// do we need to update the payment method in the $params AND in the attendee->params data (to be safe)!
					$rsvpparamsarray = $params->renderToBasicArray();
					// find the paymentoptionlist
					foreach ($rsvpparamsarray as $pield => $pm)
					{
						if ($pm["type"] == "jevrpaymentoptionlist")
						{

							$newval = Factory::getApplication()->input->get("xmlfile",array(), 'ARRAY');
							if (isset($newval[$pield]))
							{
								$attendeefields = new Registry($attendee->params);
								$oldval = $attendeefields->get($pield);
								$attendeefields->set($pield, $newval[$pield]);
								if (version_compare(PHP_VERSION, '5.4', '>='))
								{
									$attendee->params = $attendeefields->toString('JSON', array('bitmask' =>JSON_UNESCAPED_UNICODE));
								}
								else
								{
									$attendee->params = $attendeefields->toString();
								}

								//$attendee->params = str_replace('"'.$pield.'":"'.$pm["value"].'"','"'.$pield.'":"'.$newval[$pield].'"',$attendee->params);
								// must also update the database
								$db = Factory::getDbo();
								$db->setQuery("UPDATE #__jev_attendees set params=" . $db->Quote($attendee->params) . " WHERE id=" . $attendee->id);
								$db->execute();

								$params->bind($newval);
							}
						}
					}

					$feesAndBalances = $params->outstandingBalance($attendee);

					// Any transactions to update
					$db = Factory::getDbo();
					if ($attendee->id)
					{
						$db->setQuery("SELECT transaction_id FROM #__jev_rsvp_transactions where attendee_id= " . $attendee->id . " AND paymentstate=0 and gateway='manual' ORDER BY transaction_date desc limit 1 ");
						$transaction_id = $db->loadResult();
						if ($transaction_id && isset($feesAndBalances['feebalance']))
						{
							$transaction = new rsvpTransaction();
							$transaction->load($transaction_id);
							$transaction->amount = $feesAndBalances['feebalance'];
							$transaction->store();
						}
					}
				}
			}
		}

		// auto remind attendees - not those on waiting list and only if actually attending
		if ($this->params->get("autoremind", 0) == 1 && !$attendee->waiting && in_array($attendee->attendstate, array(1, 3, 4)) && ($rsvpdata->allowreminders || $this->params->get("forceautoremind", 0)))
		{
			// create reminder
			if ($row)
			{
				$mainframe = Factory::getApplication();
				$user = JEVHelper::getUser($attendee->user_id);
				if ($this->jevrreminders->remindUser($rsvpdata, $row, $user, $attendee->email_address))
				{
					$mainframe->enqueueMessage(Text::_("JEV_REMINDER_CONFIRMED"));
				}
			}
		}

		$user = JEVHelper::getUser($attendee->user_id);
		$username = $attendee->user_id > 0 ? $user->username : $attendee->email_address;
		$name = $attendee->user_id ? $user->name : $attendee->email_address;

		// send notifications ot new attendees BUT only if attending or may
		if ($this->params->get("notifyuser", 0) && $atdee == 0 && in_array($attendee->attendstate, array(1, 2, 3, 4)))
		{
			$this->notifyUser($rsvpdata, $row, $user, $name, $username, $attendee, 'ack', false);
		}
		// only send yes or subject to approval
		if ($attendee->attendstate == 1 || $attendee->attendstate == 3)
		{
			// Do not send notification if only sending cancelations!
			if ($this->params->get("notifycreator", 0) && $this->params->get("notifycreator", 0) != 3)
			{

				$this->notifyCreator($rsvpdata, $row, $name, $username, $attendee, false, $attendee->waiting);
			}
		}

		$this->countAttendees($atd_id[0]);

        // Trigger the onRegistrationStatusChange plugin
		PluginHelper::importPlugin("rsvppro");
		Factory::getApplication()->triggerEvent( 'onRegistrationStatusChange' , array($attendee, $row, $rsvpdata));

		if ($jinput->getString("tmpl") === "component")
		{
			$this->close();
			return;
		}
		else
		{
			$this->setRedirect("index.php?option=com_rsvppro&task=attendees.overview&atd_id[]=" . $atd_id[0] . "&repeating=" . $repeating . "&Itemid=" . Factory::getApplication()->input->getInt("Itemid", 0), Text::_("RSVP_ATTENDEE_SAVED"));
		}
	}

	function attendyes()
	{
	    return $this->changestate(1);
	}

	function attendno()
	{
		return $this->changestate(0);
	}

	function attendmaybe()
	{
		return $this->changestate(2);
	}

	function changestate($newstate = -1)
	{
		$atd_id = Factory::getApplication()->input->get("atd_id", array(), 'ARRAY');
		if (!isset($atd_id[0]) || strpos($atd_id[0], "|") === false)
		{
			throw new Exception( Text::_("RSVP_MISSING_ATDID"), 403);
		}

		$cid = Factory::getApplication()->input->get("cid", array(), 'ARRAY');;
		$cid = ArrayHelper::toInteger($cid);
		if (count($cid) == 0 || !isset($cid[0]) || $cid[0] < 0)
		{
			throw new Exception( Text::_("RSVP_MISSING_ATTENDEE"), 403);
		}

		$repeating = Factory::getApplication()->input->getInt("repeating", 0);

		$atdee = $cid[0];

		Table::addIncludePath(RSVP_TABLES);
		$attendee = Table::getInstance('jev_attendee');

		if ($atdee > 0)
		{
			$attendee->load($atdee);
		}
		if (!$attendee)
		{
			throw new Exception( Text::_("RSVP_MISSING_ATDID"), 403);
		}

		// get the view
		$this->view = $this->getView("attendees", "html");

		$this->view->dataModel =  $this->dataModel;

		$rp_id = $attendee->rp_id;

		$db = Factory::getDBO();
		$sql = "SELECT * FROM #__jev_attendance WHERE id=" . $attendee->at_id;
		$db->setQuery($sql);
		$rsvpdata = $db->loadObject();
		if (!$rsvpdata)
		{
			throw new Exception( Text::_("RSVP_MISSING_ATDID"), 403);
		}
		$vevent = $this->dataModel->queryModel->getEventById($rsvpdata->ev_id, false, "icaldb");
		if ($rp_id == 0)
		{
			$repeat = $vevent->getFirstRepeat();
		}
		else
		{
			list($year, $month, $day) = JEVHelper::getYMD();
			$repeatdata = $this->dataModel->getEventData($rp_id, "icaldb", $year, $month, $day);
			if ($repeatdata && isset($repeatdata["row"]))
				$repeat = $repeatdata["row"];
		}

		$this->authoriseTransactions($repeat, $atdee);

		// Add reference to current row and rsvpdata to the registry so that we have access to these in the fields
		$registry = JevRegistry::getinstance("jevents");
		$registry->set("rsvpdata", $rsvpdata);
		$registry->set("event", $repeat);

		$template = $rsvpdata->template;
		// New parameterised fields
		$eventrow = clone $repeat;
		if ($template != "")
		{
			$this->helper->attendee = $attendee;
			$this->helper->calculateBalances($this->view, $rsvpdata, $eventrow);
			$xmlfile = JevTemplateHelper::getTemplate($rsvpdata);

			if (is_int($xmlfile) || file_exists($xmlfile))
			{
				// transfer attendee specific information into the event row

				if (isset($attendee->params))
				{
					$params = new JevRsvpParameter($attendee->params, $xmlfile, $rsvpdata, $eventrow);
					$feesAndBalances = $params->outstandingBalance($attendee);
				}
				else
				{
					$params = new JevRsvpParameter("", $xmlfile, $rsvpdata, $eventrow);
					$feesAndBalances = false;
				}
			}
		}
		$db->setQuery("SELECT * FROM #__jev_attendees where id=" . $atdee);
		//$attendee = $db->loadObject();
		if (floatval($feesAndBalances["feebalance"]) <= 0.001 || !isset($feesAndBalances["feebalance"]))
		{
			$newstate = $newstate < 0 ? ($attendee->attendstate + 1) % 3 : $newstate;
			$this->setRedirect("index.php?option=com_rsvppro&task=attendees.overview&atd_id[]=" . $atd_id[0] . "&repeating=" . $repeating . "&Itemid=" . Factory::getApplication()->input->getInt("Itemid", 0), Text::_("JEV_ATTEND_STATE_CHANGED"));
		}
		else if ($attendee->attendstate == 0)
		{
			$newstate = $newstate < 0 ? ($attendee->attendstate + 4) : $newstate;
			Factory::getApplication()->enqueueMessage(Text::_("JEV_CANNOT_CONFIRM_ATTENDEE_WITH_OUTSTANDING_BALANCE"));
			$this->setRedirect("index.php?option=com_rsvppro&task=attendees.overview&atd_id[]=" . $atd_id[0] . "&repeating=" . $repeating . "&Itemid=" . Factory::getApplication()->input->getInt("Itemid", 0), Text::_("JEV_ATTEND_STATE_NOT_CHANGED"));
		}
		else
		{
			$newstate = $newstate < 0 ? ($attendee->attendstate + 1) % 3 : $newstate;
			$this->setRedirect("index.php?option=com_rsvppro&task=attendees.overview&atd_id[]=" . $atd_id[0] . "&repeating=" . $repeating . "&Itemid=" . Factory::getApplication()->input->getInt("Itemid", 0), Text::_("JEV_ATTEND_STATE_CHANGED"));
		}
		$now = JevDate::getDate("+0 seconds");
		$db->setQuery("UPDATE #__jev_attendees set attendstate=$newstate, modified=" . $db->Quote($now->toMySql()) . " where id=" . $atdee);
		$db->execute();

		// Manage auto reminders
		if ($this->params->get("autoremind", 0) == 1 && $eventrow)
		{

			/**
			 * Attendstate explanations
			 * 0 = not attending
			 * 1 = attending
			 * 2 = maybe attending
			 * 3 = attending but subject to approval i.e.pending
			 * 4 = attending but has outstanding balance to pay
			 */
			// always allow auto reminders for attendees if forced reminders enabled
			if ($rsvpdata->allowreminders || $this->params->get("forceautoremind", 0) == 1)
			{
				$user = JEVHelper::getUser($attendee->user_id);
				if ($newstate == 1)
				{
					// create reminder
					$this->jevrreminders->remindUser($rsvpdata, $eventrow, $user, $attendee->email_address);
					$mainframe = Factory::getApplication();
					$mainframe->enqueueMessage(Text::_("JEV_REMINDER_CONFIRMED"));
				}
				else if ($newstate == 0 || $newstate == 2)
				{
					// remove reminder
					$this->jevrreminders->unremindUser($rsvpdata, $eventrow, $user, $attendee->email_address);
					$mainframe = Factory::getApplication();
					$mainframe->enqueueMessage(Text::_("JEV_REMINDER_CANCELLED"));
				}
			}
		}

		// clean up update actions in plugins
		if ($eventrow)
		{
			$rsvpparams = $this->getRSVPParmeters($rsvpdata, $eventrow);
			foreach ($cid as $jevattend_id)
			{
				$sql = "SELECT * FROM #__jev_attendees WHERE id=" . $jevattend_id;
				$db = Factory::getDBO();
				$db->setQuery($sql);
				$attendee = $db->loadObject();

				$this->postUpdateActions($rsvpdata, $eventrow, $attendee, $attendee->waiting, $rsvpparams);
			}
		}

		if ($newstate == 0 && $this->params->get("notifyadmincancelled", 0))
		{
			$user = JEVHelper::getUser($attendee->user_id);
			if ($user->id == 0 && $this->params->get("attendemails", 0))
			{
				$name = $attendee->email_address;
				$username = $attendee->email_address;
			}
			else
			{
				$name = $user->name;
				$username = $user->username;
			}

			// set the attendance state to not attending for the message
			$attendee->attendstate = 0;

			$this->notifyUser($rsvpdata, $repeat, $user, $name, $username, $attendee, 'admincancelled', false);
		}

		//$this->setRedirect("index.php?option=com_rsvppro&task=attendees.overview&atd_id[]=" . $atd_id[0] . "&repeating=" . $repeating . "&Itemid=" . Factory::getApplication()->input->getInt("Itemid", 0), Text::_("JEV_ATTEND_STATE_CHANGED"));
	}

	/**
	 * List Atendee transactions
	 *
	 */
	function transactions()
	{

		$cid = Factory::getApplication()->input->get("cid", array(), 'ARRAY');;
		$cid = ArrayHelper::toInteger($cid);
		if (count($cid) == 0 || !isset($cid[0]) || $cid[0] < 0)
		{
			$atdee = 0;
		}
		else
		{
			$atdee = $cid[0];
		}

		Table::addIncludePath(RSVP_TABLES);
		$attendee = Table::getInstance('jev_attendee');

		if ($atdee > 0)
		{
			$attendee->load($atdee);
		}
		if (!$attendee)
		{
			throw new Exception( Text::_("RSVP_MISSING_ATDID"), 403);
		}

		// get the view
		$this->view = $this->getView("attendees", "html");

		$this->view->dataModel =  $this->dataModel;

		$mainframe = Factory::getApplication();
		$option = Factory::getApplication()->input->getCmd("option");

		$rp_id = $attendee->rp_id;

		$db = Factory::getDBO();
		$sql = "SELECT * FROM #__jev_attendance WHERE id=" . $attendee->at_id;
		$db->setQuery($sql);
		$rsvpdata = $db->loadObject();
		if (!$rsvpdata)
		{
			throw new Exception( Text::_("RSVP_MISSING_ATDID"), 403);
		}
		$vevent = $this->dataModel->queryModel->getEventById($rsvpdata->ev_id, true, "icaldb");
		if ($rp_id == 0)
		{
			$repeat = $vevent->getFirstRepeat();
		}
		else
		{
			list($year, $month, $day) = JEVHelper::getYMD();
			// include unpublished events
			$repeat = $this->dataModel->queryModel->listEventsById(intval($rp_id), 1, "icaldb");
			//$repeatdata = $this->dataModel->getEventData(intval($rp_id), "icaldb", $year, $month, $day);
			//if ($repeatdata && isset($repeatdata["row"]))
			//	$repeat = $repeatdata["row"];
		}

		$this->authoriseTransactions($repeat, $atdee);

		// Add reference to current row and rsvpdata to the registry so that we have access to these in the fields
		$registry = JevRegistry::getinstance("jevents");
		$registry->set("rsvpdata", $rsvpdata);
		$registry->set("event", $repeat);

		$template = $rsvpdata->template;
		// New parameterised fields
		if ($template != "")
		{
			$xmlfile = JevTemplateHelper::getTemplate($rsvpdata);

			if (is_int($xmlfile) || file_exists($xmlfile))
			{
				// transfer attendee specific information into the event row
				$eventrow = clone $repeat;
				if (isset($attendee->params))
				{
					$params = new JevRsvpParameter($attendee->params, $xmlfile, $rsvpdata, $eventrow);
					$feesAndBalances = $params->outstandingBalance($attendee);
				}
				else
				{
					$params = new JevRsvpParameter("", $xmlfile, $rsvpdata, $eventrow);
					$feesAndBalances = false;
				}
			}
		}
		else
		{
			// no template means no fees meand no transactions
			return;
		}

		$limit = intval($mainframe->getUserStateFromRequest("translistlimit", 'limit', Factory::getApplication()->get("list_limit", 10)));
		$limitstart = intval($mainframe->getUserStateFromRequest("translimitstart", 'limitstart', 0));

		if ($feesAndBalances && isset($feesAndBalances['transactions']))
		{
			$rows = $feesAndBalances['transactions'];
		}
		else
		{
			$rows = array();
		}

		// attach repeat into row
		foreach ($rows as $key => $val)
		{
			$rows[$key]->eventrepeat = $repeat;
		}

		include_once(JPATH_ADMINISTRATOR . "/components/com_rsvppro/models/transactions.php");
		$model = new TransactionsModelTransactions();
		// clone so we can get ALL the transactions
		$tempAttendee = clone $attendee;
		$tempAttendee->_feedata = null;
		// get fees with no check on payment state!
		$model->getFeesPaid($tempAttendee, false);
		$transactions = $tempAttendee->_feedata;

		// Set the layout
		$this->view->setLayout('transactions');

		$this->view->feesAndBalances = $feesAndBalances;
		$this->view->attendee = $attendee;
		$this->view->rp_id = $rp_id;
		$this->view->transactions = $transactions;
		$this->view->rsvpdata = $rsvpdata;
		$this->view->event = $repeat;
		$this->view->vevent = $vevent;
		$this->view->repeat =$repeat;

		$this->view->display();
	}

	function edittransaction()
	{
		// Check for request forgeries
		Session::checkToken('request') or jexit('Invalid Token');

		$cid = Factory::getApplication()->input->get("cid", array(), 'ARRAY');;
		$cid = ArrayHelper::toInteger($cid);
		if (count($cid) == 0 || !isset($cid[0]) || $cid[0] < 0)
		{
			$cid = 0;
		}
		else
		{
			$cid = $cid[0];
		}
		require_once(RSVP_TABLES . "/transaction.php");
		$transaction = new rsvpTransaction();
		if ($cid > 0)
		{
			$transaction->load($cid);
		}
		if (!$transaction)
		{
			throw new Exception( Text::_("RSVP_MISSING_ATDID"), 403);
		}

		$atdee_id = Factory::getApplication()->input->getInt("atdee_id", 0);
		if ($atdee_id == 0)
		{
			throw new Exception( Text::_("RSVP_MISSING_ATDID"), 403);
		}

		Table::addIncludePath(RSVP_TABLES);
		$attendee = Table::getInstance('jev_attendee');

		if ($atdee_id > 0)
		{
			$attendee->load($atdee_id);
		}
		if (!$attendee)
		{
			throw new Exception( Text::_("RSVP_MISSING_ATDID"), 403);
		}

		// get the view
		$this->view = $this->getView("attendees", "html");

		$db = Factory::getDBO();
		$sql = "SELECT * FROM #__jev_attendance WHERE id=" . intval($attendee->at_id);
		$db->setQuery($sql);
		$rsvpdata = $db->loadObject();
		$rp_id = $attendee->rp_id;

		$this->dataModel = new JEventsDataModel();
		$this->queryModel = new JEventsDBModel($this->dataModel);

		// Find the first repeat - including unpublished events
		$vevent = $this->dataModel->queryModel->getEventById($rsvpdata->ev_id, true, "icaldb");
		if ($rp_id == 0)
		{
			$repeat = $vevent->getFirstRepeat();
		}
		else
		{
			list($year, $month, $day) = JEVHelper::getYMD();
			// include unpublished events
			$repeat = $this->dataModel->queryModel->listEventsById(intval($rp_id), 1, "icaldb");
			//$repeatdata = $this->dataModel->getEventData(intval($rp_id), "icaldb", $year, $month, $day);
			//if ($repeatdata && isset($repeatdata["row"]))
			//	$repeat = $repeatdata["row"];
		}

		$this->authoriseTransactions($repeat, $atdee_id);

		$template = $rsvpdata->template;
		// New parameterised fields
		if ($template != "")
		{
			$xmlfile = JevTemplateHelper::getTemplate($rsvpdata);

			if (is_int($xmlfile) || file_exists($xmlfile))
			{
				// transfer attendee specific information into the event row
				$eventrow = clone $repeat;
				if (isset($attendee->params))
				{
					$params = new JevRsvpParameter($attendee->params, $xmlfile, $rsvpdata, $eventrow);
					$feesAndBalances = $params->outstandingBalance($attendee);
				}
				else
				{
					$params = new JevRsvpParameter("", $xmlfile, $rsvpdata, $eventrow);
					$feesAndBalances = false;
				}
			}
		}
		else
		{
			// no template means no fees meand no transactions
			return;
		}

		// Set the layout
		$this->view->setLayout('edittransaction');

		$this->view->feesAndBalances = $feesAndBalances;
		$this->view->attendee = $attendee;
		$this->view->transaction = $transaction;
		$this->view->rsvpdata = $rsvpdata;
		$this->view->sessiontemplate =$template;
		$this->view->event = $repeat;
		$this->view->vevent = $vevent;
		$this->view->repeat = $repeat;

		$this->view->display();
	}

	function deletetransaction()
	{
		// Check for request forgeries
		Session::checkToken('request') or jexit('Invalid Token');

		require_once(RSVP_TABLES . "/transaction.php");
		$transaction = new rsvpTransaction();
		$cid = Factory::getApplication()->input->get("cid", array(), 'ARRAY');;
		$cid = ArrayHelper::toInteger($cid);

		foreach ($cid as $tid)
		{
			$transaction->load($tid);

			Table::addIncludePath(RSVP_TABLES);
			$attendee = Table::getInstance('jev_attendee');
			$attendee->load($transaction->attendee_id);

			$this->authoriseTransactions(false, $attendee->id);

			if ($transaction)
			{
				// updates the attendee automatically
				$transaction->delete();
			}

			if ($attendee)
			{
				$this->countAttendees($attendee->at_id);
			}
		}

		$this->setRedirect("index.php?option=com_rsvppro&task=attendees.transactions&cid[]=" . Factory::getApplication()->input->getInt("atdee_id", 0) . "&Itemid=" . Factory::getApplication()->input->getInt("Itemid", 0), Text::_("RSVP_TRANSACTION_DELETED"));
	}

	function savetransaction()
	{
		// Check for request forgeries
		Session::checkToken('request') or jexit('Invalid Token');

		$cid = Factory::getApplication()->input->get("cid", array(), 'ARRAY');;
		$cid = ArrayHelper::toInteger($cid);
		if (count($cid) == 0 || !isset($cid[0]) || $cid[0] < 0)
		{
			$cid = 0;
		}
		else
		{
			$cid = $cid[0];
		}
		require_once(RSVP_TABLES . "/transaction.php");
		$transaction = new rsvpTransaction();
		if ($cid > 0)
		{
			$transaction->load($cid);
		}

		//$transaction->bind(Factory::getApplication()->input->post->getArray(array(), null, 'RAW'));
		$transaction->bind(Factory::getApplication()->input->post->getArray(array(), null, 'default'));

		Table::addIncludePath(RSVP_TABLES);
		$attendee = Table::getInstance('jev_attendee');
		$attendee->load($transaction->attendee_id);
		$this->authoriseTransactions(false, $attendee->id);

		$transaction->store();

		if ($attendee)
		{
			$this->countAttendees($attendee->at_id);
			$this->notifyManualPayment($transaction, $attendee);
		}
		$this->setRedirect("index.php?option=com_rsvppro&task=attendees.transactions&cid[]=" . Factory::getApplication()->input->getInt("attendee_id", 0) . "&Itemid=" . Factory::getApplication()->input->getInt("Itemid", 0), Text::_("RSVP_TRANSACTION_SAVED"));
	}

	function resendtransactionnotice()
	{
		// Check for request forgeries
		Session::checkToken('request') or jexit('Invalid Token');

		$cid = Factory::getApplication()->input->get("cid", array(), 'ARRAY');;
		$cid = ArrayHelper::toInteger($cid);
		if (count($cid) == 0 || !isset($cid[0]) || $cid[0] < 0)
		{
			$cid = 0;
		}
		else
		{
			$cid = $cid[0];
		}
		require_once(RSVP_TABLES . "/transaction.php");
		$transaction = new rsvpTransaction();
		if ($cid > 0)
		{
			$transaction->load($cid);
		}
		else
		{
			echo Text::_("RSVP_NO_TRANSACTION_SELECTED");
			return;
		}

		$transaction->bind(Factory::getApplication()->input->post->getArray(array(), null, 'RAW'));

		Table::addIncludePath(RSVP_TABLES);
		$attendee = Table::getInstance('jev_attendee');
		$attendee->load($transaction->attendee_id);
		$this->authoriseTransactions(false, $attendee->id);

		if ($attendee)
		{
			// resend the notice regardless
			$this->notifyManualPayment($transaction, $attendee, true);
			$this->setRedirect("index.php?option=com_rsvppro&task=attendees.transactions&cid[]=" . Factory::getApplication()->input->getInt("atdee_id", 0) . "&Itemid=" . Factory::getApplication()->input->getInt("Itemid", 0), Text::_("RSVP_TRANSACTION_NOTICE_SENT"));
		}
	}

	private function notifyManualPayment($transaction, $attendee, $ignorestate = false)
	{
		$db = Factory::getDBO();
		$sql = "SELECT * FROM #__jev_attendance WHERE id=" . intval($attendee->at_id);
		$db->setQuery($sql);
		$rsvpdata = $db->loadObject();

		$templateParams = RsvpHelper::getTemplateParams($rsvpdata);

		$class = "plgRsvppro" . ucfirst($transaction->gateway);
		$pluginpath = JPATH_SITE . "/plugins/rsvppro/$transaction->gateway/";
		JLoader::register($class, $pluginpath . $transaction->gateway . ".php");

		// if sending manual or paypal payment with notification enabled
		// either we ignore the payment state or the payment has been completed then send the message
		if ($ignorestate ||
			((is_callable($class . "::NotifyPayment") && call_user_func($class . "::NotifyPayment", $templateParams)) && $transaction->paymentstate == 1))
		{

			$rp_id = $attendee->rp_id;

			$this->dataModel = new JEventsDataModel();
			$this->queryModel = new JEventsDBModel($this->dataModel);

			// Find the first repeat - including unpublished events
			$vevent = $this->dataModel->queryModel->getEventById($rsvpdata->ev_id, true, "icaldb");
			if ($rp_id == 0)
			{
				$repeat = $vevent->getFirstRepeat();
			}
			else
			{
				list($year, $month, $day) = JEVHelper::getYMD();
				// include unpublished events
				$repeat = $this->dataModel->queryModel->listEventsById(intval($rp_id), 1, "icaldb");
				//$repeatdata = $this->dataModel->getEventData(intval($rp_id), "icaldb", $year, $month, $day);
				//if ($repeatdata && isset($repeatdata["row"]))
				//	$repeat = $repeatdata["row"];
			}

			PluginHelper::importPlugin('jevents');
			Factory::getApplication()->input->set("repeating", $rsvpdata->allrepeats);
			Factory::getApplication()->input->set("atd_id", array($rsvpdata->id . "|" . $repeat->rp_id()));

			Factory::getApplication()->triggerEvent('onDisplayCustomFields', array(&$repeat));

			$user = JEVHelper::getUser($attendee->user_id);
			if ($user->id == 0 && $this->params->get("attendemails", 0))
			{
				$name = $attendee->email_address;
				$username = $attendee->email_address;
			}
			else
			{
				$name = $user->name;
				$username = $user->username;
			}

			$messagetype = "pplpay";
			if (is_callable($class . "::PaymentMessageType"))
			{
				$messagetype = call_user_func($class . "::PaymentMessageType");
			}
			$this->notifyUser($rsvpdata, $repeat, $user, $name, $username, $attendee, $messagetype, false, $transaction);
		}
	}

	function invalidtransaction()
	{
		// Check for request forgeries
		Session::checkToken('request') or jexit('Invalid Token');

		$cid = Factory::getApplication()->input->get("cid", array(), 'ARRAY');;
		$cid = ArrayHelper::toInteger($cid);
		if (count($cid) == 0 || !isset($cid[0]) || $cid[0] < 0)
		{
			$cid = 0;
		}
		else
		{
			$cid = $cid[0];
		}
		require_once(RSVP_TABLES . "/transaction.php");
		$transaction = new rsvpTransaction();
		if ($cid == 0)
		{
			throw new Exception(403, "Missing Transaction Id");
		}

		$transaction->load($cid);
		$transaction->paymentstate = 0;

		Table::addIncludePath(RSVP_TABLES);
		$attendee = Table::getInstance('jev_attendee');
		$attendee->load($transaction->attendee_id);
		$this->authoriseTransactions(false, $attendee->id);

		$transaction->store();

		if ($attendee)
		{
			$this->countAttendees($attendee->at_id);
			// I must reload the attendee since the payment state has changed!
			$attendee->load($transaction->attendee_id);

			$this->setRedirect("index.php?option=com_rsvppro&task=attendees.transactions&cid[]=" . $attendee->id . "&Itemid=" . Factory::getApplication()->input->getInt("Itemid", 0), Text::_("RSVP_TRANSACTION_SAVED"));
		}
		else
		{
			$this->setRedirect("index.php?option=com_rsvppro&task=cpanel.cpanel" . "&Itemid=" . Factory::getApplication()->input->getInt("Itemid", 0), Text::_("RSVP_TRANSACTION_SAVED"));
		}
	}

	function validtransaction()
	{
		// Check for request forgeries
		Session::checkToken('request') or jexit('Invalid Token');

		$cid = Factory::getApplication()->input->get("cid", array(), 'ARRAY');;
		$cid = ArrayHelper::toInteger($cid);
		if (count($cid) == 0 || !isset($cid[0]) || $cid[0] < 0)
		{
			$cid = 0;
		}
		else
		{
			$cid = $cid[0];
		}
		require_once(RSVP_TABLES . "/transaction.php");
		$transaction = new rsvpTransaction();
		if ($cid == 0)
		{
			throw new Exception(403, "Missing Transaction Id");
		}

		$transaction->load($cid);
		$transaction->paymentstate = 1;

		Table::addIncludePath(RSVP_TABLES);
		$attendee = Table::getInstance('jev_attendee');
		$attendee->load($transaction->attendee_id);

		$this->authoriseTransactions(false, $attendee->id);

		$transaction->store();

		if ($attendee)
		{
			$this->countAttendees($attendee->at_id);
			// I must reload the attendee since the payment state has changed!
			$attendee->load($transaction->attendee_id);

			$this->notifyManualPayment($transaction, $attendee);

			$this->setRedirect("index.php?option=com_rsvppro&task=attendees.transactions&cid[]=" . $attendee->id . "&Itemid=" . Factory::getApplication()->input->getInt("Itemid", 0), Text::_("RSVP_TRANSACTION_SAVED"));
		}
		else
		{
			$this->setRedirect("index.php?option=com_rsvppro&task=cpanel.cpanel" . "&Itemid=" . Factory::getApplication()->input->getInt("Itemid", 0), Text::_("RSVP_TRANSACTION_SAVED"));
		}
	}

	protected function countAttendees($atdid = 0)
	{
		return $this->helper->countAttendees($atdid);
	}

	public function updateWaitingList($rsvpdata, $atdid = 0)
	{
		return $this->helper->updateWaitingList($rsvpdata, $atdid);
	}

	private function parseMessage($message, $rsvpdata, & $row, $name, $username, $attendee = null, $requirelogin = false, $onWaitingList = false, $transaction = false, $speciallink = false)
	{

		return $this->helper->parseMessage($message, $rsvpdata, $row, $name, $username, $attendee, $requirelogin, $onWaitingList, $transaction, $speciallink);
	}

	private function getMessageTemplate($params, $rsvpdata, $msgkey)
	{
		return $this->helper->getMessageTemplate($params, $rsvpdata, $msgkey);
	}

	// This redirects calls to the helper class is possible
	public function __call($name, $arguments)
	{
		if (is_callable(array($this->helper, $name)))
		{
			return call_user_func_array(array($this->helper, $name), $arguments);
		}
	}

	/*
	 * We cannot rely on __call magic function to pass variables by reference!
	 */
	private function notifyUser($rsvpdata, & $row, $user, $name, $username, $attendee = null, $messagetype = 'ack', $onWaitingList = false, $transaction = false, $subject = "", $message = "", $speciallink = false, $replyto = null)
	{
		if (isset($this->event) && !isset($this->helper->event))
		{
			$this->helper->event = $this->event;
		}
		return $this->helper->notifyUser($rsvpdata, $row, $user, $name, $username, $attendee, $messagetype, $onWaitingList, $transaction, $subject, $message, $speciallink, $replyto);
	}

	private function notifyCreator($rsvpdata, & $row, $name, $username, $attendee = null, $cancellation = false, $onWaitingList = false)
	{
		if (isset($this->event) && !isset($this->helper->event))
		{
			$this->helper->event = $this->event;
		}
		return $this->helper->notifyCreator($rsvpdata, $row, $name, $username, $attendee, $cancellation, $onWaitingList);
	}

	public function calculateBalances(&$view, $rsvpdata, $row)
	{
		return $this->helper->calculateBalances($view, $rsvpdata, $row);
	}

	public function analyseGuests(&$attendee, $rsvpdata, $event)
	{
		return $this->helper->analyseGuests($attendee, $rsvpdata, $event);
	}

	protected function authoriseTransactions($repeat, $atdee)
	{
		if (!$repeat)
		{
			Table::addIncludePath(RSVP_TABLES);
			$attendee = Table::getInstance('jev_attendee');
			$attendee->load($atdee);
			if (!$attendee || !$attendee->at_id)
			{
				$lang = Factory::getLanguage();
				$lang->load(RSVP_COM_COMPONENT, JPATH_ADMINISTRATOR);
				throw new Exception(403, Text::_("RSVP_NOT_AUTHORISED"));
			}
			$this->attendee = $attendee;
			$db = Factory::getDBO();
			$sql = "SELECT * FROM #__jev_attendance WHERE id=" . intval($attendee->at_id);
			$db->setQuery($sql);
			$rsvpdata = $db->loadObject();
			$this->rsvpdata = $rsvpdata;
			$rp_id = $attendee->rp_id;

			$this->dataModel = new JEventsDataModel();
			$this->queryModel = new JEventsDBModel($this->dataModel);

			// Find the first repeat - including unpublished events
			$vevent = $this->dataModel->queryModel->getEventById($rsvpdata->ev_id, true, "icaldb");

			if (!$vevent)
			{
				$user = Factory::getUser();

				if ($user->id == 0 )
				{
					$uri = JURI::getInstance();
					$link = $uri->toString();
					$link = 'index.php?option=com_users&view=login&return='.base64_encode($link);
					$link = Route::_($link, false);
					$this->setRedirect($link, Text::_('RSVP_NOT_AUTHORISED_TO_VALIDATE_TICKET'));
					$this->redirect();
					return false;
				}
				else
                {
                    $lang = Factory::getLanguage();
                    $lang->load(RSVP_COM_COMPONENT, JPATH_ADMINISTRATOR);
                    throw new Exception(403, Text::_("RSVP_NOT_AUTHORISED") . " yyy ");
                }
			}

			if ($rp_id == 0)
			{
				$repeat = $vevent->getFirstRepeat();
			}
			else
			{
				list($year, $month, $day) = JEVHelper::getYMD();
				// include unpublished events
				$repeat = $this->dataModel->queryModel->listEventsById(intval($rp_id), 1, "icaldb");
				//$repeatdata = $this->dataModel->getEventData(intval($rp_id), "icaldb", $year, $month, $day);
				//if ($repeatdata && isset($repeatdata["row"]))
				//	$repeat = $repeatdata["row"];
			}
		}

		if (!$repeat)
		{
			$lang = Factory::getLanguage();
			$lang->load(RSVP_COM_COMPONENT, JPATH_ADMINISTRATOR);
			throw new Exception(403, Text::_("RSVP_NOT_AUTHORISED") . " xxx ");
		}

		// special case for QR Code did-attend check
		if (Factory::getApplication()->input->getCmd("task") == "attendees.attend" || Factory::getApplication()->input->getCmd("task") == "attendees.notattend")
		{
			$params = ComponentHelper::getParams("com_rsvppro");

			$user = Factory::getUser();
			$userGroups = $user->getAuthorisedGroups();

			// is can approve ticket or can edit event allow through otherwise block here
			if (JEVHelper::isAdminUser($user) || array_intersect($params->get('approveticketlevel', array()), array_values($userGroups)) || JEVHelper::canEditEvent($repeat))
			{
				$gc = Factory::getApplication()->input->getInt("gc");
				$securitycheck = md5($params->get("emailkey", "email key") . " gwe did attend " . $atdee . "-" . $gc);
				$securitycheck2 = md5($params->get("emailkey", "email key") . " attendee list " . $user->id);
				$sc = trim(Factory::getApplication()->input->getString("sc"));

				if ($sc === $securitycheck || $sc === $securitycheck2 || Factory::getApplication()->isClient('administrator'))
				{
					$this->event = $repeat;
					return true;
				}
				else
				{
					$lang = Factory::getLanguage();
					$lang->load(RSVP_COM_COMPONENT, JPATH_ADMINISTRATOR);
					echo "<div style='height:100%;width:100%;background-color:red;padding:200px 50px;'><H1 style='padding:100px 20px!important;background-color:white!important;color:red!important;'>" .
					Text::_("RSVP_NOT_AUTHORISED_FAILED_SECURITY_CHECK")
					. "</h1></div>";
					Factory::getApplication()->input->set("tmpl", "component");
					return false;
				}
			}
			else
			{
				if ($user->get("id", 0) == 0)
				{
					$uri = JURI::getInstance();
					$link = $uri->toString();
					$link = 'index.php?option=com_users&view=login&return='.base64_encode($link);
					$link = Route::_($link, false);
					$this->setRedirect($link,Text::_('RSVP_NOT_AUTHORISED_TO_VALIDATE_TICKET'));
					$this->redirect();
                    return false;
					echo "<div style='height:100%;width:100%;background-color:red;padding:200px 50px;'><H1 style='padding:100px 20px!important;background-color:white!important;color:red!important;'>" .
					Text::_("RSVP_NOT_AUTHORISED_TO_VALIDATE_TICKET")
					. "</h1></div>";
					Factory::getApplication()->input->set("tmpl", "component");
					return false;
				}
				echo "<div style='height:100%;width:100%;background-color:red;padding:200px 50px;'><H1 style='padding:100px 20px!important;background-color:white!important;color:red!important;'>" .
				Text::_("RSVP_USER_NOT_AUTHORISED_TO_VALIDATE_TICKET")
				. "</h1></div>";
				Factory::getApplication()->input->set("tmpl", "component");
				return false;
				//throw new Exception(403, Text::_("RSVP_NOT_AUTHORISED_TO_VALIDATE_TICKET")."  user= ". implode(",",array_values($userGroups))."  allowed= ". implode(",",$params->get('approveticketlevel', array())));
			}
		}
		if (!JEVHelper::canEditEvent($repeat))
		{
			$lang = Factory::getLanguage();
			$lang->load(RSVP_COM_COMPONENT, JPATH_ADMINISTRATOR);
			throw new Exception(403, Text::_("RSVP_NOT_AUTHORISED"));
		}
		$this->event = $repeat;
		return true;
	}

	protected function authoriseAttendee($rsvpdata)
	{
		$this->dataModel = new JEventsDataModel();
		$this->queryModel = new JEventsDBModel($this->dataModel);

		// Find the first repeat
		$vevent = $this->dataModel->queryModel->getEventById($rsvpdata->ev_id, true, "icaldb");
		$repeat = $vevent->getFirstRepeat();

		$this->event = $repeat;
		if (!$repeat || !JEVHelper::canEditEvent($repeat))
		{
			$lang = Factory::getLanguage();
			$lang->load(RSVP_COM_COMPONENT, JPATH_ADMINISTRATOR);
			throw new Exception(403, Text::_("RSVP_NOT_AUTHORISED"));
		}
	}

	// post confirmation actions in parameter fields e.g. signup for newsletters etc.
	protected function postUpdateActions($rsvpdata, $row, $attendee, $onWaitingList, $rsvpparams)
	{
		if ($rsvpparams)
		{
			return $rsvpparams->postUpdateActions($rsvpdata, $row, $attendee, $onWaitingList);
		}
	}

	protected function getRsvpDataParams($rsvpdata)
	{
		$params = "";
		if ($rsvpdata->template != "")
		{
			$xmlfile = JevTemplateHelper::getTemplate($rsvpdata);

			if (is_int($xmlfile) || file_exists($xmlfile))
			{
				// Clean up the input
				// If the allow HTML flag is set, apply a safe HTML filter to the variable

				$params = Factory::getApplication()->input->get("xmlfile",Factory::getApplication()->input->post->get("params", array(), "ARRAY"), 'ARRAY');

				foreach ($params as $key => $ea)
				{
					/*
					 * Make the content safe first!
					* param   array()   $tagsArray   List of user-defined tags
					* param   array()   $attrArray   List of user-defined attributes
					* param   0         $tagsMethod  WhiteList method = 0, BlackList method = 1
					* param   0         $attrMethod  WhiteList method = 0, BlackList method = 1
					* param   1         $xssAuto     Only auto clean essentials = 0, Allow clean blacklisted tags/attr = 1
					* param   -1        $stripUSC    Strip 4-byte unicode characters = 1, no strip = 0, ask the database driver = -1
					 */

                    $params[$key] = JEVHelper::arrayFiltered($params[$key]);
                    //$params[$key] = InputFilter::getInstance(array(), array(), 1, 1, 1, -1)->clean($params[$key], 'html');
				}

				$jregistry = new Registry ();
				$jregistry->loadArray($params);

				// checkboxes do not return any data which causes problems when we have guests!
				$rsvpparams = new JevRsvpParameter("", $xmlfile, $rsvpdata, null);
				$emptyarray = $rsvpparams->renderToBasicArray();
				foreach ($emptyarray as $key => $ea)
				{
					if (!isset($params[$key]))
					{
						$jregistry->set($key, "");
					}
					// Store missing checkbox values
					else if ($emptyarray[$key]['type'] === "jevrcheckbox" && is_array($params[$key]) && $emptyarray[$key]['peruser'])
                    {
	                    $startindex = 0;
	                    if ($emptyarray[$key]['peruser'] === 2)
	                    {
		                    $startindex = 1;
	                    }
	                    // problem if tabs have been deleted !
	                    $maxindex = max(array_keys($params[$key]));
	                    $lastguest = Factory::getApplication()->input->getInt('lastguest', 0);
	                    if ($lastguest > 0 && ($maxindex + 1 == $lastguest))
                        {
                            for ($idx = $startindex; $idx <= $maxindex; $idx ++)
                            {
                                if (!isset($params[$key][$idx]))
                                {
                                    $params[$key][$idx] = array();
                                }
                            }
                            ksort($params[$key]);
                            $jregistry->set($key, (object) $params[$key]);
                        }
                    }
				}

				// TODO VERY IMPORTANT - CALCULATE THE FEES AND BALANCE HERE - DON'T RELY ON JAVASCRIPT!!
				if ($jregistry->get("totalfee", -1) != -1)
				{
					$jregistry->set("totalfee", 0);
					$jregistry->set("feepaid", 7.00);
					$jregistry->set("feebalance", 0);
				}

				if (version_compare(PHP_VERSION, '5.4', '>='))
				{
					$params = $jregistry->toString('JSON', array('bitmask' =>JSON_UNESCAPED_UNICODE));
				}
				else
				{
					$params = $jregistry->toString();
				}

				// Must NOT clean the data again since it breaks the JSON encoding escaping of quotes
			}
		}
		return $params;
	}

	protected function getRSVPParmeters($rsvpdata, $row)
	{
		if ($rsvpdata->template != "")
		{
			$xmlfile = JevTemplateHelper::getTemplate($rsvpdata);

			if (is_int($xmlfile) || file_exists($xmlfile))
			{
				$rsvpparams = new JevRsvpParameter("", $xmlfile, $rsvpdata, $row);
				return $rsvpparams;
			}
		}
		return false;
	}

	protected function countAttendeeContributionToAttendance(&$row, $rsvpdata, $attendee, $newattendee = false)
	{
		// TAKE CARE $attendee could be a stdClass!

		// TODO bring this in line with full count attendees code!

		// get the existing contribution to the attendance count

		$unpaidtoo = false;
		if ($attendee->attendstate == 4)
		{
			$xmlfile = JevTemplateHelper::getTemplate($rsvpdata);
			if (is_int($xmlfile) && $xmlfile > 0)
			{

				if (RsvpHelper::getTemplateSpecificParameter($rsvpdata,  "unpaidcapacity",0))
				{
					$unpaidtoo = true;
				}
			}
		}

		if ($attendee->attendstate == 1 || ($attendee->attendstate == 4 && $unpaidtoo))
		{

			$attendeeparams = new Registry($attendee->params);
			$row->currentcnt = $attendee->guestcount;
			unset($row->_reductionapplied);

			// experimental code for excluding primary attendee from the overall count
			if (RsvpHelper::getTemplateSpecificParameter($rsvpdata,  "skipprimarycount",0))
			{
				$row->currentcnt --;
			}

			// New parameterised fields
			if ($rsvpdata->template != "")
			{
				$xmlfile = JevTemplateHelper::getTemplate($rsvpdata);

				if (is_int($xmlfile) || file_exists($xmlfile))
				{
					$rsvpparams = new JevRsvpParameter($attendee->params, $xmlfile, $rsvpdata, $row);
					$params = $rsvpparams->renderInputDataToBasicArray($attendee);
					if ($params) {
					    foreach ($params as $param)
					{
						if (isset($param ["includeintotalcapacity"]) && $param ["includeintotalcapacity"] && isset($param ["nativevalue"]))
						{
							// multiple attendees
							if (is_array($param ["nativevalue"]))
							{
								for ($g = 0; $g <= $row->newcnt; $g++)
								{
									$row->currentcnt += intval($param ["nativevalue"][$g]);
									if (isset($param ["reducevaluefortotalcapacity"]) && intval($param ["nativevalue"][$g]) > 0 && !isset($row->_reductionapplied))
									{
										$row->currentcnt -= $param ["reducevaluefortotalcapacity"];
										if (!empty($param ["reducevaluefortotalcapacity"]))
										{
											// only count this as as a reduction applied IF the reduction has a value!
											$row->_reductionapplied = 1;
										}
									}
								}
							}
							else
							{
								$row->currentcnt += intval($param ["nativevalue"]);
								if (isset($param ["reducevaluefortotalcapacity"]) && intval($param ["nativevalue"]) > 0 && !isset($row->_reductionapplied))
								{
									$row->currentcnt -= $param ["reducevaluefortotalcapacity"];
									if (!empty($param ["reducevaluefortotalcapacity"]))
									{
										// only count this as as a reduction applied IF the reduction has a value!
										$row->_reductionapplied = 1;
									}
								}
							}
						}
					}
					}
				}
			}
			else
			{
				$row->currentcnt = 1;
			}
		}
		else
		{
			$row->currentcnt = 0;
		}

		$jevattend = Factory::getApplication()->input->getInt("jevattend", 0);
		// Now the new ones
		if ($jevattend == 1 || $newattendee)
		{
			$attendeeparams = new Registry(Factory::getApplication()->input->get("xmlfile",array(), 'ARRAY'));

			unset($row->_reductionapplied);
			$row->newcnt = Factory::getApplication()->input->getInt("guestcount",0);

			// experimental code for excluding primary attendee from the overall count
			if (RsvpHelper::getTemplateSpecificParameter($rsvpdata,  "skipprimarycount",0))
			{
				$row->newcnt --;
			}

			// New parameterised fields
			if ($rsvpdata->template != "")
			{
				$xmlfile = JevTemplateHelper::getTemplate($rsvpdata);

				if (is_int($xmlfile) || file_exists($xmlfile))
				{
					$rsvpparams = new JevRsvpParameter($attendeeparams, $xmlfile, $rsvpdata, $row);
					$params = $rsvpparams->renderInputDataToBasicArray($attendee);
					foreach ($params as $param)
					{
						if (isset($param ["includeintotalcapacity"]) && $param ["includeintotalcapacity"] && isset($param ["nativevalue"]))
						{
                            // multiple attendees
                            if (is_array($param ["nativevalue"])){
                                for($g=0;$g<=$row->newcnt; $g++){
                                    $row->newcnt += intval($param ["nativevalue"][$g]);
                                    if (isset($param ["reducevaluefortotalcapacity"]) && intval($param ["nativevalue"][$g]) > 0 && !isset($row->_reductionapplied))
                                    {
                                        $row->newcnt -= $param ["reducevaluefortotalcapacity"];
	                                    if (!empty($param ["reducevaluefortotalcapacity"]))
	                                    {
		                                    // only count this as as a reduction applied IF the reduction has a value!
		                                    $row->_reductionapplied = 1;
	                                    }
                                    }
                                }
                            }
                            else {
                                $row->newcnt += intval($param ["nativevalue"]);
                                if (isset($param ["reducevaluefortotalcapacity"]) && intval($param ["nativevalue"]) > 0 && !isset($row->_reductionapplied))
                                {
                                        $row->newcnt -= $param ["reducevaluefortotalcapacity"];
	                                if (!empty($param ["reducevaluefortotalcapacity"]))
	                                {
		                                // only count this as as a reduction applied IF the reduction has a value!
		                                $row->_reductionapplied = 1;
	                                }
                                }
                            }
						}
					}
				}
			}
			else
			{
				$row->newcnt = 1;
			}
		}
		else
		{
			$row->newcnt = 0;
		}
		unset($row->_reductionapplied);

	}

}
