<?php
/**
 * JEvents Component for Joomla 1.5.x
 *
 * @version     $Id: icalevent.php 1712 2010-03-04 07:33:11Z geraint $
 * @package     JEvents
 * @copyright   Copyright (C) 2008-2021 GWE Systems Ltd, 2006-2008 JEvents Project Group
 * @license     GNU/GPLv2, see http://www.gnu.org/licenses/gpl-2.0.html
 * @link        https://www.jevents.net
 */

defined( '_JEXEC' ) or die( 'Restricted Access' );

use Joomla\CMS\MVC\Controller\BaseController;
use Joomla\CMS\Language\Text;
use Joomla\CMS\Component\ComponentHelper;
use Joomla\CMS\Plugin\PluginHelper;
use Joomla\CMS\Pagination\Pagination;
use Joomla\CMS\Router\Route;
use Joomla\CMS\Factory;
use Joomla\CMS\Filesystem\File;

jimport('joomla.application.component.controller');


class AdminInviteesController extends BaseController {

	protected $params;
	protected $jomsocial = false;

	/**
	 * Controler for Sessions
	 * @param array		configuration
	 */
	function __construct($config = array())
	{
		parent::__construct($config);
		$this->registerTask( 'update',  'updateInvitees' );
		$this->registerTask( 'list',  'overview' );
		$this->registerDefaultTask("overview");

		// Must load admin language files
		$lang = Factory::getLanguage();
		$lang->load("com_jevents", JPATH_ADMINISTRATOR);

		$this->dataModel = new JEventsDataModel();
		$this->queryModel = new JEventsDBModel($this->dataModel);

		jimport('joomla.filesystem.file');
		if (File::exists(JPATH_SITE.'/components/com_community/community.php')){
			if (ComponentHelper::isEnabled("com_community")) {
				$this->jomsocial = true;
			}
		}

		$plugin = PluginHelper::getPlugin('jevents', 'jevrsvppro');
		$this->params = ComponentHelper::getParams("com_rsvppro");

		include_once(JPATH_ADMINISTRATOR."/components/com_rsvppro/libraries/inviteehelper.php");
		$this->helper = new RsvpInviteeHelper($this->params);

	}


	/**
	 * List Invitees
	 *
	 */
	function overview( )
	{
		// get the view
		$this->view = $this->getView("invitees","html");

		$this->view->params =  ComponentHelper::getParams("com_rsvppro");

		$mainframe = Factory::getApplication();$option=Factory::getApplication()->input->getCmd("option");

		$db	= Factory::getDBO();

		$atd_id = Factory::getApplication()->input->get("atd_id", array(), 'ARRAY');
		if (!isset($atd_id[0]) || strpos($atd_id[0],"|")===false){
			throw new Exception( Text::_("RSVP_MISSING_ATDID"), 403);
		}
		list($atd_id, $rp_id) = explode("|",$atd_id[0]);

		$atd_id = intval($atd_id);
		$rp_id = intval($rp_id);

		$repeating = Factory::getApplication()->input->getInt('repeating', 0 );

		$limit		= intval( $mainframe->getUserStateFromRequest( "inviteeslistlimit", 'limit', Factory::getApplication()->get("list_limit", 10) ));
		$limitstart = intval( $mainframe->getUserStateFromRequest( "inviteeslimitstart", 'limitstart', 0 ));

		$where = array();
		$join = array();

		$where[] = "ev.ev_id IS NOT NULL";
		$where[] = "atd.id = $atd_id";
		$where[] = "inv.rp_id = $rp_id";

		$filter_order		= $mainframe->getUserStateFromRequest( $option.'inv_filter_order',		'filter_order',		'atd.id',	'cmd' );
		if ($filter_order=="invcount" ||  $filter_order=="ev.created_by") $filter_order='atd.id';
		$filter_order_Dir	= $mainframe->getUserStateFromRequest( $option.'inv_filter_order_Dir',	'filter_order_Dir',	' ASC',				'word' );
		$orderby 	= ' ORDER BY '.$filter_order.' '.$filter_order_Dir;

		$search		= $mainframe->getUserStateFromRequest( "search{".RSVP_COM_COMPONENT."}", 'search', '' );
		$search		= $db->escape( trim( strtolower( $search ) ) );

		if( $search ){
			$where[] = "(inv.email_address LIKE '%$search%' OR ju.username LIKE '%$search%' OR ju.email LIKE '%$search%' )";
		}

		if ($repeating){
			$where[] = "atd.allinvites=1 AND atd.invites>0 ";
		}
		else {
			$where[] = "atd.allinvites=0  AND atd.invites>0 ";
		}

		$query = "SELECT count(distinct inv.id)"
		. "\n FROM #__jevents_vevent AS ev "
		. "\n LEFT JOIN #__jevents_vevdetail as det ON ev.detail_id=det.evdet_id"
		. "\n LEFT JOIN #__jev_attendance AS atd ON atd.ev_id = ev.ev_id"
		. "\n LEFT JOIN #__jev_invitees AS inv ON inv.at_id = atd.id"
		. "\n LEFT JOIN #__users AS ju ON ju.id = inv.user_id"
		. ( count( $join) ? "\n LEFT JOIN  " . implode( ' LEFT JOIN ', $join) : '' )
		. ( count( $where ) ? "\n WHERE " . implode( ' AND ', $where ) : '' );
		$db->setQuery( $query);
		try {
			$total = $db->loadResult();
		}
		catch (RuntimeException $e)
		{
			$error = $e->getMessage();
			echo $error;
			$total = 0;
		}
		if( $limit > $total ) {
			$limitstart = 0;
		}

		$query = "SELECT det.*, atd.* , atd.id as atd_id, atdc.atdcount, inv.*,inv.id as inv_id,   a.id as attending, a.attendstate, "
		. " CASE WHEN inv.user_id=0 THEN inv.email_address ELSE CONCAT_WS(' - ',ju.username,ju.email) END as attendee "
		. "\n FROM #__jevents_vevent as ev "
		. "\n LEFT JOIN #__jevents_vevdetail as det ON ev.detail_id=det.evdet_id"
		. "\n LEFT JOIN #__jev_attendance AS atd ON atd.ev_id = ev.ev_id"
		. "\n LEFT JOIN #__jev_attendeecount AS atdc ON atd.id = atdc.at_id"
		. "\n LEFT JOIN #__jev_invitees AS inv ON inv.at_id = atd.id"
		." LEFT JOIN #__jev_attendees as a ON a.user_id=inv.user_id AND a.at_id=inv.at_id AND a.rp_id=inv.rp_id"
		. "\n LEFT JOIN #__users AS ju ON ju.id = inv.user_id"
		. ( count( $join) ? "\n LEFT JOIN  " . implode( ' LEFT JOIN ', $join) : '' )
		. ( count( $where ) ? "\n WHERE " . implode( ' AND ', $where ) : '' )
		. "\n GROUP BY inv.id $orderby"
		;

		if ($limit>0){
			$query .= "\n LIMIT $limitstart, $limit";
		}
		$db->setQuery( $query );

		try {
			$rows = $db->loadObjectList();
		}
		catch (RuntimeException $e)
		{
			$error = $e->getMessage();
			echo $error;
			$rows = array();
		}

		foreach ($rows as $key=>$val) {

		}

		$db = Factory::getDBO();
		$sql = "SELECT * FROM #__jev_attendance WHERE id=".intval($atd_id);
		$db->setQuery($sql);
		$rsvpdata = $db->loadObject();

		$this->dataModel = new JEventsDataModel();
		$this->queryModel = new JEventsDBModel($this->dataModel);

		// Find the first repeat - iclude unpublished events
		$vevent = $this->dataModel->queryModel->getEventById(  $rsvpdata->ev_id, true, "icaldb");
		if (!$vevent) {
			Factory::getApplication()->enqueueMessage( Text::_('RSVP_PRO_EVENT_INACCESSIBLE_OR_MISSING'));
			Factory::getApplication()->redirect(Route::_("index.php?option=com_rsvppro&task=sessions.list"));
		}
		if ($rp_id==0){
			$repeat = $vevent->getFirstRepeat();
		}
		else {
			list($year,$month,$day) = JEVHelper::getYMD();
                        // include unpublished events
                        $repeat = $this->dataModel->queryModel->listEventsById (intval($rp_id), 1, "icaldb");
			//$repeatdata = $this->dataModel->getEventData(intval($rp_id), "icaldb", $year, $month, $day);
			//if ($repeatdata && isset($repeatdata["row"])) $repeat = $repeatdata["row"];

		}

		jimport('joomla.html.pagination');
		$pageNav = new Pagination( $total, $limitstart, $limit  );

		// Set the layout
		$this->view->setLayout('overview');

		$this->view->atd_id = $atd_id ;
		$this->view->rp_id = $rp_id;
		$this->view->rows = $rows;
		$this->view->rsvpdata = $rsvpdata;
		$this->view->repeat = $repeat;
		$this->view->vevent = $vevent;
		$this->view->search = $search;
		$this->view->order = $filter_order;
		$this->view->orderdir = $filter_order_Dir;
		$this->view->repeating = $repeating;
		$this->view->pageNav = $pageNav;

		$this->view->display();
	}


	public function updateInvitees(){
		$user=Factory::getUser();
		if ($user->id==0){
			return "";
		}

		$redirect = true;

        $db =  Factory::getDBO ();
		$at_id = Factory::getApplication()->input->getInt("at_id",-1);
        $sql = "SELECT * FROM #__jev_attendance WHERE id=" . $at_id;
        $db->setQuery($sql);
        $rsvpdata = $db->loadObject();
		if (!$rsvpdata) return false;

		$rp_id = Factory::getApplication()->input->getInt("rp_id",-1);
		$datamodel = new JEventsDataModel();
		$row = $datamodel->queryModel->listEventsById($rp_id, 1, "icaldb");
		if (!$row) return false;

		return $this->helper->updateInvitees($rsvpdata, $row, $redirect);

	}


}
