<?php

/**
 * JEvents Component for Joomla 1.5.x
 *
 * This file based on Joomla config component Copyright (C) 2005 - 2008 Open Source Matters.
 *
 * @version     $Id: params.php 3548 2012-04-20 09:25:43Z geraintedwards $
 * @package     JEvents
 * @copyright   Copyright (C) 2008-2021 GWE Systems Ltd, 2006-2008 JEvents Project Group
 * @license     GNU/GPLv2, see http://www.gnu.org/licenses/gpl-2.0.html
 * @link        https://www.jevents.net
 */
defined('_JEXEC') or die('Restricted access');

use Joomla\CMS\Session\Session;
use Joomla\CMS\MVC\Controller\AdminController;
use Joomla\CMS\Table\Table;
use Joomla\CMS\Language\Text;
use Joomla\CMS\Component\ComponentHelper;
use Joomla\CMS\Router\Route;
use Joomla\CMS\Cache\Cache;
use Joomla\CMS\Form\Form;
use Joomla\CMS\Factory;

jimport('joomla.application.component.controlleradmin');

class AdminParamsController extends AdminController
{

	/**
	 * Custom Constructor
	 */
	function __construct($default = array())
	{
		$user = Factory::getUser();

		if (!JEVHelper::isAdminUser())
		{
			Factory::getApplication()->enqueueMessage( Text::_("Not Authorised - must be admin"));
			Factory::getApplication()->redirect("index.php?option=" . RSVP_COM_COMPONENT . "&task=cpanel.cpanel");
			return;
		}

		$default['default_task'] = 'edit';
		parent::__construct($default);

		$this->registerTask('apply', 'save');

	}

	/**
	 * Show the configuration edit form
	 * @param string The URL option
	 */
	function edit()
	{

		// get the view
		$this->view = $this->getView("params", "html");

		//$model = $this->getModel('params');
		$model = $this->getModel('component');
		$table =  Table::getInstance('extension');
		if (!$table->load(array("element" => "com_rsvppro", "type" => "component"))) // 1.6 mod
		{
			\Joomla\CMS\Factory::getApplication()->enqueueMessage('Not a valid component', "warning");
			return false;
		}
		// Backwards compatatbility
		$table->id = $table->extension_id;
		$table->option = $table->element;

		// Set the layout
		$this->view->setLayout('edit');

		$this->view->component = $table;
		$this->view->setModel($model, true);
		$this->view->display();

	}

	/**
	 * Save the configuration
	 */
	function save()
	{
		// Check for request forgeries
		Session::checkToken() or jexit('Invalid Token');
		//echo $this->getTask();
		//exit;
		$component = RSVP_COM_COMPONENT;

		$model = $this->getModel('params');
		$table =  Table::getInstance('extension');
		//if (!$table->loadByOption( $component ))
		if (!$table->load(array("element" => "com_rsvppro", "type" => "component"))) // 1.6 mod
		{
			\Joomla\CMS\Factory::getApplication()->enqueueMessage( 'Not a valid component', "warning");
			return false;
		}


		$post = Factory::getApplication()->input->get('post');
		$post['params'] = Factory::getApplication()->input->get("jform",array(), 'ARRAY');
		$post['option'] = $component;
		$table->bind($post);

		// pre-save checks
		if (!$table->check())
		{
			\Joomla\CMS\Factory::getApplication()->enqueueMessage($table->getError(), "warning");
			return false;
		}

		// save the changes
		if (!$table->store())
		{
			\Joomla\CMS\Factory::getApplication()->enqueueMessage( $table->getError(), "warning");
			return false;
		}

		// Now save the form permissions data
		$data = Factory::getApplication()->input->get("jform",array(), 'ARRAY');
		$option = RSVP_COM_COMPONENT;
		$comp = ComponentHelper::getComponent(RSVP_COM_COMPONENT);
		$id = $comp->id;
		// Validate the posted data.
		Form::addFormPath(JPATH_COMPONENT);
		Form::addFieldPath(JPATH_COMPONENT . '/fields');

		$form = $model->getForm();
		$return = $model->validate($form, $data);

		// Check for validation errors.
		if ($return === false)
		{
			// Get the validation messages.
			$errors = $model->getErrors();
			$app = Factory::getApplication();
			// Push up to three validation messages out to the user.
			for ($i = 0, $n = count($errors); $i < $n && $i < 3; $i++)
			{
				if (JError::isError($errors[$i]))
				{
					$app->enqueueMessage($errors[$i]->getMessage(), 'notice');
				}
				else
				{
					$app->enqueueMessage($errors[$i], 'notice');
				}
			}

			// Save the data in the session.
			$app->setUserState('com_config.config.global.data', $data);
			// Redirect back to the edit screen.
			$this->setRedirect(Route::_('index.php?option=' . RSVP_COM_COMPONENT . '&task=params.edit', false));
			return false;
		}

		// Attempt to save the configuration.
		$data = array(
			'params' => $return,
			'id' => $id,
			'option' => $option
		);
		$return = $model->saveRules($data);

		// Clear cache of com_config component.
		$this->cleanCache('_system', 1); // admin
		$this->cleanCache('_system', 0); // site

		//SAVE AND APPLY CODE FROM PRAKASH
		switch ($this->getTask()) {
			case 'apply':
				$this->setRedirect('index.php?option=' . RSVP_COM_COMPONENT . '&task=params.edit', Text::_('RSVP_CONFIG_SAVED'));
				break;
			default:
				$this->setRedirect('index.php?option=' . RSVP_COM_COMPONENT . "&task=cpanel.cpanel", Text::_('RSVP_CONFIG_SAVED'));
				break;
		}
		//$this->setRedirect( 'index.php?option='.RSVP_COM_COMPONENT."&task=cpanel.cpanel", Text::_( 'RSVP_CONFIG_SAVED' ) );
		//$this->setMessage(Text::_( 'RSVP_CONFIG_SAVED' ));
		//$this->edit();

	}

	/**
	 * Apply the configuration
	 */
	function apply()
	{
		// Check for request forgeries
		Session::checkToken() or jexit('Invalid Token');

		$component = RSVP_COM_COMPONENT;

		$model = $this->getModel('params');
		$table =  Table::getInstance('extension');
		//if (!$table->loadByOption( $component ))
		if (!$table->load(array("element" => "com_rsvppro", "type" => "component"))) // 1.6 mod
		{
			\Joomla\CMS\Factory::getApplication()->enqueueMessage('Not a valid component', "warning");
			return false;
		}


		$post = Factory::getApplication()->input->get('post');
		$post['option'] = $component;
		$table->bind($post);

		// pre-save checks
		if (!$table->check())
		{
			\Joomla\CMS\Factory::getApplication()->enqueueMessage( $table->getError(), "warning");
			return false;
		}

		// save the changes
		if (!$table->store())
		{
			\Joomla\CMS\Factory::getApplication()->enqueueMessage( $table->getError(), "warning");
			return false;
		}

		// Now save the form permissions data
		$data = Factory::getApplication()->input->get("jform",array(), 'ARRAY');
		$option = RSVP_COM_COMPONENT;
		$comp = ComponentHelper::getComponent(RSVP_COM_COMPONENT);
		$id = $comp->id;
		// Validate the posted data.
		Form::addFormPath(JPATH_COMPONENT);
		Form::addFieldPath(JPATH_COMPONENT . '/fields');
		$form = $model->getForm();
		$return = $model->validate($form, $data);

		// Check for validation errors.
		if ($return === false)
		{
			// Get the validation messages.
			$errors = $model->getErrors();

			$app = Factory::getApplication();
			// Push up to three validation messages out to the user.
			for ($i = 0, $n = count($errors); $i < $n && $i < 3; $i++)
			{
				if (JError::isError($errors[$i]))
				{
					$app->enqueueMessage($errors[$i]->getMessage(), 'notice');
				}
				else
				{
					$app->enqueueMessage($errors[$i], 'notice');
				}
			}

			// Save the data in the session.
			$app->setUserState('com_config.config.global.data', $data);
			// Redirect back to the edit screen.
			$this->setRedirect(Route::_('index.php?option=' . RSVP_COM_COMPONENT . '&task=params.edit', false));
			return false;
		}

		// Attempt to save the configuration.
		$data = array(
			'params' => $return,
			'id' => $id,
			'option' => $option
		);
		$return = $model->saveRules($data);

		// Clear cache of com_config component.
		$this->cleanCache('_system');

		$this->setRedirect('index.php?option=' . RSVP_COM_COMPONENT . "&task=params.edit", Text::_('RSVP_CONFIG_SAVED'));
		//$this->setMessage(Text::_( 'RSVP_CONFIG_SAVED' ));
		//$this->edit();

	}

	/**
	 * Cancel operation
	 */
	function cancel()
	{
		$this->setRedirect('index.php');

	}


	/**
	 * Clean the cache
	 *
	 * @param   string   $group      The cache group
	 * @param   integer  $client_id  The ID of the client
	 *
	 * @return  void
	 *
	 * @since   12.2
	 */
	protected function cleanCache($group = null, $client_id = 0)
	{
		$conf = Factory::getConfig();

		$options = array(
			'defaultgroup' => ($group) ? $group : (isset($this->option) ? $this->option : Factory::getApplication()->input->get('option')),
			'cachebase' => ($client_id) ? JPATH_ADMINISTRATOR . '/cache' : $conf->get('cache_path', JPATH_SITE . '/cache'));

		$cache = Cache::getInstance('callback', $options);
		$cache->clean();

		// If caching is enabled then remove the component params from the cache!
		// Bug fixed in Joomla 3.2.5 ??
		$joomlaconfig = Factory::getConfig();
		if ($joomlaconfig->get("caching",0)){
			$cacheController = Factory::getCache('_system', 'callback');
			$cacheController->cache->remove("com_jevents");
			$cacheController->cache->remove("com_rsvppro");
		}

		// Trigger the onContentCleanCache event.
		$this->event_clean_cache = 'onContentCleanCache';
		Factory::getApplication()->triggerEvent($this->event_clean_cache, $options);
	}

}
