<?php
/**
 * JEvents Component for Joomla 1.5.x
 *
 * @version     $Id: cpanel.php 1429 2009-04-28 16:45:57Z geraint $
 * @package     JEvents
 * @copyright   Copyright (C) 2008-2021 GWE Systems Ltd
 * @license     GNU/GPLv2, see http://www.gnu.org/licenses/gpl-2.0.html
 * @link        https://www.jevents.net
 */

defined( 'JPATH_BASE' ) or die( 'Direct Access to this location is not allowed.' );

use Joomla\Utilities\ArrayHelper;
use Joomla\CMS\Session\Session;
use Joomla\CMS\MVC\Controller\BaseController;
use Joomla\CMS\Language\Text;
use Joomla\CMS\Router\Route;
use Joomla\CMS\Factory;

jimport('joomla.application.component.controller');

class AdminTemplatesController extends BaseController {
	/**
	 * Controler for the Control Panel
	 * @param array		configuration
	 */
	function __construct($config = array())
	{
		parent::__construct($config);
		$this->registerTask( 'list',  'overview' );
		$this->registerTask( 'new',  'edit' );
		$this->registerTask( 'editcopy',  'edit' );
		$this->registerTask( 'apply',  'save' );
		$this->registerDefaultTask("overview");

		if (!JevTemplateHelper::canCreateOwn()){
			throw new Exception( Text::_('ALERTNOTAUTH') , 403);
		}

	}

	/**
	 * List Ical Events
	 *
	 */
	function overview( )
	{
		// get the view
		$this->view = $this->getView("templates","html");

		// Set the layout
		$this->view->setLayout('overview');

		// Get/Create the model
		if ($model =  $this->getModel("templates", "rsvpprotemplatesModel")) {
			// Push the model into the view (as template)
			$this->view->setModel($model, true);
		}

		$this->view->overview();
	}

	function edit(){
		// get the view
		$this->view = $this->getView("templates","html");

		// Set the layout
		$this->view->setLayout('edit');

		// Get/Create the model
		if ($model =  $this->getModel("template", "rsvpprotemplatesModel")) {
			// Push the model into the view (as template)
			$this->view->setModel($model, true);
		}

		$this->view->edit();

	} // edittemplates()

	function translate(){
		// get the view
		$this->view = $this->getView("templates","html");

		// Set the layout
		$this->view->setLayout('translate');

		// Get/Create the model
		if ($model =  $this->getModel("template", "RsvpprotemplatesModel")) {
			// Push the model into the view (as template)
			$this->view->setModel($model, true);
		}

		$this->view->translate();

	} // edittemplates()


	function cancel(){
		if (Factory::getApplication()->input->getString("tmpl")=="component"){
			$this->close();
			return;
		}
		else {
			$this->setRedirect(Route::_("index.php?option=".RSVP_COM_COMPONENT."&task=templates.overview", false) );
		}
	}


	function save() {

		$cid = Factory::getApplication()->input->get("cid", array(0), 'ARRAY');;
		$cid = ArrayHelper::toInteger($cid);
		if (count($cid)>0){

			// Get/Create the model
			if ($model =  $this->getModel("template", "RsvpprotemplatesModel")) {
				//$data = JEVHelper::arrayFiltered(Factory::getApplication()->input->getArray(array(), null, 'RAW'));
				$input      = Factory::getApplication()->input;
				$data   = $input->getArray(array(), null, 'RAW');

				try {
					if (!$model->store($data))
					{
						echo "<script> alert('" . Text::_('JLIB_DATABASE_ERROR_EMPTY_ROW_RETURNED', true) . "'); window.history.go(-1); </script>\n";
						exit();
					}
					$template = $model->template;
					// A New one edited directly from the event
					if ($this->getTask()=="apply"){
						$link = "index.php?option=".RSVP_COM_COMPONENT."&task=templates.edit&cid[0]=".$template->id;
						if (Factory::getApplication()->input->getInt("customise",0)){
							$link .= "&customise=1";
							$link .= "&tmpl=component";
						}
						$link .= "&evid=".Factory::getApplication()->input->getInt("evid");
						$action = "document.location = '$link';";
					}
					else {
						$action = 	"try {	window.parent.jQuery('#customisetemplate').modal('hide');}	catch (e){}";
						$action .= 	"try {	window.parent.closeJevModalBySelector('#customisetemplate');}	catch (e){}";

						$link = "index.php?option=".RSVP_COM_COMPONENT."&task=templates.overview";
					}
					if (intval($cid[0])<=0 && Factory::getApplication()->input->getInt("customise",0)==1){
						echo "<script type='text/javascript'>window.parent.setTemplate(".$template->id.",'".addslashes($template->title)."');window.parent.jQuery('#custom_rsvp_template').trigger('liszt:updated');alert('".Text::_("JEV_TEMPLATE_SAVED", true)."');$action</script>";
						if ($this->getTask()=="apply"){
							$this->setRedirect($link);
						}
						else {
							exit();
						}
					}
					else if ( Factory::getApplication()->input->getInt("customise",0)==1){
						echo "<script type='text/javascript'>window.parent.setTemplateTitle(".$template->id.",'".addslashes($template->title)."');window.parent.jQuery('#custom_rsvp_template').trigger('liszt:updated');alert('".Text::_("JEV_TEMPLATE_SAVED", true)."');$action</script>";
						if ($this->getTask()=="apply"){
							$this->setRedirect($link);
						}
						else {
							exit();
						}
					}
					else {
						if ($this->getTask()=="apply"){
							$this->setRedirect($link, Text::_("RSVP_TEMPLATE_SAVED"));
							echo "<script> alert('".addslashes(Text::_("RSVP_TEMPLATE_SAVED"))."'); window.history.go(-1); </script>\n";
						}
						else {
							$this->setRedirect($link, Text::_("RSVP_TEMPLATE_SAVED"));
						}
					}
					return;
				}
				catch (Exception $e)
				{
					echo "<script> alert('".addslashes($e->getMessage())."'); window.history.go(-1); </script>\n";
					exit();
				}
			}
		}

	}

	function savetranslation() {

		$cid = Factory::getApplication()->input->get("cid", array(0), 'ARRAY');;
		$cid = ArrayHelper::toInteger($cid);
		if (count($cid)>0){

			// clean out the cache
			$cache = Factory::getCache('com_jevents');
			$cache->clean(JEV_COM_COMPONENT);

			// Get/Create the model
			if ($model =  $this->getModel("template", "RsvpprotemplatesModel"))
			{
				$model->saveTranslation();
			}
		}
		ob_end_clean();
		if (!headers_sent())
		{
			header('Content-Type:text/html;charset=utf-8');
		}
		$link = Route::_('index.php?option=com_rsvppro&task=templates.overview', false);
		?>
		<script type="text/javascript">
			window.parent.location="<?php echo $link; ?>";
		</script>
		<?php
		exit();


	}

	function deletetranslation() {

		$cid = Factory::getApplication()->input->get("cid", array(0), 'ARRAY');;
		$cid = ArrayHelper::toInteger($cid);
		if (count($cid)>0){

			// clean out the cache
			$cache = Factory::getCache('com_jevents');
			$cache->clean(JEV_COM_COMPONENT);

			// Get/Create the model
			if ($model =  $this->getModel("template", "RsvpprotemplatesModel"))
			{
				$model->deleteTranslation();
			}
		}
		ob_end_clean();
		if (!headers_sent())
		{
			header('Content-Type:text/html;charset=utf-8');
		}
		$link = Route::_('index.php?option=com_rsvppro&task=templates.overview', false);
		?>
		<script type="text/javascript">
			window.parent.location="<?php echo $link; ?>";
		</script>
		<?php
		exit();


	}

	function savecopy() {

		$cid = ArrayHelper::toInteger(Factory::getApplication()->input->get("cid", array(0), 'ARRAY'));

		if (count($cid)>0){

			// Get/Create the model
			if ($model =  $this->getModel("template", "RsvpprotemplatesModel")) {

				$input      = Factory::getApplication()->input;
				$postVars   = $input->getArray(array(), null, 'RAW');

				//$postVars = JEVHelper::arrayFiltered(Factory::getApplication()->input->getArray(array(), null, 'RAW'));

				if ($model->storecopy($postVars)){
					// Now need to copy any associated translations
					$this->setRedirect(Route::_("index.php?option=".RSVP_COM_COMPONENT."&task=templates.overview", false), Text::_("RSVP_TEMPLATE_SAVED"));
					return;
				}
				else {
					echo "<script> alert('".addslashes($model->getError())."'); window.history.go(-1); </script>\n";
					exit();
				}
			}
		}

	}

	function globalise()
	{
		// Check for request forgeries
		Session::checkToken() or jexit( 'Invalid Token' );

		if (JevTemplateHelper::canCreateGlobal()){
			$cid = Factory::getApplication()->input->get("cid", array(), 'ARRAY');
			$cid = ArrayHelper::toInteger($cid);

			if (count( $cid ) < 1) {
				throw new Exception(500, Text::_( 'Select an item to publish' ) );
			}

			$this->_authoriseAccess($cid);

			$model =  $this->getModel("template", "RsvpprotemplatesModel");
			if(!$model->globalise($cid, 1)) {
				echo "<script> alert('".$model->getError(true)."'); window.history.go(-1); </script>\n";
			}
			$returntask	= Factory::getApplication()->input->getCmd( 'returntask', "templates.overview");
			if ($returntask!="templates.list" && $returntask!="templates.overview" && $returntask!="templates.select"){
				$returntask="templates.overview";
			}
			if (method_exists($this,str_replace("templates.","",$returntask))){
				$returntask = str_replace("templates.","",$returntask);
				return $this->$returntask();
			}

			$tmpl = "";
			if ( Factory::getApplication()->input->getString("tmpl","")=="component"){
				$tmpl ="&tmpl=component";
			}
		}
		$this->setRedirect(Route::_( 'index.php?option=com_rsvppro&task=templates.list' . $tmpl, false));
	}

	function lock()
	{
		// Check for request forgeries
		Session::checkToken() or jexit( 'Invalid Token' );

		if (JevTemplateHelper::canCreateGlobal()){
			$cid = Factory::getApplication()->input->get("cid", array(), 'ARRAY');
			$cid = ArrayHelper::toInteger($cid);

			if (count( $cid ) < 1) {
				throw new Exception(500, Text::_( 'Select an item to lock' ) );
			}

			$this->_authoriseAccess($cid);

			$model =  $this->getModel("template", "RsvpprotemplatesModel");
			if(!$model->lock($cid, 1)) {
				echo "<script> alert('".$model->getError(true)."'); window.history.go(-1); </script>\n";
			}

			$returntask	= Factory::getApplication()->input->getCmd( 'returntask', "templates.overview");
			if ($returntask!="templates.list" && $returntask!="templates.overview" && $returntask!="templates.select"){
				$returntask="templates.overview";
			}
			if (method_exists($this,str_replace("templates.","",$returntask))){
				$returntask = str_replace("templates.","",$returntask);
				return $this->$returntask();
			}

			$tmpl = "";
			if ( Factory::getApplication()->input->getString("tmpl","")=="component"){
				$tmpl ="&tmpl=component";
			}
		}
		$this->setRedirect(Route::_( 'index.php?option=com_rsvppro&task=templates.list' . $tmpl, false));
	}

	function unlock()
	{
		// Check for request forgeries
		Session::checkToken() or jexit( 'Invalid Token' );

		if (JevTemplateHelper::canCreateGlobal()){
			$cid = Factory::getApplication()->input->get("cid", array(), 'ARRAY');
			$cid = ArrayHelper::toInteger($cid);

			if (count( $cid ) < 1) {
				throw new Exception(500, Text::_( 'Select an item to unlock' ) );
			}

			$this->_authoriseAccess($cid);

			$model =  $this->getModel("template", "RsvpprotemplatesModel");
			if(!$model->lock($cid, 0)) {
				echo "<script> alert('".$model->getError(true)."'); window.history.go(-1); </script>\n";
			}

			$returntask	= Factory::getApplication()->input->getCmd( 'returntask', "templates.overview");
			if ($returntask!="templates.list" && $returntask!="templates.overview" && $returntask!="templates.select"){
				$returntask="templates.overview";
			}
			if (method_exists($this,str_replace("templates.","",$returntask))){
				$returntask = str_replace("templates.","",$returntask);
				return $this->$returntask();
			}

			$tmpl = "";
			if ( Factory::getApplication()->input->getString("tmpl","")=="component"){
				$tmpl ="&tmpl=component";
			}
		}
		$this->setRedirect(Route::_( 'index.php?option=com_rsvppro&task=templates.list' . $tmpl, false));
	}

	function privatise()
	{
		// Check for request forgeries
		Session::checkToken() or jexit( 'Invalid Token' );

		if (JevTemplateHelper::canCreateGlobal()){

			$cid = Factory::getApplication()->input->get("cid", array(), 'ARRAY');
			$cid = ArrayHelper::toInteger($cid);

			if (count( $cid ) < 1) {
				throw new Exception(500, Text::_( 'Select an item to unpublish' ) );
			}

			$this->_authoriseAccess($cid);

			$model =  $this->getModel("template", "RsvpprotemplatesModel");
			if(!$model->globalise($cid, 0)) {
				echo "<script> alert('".$model->getError(true)."'); window.history.go(-1); </script>\n";
			}

			$returntask	= Factory::getApplication()->input->getCmd( 'returntask', "templates.overview");
			if ($returntask!="templates.list" && $returntask!="templates.overview" && $returntask!="templates.select"){
				$returntask="templates.overview";
			}
			if (method_exists($this,str_replace("templates.","",$returntask))){
				$returntask = str_replace("templates.","",$returntask);
				return $this->$returntask();
			}

			$tmpl = "";
			if (Factory::getApplication()->input->getString("tmpl","")=="component"){
				$tmpl ="&tmpl=component";
			}
		}

		//$this->list();
		$this->setRedirect( Route::_('index.php?option=com_rsvppro&task=templates.list'.$tmpl, false) );
	}

	function maketemplate()
	{
		// Check for request forgeries
		Session::checkToken() or jexit( 'Invalid Token' );

		if (JevTemplateHelper::canCreateGlobal()){
			$cid = Factory::getApplication()->input->get("cid", array(), 'ARRAY');
			$cid = ArrayHelper::toInteger($cid);

			if (count( $cid ) < 1) {
				throw new Exception(500, Text::_( 'Select an item to publish' ) );
			}

			$this->_authoriseAccess($cid);

			$model =  $this->getModel("template", "RsvpprotemplatesModel");
			if(!$model->templatise($cid, 1)) {
				echo "<script> alert('".$model->getError(true)."'); window.history.go(-1); </script>\n";
			}

			$returntask	= Factory::getApplication()->input->getCmd( 'returntask', "templates.overview");
			if ($returntask!="templates.list" && $returntask!="templates.overview" && $returntask!="templates.select"){
				$returntask="templates.overview";
			}
			if (method_exists($this,str_replace("templates.","",$returntask))){
				$returntask = str_replace("templates.","",$returntask);
				return $this->$returntask();
			}

			$tmpl = "";
			if ( Factory::getApplication()->input->getString("tmpl","")=="component"){
				$tmpl ="&tmpl=component";
			}
		}
		$this->setRedirect(Route::_( 'index.php?option=com_rsvppro&task=templates.list' . $tmpl, false));
	}


	function unmaketemplate()
	{
		// Check for request forgeries
		Session::checkToken() or jexit( 'Invalid Token' );

		if (JevTemplateHelper::canCreateGlobal()){
			$cid = Factory::getApplication()->input->get("cid", array(), 'ARRAY');
			$cid = ArrayHelper::toInteger($cid);

			if (count( $cid ) < 1) {
				throw new Exception(500, Text::_( 'Select an item to publish' ) );
			}

			$this->_authoriseAccess($cid);

			$model =  $this->getModel("template", "RsvpprotemplatesModel");
			if(!$model->templatise($cid, 0)) {
				echo "<script> alert('".$model->getError(true)."'); window.history.go(-1); </script>\n";
			}

			$returntask	= Factory::getApplication()->input->getCmd( 'returntask', "templates.overview");
			if ($returntask!="templates.list" && $returntask!="templates.overview" && $returntask!="templates.select"){
				$returntask="templates.overview";
			}
			if (method_exists($this,str_replace("templates.","",$returntask))){
				$returntask = str_replace("templates.","",$returntask);
				return $this->$returntask();
			}

			$tmpl = "";
			if ( Factory::getApplication()->input->getString("tmpl","")=="component"){
				$tmpl ="&tmpl=component";
			}
		}
		$this->setRedirect(Route::_( 'index.php?option=com_rsvppro&task=templates.list' . $tmpl, false));
	}

	function delete ()
	{
		// get the view
		$this->view = $this->getView("templates","html");

		// Set the layout
		$this->view->setLayout('edit');

		// Check if on tmpl or not.
		$tmpl = "";
		if ( Factory::getApplication()->input->getString("tmpl","")=="component"){
			$tmpl ="&tmpl=component";
		}

		// Get/Create the model
		if ($model =  $this->getModel("template", "RsvpprotemplatesModel")) {
			$cid = Factory::getApplication()->input->get("cid", array(0), 'ARRAY');;
			$cid = ArrayHelper::toInteger($cid);
			$model->delete($cid);

			$this->setRedirect( Route::_('index.php?option=com_rsvppro&task=templates.overview'.$tmpl, false) );

		}

	}
	function _authoriseAccess($locid=0)
	{
		// TODO
		return 1;
	}

	function close(){
		ob_end_clean();
		?>
		<script type="text/javascript">
            try {	window.parent.closeJevModalBySelector('#customisetemplate');}	catch (e){};
			try {
				window.parent.jQuery('.jevmodal').modal('hide');
				//window.parent.jQuery('#translationPopup').modal('hide');
			}
			catch (e){}
			try {
				window.parent.SqueezeBox.close();
			}
			catch (e){}
			try {
				window.parent.closedialog();
			}
			catch (e){}
		</script>
		<?php
		exit();
	}

	// export templates
	function export()
	{
		// Check for request forgeries
		Session::checkToken() or jexit( 'Invalid Token' );

		if (JevTemplateHelper::canCreateOwn())
		{
			$cid = Factory::getApplication()->input->get("cid", array(), 'ARRAY');
			$cid = ArrayHelper::toInteger($cid);

			if (count($cid) < 1)
			{
				throw new Exception(500, Text::_('Select an item to export'));
			}

			$this->_authoriseAccess($cid);

			$model = $this->getModel("templates", "RsvpprotemplatesModel");
			$items = $model->getData();

			$selectedItems = array();
			foreach ($cid as $id)
			{
				foreach($items as $item)
				{
					if ($id == $item->id)
					{
						$selectedItems[]= $item;
					}
				}
			}
			// get the view
			$this->view = $this->getView("templates", "html");

			// Set the layout
			$this->view->setLayout('export');

			$this->view->items = $selectedItems;

			$this->view->displayNoWrapper();

			return;
			//$this->setRedirect(Route::_('index.php?option=com_rsvppro&task=templates.list', false));
		}
	}

	// import templates
	function import()
	{
		// Check for request forgeries
		Session::checkToken() or jexit( 'Invalid Token' );

		if (JevTemplateHelper::canCreateOwn())
		{
			// get the view
			$this->view = $this->getView("templates", "html");

			$files   = Factory::getApplication()->input->files;
			$jsonFile = $files->get("jsonfile", false, "raw");

			if (!$jsonFile || $jsonFile["error"] ||  $jsonFile["type"] !== "application/json")
			{
				if ($jsonFile && $jsonFile["error"])
				{
					Factory::getApplication()->enqueueMessage(Text::_('COM_RSVPPRO_PROBLEM_UPLOADING_JSON_FILE'), 'error');
				}
				if ($jsonFile && $jsonFile["type"] !== "application/json")
				{
					Factory::getApplication()->enqueueMessage(Text::_('COM_RSVPPRO_JSON_FILE_INVALID_MIME_TYPE'), 'error');
				}

				// Set the layout
				$this->view->setLayout('import_select');

				$this->view->display();

			}
			else
			{
				$model = $this->getModel("templates", "RsvpprotemplatesModel");
				$success = $model->import($jsonFile);
				if ($success)
				{
					Factory::getApplication()->enqueueMessage(Text::_('COM_RSVPPRO_TEMPLATES_IMPORTED'));
				}
				else
				{
					Factory::getApplication()->enqueueMessage(Text::_('COM_RSVPPRO_TEMPLATES_NOT_IMPORTED'), 'error');
				}

				$this->setRedirect(Route::_('index.php?option=com_rsvppro&task=templates.list', false));
			}
		}
	}

}
