<?php

/**
 * JEvents Component for Joomla 1.5.x
 *
 * @version     $Id$
 * @package     JEvents
 * @copyright   Copyright (C) 2008-2021 GWE Systems Ltd
 * @license     GNU/GPLv2, see http://www.gnu.org/licenses/gpl-2.0.html
 * @link        https://www.jevents.net
 */

// Check to ensure this file is within the rest of the framework

defined('JPATH_BASE') or die();

use Joomla\CMS\Language\Text;
use Joomla\CMS\Factory;
use Joomla\CMS\HTML\HTMLHelper;


include_once(JPATH_ADMINISTRATOR . "/components/com_rsvppro/fields/JevrField.php");


class JFormFieldJevrjsnfield extends JevrField

{


	/**
	 * Element name
	 *
	 * @access    protected
	 * @var        string
	 */

	var $_name = 'Jevrjsnfield';

	const name = 'jevrjsnfield';


	static function isEnabled()

	{

		return is_dir(JPATH_SITE . '/components/com_jsn') && is_dir(JPATH_ADMINISTRATOR . '/components/com_jsn');


	}


	public static function loadScript($field = false)

	{

		HTMLHelper::script('administrator/components/' . RSVP_COM_COMPONENT . '/customfields/js/jevrjsnfield.js');


		if ($field)

		{

			$id = 'field' . $field->field_id;

		}

		else

		{

			$id = '###';

		}

		ob_start();

		?>

		<div class='rsvpfieldinput'>

			<div class="gsl-form-label">Easy Profile Field</div>

			<div class="gsl-form-controls gsl-form-small" style="font-weight:bold;">jsnField<?php RsvpHelper::fieldId($id); ?></div>

			<input type="hidden" name="dv[<?php echo $id; ?>]" value=""/>

			<select name="params[<?php echo $id; ?>][fieldname]" id="fieldname<?php echo $id; ?>"
			        onchange="jevrjsnfield.setvalue('<?php echo $id; ?>');">

				<?php

				// get the jsn language file

				$lang = Factory::getLanguage();

				$lang->load("com_jsn", JPATH_SITE);


				$db = Factory::getDBO();

				$db->setQuery("SELECT title, alias FROM #__jsn_fields as jsn WHERE (( alias != 'NA' ) AND ( published = 1 )) ");

				$rows = $db->loadObjectList();


				$fieldname = "";

				$activeField = "";

				if ($field)

				{

					try
					{

						$params = json_decode($field->params);

						$fieldname = isset($params->fieldname) ? $params->fieldname : "";

					}

					catch (Exception $e)
					{


					}

				}


				if (!$db->getErrorNum())

				{

					//foreach ($rows as $row) {

					for ($i = 0; $i < count($rows); $i++)

					{

						$row = $rows[$i];

						if (defined($row->alias))

						{

							$jsnFieldName = constant($row->alias);

						}

						else

						{

							$jsnFieldName = $row->alias;

						}


						$selected = "";

						if ($field && $fieldname == $row->alias)

						{

							$selected = "selected='selected'";

							$activeField = $jsnFieldName;

						}

						?>


						<option value="<?php echo $row->alias; ?>" <?php echo $selected; ?> ><?php echo $jsnFieldName ?></option>

						<?php

					}

				}

				?>

			</select>


			<div class="rsvpclear"></div>


			<?php

			RsvpHelper::hidden($id, $field, self::name);

			RsvpHelper::label($id, $field, self::name);

			RsvpHelper::conditional($id, $field);

			RsvpHelper::peruser($id, $field);

			RsvpHelper::formonly($id, $field);

			RsvpHelper::showinform($id, $field);

			RsvpHelper::showindetail($id, $field);

			RsvpHelper::showinlist($id, $field);
			RsvpHelper::showinticket($id, $field);

			RsvpHelper::includeinexport($id,  $field);		

			RsvpHelper::allowoverride($id, $field);

			RsvpHelper::accessOptions($id, $field);

			RsvpHelper::applicableCategories("facc[$id]", "facs[$id]", $id, $field ? $field->applicablecategories : "all");

			?>

			<div class="rsvpclear"></div>

		</div>

		<div class='rsvpfieldpreview' id='<?php echo $id; ?>preview'>

			<div class="previewlabel"><?php echo Text::_("RSVP_PREVIEW"); ?></div>

			<div class="gsl-form-label rsvppl"
			     id='pl<?php echo $id; ?>'><?php echo $field ? $field->label : Text::_("RSVP_FIELD_LABEL"); ?></div>

			<div id="pdv<?php echo $id; ?>">

				<?php

				echo $activeField;

				?>

			</div>

		</div>

		<div class="rsvpclear"></div>

		<?php

		$html = ob_get_clean();


		return RsvpHelper::setField($id, $field, $html, self::name);


	}


	function getInput()

	{

		if (!$this->isEnabled())
		{

			return;

		}

		require_once(JPATH_SITE . '/components/com_jsn/helpers/helper.php');


		$node = $this->element;

		$name = $this->name;

		$id = $this->id;

		$value = $this->value;

		$fieldname = $this->fieldname;

		//$user = JsnHelper::getUser();


		if (Factory::getApplication()->input->getCmd("task") == "attendees.edit" && isset($this->attendee) && $this->attendee->user_id > 0 && $this->attendee->user_id != $user->getField("id"))
		{

			$value = $this->attendee->user_id;

		}

		else if (Factory::getApplication()->input->getCmd("task") == "attendees.edit" && isset($this->attendee) && $this->attendee->id > 0 && $this->attendee->user_id == 0)
		{

			$value = 0;

		}


		$showinform = $this->attribute("showinform");


		$fieldname = $this->attribute("fieldname");

		$html = "";


		if (!$this->isEnabled())
		{

			$showinform = false;

		}


		if ($showinform)
		{


			if (is_null($value) || $value == "" || $value == "0" || $value == 0)

			{

				$user = JsnHelper::getUser();

				if (isset($this->attendee) && isset($this->attendee->user_id) && $this->attendee->user_id > 0)

				{

					$value = $this->attendee->user_id;

				}

				else if ($user->id == 0)

					return $value;

				else
				{

					if (Factory::getApplication()->input->getCmd("task") == "attendees.edit" && isset($this->attendee) && $this->attendee->user_id == 0)
					{

						return $value;

					}

					else
					{

						$value = $user->id;

					}

				}

			}


			$user = JsnHelper::getUser($value);


			if (!isset($user->jsnProfile))

			{

				$db = Factory::getDBO();

				$user->jsnProfile = new stdClass();

				$db->setQuery('SELECT jsnprofile.*, user.name, user.username, user.email, user.lastvisitDate, user.registerDate ' .

					'FROM #__jsn_users AS jsnprofile ' .

					'LEFT JOIN #__users AS user ON ( user.id = jsnprofile.id ) ' .

					' WHERE ( jsnprofile.id = \'' . $value . '\' ) ');

				$user->jsnProfile = $db->loadObject();

			}


			static $fieldclasses;

			if (!isset($fieldclasses))

			{

				$db = Factory::getDBO();

				$db->setQuery('SELECT * FROM #__jsn_fields AS fld WHERE fld.level = 2');

				$fieldclasses = $db->loadObjectList('alias');

			}


			if ($user->jsnProfile)

			{

				if (isset($user->jsnProfile->$fieldname))

				{

					if (array_key_exists($fieldname, $fieldclasses))

					{

						$html = '<span>' . $user->getField($fieldname) . '</span>';

						if (trim($html) == "")

						{

							$html = '<span>' . $user->jsnProfile->$fieldname . '</span>';

						}

					}

					else

					{

						$html = '<span>' . $user->jsnProfile->$fieldname . '</span>';

					}

				}

				else

				{

					$html = '';

				}

			}

		}


		if ($user->id == 0 || $html == '')

		{
			$value = '';

		}

		else

		{
			$value = $user->id;

		}

		$html .= '<input type="hidden" name="' . $name . '" id="' . $id . '" value="' . $value . '" />';


		return $html;


	}

	function currentAttendeeCount($node, $value)
	{
		if (is_array($value) && count($value) > 1)
		{
			return count($value) - 1;
		}

		return 1;
	}

	function render_NOT_USED(&$xmlElement, $value, $control_name = 'params')

	{

		$showinform = $xmlElement->attributes("showinform");


		$name = $xmlElement->attributes()->name;

		$label = $xmlElement->attributes('label');

		$descr = $xmlElement->attributes('description');

		if (!$showinform)

			$label = "";


		$result[0] = $label . " ";

		$result[1] = $this->fetchElement($name, $value, $xmlElement, $control_name);

		$result[2] = $descr;

		$result[3] = $label;

		$result[4] = $value;

		$result[5] = $name;


		return $result;


	}


	function toXML($field)

	{

		$result = array();

		$result[] = "<field ";

		if (is_string($field->params) && strpos($field->params, "{") === 0)

		{

			$field->params = json_decode($field->params);

		}

		foreach (get_object_vars($field) as $k => $v)

		{

			if ($k == "options" || $k == "html" || $k == "defaultvalue" || $k == "name")

				continue;

			if ($k == "field_id")

			{

				$k = "name";

				$v = "field" . $v;

			}

			if ($k == "params")

			{

				if (is_object($field->params))

				{

					foreach (get_object_vars($field->params) as $label => $value)

					{

						$result[] = $label . '="' . addslashes(htmlspecialchars($value)) . '" ';

					}

				}

				continue;

			}

			$result[] = $k . '="' . addslashes(htmlspecialchars($v)) . '" ';

		}

		$result[] = " />";

		$xml = implode(" ", $result);

		return $xml;


	}


	public function convertValue($value)

	{


		if (!$this->isEnabled())
		{

			return "";

		}


		require_once(JPATH_SITE . '/components/com_jsn/helpers/helper.php');


		$fieldname = $this->attribute("fieldname");


		$attendeeUserId = 0;

		if (isset($this->attendee) && isset($this->attendee->user_id) && $this->attendee->user_id > 0)

		{

			$attendeeUserId = $this->attendee->user_id;

		}


		$html = "";

		if (is_null($value) || $value == "" || $value == "0" || (is_numeric($value) && intval($value) == 0))

		{

			$user = JsnHelper::getUser();

			if (isset($this->attendee) && isset($this->attendee->user_id) && $this->attendee->user_id > 0)

			{

				$attendeeUserId = $value = $this->attendee->user_id;

			}

			else if ($user->id == 0)

				return $value;

			else
			{

				if (Factory::getApplication()->input->getCmd("task") == "attendees.edit" && isset($this->attendee) && $this->attendee->user_id == 0)
				{

					return $value;

				}

				else
				{

					$value = $user->id;

				}

			}

		}


		if (is_numeric($value) && $value == $attendeeUserId)
		{

			$user = JsnHelper::getUser($value);

			if (!isset($user->jsnProfile) || !$user->jsnProfile)

			{

				$db = Factory::getDBO();

				$user->jsnProfile = new stdClass();

				$db->setQuery('SELECT jsnprofile.*, user.name, user.username, user.email, user.lastvisitDate, user.registerDate ' .

					'FROM #__jsn_users AS jsnprofile ' .

					'LEFT JOIN #__users AS user ON ( user.id = jsnprofile.id ) ' .

					' WHERE ( jsnprofile.id = \'' . $value . '\' ) ');

				$user->jsnProfile = $db->loadObject();

			}


			static $fieldclasses;

			if (!isset($fieldclasses))

			{

				$db = Factory::getDBO();

				$db->setQuery('SELECT * FROM #__jsn_fields AS fld WHERE fld.level = 2');

				$fieldclasses = $db->loadObjectList('alias');

			}


			if ($user->jsnProfile)

			{

				if (isset($user->jsnProfile->$fieldname))

				{

					if (array_key_exists($fieldname, $fieldclasses))

					{

						$html = '<span>' . $user->getField($fieldname) . '</span>';

						if (trim($html) == "")

						{

							$html = '<span>' . $user->jsnProfile->$fieldname . '</span>';

						}

					}

					else

					{

						$html = '<span>' . $user->jsnProfile->$fieldname . '</span>';

					}

				}

				else

				{

					$html = '<span>' . $user->getField($fieldname) . '</span>';

					if (trim($html) == "" && isset($user->$fieldname))

					{

						$html = '<span>' . $user->$fieldname . '</span>';

					}

				}

			}

		}

		else
		{

			$html = $value;

		}

		if (Factory::getApplication()->input->getCmd("task") == "attendees.export")
		{

			// there is a HTML error in some CB fields so missmatched tags and cannot use strip_tags

			//$value =  strip_tags($html);

			$search = array('@<*?[^<>]*?>@si');

			$value = preg_replace($search, '', $html);

			return $value;

		}


		if (!isset($this->attendee->guestcount))
		{

			$this->attendee->guestcount = 1;

		}


		if ($this->attribute("peruser") == 1 && $this->attendee->guestcount > 0)
		{

			return array_fill(0, $this->attendee->guestcount, $html);

		}

		else if ($this->attribute("peruser") == 2 && $this->attendee->guestcount > 0)
		{

			$return = array_fill(0, $this->attendee->guestcount, $html);

			//$return[0]="";

			return $return;

		}

		return $html;


	}


}


class_alias("JFormFieldJevrjsnfield", "FormFieldJevrjsnfield");