<?php
/**
 * JEvents Component for Joomla 1.5.x
 *
 * @version     $Id: jevboolean.php 1569 2009-09-16 06:22:03Z geraint $
 * @package     JEvents
 * @copyright   Copyright (C) 2008-2021 GWE Systems Ltd
 * @license     GNU/GPLv2, see http://www.gnu.org/licenses/gpl-2.0.html
 * @link        https://www.jevents.net
 */
// Check to ensure this file is within the rest of the framework
defined('JPATH_BASE') or die();

use Joomla\CMS\Language\Text;
use Joomla\CMS\Factory;
use Joomla\CMS\HTML\HTMLHelper;

include_once(JPATH_ADMINISTRATOR.'/'."components/com_rsvppro/fields/JevrFieldRadio.php");
// need this for this list  interface
include_once("jevrlist.php");

class JFormFieldJevrboolean extends JevrFieldRadio
{

	/**
	 * Element name
	 *
	 * @access	protected
	 * @var		string
	 */
	var $_name = 'Jevrboolean';
	const name = 'jevrboolean';

	public static function loadScript($field=false)
	{
		HTMLHelper::script('administrator/components/' . RSVP_COM_COMPONENT . '/fields/js/jevrboolean.js');

		if ($field)
		{
			$id = 'field' . $field->field_id;
		}
		else
		{
			$id = '###';
		}
		ob_start();
		?>
		<div class='rsvpfieldinput rsvpbooleanfield'>

			<div class="gsl-form-label"><?php echo Text::_("RSVP_FIELD_TYPE"); ?></div>
			<div class="gsl-form-controls gsl-form-small" style="font-weight:bold;"><?php echo Text::_("RSVP_TEMPLATE_TYPE_JEVRBOOLEAN"); ?><?php RsvpHelper::fieldId($id); ?></div>
			<div class="rsvpclear"></div>

			<?php
			RsvpHelper::hidden($id, $field, self::name);
			RsvpHelper::label($id, $field, self::name);
			RsvpHelper::tooltip($id, $field);
			?>

			<div class="gsl-form-label"><?php echo Text::_("RSVP_DEFAULT_VALUE"); ?></div>
			<div class="gsl-form-controls btn-group">
				<label for="dv1<?php echo $id; ?>" class="btn"><?php echo Text::_("RSVP_YES"); ?>
				<input type="radio" name="dv[<?php echo $id; ?>]"  id="dv1<?php echo $id; ?>" value="1" <?php if ($field && $field->defaultvalue == 1)
				echo 'checked="checked"'; if (!$field)
				echo 'checked="checked"'; ?> onclick="jevrboolean.settrue('<?php echo $id; ?>');" />
				</label>
				<label for="dv0<?php echo $id; ?>"  class="btn"><?php echo Text::_("RSVP_NO"); ?>
				<input type="radio" name="dv[<?php echo $id; ?>]" id="dv0<?php echo $id; ?>" value="0" <?php if ($field && $field->defaultvalue == 0)
			echo 'checked="checked"'; ?> onclick="jevrboolean.setfalse('<?php echo $id; ?>');"/>
				</label>
			</div>
			<div class="rsvpclear"></div>

			<?php
			RsvpHelper::required($id, $field);
			RsvpHelper::requiredMessage($id, $field);
			RsvpHelper::peruser($id, $field);
			RsvpHelper::conditional($id, $field);
            RsvpHelper::className($id, $field);
            RsvpHelper::formonly($id, $field);
			RsvpHelper::showinform($id, $field);
			RsvpHelper::showindetail($id, $field);
			RsvpHelper::showinlist($id, $field);
			RsvpHelper::showinticket($id, $field);
			RsvpHelper::includeinexport($id,  $field);
			RsvpHelper::allowoverride($id, $field);
			RsvpHelper::accessOptions($id, $field);
			RsvpHelper::applicableCategories("facc[$id]", "facs[$id]", $id, $field ? $field->applicablecategories : "all");
			?>

			<div class="rsvpclear"></div>

		</div>
		<div class='rsvpfieldpreview' id='<?php echo $id; ?>preview'>
			<div class="previewlabel"><?php echo Text::_("RSVP_PREVIEW"); ?></div>
			<div class="gsl-form-label rsvppl" id='pl<?php echo $id; ?>' ><?php echo $field ? $field->label : Text::_("RSVP_FIELD_LABEL"); ?></div>
			<div class="btn-group">
			<label for="pdv1<?php echo $id; ?>"  class="btn"> <?php echo Text::_("RSVP_YES"); ?>
			<input type="radio" name="pdv[<?php echo $id; ?>]"  id="pdv1<?php echo $id; ?>" value="1"  <?php if ($field && $field->defaultvalue == 1)
			echo 'checked="checked"'; if (!$field)
			echo 'checked="checked"'; ?> />
			</label>
			<label for="pdv0<?php echo $id; ?>" class="btn"><?php echo Text::_("RSVP_NO"); ?>
			<input type="radio" name="pdv[<?php echo $id; ?>]" id="pdv0<?php echo $id; ?>" value="0" <?php if ($field && $field->defaultvalue == 0)
			echo 'checked="checked"'; ?> />
			</label>
			</div>
		</div>
		<div class="rsvpclear"></div>
		<?php
		$html = ob_get_clean();

		return RsvpHelper::setField($id, $field, $html, self::name);

	}

	public function getOptions()
	{
		// Must load admin language files
		$lang = Factory::getLanguage();
		$lang->load("com_jevents", JPATH_ADMINISTRATOR);

		$options = array ();
		$options[] = JHTML::_('select.option', 0, Text::_("Jev_No"));
		$options[] = JHTML::_('select.option', 1, Text::_("jev_Yes"));

		return $options;


	}

	function getInput()
	{
		$node =  $this->element;
		$name = $this->name;
		$id = $this->id;
		$value = $this->value;
		$fieldname = $this->fieldname;

		// This was causing problems with form in Protostar template (and probably others too)
		if (strpos($this->element['class'],"gsl-button-group")===false){
			//$this->element['class'] .= " gsl-button-group";
		}

		$class = ( $this->attribute('class') ? 'class="' . $this->attribute('class') . ' xxx"' : 'class=" xxx"' );

		// Must load admin language files
		$lang = Factory::getLanguage();
		$lang->load("com_jevents", JPATH_ADMINISTRATOR);

		$options = array();
		$options[] = HTMLHelper::_('select.option', 1, Text::_("JEV_YES"));
		$options[] = HTMLHelper::_('select.option', 0, Text::_("JEV_NO"));

		$html = "";
		$peruser = $this->attribute("peruser");
		if ($this->attribute("peruser") == 1 || $this->attribute("peruser") == 2)
		{
			if (is_object($value))
			{
				$value = get_object_vars($value);
			}
			if (!is_array($value))
			{
				$value = array($value);
			}
			if (count($value) < $this->currentAttendees)
			{
				// flesh out the value if there are not the right number of items
				for ($i = 0; $i <= $this->currentAttendees - count($value); $i++)
				{
					$value[] = $this->attribute("default");
				}
			}
			$html = "";
			$i = 0;
			foreach ($value as $val)
			{
				$elementname =  $name . '[' . $i . ']';
				if ($i == 0)
				{
					if ($this->attribute("peruser") == 2)
					{
						$thisclass = str_replace(" xxx", " disabledfirstparam rsvpparam rsvpparam0 rsvp_" . $fieldname, $class);
					}
					else
					{
						$thisclass = str_replace(" xxx", " rsvpparam rsvpparam0 rsvp_" . $fieldname, $class);
					}
				}
				else
				{
					$thisclass = str_replace(" xxx", " rsvpparam rsvpparam$i  rsvp_" . $fieldname, $class);
				}
				// XSS protection
				if (!in_array((int) $val, array(0,1)))
				{
					$val = 0;
				}
				$attribs = $thisclass;
				$inputattribs = " onclick='JevrConditionFieldState.changeState(jQuery(this).parent(), \"" . $this->fieldname . "\");' ";
				$attribs .= " id = 'rsvp_" . $fieldname . "_span_$i' ";
				$html .= HTMLHelper::_('jevrList.radiolist', $options, '' . $elementname, $attribs, 'value', 'text', $val,  $id. "_" . $i, $inputattribs);
				$script = "JevrConditionFieldState.append( { '" . $this->fieldname . "_" . $i . "' : {'name':'" . $this->fieldname . "', 'value':'" . $val . "', 'guestcount':'" . $i . "',  'peruser' :" . $peruser . "}}); ";
				$doc = Factory::getDocument();
				$doc->addScriptDeclaration($script);
				$i++;
			}
			$val = $this->attribute("default");
			$elementname = $name . '[\'xxxyyyzzz\']';
			$thisclass = str_replace(" xxx", " paramtmpl rsvp_" . $fieldname, $class);
			$attribs = $thisclass;
			$inputattribs = $thisclass . " onclick='JevrConditionFieldState.changeState(jQuery(this).parent(),\"" . $this->fieldname . "\");' ";
			$attribs .= " id = 'rsvp_" .$fieldname . "_span_xxxyyyzzz' ";
			$html .= HTMLHelper::_('jevrList.radiolist', $options, 'paramtmpl_' . $elementname, $attribs, 'value', 'text', $val,  $id. "_xxx", $inputattribs);
			$script = "JevrConditionFieldState.append( { '" . $this->fieldname . "_xxxyyyzzz' : {'name':'" . $this->fieldname. "', 'value':'" . $val . "', 'guestcount':'xxxyyyzzz',  'peruser' :" . $peruser . "}}); ";
			$doc = Factory::getDocument();
			$doc->addScriptDeclaration($script);
		}
		else
		{
			$elementname =  $name ;
			$thisclass = str_replace(" xxx", " rsvpparam rsvpparam0 rsvp_" . $fieldname, $class);
			$val = $value;
			if (!$this->attendee)
			{
				$val = $this->attribute("default");
			}
			// XSS protection
			if (!in_array((int) $val, array(0,1)))
			{
				$val = 0;
			}

			$attribs = $thisclass;
			$inputattribs = " onclick='JevrConditionFieldState.changeState(jQuery(this).parent(),\"" . $this->fieldname . "\");' ";
			// per user is 0 so don't put a guest number on the element!
			$attribs .= " id = 'rsvp_" . $fieldname . "_span_' ";
			$html .= HTMLHelper::_('jevrList.radiolist', $options, '' . $elementname, $attribs, 'value', 'text', $val, $id."_", $inputattribs);
			$script = "JevrConditionFieldState.append( { '" . $this->fieldname . "' : {'name':'" . $this->fieldname . "', 'value':'" . $val . "',  'peruser' :" . $peruser . "}}); ";
			$doc = Factory::getDocument();
			$doc->addScriptDeclaration($script);
		}

		return $html;

	}

	function fetchRequiredScript()
	{
		$elementid = $this->id;
		$elementname = $this->name;

		return "jevrsvpRequiredFields.fields.push({'name':'" . $elementname. "', 'type':'radio', 'id':'" . $elementid . "', 'default' :'" . $this->attribute("default") . "' ,'reqmsg':'" . trim(Text::_($this->attribute('requiredmessage'), true)) . "'}); ";

	}

	public function RADIOfetchRequiredScript($controlname,  $name)
	{

		$script = "";
		if ($this->attribute("peruser") == 1 || $this->attribute("peruser") == 2)
		{
			$this->fixValue($value, $node);

			$i = 0;
			foreach ($value as $val)
			{
				if ($val == "#%^£xx£^%#" || $i > 2)
					continue;
				$elementid = $name . $controlname . '_' . $i;
				$elementname = $name . "[" . $controlname . "][xxxyyyzzz]";
				$script .= "jevrsvpRequiredFields.fields.push({'name':'" . $elementname . "', 'type':'radio', 'id':'" . $elementid . "',  'default' :'" . $this->attribute("default") . "' ,'reqmsg':'" . trim(Text::_($this->attribute('requiredmessage'), true)) . "'}); ";
				break;
				$i++;
			}
		}
		else
		{
			$elementid = $name . $controlname;
			$elementname = $name . "[" . $controlname . "]";
			$script = "jevrsvpRequiredFields.fields.push({'name':'" . $elementname . "','type':'radio', 'id':'" . $elementid . "',  'default' :'" . $this->attribute("default") . "' ,'reqmsg':'" . trim(Text::_($this->attribute('requiredmessage'), true)) . "'}); ";
		}
		return $script;

	}

	public function convertValue($value)
	{
		static $values;
		if (!isset($values))
		{
			$values = array();
			$values[0] = Text::_("JEV_NO");
			$values[1] = Text::_("JEV_YES");
		}
		return $values[$value];

	}

}
class_alias("JFormFieldJevrboolean", "FormFieldJevrboolean");
