<?php
/**
 * JEvents Component for Joomla 1.5.x
 *
 * @version     $Id$
 * @package     JEvents
 * @copyright   Copyright (C) 2008-2021 GWE Systems Ltd
 * @license     GNU/GPLv2, see http://www.gnu.org/licenses/gpl-2.0.html
 * @link        https://www.jevents.net
 */
// Check to ensure this file is within the rest of the framework
defined('JPATH_BASE') or die();

use Joomla\CMS\Language\Text;
use Joomla\CMS\Component\ComponentHelper;
use Joomla\CMS\Plugin\PluginHelper;
use Joomla\CMS\Factory;

include_once(JPATH_ADMINISTRATOR ."/components/com_rsvppro/fields/JevrField.php");

class JFormFieldJevrtransactions extends JevrField
{

	/**
	 * Element name
	 *
	 * @access	protected
	 * @var		string
	 */
	var $_name = 'Jevrtransactions';
	const name = 'jevrtransactions';

	public static function loadScript($field=false)
	{

		if ($field)
		{
			$id = 'field' . $field->field_id;
		}
		else
		{
			$id = '###';
		}
		ob_start();
?>
		<div class='rsvpfieldinput'>

			<div class="gsl-form-label"><?php echo Text::_("RSVP_FIELD_TYPE"); ?></div>
		<div class="gsl-form-controls gsl-form-small" style="font-weight:bold;"><?php echo Text::_("RSVP_TEMPLATE_TYPE_JEVRTRANSACTIONS"); ?><?php RsvpHelper::fieldId($id);?></div>
		<div class="rsvpclear"></div>

	<?php
		RsvpHelper::hidden($id, $field, self::name);
		RsvpHelper::label($id, $field);
		RsvpHelper::tooltip($id, $field);
                RsvpHelper::className($id, $field);

	?>
		<input type="hidden" name="peruser[<?php echo $id; ?>]" value="-1" />
	<?php
		RsvpHelper::formonly($id, $field);
		RsvpHelper::showinform($id, $field);
		RsvpHelper::showindetail($id, $field);
		// by default don't show transaction summary in list view
		RsvpHelper::showinlist($id, $field,0);
		RsvpHelper::showinticket($id, $field);
		RsvpHelper::includeinexport($id, $field);
		RsvpHelper::allowoverride($id, $field);
		RsvpHelper::accessOptions($id, $field);
		RsvpHelper::applicableCategories("facc[$id]", "facs[$id]", $id, $field ? $field->applicablecategories : "all");
	?>

		<div class="rsvpclear"></div>

	</div>
	<div class='rsvpfieldpreview'  id='<?php echo $id; ?>preview'>
		<div class="previewlabel"><?php echo Text::_("RSVP_PREVIEW"); ?></div>
		<div class="gsl-form-label rsvppl" id='pl<?php echo $id; ?>'><?php echo $field ? $field->label : Text::_("RSVP_FIELD_LABEL"); ?></div>
		<div id="pdv<?php echo $id; ?>">
			</div>
		</div>
		<div class="rsvpclear"></div>
<?php
		$html = ob_get_clean();

		return RsvpHelper::setField($id, $field, $html, self::name);

	}

	public static function paidOption()
	{
		return 1;

	}

	function getInput()
	{
		$node =  $this->element;
		$name = $this->name;
		$id = $this->id;
		$value = $this->value;
		$fieldname = $this->fieldname;

		if (!$this->attribute("showinform"))
		{
			return "";
		}
		// NB the value depends on the Attendee
		if (isset($this->attendee) && isset($this->attendee->outstandingBalances))
		{
			PluginHelper::importPlugin("rsvppro");

			Factory::getApplication()->triggerEvent('activeGatewayClass', array(&$activePlugin, "paymentpage"));

			$html = array();

			foreach ($this->attendee->outstandingBalances["transactions"] as $transaction)
			{
				$link = "";
				$class = "plgRsvppro" . ucfirst($transaction->gateway);
				$pluginpath = version_compare(JVERSION, "1.6.0", 'ge')?JPATH_SITE."/plugins/rsvppro/$transaction->gateway/" : JPATH_SITE."/plugins/rsvppro/";
				JLoader::register($class, $pluginpath . $transaction->gateway . ".php");
				$link = call_user_func_array(array($class, "transactionDetailLink"), array($transaction, $this->rsvpdata, $this->attendee, $this->event));

				$this->params = ComponentHelper::getParams("com_rsvppro");
				$format = $this->params->get("timestampformat", "%Y-%m-%d %H:%M");
				$transactiondate = new JevDate($transaction->transaction_date);
				$html[] = RsvpHelper::phpMoneyFormat($transaction->amount) . "  (" . $transactiondate->toFormat($format) . ") " . $link;
			}

			return implode("<br/>", $html);
		}
		return;

	}

	// add dummy values so that convertValue in renderToBasicArray isn't skipped
	public function fixRawValues(&$values) {
		if (!is_array($values))
		{
			$values = 1;
		}
		else {
			foreach ($values as $k => $v){
				$values[$k] = 1;
			}
		}
	}

	public function convertValue($value)
	{
		// NB the value depends on the Attendee
		if (isset($this->attendee) && isset($this->attendee->outstandingBalances))
		{
			$html = array();

			foreach ($this->attendee->outstandingBalances["transactions"] as $transaction)
			{
				$link = "";
				$class = "plgRsvppro" . ucfirst($transaction->gateway);
				$pluginpath = version_compare(JVERSION, "1.6.0", 'ge')?JPATH_SITE."/plugins/rsvppro/$transaction->gateway/" : JPATH_SITE."/plugins/rsvppro/";
				JLoader::register($class, $pluginpath . $transaction->gateway . ".php");
				$link = call_user_func_array(array($class, "transactionDetailLink"), array($transaction, $this->rsvpdata, $this->attendee, $this->event));

				$this->params = ComponentHelper::getParams("com_rsvppro");
				$format = $this->params->get("timestampformat", "%Y-%m-%d %H:%M");
				$transactiondate = new JevDate($transaction->transaction_date);
				$html[] = RsvpHelper::phpMoneyFormat($transaction->amount) . "  (" . $transactiondate->toFormat($format) . ") " . $link;
			}

			return implode("<br/>", $html);
		}
		return "";

	}

}
class_alias("JFormFieldJevrtransactions", "FormFieldJevrtransactions");
