<?php
/**
 * JEvents Component for Joomla 1.5.x
 *
 * @version     $Id: jevbuttons.php 2749 2011-10-13 08:54:34Z geraintedwards $
 * @package     JEvents
 * @copyright   Copyright (C)  2008-2021 GWE Systems Ltd
 * @license     GNU/GPLv2, see http://www.gnu.org/licenses/gpl-2.0.html
 * @link        https://www.jevents.net
 */

namespace Joomla\CMS\Toolbar\Button;

// Check to ensure this file is within the rest of the framework
defined('JPATH_BASE') or die();

use Joomla\CMS\Language\Text;
use Joomla\CMS\Layout\LayoutInterface;
use Joomla\CMS\Toolbar\Toolbar;
use Joomla\CMS\Toolbar\ToolbarButton;
use Joomla\CMS\Toolbar\Button\StandardButton;
use Joomla\CMS\Layout\FileLayout;
use Joomla\String\StringHelper;

class Jeveditbutton extends ToolbarButton
{
	/**
	 * Button type
	 *
	 * @access	protected
	 * @var		string
	 */
	var $_name = 'JevEdit';

	function fetchButton( $type='JevEdit', $name = '', $text = '', $task = '', $list = true, $hideMenu = false )
	{
		$text	= Text::_($text);
		$class	= $this->fetchIconClass($name);
		$doTask	= $this->_getCommand( $name, $task, $list, $hideMenu);

		if ($name == 'apply' || $name == 'new')
		{
			$btnClass = 'gsl-button gsl-button-success button-' . $name;
			$class .= ' icon-white';
		}
		else
		{
			$btnClass = 'gsl-button gsl-button-primary button-' . $name;
		}


		$html = "<button href=\"#\" onclick=\"document.getElementById('adminForm').setAttribute('target','_blank');document.querySelector('input[name=option]').value = 'com_jevents';$doTask;document.querySelector('input[name=option]').value = 'com_rsvppro';document.getElementById('adminForm').setAttribute('target','_self');\" class=\"$btnClass\">\n";
		$html .= "<span class=\"$class\">\n";
		$html .= "</span>\n";
		$html .= "$text\n";
		$html .= "</button>\n";

		return $html;
	}

	/**
	 * Get the button CSS Id
	 *
	 * @access	public
	 * @return	string	Button CSS Id
	 * @since	1.5
	 */
	function fetchId( $type='Confirm',  $msg='', $name = '', $text = '', $task = '', $list = true, $hideMenu = false , $jstestvar = false)
	{
		return $this->_parent->getName().'-'.$name;
	}

	/**
	 * Get the JavaScript command for the button
	 *
	 * @param   string   $name  The task name as seen by the user
	 * @param   string   $task  The task used by the application
	 * @param   boolean  $list  True is requires a list confirmation.
	 *
	 * @return  string   JavaScript command string
	 *
	 * @since   3.0
	 */
	protected function _getCommand($name, $task, $list)
	{
		$message = Text::_('JLIB_HTML_PLEASE_MAKE_A_SELECTION_FROM_THE_LIST');
		$message = addslashes($message);

		if ($list)
		{
			$cmd = "if (document.adminForm.boxchecked.value==0){alert('$message');}else{ Joomla.submitbutton('$task')}";
		}
		else
		{
			$cmd = "Joomla.submitbutton('$task')";
		}

		return $cmd;
	}
}

class Jevstandardbutton extends StandardButton
{
	/**
	 * Button type
	 *
	 * @access	protected
	 * @var		string
	 */
	var $_name = 'Jevstandard';

	/**
	 * Fetch the HTML for the button
	 *
	 * @param   string   $type  Unused string.
	 * @param   string   $name  The name of the button icon class.
	 * @param   string   $text  Button text.
	 * @param   string   $task  Task associated with the button.
	 * @param   boolean  $list  True to allow lists
	 * @param string $formId The id of action form.
	 * @param   float  $ratio  Ratio for uikit icon scaling
	 *
	 * @return  string  HTML string for the button
	 *
	 * @since   3.0
	 */
	public function fetchButton($type = 'Standard', $name = '', $text = '', $task = '', $list = true, $formId = null, $ratio = 1)
	{
		// Store all data to the options array for use with LayoutInterface
		$options = array();
		$options['text']     = Text::_($text);
		$options['class']    = $this->fetchIconClass($name);
		$options['doTask']   = $this->_getCommand($options['text'], $task, $list);
		$options['btnClass'] = 'gsl-button gsl-button-primary button-' . $name;
		$options['ratio']    = $ratio;

		// Add name as class too
		$options['btnClass'] .= " " . str_replace(".", "", $task);

		/*
		if ($name === 'apply' || $name === 'new' || $name === 'save')
		{
			$options['btnClass'] .= ' gsl-button-success';
			$options['class'] .= ' icon-white';
		}
		*/

		// Instantiate a new FileLayout instance and render the layout
		$layout = new FileLayout('joomla.toolbar.standard');

		return $layout->render($options);
	}

	/**
	 * Get the JavaScript command for the button
	 *
	 * @param   string   $name  The task name as seen by the user
	 * @param   string   $task  The task used by the application
	 * @param   boolean  $list  True is requires a list confirmation.
	 *
	 * @return  string   JavaScript command string
	 *
	 * @since   3.0
	 */
	protected function _getCommand($name, $task, $list)
	{
		Text::script('JLIB_HTML_PLEASE_MAKE_A_SELECTION_FROM_THE_LIST');

		$cmd = "Joomla.submitbutton('" . $task . "');";

		if ($list)
		{
			$alert = "alert(Joomla.Text._('JLIB_HTML_PLEASE_MAKE_A_SELECTION_FROM_THE_LIST'));";
			$cmd   = "if (document.adminForm.boxchecked.value == 0) { " . $alert . " } else { " . $cmd . " }";
		}

		return $cmd;
	}
}

class JevconfirmButton extends ConfirmButton
{
	/**
	 * Button type
	 *
	 * @access    protected
	 * @var        string
	 */
	var $_name = 'Jevconfirm';

	/**
	 * Fetch the HTML for the button
	 *
	 * @param   string  $type     Unused string.
	 * @param   string  $msg      Message to render
	 * @param   string  $name     Name to be used as apart of the id
	 * @param   string  $text     Button text
	 * @param   string  $task     The task associated with the button
	 * @param   boolean $list     True to allow use of lists
	 * @param   boolean $hideMenu True to hide the menu on click
	 *
	 * @return  string   HTML string for the button
	 *
	 * @since   3.0
	 */
	public function fetchButton($type = 'Confirm', $msg = '', $name = '', $text = '', $task = '', $list = true, $hideMenu = false, $btnClass = false)
	{
		// Store all data to the options array for use with JLayout
		$options           = array();
		$options['text']   = \JText::_($text);
		$options['msg']    = \JText::_($msg, true);

		// We want to make sure carriage returns in the message are respected
		//$options['msg'] = str_replace("\\\\n" , "\n", $options['msg']);

		$options['class']  = $this->fetchIconClass($name);
		$options['doTask'] = $this->getButtonCommand($options['msg'], $name, $task, $list);

		$options['btnClass'] = 'btn btn-small button-' . $name;

		// Add name as class too
		$options['btnClass'] .= " " . str_replace(array(".", "(", ")") , "", $task);

		if ($btnClass)
		{
			$options['btnClass'] .= " " . $btnClass;
		}

		// Instantiate a new JLayoutFile instance and render the layout
		$layout = new FileLayout('joomla.toolbar.confirm');

		$output =  $layout->render($options);

		return $output;

	}

	protected function getButtonCommand($msg, $name, $task, $list)
	{
		\JText::script('JLIB_HTML_PLEASE_MAKE_A_SELECTION_FROM_THE_LIST');

		$msg = str_replace("\\n", "<br>", $msg);
		$msg = str_replace("\n", "<br>", $msg);
		if (strpos($task, ".") !== false)
		{
			$cmd = "let msg = '" . $msg . "'; gslUIkit.modal.confirm(msg).then(() => { Joomla.submitbutton('" . $task . "'); }, () => {})";
		}
		else
		{
			// Javascript !
			$cmd = "let msg = '" . $msg . "'; gslUIkit.modal.confirm(msg).then( () => { " . $task . "; }, () => {})";
		}

		if ($list)
		{
			$alert = "alert(Joomla.JText._('JLIB_HTML_PLEASE_MAKE_A_SELECTION_FROM_THE_LIST'));";
			$cmd   = "if (document.adminForm.boxchecked.value == 0) { " . $alert . " } else { " . $cmd . " }";
		}

		return $cmd;
	}

}

class_alias("Joomla\CMS\Toolbar\Button\JevconfirmButton", "Joomla\CMS\Toolbar\Button\JToolbarButtonJevconfirm");

class JevconfirmvarButton extends ConfirmButton
{
	/**
	 * Button type
	 *
	 * @access    protected
	 * @var        string
	 */
	var $_name = 'Jevconfirm';

	/**
	 * Fetch the HTML for the button
	 *
	 * @param   string  $type     Unused string.
	 * @param   string  $msgvar   Message to render as a javascript variable!
	 * @param   string  $name     Name to be used as apart of the id
	 * @param   string  $text     Button text
	 * @param   string  $task     The task associated with the button
	 * @param   boolean $list     True to allow use of lists
	 * @param   boolean $hideMenu True to hide the menu on click
	 *
	 * @return  string   HTML string for the button
	 *
	 * @since   3.0
	 */
	public function fetchButton($type = 'Confirmvar', $msgvar = '', $name = '', $text = '', $task = '', $list = true, $hideMenu = false, $btnClass = false, $tooltiptitle = "", $tooltipcontent = "")
	{
		// Store all data to the options array for use with JLayout
		$options           = array();
		$options['text']   = \JText::_($text);
		$options['msgvar'] = $msgvar;

		// We want to make sure carriage returns in the message are respected
		//$options['msg'] = str_replace("\\\\n" , "\n", $options['msg']);

		$options['class']  = $this->fetchIconClass($name);
		$options['gsl-icon']  = $name;
		$options['doTask'] = $this->getButtonCommand($options['msgvar'], $name, $task, $list);

		$options['btnClass'] = 'btn btn-small button-' . $name;

		// Add name as class too
		$options['btnClass'] .= " " . str_replace(array(".", "(", ")") , "", $task);

		if ($btnClass)
		{
			$options['btnClass'] .= " " . $btnClass;
		}

		$tooltip = '';
		if ($tooltiptitle && $tooltipcontent)
		{
			$tooltip = ' data-yspoptitle = "' . \JText::_($tooltiptitle, true) . '"'
				. '  data-yspopcontent = "' . \JText::_($tooltipcontent, true) . '" '
				. ' data-yspopoptions= \'{"mode" : "hover", "offset" : 20,"delayHide" : 200, "pos" : "right"}\'';

			$options['btnClass'] .= " hasYsPopover";
		}
		$options['tooltip'] = $tooltip;

		// Instantiate a new JLayoutFile instance and render the layout
		$layout = new FileLayout('joomla.toolbar.confirmvar');

		$output =  $layout->render($options);

		return $output;

	}

	protected function getButtonCommand($msgvar, $name, $task, $list)
	{
		\JText::script('JLIB_HTML_PLEASE_MAKE_A_SELECTION_FROM_THE_LIST');

		$msgvar = str_replace("\\n", "<br>", $msgvar);
		$msgvar = str_replace("\n", "<br>", $msgvar);

		if (strpos($task, ".") !== false)
		{
			//$cmd = "let msg = " . $msgvar . "; if (confirm(msg.replace(/\\\\n/g, '\\n'))) { Joomla.submitbutton('" . $task . "'); }";
			$cmd = "let msg = " . $msgvar . "; if (msg.length == 0) { Joomla.submitbutton('" . $task . "');return false; };gslUIkit.modal.confirm(msg.replace(/\\n/g, '<br>')).then(() => { Joomla.submitbutton('" . $task . "'); }, () => {try { event.stopPropagation();} catch (e) {}; return false;})";
		}
		else
		{
			// Javascript !
			//$cmd = "let msg = " . $msgvar . "; if (confirm(msg.replace(/\\\\n/g, '\\n'))) { " . $task . "; }";
			$cmd = "let msg = " . $msgvar . "; gslUIkit.modal.confirm(msg.replace(/\\n/g, '<br>')).then( () => { " . $task . "; }, () => {try { event.stopPropagation();} catch (e) {}; return false;})";
		}

		if ($list)
		{
			$alert = "alert(Joomla.JText._('JLIB_HTML_PLEASE_MAKE_A_SELECTION_FROM_THE_LIST'));";
			$cmd   = "if (document.adminForm.boxchecked.value == 0) { " . $alert . " } else { " . $cmd . " }";
		}

		return $cmd;
	}

}

class JToolbarButtonJevscript extends ToolbarButton
{

	/**
	 * Button type
	 *
	 * @access	protected
	 * @var		string
	 */
	var $_name = 'Jevscript';

	function fetchButton($type = 'Jevscript', $iconclass = '', $name = '', $text = '', $jsaction = '', $list = true, $hideMenu = false, $targetModalId = false, $tooltiptitle = "", $tooltipcontent = "", $mouseover = "")
	{
		$i18n_text = Text::_($text);
		$class = $this->fetchIconClass($iconclass);
		$doTask = $this->_getCommand($text, $jsaction, $list);
		$tooltip = '';

		if ($name == "apply" || $name == "new" || $name == "save")
		{
			$btnClass = "gsl-button gsl-button-successs";
			$iconWhite = " icon-white";
		}
		else
		{
			$btnClass = "gsl-button gsl-button-primary ";
			$iconWhite = " ";
		}

		if (!empty($mouseover))
		{
			$mouseover = " onmouseover='" . $mouseover . "' ";
		}
		// Add name as class too
		$btnClass .= " " . str_replace(".", "_", $name);

		if ($tooltiptitle && $tooltipcontent)
		{
			$tooltip = ' data-yspoptitle = "' . Text::_($tooltiptitle, true) . '"'
				. '  data-yspopcontent = "' . Text::_($tooltipcontent, true) . '" '
				. ' data-yspopoptions= \'{"mode" : "hover", "offset" : 20,"delayHide" : 200, "pos" : "right"}\'';

			$btnClass .= " hasYsPopover";
		}

		if ($targetModalId == "fred")
		{

			$html = '<button data-toggle="modal" onclick="'.$doTask.'" data-target="#' . $targetModalId . '" class="' . $btnClass . '" ' . $tooltip . $mouseover . ' >
				<span class="' . $class . $iconWhite . '" title="' . $i18n_text . '"></span> ' . $i18n_text . '</button>';
		}
		else
		{

			$html = "<button href=\"#\" onclick=\"$doTask\" class=\"" . $btnClass . "\"  $tooltip $mouseover>\n";
			$html .= "<span  gsl-icon='icon:" . $iconclass . "' title='" . $i18n_text . "s'></span>\n";

			$html .= "$i18n_text\n";
			$html .= "</button>\n";
		}

		return $html;
	}

	/**
	 * Get the button CSS Id
	 *
	 * @access	public
	 * @return	string	Button CSS Id
	 * @since	1.5
	 */
	function fetchId($type = 'Jevscript', $icon = '', $text = '', $jsaction = '', $listSelect = '', $js = '')
	{
		return $this->_parent->getName() . '-' . $icon;
	}

	/**
	 * Get the JavaScript command for the button
	 *
	 * @access	private
	 * @param	object	$definition	Button definition
	 * @return	string	JavaScript command string
	 * @since	1.5
	 */
	function _getCommand($name, $jsaction, $list)
	{
		$todo = StringHelper::strtolower(Text::_($name));
		$message = Text::sprintf('Please make a selection from the list to %s', $todo);
		$message = addslashes($message);
		$submitbutton = "Joomla.submitbutton";

		if ($list)
		{
			$cmd = "(function(e) {if(document.adminForm.boxchecked.value==0){alert('$message');e.preventDefault(); e.stopPropagation();}else{ $jsaction; }})(event); ";
		}
		else
		{
			// we MUST have a space before $action to stop {$ from being parsed by php
			$cmd = "(function(e) { $jsaction;return false;})(event);";
		}

		return $cmd;
	}

}

class JToolbarButtonJevraw extends ToolbarButton
{

	/**
	 * Button type
	 *
	 * @access	protected
	 * @var		string
	 */
	var $_name = 'Jevraw';

	/**
	 * Get the HTML for a separator in the toolbar
	 *
	 * @param   array  &$definition  Class name and custom width
	 *
	 * @return  string  The HTML for the separator
	 *
	 * @see     ToolbarButton::render()
	 * @since   3.0
	 */
	public function render(&$definition = NULL)
	{
		$html  = empty($definition[1]) ? '' : $definition[1];
		return $html . "\n";
	}

	/**
	 * Empty implementation (not required for raw)
	 *
	 * @return  void
	 *
	 * @since   3.0
	 */
	public function fetchButton()
	{
	}
}

class Jevgroupbutton extends ToolbarButton
{
	/**
	 * Button type
	 *
	 * @access	protected
	 * @var		string
	 */
	var $_name = 'Jevgroup';

	/**
	 * Fetch the HTML for the button
	 *
	 * @param   string   $type  Unused string.
	 * @param   array    $buttongroup  buttons to add
	 *
	 * @return  string  HTML string for the button
	 *
	 * @since   3.0
	 */
	public function fetchButton($type = 'Group', $label = "", $wrappername = "buttongroup", $buttongroup = array())
	{
		$buttonbar = new Toolbar();
		foreach ($buttongroup as $button)
		{
			call_user_func_array(array($buttonbar, 'appendButton'), $button);
		}

		$buttonbaroputput =  '<div id="' . $wrappername . '_wrapper" class="ystoolbar_wrapper gsl-button-group ">';
		$buttonbaroputput .= '<button class="gsl-button gsl-button-primary" type="button"><span gsl-icon="icon:  triangle-down"></span>' . Text::_($label) . '</button>';
		$buttonbaroputput .= '<div class="gsl-padding-remove" gsl-dropdown="mode: click, hover;pos: bottom-left;offset: 0">';
		$buttonbaroputput .= '<div class="gsl-nav gsl-dropdown-nav " >';

		$toolbarButtons = $buttonbar->getItems();
		foreach ($toolbarButtons as $toolbarButton)
		{
			$buttonbaroputput .= $buttonbar->renderButton($toolbarButton);
		}
		$buttonbaroputput .= "</div>";
		$buttonbaroputput .= "</div>";
		$buttonbaroputput .= "</div>";

		return $buttonbaroputput;
	}

	function fetchId( $type='Group',  $msg='', $name = '')
	{
		return $this->_parent->getName().'-'.$name;
	}


}
