<?php

/**
 * @version    3.7.4
 * @package    com_rsvppro
 * @author     Tony Partridge <tony@jevents.net>
 * @copyright  2018 Tony Partridge - GWE Systems Ltd
 * @license    GNU General Public License version 2 or later; see LICENSE.txt
 */
defined('_JEXEC') or die;

use Joomla\CMS\MVC\Model\ListModel;
use Joomla\CMS\Component\ComponentHelper;
use Joomla\CMS\Factory;

jimport('joomla.application.component.modellist');

/**
 * Methods supporting a list of Rsvpprotest records.
 *
 * @since  1.6
 */
class RsvpproModelEmailreminders extends ListModel
{
/**
	* Constructor.
	*
	* @param   array  $config  An optional associative array of configuration settings.
	*
	* @see        JController
	* @since      1.6
	*/
	public function __construct($config = array())
	{
		if (empty($config['filter_fields']))
		{
			$config['filter_fields'] = array(
				'id', 'a.`id`',
				'name', 'a.`name`',
				'at_id', 'a.`at_id`',
				'user_id', 'a.`user_id`',
				'rp_id', 'a.`rp_id`',
				'sentmessage', 'a.`sentmessage`',
				'sentdate', 'a.`sentdate`',
				'email_name', 'a.`email_name`',
				'email_address', 'a.`email_address`',
				'status', 'a.`sentmessage`',
			);
		}

		parent::__construct($config);
	}

	/**
	 * Method to auto-populate the model state.
	 *
	 * Note. Calling getState in this method will result in recursion.
	 *
	 * @param   string  $ordering   Elements order
	 * @param   string  $direction  Order direction
	 *
	 * @return void
	 *
	 * @throws Exception
	 */
	protected function populateState($ordering = null, $direction = null)
	{
		// Initialise variables.
		$app = Factory::getApplication('administrator');

		// Load the filter state.
		$search = $app->getUserStateFromRequest($this->context . '.filter.search', 'filter_search');
		$this->setState('filter.search', $search);

		// Filtering name
		$this->setState('filter.name', $app->getUserStateFromRequest($this->context.'.filter.name', 'filter_name', '', 'string'));

		// Filtering user_id
		$this->setState('filter.user_id', $app->getUserStateFromRequest($this->context.'.filter.user_id', 'filter_user_id', '', 'string'));

		// Filtering sent status
		$this->setState('filter.status', $app->getUserStateFromRequest($this->context.'.filter.status', 'filter_status', '', 'string'));

		// Filtering sentdate
		$this->setState('filter.sentdate.from', $app->getUserStateFromRequest($this->context.'.filter.sentdate.from', 'filter_from_sentdate', '', 'string'));
		$this->setState('filter.sentdate.to', $app->getUserStateFromRequest($this->context.'.filter.sentdate.to', 'filter_to_sentdate', '', 'string'));


		// Load the parameters.
		$params = ComponentHelper::getParams('com_rsvppro');
		$this->setState('params', $params);

		// List state information.
		parent::populateState('a.id', 'asc');
	}

	/**
	 * Method to get a store id based on model configuration state.
	 *
	 * This is necessary because the model is used by the component and
	 * different modules that might need different sets of data or different
	 * ordering requirements.
	 *
	 * @param   string  $id  A prefix for the store id.
	 *
	 * @return   string A store id.
	 *
	 * @since    1.6
	 */
	protected function getStoreId($id = '')
	{
		// Compile the store id.
		$id .= ':' . $this->getState('filter.search');

		return parent::getStoreId($id);
	}

	/**
	 * Build an SQL query to load the list data.
	 *
	 * @return   JDatabaseQuery
	 *
	 * @since    1.6
	 */
	protected function getListQuery()
	{
		// Create a new query object.
		$db    = $this->getDbo();
		$query = $db->getQuery(true);

		// Select the required fields from the table.
		$query->select(
			$this->getState(
				'list.select', 'DISTINCT a.*'
			)
		);
		$query->from('`#__jev_reminders` AS a');

		// Join over the foreign key 'name'
		// Create the base subQuery select statement.
		$subQuery = $db->getQuery(true);

		$subQuery->select('ev_id')
			->from($db->quoteName('#__jev_attendance'))
			->where($db->quoteName('id') . ' = a.`at_id`');

		$query->select('`eventDetail`.evdet_id AS `ev_id`, `eventDetail`.summary AS `name`, `eventDetail`.dtstart AS `dtstart`');
		$query->join('LEFT', '#__jevents_vevent AS `vevent` ON `vevent`.ev_id IN ( ' . $subQuery . ')');
		$query->join('LEFT', '#__jevents_vevdetail AS `eventDetail` ON `eventDetail`.evdet_id = `vevent`.detail_id');

		// Join over the user field 'user_id'
		$query->select('`user_id`.name AS `user_id`');
		$query->join('LEFT', '#__users AS `user_id` ON `user_id`.id = a.`user_id`');

		// Filter by search in title
		$search = $this->getState('filter.search');

		if (!empty($search))
		{
			if (stripos($search, 'id:') === 0)
			{
				$query->where('a.id = ' . (int) substr($search, 3));
			}
			else
			{
				$search = $db->Quote('%' . $db->escape($search, true) . '%');
				$query->where('(eventDetail.summary LIKE ' . $search . '  OR  a.email_name LIKE ' . $search . '  OR  a.email_address LIKE ' . $search . ' )');
			}
		}

		// Filtering Sent Status
		$filter_sentmessage = $this->state->get("filter.status");

		if ($filter_sentmessage !== null && !empty($filter_sentmessage))
		{
			$query->where("a.`sentmessage` = '".$db->escape($filter_sentmessage)."'");
		}

		// Filtering name
		$filter_name = $this->state->get("filter.name");

		if ($filter_name !== null && !empty($filter_name))
		{
			$query->where("eventDetail.`evdet_id` = " . $db->escape($filter_name) . "");
		}

		// Filtering user_id
		$filter_user_id = $this->state->get("filter.user_id");

		if ($filter_user_id !== null && !empty($filter_user_id))
		{
			$query->where("a.`user_id` = '".$db->escape($filter_user_id)."'");
		}

		// Filtering sentdate
		$filter_sentdate_from = $this->state->get("filter.sentdate.from");

		if ($filter_sentdate_from !== null && !empty($filter_sentdate_from))
		{
			$query->where("a.`sentdate` >= '".$db->escape($filter_sentdate_from)."'");
		}
		$filter_sentdate_to = $this->state->get("filter.sentdate.to");

		if ($filter_sentdate_to !== null  && !empty($filter_sentdate_to))
		{
			$query->where("a.`sentdate` <= '".$db->escape($filter_sentdate_to)."'");
		}
		// Add the list ordering clause.
		$orderCol  = $this->state->get('list.ordering');
		$orderDirn = $this->state->get('list.direction');

		if ($orderCol && $orderDirn)
		{
			$query->order($db->escape($orderCol . ' ' . $orderDirn));
		}

		return $query;
	}

	/**
	 * Get an array of data items
	 *
	 * @return mixed Array of data items on success, false on failure.
	 */
	public function getItems()
	{
		$_items = parent::getItems();

		$items  = $_items ? $_items : array();

		foreach ($items as $oneItem)
		{

			if (isset($oneItem->name))
			{
				$values    = explode(',', $oneItem->name);
				$textValue = array();
				$dtstart   = array();
				$evdet_id  = array();

				foreach ($values as $value)
				{
					$db    = Factory::getDbo();
					$query = $db->getQuery(true);
					$query
						->select('`#vevdet`.`summary`, `#vevdet`.`evdet_id`, `#vevdet`.`dtstart`')
						->from($db->quoteName('#__jevents_vevdetail', '#vevdet'))
						->where($db->quoteName('evdet_id') . ' = '. $db->quote($db->escape($value)));

					$db->setQuery($query);
					$results = $db->loadObject();

					if ($results)
					{
						$summaryValue[] = $results->summary;
						$evdet_id[]     = $results->evdet_id;
						$dtstart[]      = $results->dtstart;
					}
				}

				$oneItem->name = !empty($textValue) ? implode(', ', $textValue) : $oneItem->name;
				$oneItem->ev_id = !empty($evdet_id) ? implode(', ', $evdet_id) : $oneItem->ev_id;
				$oneItem->dtstart = !empty($dtstart) ? implode(', ', $dtstart) : $oneItem->dtstart;
			}
		}

		return $items;
	}
}
