<?php

// Check to ensure this file is included in Joomla!
defined('_JEXEC') or die( 'Restricted Access' );

use Joomla\CMS\MVC\Model\BaseDatabaseModel;
use Joomla\CMS\Pagination\Pagination;
use Joomla\CMS\Factory;
use Joomla\String\StringHelper;
use Joomla\CMS\Date\Date;

jimport('joomla.application.component.model');

class RsvpprotemplatesModelTemplates extends BaseDatabaseModel
{

	/**
	 * Category ata array
	 *
	 * @var array
	 */
	var $_data = null;

	/**
	 * Category total
	 *
	 * @var integer
	 */
	var $_total = null;

	/**
	 * Pagination object
	 *
	 * @var object
	 */
	var $_pagination = null;

	/**
	 * Constructor
	 *
	 * @since 1.5
	 */
	function __construct()
	{
		parent::__construct();

		$mainframe = Factory::getApplication();$option=Factory::getApplication()->input->getCmd("option");

		// Get the pagination request variables
		$limit		= $mainframe->getUserStateFromRequest( 'templatelistlimit', 'limit', $mainframe->getCfg('list_limit'), 'int' );
		$limitstart	= $mainframe->getUserStateFromRequest( $option.'.limitstart', 'limitstart', 0, 'int' );

		// In case limit has been changed, adjust limitstart accordingly
		$limitstart = ($limit != 0 ? (floor($limitstart / $limit) * $limit) : 0);

		$this->setState('limit', $limit);
		$this->setState('limitstart', $limitstart);
	}

	/**
	 * Method to get location item data
	 *
	 * @access public
	 * @return array
	 */
	function getData()
	{
		// Lets load the content if it doesn't already exist
		if (empty($this->_data))
		{
			$query = $this->_buildQuery();
			try {
				$this->_data = $this->_getList($query, $this->getState('limitstart'), $this->getState('limit'));
			}
			catch (RuntimeException $e)
			{
				$error = $e->getMessage();
				echo $error;
			}

		}

		return $this->_data;
	}

	/**
	 * Method to get the total number of location items
	 *
	 * @access public
	 * @return integer
	 */
	function getTotal()
	{
		// Lets load the content if it doesn't already exist
		if (empty($this->_total))
		{
			$query = $this->_buildQuery();
			$this->_total = $this->_getListCount($query);
		}

		return $this->_total;
	}

	/**
	 * Method to get a pagination object for the locations
	 *
	 * @access public
	 * @return integer
	 */
	function getPagination()
	{
		// Lets load the content if it doesn't already exist
		if (empty($this->_pagination))
		{
			$mainframe = Factory::getApplication();
			if ($mainframe->isClient('administrator')){
				jimport('joomla.html.pagination');
				$this->_pagination = new Pagination( $this->getTotal(), $this->getState('limitstart'), $this->getState('limit') );
			}
			else {
				include_once(JPATH_COMPONENT_ADMINISTRATOR."/libraries/JevPagination.php");
				$this->_pagination = new JevPagination( $this->getTotal(), $this->getState('limitstart'), $this->getState('limit'),true);
			}
		}

		return $this->_pagination;
	}


	function _buildQuery()
	{
		// Get the WHERE and ORDER BY clauses for the query
		$where		= $this->_buildContentWhere();
		$orderby	= $this->_buildContentOrderBy();

		$query = ' SELECT tmpl.* FROM #__jev_rsvp_templates AS tmpl '
		. $where
		. $orderby
		;
		return $query;
	}


	function _buildContentOrderBy()
	{
		$mainframe = Factory::getApplication();$option=Factory::getApplication()->input->getCmd("option");

		$filter_order		= $mainframe->getUserStateFromRequest( $option.'tmpl_filter_order',		'filter_order',		'tmpl.title',	'cmd' );
		$filter_order_Dir	= $mainframe->getUserStateFromRequest( $option.'tmpl_filter_order_Dir',	'filter_order_Dir',	'ASC',			'word' );

		$orderby 	= ' ORDER BY '.$filter_order.' '.$filter_order_Dir;

		return $orderby;
	}

	function _buildContentWhere()
	{
		$mainframe = Factory::getApplication();$option=Factory::getApplication()->input->getCmd("option");
		$db					= Factory::getDBO();
		$filter_state		= $mainframe->getUserStateFromRequest( $option.'tmpl_filter_state',		'filter_state',		'',				'word' );
		$filter_order		= $mainframe->getUserStateFromRequest( $option.'tmpl_filter_order',		'filter_order',		'tmpl.title',	'cmd' );
		$filter_order_Dir	= $mainframe->getUserStateFromRequest( $option.'tmpl_filter_order_Dir',	'filter_order_Dir',	'ASC',				'word' );
		$search				= $mainframe->getUserStateFromRequest( $option.'tmpl_search',			'search',			'',				'string' );
		$search				= StringHelper::strtolower( $search );
		$customised			= $mainframe->getUserStateFromRequest( $option.'rsvp_customised', 'customised', 0, 'int' );

		$where = array();

		if (trim($search)!="") {
			$where[] = 'LOWER(tmpl.title) LIKE '.$db->Quote( '%'.$db->escape( $search, true ).'%', false );
		}
		if ( $filter_state ) {
			if ( $filter_state == 'P' ) {
				$where[] = 'tmpl.published = 1';
			} else if ($filter_state == 'U' ) {
				$where[] = 'tmpl.published = 0';
			}
		}
		$user = Factory::getUser();

		if (!JevTemplateHelper::canCreateGlobal()){
			$where[] = 'tmpl.global = 0 ';
			$where[] = 'tmpl.created_by =  ' . $user->id;
		}
		else {
			if($customised !== 2)
			{
				$where[] = ' (tmpl.global = 1 OR tmpl.created_by =  ' . $user->id . ')';
			}
		}

		if ($customised === 1){
			$where[] = 'tmpl.istemplate = 0';
		}
		else if($customised === 0) {
			$where[] = 'tmpl.istemplate = 1';
		}

		$where 		= ( count( $where ) ? ' WHERE '. implode( ' AND ', $where ) : '' );

		return $where;
	}

	public function getLanguages()
	{
		static  $languages;
		if (!isset($languages)){
			$db = Factory::getDBO();

			// get the list of languages first
			$query	= $db->getQuery(true);
			$query->select("l.*");
			$query->from("#__languages as l");
			$query->where('l.lang_code <> "xx-XX"');
			$query->order("l.lang_code asc");

			$db->setQuery($query);
			$languages  = $db->loadObjectList('lang_code');
		}
		return $languages;
	}

	public function import($files)
	{
		$success = true;
		$filedata = file_get_contents($files['tmp_name']);
		$templates = json_decode($filedata, true);
		foreach ($templates as $template)
		{
			$user = Factory::getUser();
			//$template['created_by'] = $user->id;
			$template['modified_by'] = 0;
			//$now = new Date('+0 seconds');

			//$template['created'] = $now->toSql();

			// Change name to something unique e.g. add '(imported)' to the end with a number if needed
			// ToDo check title is unique
			$template['title'] .= " (imported)";

			$templateEntry = $this->getTable('template');
			$templateEntry->bindimport($template);
			$success &= $templateEntry->store();
		}
		return $success;
	}
}
