<?php
/**
 * JEvents Component for Joomla 1.5.x
 *
 * @version     $Id: jevents.php 1402 2009-04-03 13:00:38Z geraint $
 * @package     JEvents
 * @copyright   Copyright (C)  2008-2021 GWE Systems Ltd
 * @license     GNU/GPLv2, see http://www.gnu.org/licenses/gpl-2.0.html
 * @link        https://www.jevents.net
 */


defined( 'JPATH_BASE' ) or die( 'Direct Access to this location is not allowed.' );

use Joomla\CMS\MVC\Controller\BaseController;
use Joomla\CMS\Language\Text;
use Joomla\CMS\Component\ComponentHelper;
use Joomla\CMS\Factory;

if (version_compare(phpversion(), '5.2.0', '<')===true) {
	echo  '<div style="font:12px/1.35em arial, helvetica, sans-serif;"><div style="margin:0 0 25px 0; border-bottom:1px solid #ccc;"><h3 style="margin:0; font-size:1.7em; font-weight:normal; text-transform:none; text-align:left; color:#2f2f2f;">'.Text::_("RSVP_INVALID_PHP1").'</h3></div>'.Text::_("RSVP_INVALID_PHP2").'</div>';
	return;
}

if (isset($_SERVER['HTTP_USER_AGENT']) && (strpos($_SERVER['HTTP_USER_AGENT'], "MSIE") !== false || strpos($_SERVER['HTTP_USER_AGENT'], "Internet Explorer") !== false))
{
	define ("GSLMSIE10" , 1);
}
else
{
	// Also needs JEvents 3.5.0 installed
	JLoader::register('JEventsVersion',JPATH_ADMINISTRATOR."/components/com_jevents/libraries/version.php");
	$jevversion = JEventsVersion::getInstance();

	define ("GSLMSIE10" , version_compare($jevversion->get("RELEASE"), "3.4.999", "lt") ? 1 : 0);

}

jimport("joomla.filesystem.file");
jimport('joomla.filesystem.path');
JLoader::register('JevRegistry', JPATH_SITE . "/components/com_jevents/libraries/registry.php");

$app    = Factory::getApplication();
$input  = $app->input;
$option = $input->getCmd("option");
include_once(JPATH_ADMINISTRATOR."/components/com_jevents/jevents.defines.php");
include_once(JPATH_COMPONENT_ADMINISTRATOR.'/'.str_replace("com_","",$option).".defines.php");
include_once(JEV_ADMINPATH.'/'.JEV_COMPONENT.".defines.php");

$registry	= JevRegistry::getinstance("jevents");
$jevparams = ComponentHelper::getParams(JEV_COM_COMPONENT);

// See http://www.php.net/manual/en/timezones.php
$tz = $jevparams->get("icaltimezonelive","");
if ($tz!="" && is_callable("date_default_timezone_set")){
	$timezone= date_default_timezone_get();
	date_default_timezone_set($tz);
	$registry->set("jevents.timezone",$timezone);
}

$lang = Factory::getLanguage();
$lang->load(RSVP_COM_COMPONENT, JPATH_SITE);

$rsvpparams = ComponentHelper::getParams(RSVP_COM_COMPONENT);
if ($rsvpparams->get("attendance", -99) === -99 && $input->getCmd('task', 'cpanel.show') !== "params.edit" && $input->getCmd('task', 'cpanel.show') !== "params.apply"  && $input->getCmd('task', 'cpanel.show') !== "params.save")
{
	Factory::getApplication()->enqueueMessage( Text::_('RSVP_PLEASE_CHECK_CONFIGURATION_AND_SAVE'));
	Factory::getApplication()->redirect("index.php?option=" . RSVP_COM_COMPONENT . "&task=params.edit");
	return;
}

// also load the plugin language file!
$lang->load( 'plg_jevents_jevrsvppro',JPATH_ADMINISTRATOR );

$params = ComponentHelper::getParams(RSVP_COM_COMPONENT);

// access control
$juser = Factory::getUser();
$authorised = false;
if (version_compare(JVERSION, "1.6.0", 'ge'))
{
	if ($juser->authorise('core.manage', 'com_rsvppro'))
	{
		$authorised = true;
	}
}
else
{
	// >= admins
	if ($juser->gid >= 24)
	{
		$authorised = true;
	}
}
if (!$authorised)
{
	return;
}

// Tell RSVP Pro javascript which framework to use
$frontendstyling = $params->get("frontendstyling", "B2");
Factory::getDocument()->addScriptDeclaration("var rsvpprofrontendstyling = '$frontendstyling';\n");

// disable Zend php4 compatability mode
@ini_set("zend.ze1_compatibility_mode","Off");

$cmd 	= $input->getCmd('task', 'cpanel.show');
$viewRequest	= $input->get('view', '');
if (strpos($viewRequest, 'emailreminders') !== false || strpos($viewRequest, 'emailinvitees') !== false) {

	// The J3 way!
	// Include dependancies
	jimport('joomla.application.component.controller');

	JLoader::registerPrefix('Rsvppro', JPATH_COMPONENT_ADMINISTRATOR);
	JLoader::register('RsvpproHelper', JPATH_COMPONENT_ADMINISTRATOR . DIRECTORY_SEPARATOR . 'helpers' . DIRECTORY_SEPARATOR . 'rsvppro.php');

	$controller = BaseController::getInstance('Rsvppro');
	$controller->execute(Factory::getApplication()->input->get('task'));
	$controller->redirect();
	return;

}

if (strpos($cmd, '.') != false) {
	// We have a defined controller/task pair -- lets split them out
	list($controllerName, $task) = explode('.', $cmd);

	// Define the controller name and path
	$controllerName	= strtolower($controllerName);
	$controllerPath	= JPATH_COMPONENT.'/'.'controllers'.'/'.$controllerName.'.php';
	$controllerName = "Admin".$controllerName;

	// If the controller file path exists, include it ... else lets die with a 500 error
	if (file_exists($controllerPath)) {
		require_once($controllerPath);
	} else {
		throw new Exception(500, 'Invalid Controller: ' . $controllerName);
	}
} else {
	// Base controller, just set the task
	$controllerName = null;
	$task = $cmd;
}

// Set the name for the controller and instantiate it
$controllerClass = ucfirst($controllerName).'Controller';
if (class_exists($controllerClass)) {
	$controller = new $controllerClass();

	// Perform the Request task
	$controller->execute($task);

	// Must reset the timezone back!!
	if ($tz && is_callable("date_default_timezone_set")){
		date_default_timezone_set($timezone);
	}


	// Redirect if set by the controller
	$controller->redirect();

} else {
	echo "missing controllerClass $controllerClass";
	exit();
}
