<?php
/**
 * JEvents Component for Joomla 1.5.x
 *
 * @version     $Id: view.html.php 1703 2010-02-16 12:23:46Z geraint $
 * @package     JEvents
 * @copyright   Copyright (C)  2008-2021 GWE Systems Ltd
 * @license     GNU/GPLv2, see http://www.gnu.org/licenses/gpl-2.0.html
 * @link        https://www.jevents.net
 */

// Check to ensure this file is included in Joomla!
defined('_JEXEC') or die();

use Joomla\CMS\Language\Text;
use Joomla\CMS\Component\ComponentHelper;
use Joomla\CMS\Session\Session;
use Joomla\CMS\Factory;
use Joomla\CMS\HTML\HTMLHelper;
use Joomla\CMS\Toolbar\Toolbar;

/**
 * HTML View class for the component
 *
 * @static
 */
class AdminAttendeesViewAttendees extends RSVPAbstractView
{
	protected $xmlparams = array();

	function __construct($config = array()){
		parent::__construct($config);
		JLoader::register('Joomla\CMS\Toolbar\Button\JevgroupButton', JPATH_COMPONENT_ADMINISTRATOR . "/libraries/jevtoolbarbuttons.php");
		JLoader::register('Joomla\CMS\Toolbar\Button\JevconfirmButton', JPATH_COMPONENT_ADMINISTRATOR . "/libraries/jevtoolbarbuttons.php");

		//JEVHelper::stylesheet('eventsadmin.css', 'components/com_jevents/assets/css/');

	}

	function overview($tpl = null)
	{
	        jimport( 'joomla.application.component.view' );
		HTMLHelper::stylesheet(  'components/'.RSVP_COM_COMPONENT.'/assets/css/rsvpadmin.css' );
		$document = Factory::getDocument();
		$document->setTitle(Text::_('JEV_ATTENDEES_AND_WAITING'));

		// Set toolbar items for the page
        $this->rows = $this->items;
		if (count($this->rows) > 0)
		{
			$attendee = $this->rows[0];
		}
		$params = ComponentHelper::getParams('com_rsvppro');
		JToolBarHelper::title( Text::_( 'JEV_ATTENDEES_AND_WAITING' )." : ".$this->repeat->summary. " (".strftime($params->get("listdateformat", "%Y-%m-%d"), $this->event->getUnixStartTime()).")", 'jevents' );
		if (JEVHelper::canEditEvent($this->repeat)){
			JToolBarHelper::addNew( 'attendees.edit');
			JToolBarHelper::deleteList("RSVP_DELETE_ATTENDEE","attendees.delete");

			//JToolBarHelper::custom( 'attendees.attend', 'check', 'check', 'RSVP_SET_DID_ATTEND', true );
			//JToolBarHelper::custom( 'attendees.notattend', 'close', 'close', 'RSVP_SET_DID_NOT_ATTEND', true );

			$bar = Toolbar::getInstance('toolbar');
			$bar->appendButton('jevgroup', 'RSVP_SET_ATTEND', 'attend', array(
			        array('Standard', 'check', 'RSVP_SET_DID_ATTEND', 'attendees.attend',  true ),
                    array('Standard', 'close', 'RSVP_SET_DID_NOT_ATTEND', 'attendees.notattend',  true )
            ));

			$bar = Toolbar::getInstance('toolbar');
			$bar->appendButton('jevgroup', 'RSVP_SET_CONFIRMED', 'confirm', array(
				array('Standard', 'check', 'RSVP_SET_CONFIRMED_YES', 'attendees.confirm',  true ),
				array('Standard', 'close', 'RSVP_SET_CONFIRMED_NO', 'attendees.unconfirm',  true )
			));

			$bar = Toolbar::getInstance('toolbar');
			$bar->appendButton('jevgroup', 'RSVP_SET_ATTENDANCE', 'attendance', array(
				array('Standard', 'check', 'RSVP_SET_ATTENDANCE_YES', 'attendees.attendyes',  true ),
				array('Standard', 'close', 'RSVP_SET_ATTENDANCE_NO', 'attendees.attendno',  true ),
				array('Standard', 'question', 'RSVP_SET_ATTENDANCE_MAYBE', 'attendees.attendmaybe',  true )
			));

			$bar = Toolbar::getInstance('toolbar');
			$bar->appendButton('jevgroup', 'RSVP_SET_WAITING', 'waiting', array(
				array('Standard', 'check', 'RSVP_SET_WAITING_YES', 'attendees.waiting',  true ),
				array('Confirm', 'RSVP_PROMOTE_FROM_WAITING_LIST', 'close', 'RSVP_SET_WAITING_NO', 'attendees.notwaiting', true),
				//array('Standard', 'close', 'RSVP_SET_WAITING_NO', 'attendees.notwaiting',  true )
			));

			JToolBarHelper::custom( 'attendees.export', 'file-text', 'file-text', 'RSVP_EXPORT', false );
		}

		$user = Factory::getUser();
		 if ($user->id == $this->repeat ->created_by() || JEVHelper::isAdminUser($user) || JEVHelper::canDeleteEvent($this->repeat, $user) || $user->authorise('core.sendmessage', 'com_rsvppro')){
			//JToolBarHelper::custom( 'attendees.messageall', 'mail', 'mail', 'JEV_RSVP_SEND_MESSAGE_TO_ALL', false );
			JToolBarHelper::custom( 'attendees.message', 'mail', 'mail', 'JEV_RSVP_SEND_MESSAGE_TO_SELECTED',true);
		 }

		 JToolBarHelper::custom( 'sessions.list', 'calendar', 'calendar', 'RSVP_SESSIONS', false );

		if (Factory::getApplication()->isClient('administrator')){
			JToolBarHelper::custom( 'cpanel.cpanel', 'home', 'home', 'JEV_ADMIN_CPANEL', false );
		}
		else {
		}

		$this->showToolBar();

		$params = ComponentHelper::getParams(RSVP_COM_COMPONENT);

		$this->params = $params;

	}

	function transactions($tpl = null)
	{

		HTMLHelper::stylesheet(  'components/'.RSVP_COM_COMPONENT.'/assets/css/rsvpadmin.css' );

		$document = Factory::getDocument();
		$document->setTitle(Text::_('RSVP_TRANSACTIONS'));

		$attendeeName = "";
		if ($this->attendee->user_id>0){
			$user =JEVHelper::getUser($this->attendee->user_id);
			$attendeeName = $user->name." (".$user->username.")";
		}
		else {
			$attendeeName = $this->attendee->email_address;
		}
                if (JevJoomlaVersion::isCompatible("3.0"))
		{
        JHtmlSidebar::addEntry(
			$this->event->title(),
			"index.php?option=com_rsvppro&task=attendees.overview&atd_id[]=" .  $this->rsvpdata->id ."|". $this->attendee->rp_id ."&repeating=" . $this->rsvpdata->allrepeats,
			false
		);
                }
                else {
        JSubMenuHelper::addEntry(
			$this->event->title(),
			"index.php?option=com_rsvppro&task=attendees.overview&atd_id[]=" .  $this->rsvpdata->id ."|". $this->attendee->rp_id ."&repeating=" . $this->rsvpdata->allrepeats,
			false
		);
                }

		JToolBarHelper::title( Text::_( 'RSVP_TRANSACTIONS' )." : ".$this->event->title()." :: ".$attendeeName  , 'jevents' );
		JToolBarHelper::addNew( 'attendees.newtransaction');
		JToolBarHelper::deleteList(Text::_("RSVP_DELETE_TRANSACTION", true),"attendees.deletetransaction");
		JToolBarHelper::custom( 'attendees.overview', 'users.png', 'users.png', 'JEV_ATTENDEES', false );

		if (Factory::getApplication()->isClient('administrator')){
			JToolBarHelper::custom( 'cpanel.cpanel', 'home', 'home', 'JEV_ADMIN_CPANEL', false );
		}
		else {
			JToolBarHelper::cancel( 'attendees.overview');
		}

		$this->showToolBar();

		$params = ComponentHelper::getParams(RSVP_COM_COMPONENT);

		$this->params = $params;

	}

	function export($tpl = null)
	{
        $this->rows = $this->items;

	}

	function message($tpl = null)
	{
		$user = Factory::getUser();
		 if ($user->id != $this->repeat ->created_by() && !JEVHelper::isAdminUser($user) && !JEVHelper::canDeleteEvent($this->repeat, $user) && ! $user->authorise('core.sendmessage', 'com_rsvppro')){
			 return;
		 }

		HTMLHelper::_('jquery.framework');
		// Load the modal behavior script.
JLoader::register('JevModal', JPATH_LIBRARIES . "/jevents/jevmodal/jevmodal.php");
JevModal::modal("a.jevmodal");
		// sequence is important since rsvpadmin overwrites the find user method
		HTMLHelper::script( 'plugins/jevents/jevrsvppro/rsvppro/rsvp.js' );
		HTMLHelper::stylesheet('plugins/jevents/jevrsvppro/rsvppro/rsvp.css' );
		HTMLHelper::stylesheet(  'components/'.RSVP_COM_COMPONENT.'/assets/css/rsvpadmin.css' );
		HTMLHelper::script( 'components/com_rsvppro/assets/js/rsvpadmin.js' );
		HTMLHelper::script(  'components/'.RSVP_COM_COMPONENT.'/assets/js/forms.js' );

		$script = "var urlroot = '".JURI::root()."';\n";
		$script .= "var jsontoken = '".Session::getFormToken()."';\n";
		$script .= "var jsonclient = '".Factory::getApplication()->getClientId()."';\n";

		$document = Factory::getDocument();
		$document->addScriptDeclaration($script);

		$document->setTitle(Text::_('RSVP_EDIT_ATTENDEE_MESSAGE'));


		// Set toolbar items for the page
		JToolBarHelper::title( Text::_( 'RSVP_EDIT_ATTENDEE_MESSAGE' ), 'jevents' );
		JToolBarHelper::custom( 'attendees.message', 'send.png', 'send.png', 'JEV_RSVP_SEND_MESSAGE',false);

		// Don't show this button - we use the update button beneath!
		JToolBarHelper::cancel('attendees.overview');

		RsvpproHelper::addSubmenu();


		$this->showToolBar();

		$params = ComponentHelper::getParams(RSVP_COM_COMPONENT);





	}

	function edit($tpl = null)
	{

		// Load the modal behavior script.
JLoader::register('JevModal', JPATH_LIBRARIES . "/jevents/jevmodal/jevmodal.php");
JevModal::modal("a.jevmodal");
		// sequence is important since rsvpadmin overwrites the find user method
		HTMLHelper::script( 'plugins/jevents/jevrsvppro/rsvppro/rsvp.js' );
		HTMLHelper::stylesheet('plugins/jevents/jevrsvppro/rsvppro/rsvp.css' );
		HTMLHelper::script( 'components/com_rsvppro/assets/js/rsvpadmin.js' );

		$script = "var urlroot = '".JURI::root()."';\n";
		$script .= "var jsontoken = '".Session::getFormToken()."';\n";
		$script .= "var jsonclient = '".Factory::getApplication()->getClientId()."';\n";

		$document = Factory::getDocument();
		$document->addScriptDeclaration($script);

		$document->setTitle(Text::_('RSVP_EDIT_ATTENDEE'));

		// Set toolbar items for the page
		JToolBarHelper::title( Text::_( 'RSVP_EDIT_ATTENDEE' ), 'jevents' );

		// Don't show this button - we use the update button beneath!
		//JToolBarHelper::save( 'attendees.save');
		JToolBarHelper::cancel('attendees.overview');

		RsvpproHelper::addSubmenu();

		$params = ComponentHelper::getParams(RSVP_COM_COMPONENT);


	}

	function edittransaction($tpl = null)
	{

		// Load the modal behavior script.
JLoader::register('JevModal', JPATH_LIBRARIES . "/jevents/jevmodal/jevmodal.php");
JevModal::modal("a.jevmodal");
		// sequence is important since rsvpadmin overwrites the find user method
		HTMLHelper::script( 'plugins/jevents/jevrsvppro/rsvppro/rsvp.js' );
		HTMLHelper::script('components/com_rsvppro/assets/js/rsvpadmin.js' );
		HTMLHelper::stylesheet(  'components/'.RSVP_COM_COMPONENT.'/assets/css/rsvpadmin.css' );

		$script = "var urlroot = '".JURI::root()."';\n";
		$script .= "var jsontoken = '".Session::getFormToken()."';\n";
		$script .= "var jsonclient = '".Factory::getApplication()->getClientId()."';\n";

		$document = Factory::getDocument();
		$document->addScriptDeclaration($script);

		$document->setTitle(Text::_('RSVP_EDIT_TRANSACTION'));

		$attendeeName = "";
		if ($this->attendee->user_id>0){
			$user =JEVHelper::getUser($this->attendee->user_id);
			$attendeeName = $user->name." (".$user->username.")";
		}
		else {
			$attendeeName = $this->attendee->email_address;
		}
		JToolBarHelper::title( Text::_( 'RSVP_TRANSACTIONS' )." : ".$this->event->title()." :: ".$attendeeName  , 'jevents' );
		JToolBarHelper::save( 'attendees.savetransaction');
		JToolBarHelper::cancel('attendees.transactions');

		RsvpproHelper::addSubmenu();


		$this->showToolBar();

		$params = ComponentHelper::getParams(RSVP_COM_COMPONENT);


	}

	function showToolBar(){
		$mainframe = Factory::getApplication();
		if (Factory::getApplication()->input->getCmd("tmpl","")=="component" || !$mainframe->isClient('administrator')){
			?>
		<div class='jevrsvppro'>
				<div id="toolbar-box" >
					<div class="t">
					<div class="t">
						<div class="t"></div>
					</div>
				</div>
				<div class="m">
				<?php
				$bar =  JToolBar::getInstance('toolbar');
				$barhtml = $bar->render();
				$barhtml = preg_replace('/onclick="(.*)" /','onclick="$1;return false;" ',$barhtml);
				echo $barhtml;
				if (version_compare(JVERSION, "3.0", 'ge')){
					$title = Factory::getApplication()->JComponentTitle;
				}
				else {
					$title = Factory::getApplication()->get('JComponentTitle');
				}
				echo $title;
				?>
				<div class="clr"></div>
				</div>
				<div class="b">
					<div class="b">
						<div class="b"></div>
					</div>
				</div>
			</div>
		</div>
		<?php
		}
		// Kepri doesn't load icons etc. when using tmpl=component - but we want them!
		if (Factory::getApplication()->input->getCmd("tmpl","")=="component" && $mainframe->isClient('administrator')){
			HTMLHelper::stylesheet( 'administrator/templates/'.$mainframe->getTemplate().'/css/template.css' );

		}
	}

}
