<?php

/**
 * @version    3.7.4
 * @package    COM_RSVPPRO
 * @author     Tony Partridge <tony@jevents.net>
 * @copyright  2018 - 2021 GWE Systems Ltd
 * @license    GNU General Public License version 2 or later; see LICENSE.txt
 */
// No direct access
defined('_JEXEC') or die;

use Joomla\CMS\MVC\View\HtmlView;
use Joomla\CMS\Language\Text;

jimport('joomla.application.component.view');

/**
 * View class for a list of Rsvppro.
 *
 * @since  1.6
 */
class RsvpproViewEmailinvitees extends HtmlView
{
	protected $items;

	protected $pagination;

	protected $state;

	/**
	 * Display the view
	 *
	 * @param   string  $tpl  Template name
	 *
	 * @return void
	 *
	 * @throws Exception
	 */
	public function display($tpl = null)
	{
		$this->state = $this->get('State');
		$this->items = $this->get('Items');
		$this->pagination = $this->get('Pagination');
        $this->filterForm = $this->get('FilterForm');
        $this->activeFilters = $this->get('ActiveFilters');

		// Check for errors.
		if (count($errors = $this->get('Errors')))
		{
			throw new Exception(implode("\n", $errors));
		}

		RsvpproHelper::addSubmenu('emailinvitees');

		$this->addToolbar();

		parent::display($tpl);
	}

	/**
	 * Add the page title and toolbar.
	 *
	 * @return void
	 *
	 * @since    1.6
	 */
	protected function addToolbar()
	{
		$state = $this->get('State');
		$canDo = RsvpproHelper::getActions();

		JToolBarHelper::title(Text::_('COM_RSVPPRO_TITLE_EMAILINVITEES'), 'emailinvitees.png');


		if ($canDo->get('core.admin'))
		{
			JToolBarHelper::deleteList('', 'emailinvitees.delete', 'JTOOLBAR_DELETE');

			JToolBarHelper::deleteList('JEV_RSVPPRO_PURGE_MSG', 'emailinvitees.purge', 'JEV_RSVPPRO_PURGE_LBL');

			JToolBarHelper::preferences('com_rsvppro');
		}

	}

	/**
	 * Method to order fields
	 *
	 * @return void
	 */
	protected function getSortFields()
	{
		return array(
			'a.`id`' => Text::_('JGRID_HEADING_ID'),
			'a.`user_id`' => Text::_('COM_RSVPPRO_EMAILINVITEES_USER_ID'),
			'a.`email_address`' => Text::_('COM_RSVPPRO_EMAILINVITEES_EMAIL_ADDRESS'),
			'a.`email_name`' => Text::_('COM_RSVPPRO_EMAILINVITEES_EMAIL_NAME'),
			'a.`sentmessage`' => Text::_('COM_RSVPPRO_EMAILINVITEES_SENTMESSAGE'),
			'a.`viewedevent`' => Text::_('COM_RSVPPRO_EMAILINVITEES_VIEWEDEVENT'),
			'a.`invitedate`' => Text::_('COM_RSVPPRO_EMAILINVITEES_INVITEDATE'),
			'a.`name`' => Text::_('COM_RSVPPRO_EMAILINVITEES_NAME'),
		);
	}

    /**
     * Check if state is set
     *
     * @param   mixed  $state  State
     *
     * @return bool
     */
    public function getState($state)
    {
        return isset($this->state->{$state}) ? $this->state->{$state} : false;
    }
}
