<?php

/**
 * @version    3.7.4
 * @package    COM_RSVPPRO
 * @author     Tony Partridge <tony@jevents.net>
 * @copyright  2018 Tony Partridge - GWE Systems Ltd
 * @license    GNU General Public License version 2 or later; see LICENSE.txt
 */
// No direct access
defined('_JEXEC') or die;

use Joomla\CMS\MVC\View\HtmlView;
use Joomla\CMS\Language\Text;

jimport('joomla.application.component.view');

/**
 * View class for a list of Rsvppro.
 *
 * @since  1.6
 */
class RsvpproViewEmailreminders extends HtmlView
{
	protected $items;

	protected $pagination;

	protected $state;

	/**
	 * Display the view
	 *
	 * @param   string  $tpl  Template name
	 *
	 * @return void
	 *
	 * @throws Exception
	 */
	public function display($tpl = null)
	{
		$this->state = $this->get('State');
		$this->items = $this->get('Items');
		$this->pagination = $this->get('Pagination');
        $this->filterForm = $this->get('FilterForm');
        $this->activeFilters = $this->get('ActiveFilters');

		// Check for errors.
		if (count($errors = $this->get('Errors')))
		{
			throw new Exception(implode("\n", $errors));
		}

		RsvpproHelper::addSubmenu('emailreminders');

		$this->addToolbar();

		$this->sidebar = JHtmlSidebar::render();
		parent::display($tpl);
	}

	/**
	 * Add the page title and toolbar.
	 *
	 * @return void
	 *
	 * @since    1.6
	 */
	protected function addToolbar()
	{
		$canDo = RsvpproHelper::getActions();
		JToolBarHelper::title(Text::_('COM_RSVPPRO_TITLE_EMAILREMINDERS'), 'emailreminders.png');

		// Check if the form exists before showing the add/edit buttons
		$formPath = JPATH_COMPONENT_ADMINISTRATOR . '/views/emailreminder';

		if ($canDo->get('core.admin'))
		{
			JToolBarHelper::deleteList('JTOOLBAR_DELETE', 'emailreminders.delete', 'JTOOLBAR_DELETE');

			JToolBarHelper::deleteList('JEV_RSVPPRO_PURGE_MSG', 'emailreminders.purge', 'JEV_RSVPPRO_PURGE_LBL');

			JToolBarHelper::preferences('com_rsvppro');
		}

		// Set sidebar action - New in 3.0
		JHtmlSidebar::setAction('index.php?option=com_rsvppro&view=emailreminders');
	}

	/**
	 * Method to order fields
	 *
	 * @return void
	 */
	protected function getSortFields()
	{
		return array(
			'a.`id`' => Text::_('JGRID_HEADING_ID'),
			'a.`name`' => Text::_('COM_RSVPPRO_EMAILREMINDERS_NAME'),
			'a.`user_id`' => Text::_('COM_RSVPPRO_EMAILREMINDERS_USER_ID'),
			'a.`sentmessage`' => Text::_('COM_RSVPPRO_EMAILREMINDERS_SENTMESSAGE'),
			'a.`sentdate`' => Text::_('COM_RSVPPRO_EMAILREMINDERS_SENTDATE'),
			'a.`email_name`' => Text::_('COM_RSVPPRO_EMAILREMINDERS_EMAIL_NAME'),
			'a.`email_address`' => Text::_('COM_RSVPPRO_EMAILREMINDERS_EMAIL_ADDRESS'),
		);
	}
}
