<?php

/**
 * JEvents Component for Joomla 1.5.x
 *
 * @version     $Id$
 * @package     JEvents
 * @copyright   Copyright (C) 2008-2021 GWE Systems Ltd
 * @license     GNU/GPLv2, see http://www.gnu.org/licenses/gpl-2.0.html
 * @link        https://www.jevents.net
 */
defined('JPATH_BASE') or die('Direct Access to this location is not allowed.');

use Joomla\CMS\Language\Text;
use Joomla\CMS\Component\ComponentHelper;
use Joomla\CMS\Plugin\PluginHelper;
use Joomla\CMS\Filter\InputFilter;
use Joomla\CMS\Router\Route;
use Joomla\CMS\Uri\Uri;
use Joomla\CMS\Session\Session;
use Joomla\CMS\User\User;
use Joomla\CMS\User\UserHelper;
use Joomla\CMS\Factory;
use Joomla\CMS\Input\Input;
use Joomla\CMS\Filesystem\File;

include_once(RSVP_ADMINPATH . "/controllers/attendees.php");

/**
 * Attendstate explanations
 * 0 = not attending
 * 1 = attending
 * 2 = maybe attending
 * 3 = attending but subject to approval i.e.pending
 * 4 = attending but has outstanding balance to pay
 */
class FrontAttendeesController extends AdminAttendeesController
{

	function __construct($config = array())
	{
		parent::__construct($config);

		$this->registerTask('record', 'recordAttendance');
		$this->registerTask('pdfticket', 'ticket');
		$this->registerTask('htmlticket', 'ticket');

		// Load admin language file
		$lang = Factory::getLanguage();
		$lang->load(RSVP_COM_COMPONENT, JPATH_ADMINISTRATOR);

		JLoader::register('JEventDate', JEV_PATH . "/libraries/jeventdate.php");

	/*
	 * All done in parent constructor
		jimport('joomla.filesystem.file');
		if (File::exists(JPATH_SITE . '/components/com_community/community.php'))
		{
			if (ComponentHelper::isEnabled("com_community"))
			{
				$this->jomsocial = true;
			}
		}

		$this->params = ComponentHelper::getParams("com_rsvppro");

		JLoader::register('JevRsvpReminders', JPATH_ADMINISTRATOR . "/components/com_rsvppro/libraries/jevrreminders.php");
		$this->jevrreminders = new JevRsvpReminders($this->params, $this->jomsocial);

		include_once(JPATH_ADMINISTRATOR . "/components/com_rsvppro/libraries/attendeehelper.php");
		$this->helper = new RsvpAttendeeHelper($this->params);
	 */

	}

	// This redirects calls to the helper class is possible
	public function __call($name,  $arguments)
	{
		if (isset($this->event) && !isset($this->helper->event)){
			$this->helper->event = $this->event;
		}
		if (is_callable(array($this->helper, $name)))
		{
			return call_user_func_array(array($this->helper, $name),  $arguments);
		}

	}

	function overview()
	{
		$user = Factory::getUser();
		$params = ComponentHelper::getParams("com_rsvppro");
		if ($user->id == 0 && !$params->get("showtoanon",0))
			return;


		if (! JEVHelper::isAdminUser($user) )
		{
			$atd_id = Factory::getApplication()->input->get("atd_id", array(), 'ARRAY');
			if (!isset($atd_id[0]) || strpos($atd_id[0], "|") === false)
			{
				throw new Exception(Text::_("RSVP_MISSING_ATDID"), 403);
			}

			list($atd_id, $rp_id) = explode("|", $atd_id[0]);
			$db = Factory::getDBO();
			$sql = "SELECT * FROM #__jev_attendance WHERE id=" . intval($atd_id);
			$db->setQuery($sql);
			$rsvpdata = $db->loadObject();
			if (!$rsvpdata)
			{
				throw new Exception(Text::_("RSVP_MISSING_ATDID"), 403);
			}

			if (!$rsvpdata->showattendees)
			{
				$vevent = $this->dataModel->queryModel->getEventById($rsvpdata->ev_id, false, "icaldb");
				if (!$vevent || !($vevent->created_by() == intval($user->id) || JEVHelper::canEditEvent($vevent)  || RsvpHelper::canManageSessionAttendees(false, $vevent)))
				{
					return "";
				}
			}
		}


		return parent::overview();

	}

	function export()
	{
		$user = Factory::getUser();
		if ($user->id == 0)
			return;

		if (true || ! JEVHelper::isAdminUser($user))
		{
			$atd_id = Factory::getApplication()->input->get("atd_id", array(), 'ARRAY');
			if (!isset($atd_id[0]) || strpos($atd_id[0], "|") === false)
			{
				throw new Exception(Text::_("RSVP_MISSING_ATDID"), 403);
			}

			list($atd_id, $rp_id) = explode("|", $atd_id[0]);
			$db = Factory::getDBO();
			$sql = "SELECT * FROM #__jev_attendance WHERE id=" . intval($atd_id);
			$db->setQuery($sql);
			$rsvpdata = $db->loadObject();
			if (!$rsvpdata)
			{
				throw new Exception(Text::_("RSVP_MISSING_ATDID"), 403);
			}

			$vevent = $this->dataModel->queryModel->getEventById($rsvpdata->ev_id, false, "icaldb");
			if (JEVHelper::canEditEvent($vevent)){
				parent::export();
			}
		}

	}

	function confirm()
	{
		return parent::confirm();
	}

	function remindconfirm()
	{
		return parent::remindconfirm();

	}

	function approve()
	{
		return parent::approve();

	}

	function delete()
	{
		return parent::delete();
	}

	function edit()
	{
		return parent::edit();
	}

	function save()
	{
		return parent::save();

	}

	function ticket()
	{
		return parent::ticket();
	}

	public function recordAttendance()
	{
		$mainframe = Factory::getApplication();

		// special case where we are confirming an attendance but are logged in as a separate user - it happens mainly during testing but it is a pain
		$em1 = $this->getEmailAddress('em');
		$em2 = $this->getEmailAddress('em2');
		if (!empty($em) || !empty($em2) )
		{
			$user = Factory::getUser(0);
		}
		else {
			$user = Factory::getUser();
		}

		// I may also need to trigger a plugin to say that a registration has been made
		PluginHelper::importPlugin("rsvppro");

		// Check for request forgeries
		//Session::checkToken('request') or jexit( 'Invalid Token' );

		$db = Factory::getDBO();
		$at_id = Factory::getApplication()->input->getInt("at_id", -1);
		$sql = "SELECT * FROM #__jev_attendance WHERE id=" . $at_id;
		$db->setQuery($sql);
		$rsvpdata = $db->loadObject();
		if (!$rsvpdata)
			return false;

		if ($user->id == 0 && !RsvpHelper::getTemplateSpecificParameter($rsvpdata, "attendemails", 0))
		{
			return false;
		}

		$rp_id = Factory::getApplication()->input->getInt("rp_id", -1);
		$datamodel = new JEventsDataModel();
		$row = $datamodel->queryModel->listEventsById($rp_id, 1, "icaldb");
		if (!$row)
			return false;

		$this->event = $row;

		// Do we need to register this visitor in Joomla as a user
		$createNewAccount = false;
		$rsvpdata->newid = 0;
		if ($user->id == 0 && RsvpHelper::getTemplateSpecificParameter($rsvpdata, "attendemails", 0) && RsvpHelper::getTemplateSpecificParameter($rsvpdata, "autoregister", 0))
		{
			$user->newid  = $this->createUser($row, $rsvpdata);
			// useful place for holding this information that can then be used in plugins
			$rsvpdata->newid = $user->newid;
			$createNewAccount = true;
		}

		$cache =  Factory::getCache('com_jevents');
		$cache->clean();

		// Add reference to current row and rsvpdata to the registry so that we have access to these in the fields
		$registry = JevRegistry::getinstance("jevents");
		$registry->set("rsvpdata", $rsvpdata);
		$registry->set("event", $row);

		$db = Factory::getDBO();

		$this->countAttendees($rsvpdata->id);
		$guestcount = Factory::getApplication()->input->getInt("guestcount", 1);

		$sql = "SELECT atdcount FROM #__jev_attendeecount as a WHERE a.at_id=" . $rsvpdata->id;
		if (!$rsvpdata->allrepeats)
		{
			$sql .= " and a.rp_id=" . $row->rp_id();
		}
		$db->setQuery($sql);
		$attendeeCount = $db->loadResult();

		// Hygene trap
		$jevattend_hidden = Factory::getApplication()->input->getInt("jevattend_hidden", 0);
		if ($jevattend_hidden)
		{

			// Is this an admin user cancelling an attendance record
			//$this->adminCancelAttendance($rsvpdata, $row);

			// Is the admin approving an attendance record
			//$this->adminApproveAttendance($rsvpdata, $row);

			// Is this an existing attendee
			// if anon user and email attendance is allowed then find accordingly
			if ($user->id == 0 && RsvpHelper::getTemplateSpecificParameter($rsvpdata, "attendemails", 0))
			{
				$emailaddress = $this->getEmailAddress();
				// Make sure can only cancel or modify their own email address!!!
				if ($emailaddress == "" || $emailaddress != trim(strtolower(Factory::getApplication()->input->getString("jevattend_email"))))
				{
					$attendee = false;

					$emailaddress = trim(strtolower(Factory::getApplication()->input->getString("jevattend_email")));

					// block attempts to double book the same email
					if ($emailaddress != "" && filter_var($emailaddress, FILTER_VALIDATE_EMAIL))
					{
						$emailaddress = trim(strtolower(Factory::getApplication()->input->getString("jevattend_email")));
						// Check in case this email address is associated with a Joomla user account

						$db = Factory::getDbo();
						$query = $db->getQuery(true);
						$query->select('*');
						$query->from($db->quoteName('#__users'));
						$query->where($db->quoteName('email') . ' = ' . $db->quote($emailaddress));
						$db->setQuery($query, 0, 1);
						$testuser = $db->loadObject();
						if ($testuser && !$createNewAccount && RsvpHelper::getTemplateSpecificParameter($rsvpdata, "attendemailscheck", 1)){
							$Itemid = Factory::getApplication()->input->getInt("Itemid");
							list($year, $month, $day) = JEVHelper::getYMD();
							$link = Route::_($row->viewDetailLink($year, $month, $day, false, $Itemid));
							if ($testuser->activation ){
								Factory::getApplication()->enqueueMessage(Text::_("JEV_INACTIVE_USER_ACCOUNT_EXISTS_PLEASE_ACTIVATE"),"info");
								Factory::getApplication()->redirect($link);
							}
							else {
								Factory::getApplication()->enqueueMessage(Text::_("JEV_USER_ACCOUNT_EXISTS_PLEASE_LOGIN"),"info");
								Factory::getApplication()->redirect($link);
							}
						}

						// check  if we already have this attendee
						$sql = "SELECT * FROM #__jev_attendees WHERE at_id=" . $rsvpdata->id . " and LOWER(email_address)=" . $db->Quote($emailaddress);
						if (!$rsvpdata->allrepeats)
						{
							$sql .= " AND rp_id=" . $row->rp_id();
						}
						$db->setQuery($sql);
						$testattendee = $db->loadObject();

						// if we allowunconfirmedpendingunpaidchanges then we remove this attendee to create another but only if unconfirmed and confirmation IS required
						if ($testattendee)
						{
							if (!isset($testattendee->outstandingBalances))
							{
								// This MUST be called before renderToBasicArray to populate the balance fields - so we do it here to be safe
								if (isset($testattendee->params))
								{
									$xmlfile = JevTemplateHelper::getTemplate($rsvpdata);

									if (is_int($xmlfile) || file_exists($xmlfile))
									{
										$params = new JevRsvpParameter($testattendee->params, $xmlfile, $rsvpdata, null);
										$feesAndBalances = $params->outstandingBalance($testattendee);
									}
								}
							}

							if (RsvpHelper::getTemplateSpecificParameter($rsvpdata, "allowunconfirmedpendingunpaidchanges",0) && !$testattendee->confirmed && (
									(RsvpHelper::getTemplateSpecificParameter($rsvpdata, "requireconfirmationpaid",0) && isset(  $testattendee->outstandingBalances) &&  $testattendee->outstandingBalances['hasfees'] && $testattendee->outstandingBalances['feepaid'] ==0 )
									|| (RsvpHelper::getTemplateSpecificParameter($rsvpdata, "requireconfirmation",0) && (!isset(  $testattendee->outstandingBalances) || !$testattendee->outstandingBalances['hasfees'] )))
								){

								$testattendee = null;
								$sql = "DELETE FROM #__jev_attendees WHERE at_id=" . $rsvpdata->id . " and LOWER(email_address)=" . $db->Quote($emailaddress);

								if (!$rsvpdata->allrepeats)
								{
									$sql .= " AND rp_id=" . $row->rp_id();
								}
								$db->setQuery($sql);
								$db->execute();
							}
                            else {
                                $Itemid = Factory::getApplication()->input->getInt("Itemid");
                                list($year, $month, $day) = JEVHelper::getYMD();
                                $link = Route::_($row->viewDetailLink($year, $month, $day, false, $Itemid), false);
	                            if (!$testattendee->confirmed && RsvpHelper::getTemplateSpecificParameter($rsvpdata, "requireconfirmation",0))
	                            {
		                            $code = base64_encode($emailaddress . ":" . md5(RsvpHelper::getTemplateSpecificParameter($rsvpdata, "emailkey", "email key") . $emailaddress));

		                            // re-send email with confirmation details
		                            $creator = JEVHelper::getUser($row->created_by());
		                            if ($creator)
		                            {
			                            $link = Route::_($row->viewDetailLink($year, $month, $day, false, $Itemid) . "&em=" . $code, false);
			                            // some sef addons have a but that adds the root already
			                            if (strpos($link, "/") !== 0 && strpos($link, 'http://') !== 0)
			                            {
				                            $link = "/" . $link;
			                            }

			                            $uri  = JURI::getInstance(JURI::base());
			                            $root = $uri->toString(array('scheme', 'host', 'port'));
			                            if (strpos($link, 'http://') !== 0)
			                            {
				                            $link = $root . $link;
			                            }

			                            // confirm email message
			                            $this->notifyUser($rsvpdata, $this->event, $user, $emailaddress, $emailaddress, $testattendee, 'cem', $testattendee->waiting, "", "", $link);
		                            }
	                            }
	                            else
	                            {
		                            $this->notifyUser($rsvpdata, $row, $user, $testattendee->email_address, $testattendee->email_address, $testattendee, 'ack', $testattendee->waiting);
	                            }
                                $mainframe->enqueueMessage(Text::_("JEV_ATTENDANCE_CANNOT_REGISTER_A_SECOND_TIME_USING_THE_SAME_EMAIL_ADDRESS"),"warning");
                                $mainframe->redirect($link);
                            }
						}

						if ($testattendee && RsvpHelper::getTemplateSpecificParameter($rsvpdata, "requireconfirmation",0))
						{
							$Itemid = Factory::getApplication()->input->getInt("Itemid");
							list($year, $month, $day) = JEVHelper::getYMD();

							$code = base64_encode($emailaddress . ":" . md5(RsvpHelper::getTemplateSpecificParameter($rsvpdata, "emailkey", "email key") . $emailaddress));

							// re-send email with confirmation details
							$creator = JEVHelper::getUser($row->created_by());
							if ($creator)
							{
								$link = Route::_($row->viewDetailLink($year, $month, $day, false, $Itemid) . "&em=" . $code, false);
								// some sef addons have a but that adds the root already
								if (strpos($link, "/") !== 0 && strpos($link, 'http://') !== 0)
								{
									$link = "/" . $link;
								}

								$uri =  JURI::getInstance(JURI::base());
								$root = $uri->toString(array('scheme', 'host', 'port'));
								if (strpos($link, 'http://') !== 0){
									$link = $root . $link;
								}

								// confirm email message
								$this->notifyUser($rsvpdata, $this->event, $user, $emailaddress, $emailaddress, $testattendee,'cem', $testattendee->waiting, "","",$link);
								/*
								if ($testattendee->confirmed){
									$subject = Text::sprintf("JEV_CONFIRM_EMAIL_ATTEND_SUBJECT", $row->title());
									$message = Text::sprintf("JEV_CONFIRM_EMAIL_ATTEND", $link, $row->title(), $row->title());
								}
								else {
									// TODO offer a different message here!
									$subject = Text::sprintf("JEV_CONFIRM_EMAIL_ATTEND_SUBJECT", $row->title());
									$message = Text::sprintf("JEV_CONFIRM_EMAIL_ATTEND", $link, $row->title(), $row->title());
								}

								$this->helper->sendMail($creator->email, $creator->name, $emailaddress, $subject, $message, 1);
								 */
							}

							// Must remove the authentication code otherwise there is no point!
							$link = Route::_($row->viewDetailLink($year, $month, $day, false, $Itemid));

							$mainframe->enqueueMessage(Text::_("JEV_ATTENDANCE_MISSING_AUTHENTICATION_CODE"));
							Factory::getApplication()->enqueueMessage(Text::_("JEV_ATTENDANCE_AUTHENTICATION_CODE_RESENT"),"info");
							$mainframe->redirect($link);
						}
					}
					else {
						if ($emailaddress != "" && !filter_var($emailaddress, FILTER_VALIDATE_EMAIL)){
							$Itemid = Factory::getApplication()->input->getInt("Itemid");
							list($year, $month, $day) = JEVHelper::getYMD();
							$link = Route::_($row->viewDetailLink($year, $month, $day, false, $Itemid));
							Factory::getApplication()->enqueueMessage(Text::_("JEV_INVALID_EMAIL"),"warning");
							Factory::getApplication()->redirect($link);
						}
						else {
							$Itemid = Factory::getApplication()->input->getInt("Itemid");
							list($year, $month, $day) = JEVHelper::getYMD();
							$link = Route::_($row->viewDetailLink($year, $month, $day, false, $Itemid));
							Factory::getApplication()->enqueueMessage(Text::_("JEV_MISSING_EMAIL"),"warning");
							Factory::getApplication()->redirect($link);
						}
					}
				}
				else
				{
					// fetch attendee for auto remind
					$sql = "SELECT * FROM #__jev_attendees WHERE at_id=" . $rsvpdata->id . " and email_address=" . $db->Quote($emailaddress);
					if (!$rsvpdata->allrepeats)
					{
						$sql .= " AND rp_id=" . $row->rp_id();
					}
					$db->setQuery($sql);
					$attendee = $db->loadObject();
				}
			}
			else
			{
				// fetch attendee for auto remind
				$sql = "SELECT * FROM #__jev_attendees WHERE at_id=" . $rsvpdata->id . " and user_id=" . $user->id;
				if (!$rsvpdata->allrepeats)
				{
					$sql .= " AND rp_id=" . $row->rp_id();
				}
				$db->setQuery($sql);
				$attendee = $db->loadObject();
			}

			$jevattend = Factory::getApplication()->input->getInt("jevattend", 0);

			// if template has fees then no need to confirm the email attendee if requireconfirmationpaid is false
			$templateHasFees = $this->templateHasFees($rsvpdata, $row);
			if ($templateHasFees && RsvpHelper::getTemplateSpecificParameter($rsvpdata, "requireconfirmation", 0) && !RsvpHelper::getTemplateSpecificParameter($rsvpdata, "requireconfirmationpaid", 0)  && $user->id == 0 && RsvpHelper::getTemplateSpecificParameter($rsvpdata, "attendemails", 0)){
				RsvpHelper::setTemplateSpecificParameter($rsvpdata, "requireconfirmation", 0 );
				// must also set em in the URL!
				$code = base64_encode($emailaddress . ":" . md5(RsvpHelper::getTemplateSpecificParameter($rsvpdata, "emailkey", "email key") . $emailaddress));
				Factory::getApplication()->input->set("em",$code);
			}
			else if (isset($emailaddress)) {
				$code = base64_encode($emailaddress . ":" . md5(RsvpHelper::getTemplateSpecificParameter($rsvpdata, "emailkey", "email key") . $emailaddress. "invited"));
				if (Factory::getApplication()->input->getString("em2","")==$code){
					RsvpHelper::setTemplateSpecificParameter($rsvpdata,"requireconfirmation", 0 );
					$this->params->set("requireconfirmation", 0 );
					Factory::getApplication()->input->set("em",$code);
				}
			}

			// Scenarios that matter
			// x 1. No attendee record - saying no
			// x 2. No attendee record - saying maybe
			// x 3. No attendee record - saying subject to approval
			// x 4. No attendee record - saying yes
			// x 5. Existing attendee record was yes - now no
			// x 6. Existing attendee record was yes - now maybe
			// x 7. Existing attendee record was yes - now subject to approval
			// x 8. Existing attendee record was yes - now yes (params changed)
			// x 9. Existing attendee record was no - now maybe
			// x 10. Existing attendee record was no - now yes
			// x 11. Existing attendee record was no - now subject to approval
			// x 12. Existing attendee record was maybe - now yes
			// x 13. Existing attendee record was maybe - now no
			// x 14. Existing attendee record was maybe - now subject to approval
			// x 15. Existing attendee record was maybe - now maybe  (params changed)
			// x 16. Existing attendee record was subject to approval - now no
			// x 17. Existing attendee record was subject to approval - now maybe
			// x 18. Existing attendee record was subject to approval - now yes
			// x 19. Existing attendee record was subject to approval - now subject to approval  (params changed)
			// x 20. Existing attendee record was no - now no (params changed)

			// Trap recaptcha if any and check coupons
			$rsvpparams = $this->getRSVPParmeters($rsvpdata, $row);
			if ($rsvpparams ) {
				$rsvpparamsarray = $rsvpparams->renderToBasicArray();
				// find the paymentoptionlist
				foreach ($rsvpparamsarray as $pield => $pm)
				{
					// only use the recaptcha when signing up for a registration and not for cancelling which has other traps built in
					if (($pm["type"] == "jevrecaptcha" || $pm["type"] == "jevrecaptcha_v2" ) && $jevattend)
					{
						if (isset($attendee) && isset($attendee->user_id) && $attendee->user_id==0 &&  $attendee->confirmed){
							continue;
						}

						// check access
						$accessible = true;
						if (isset($pm["accessflag"])){
							$levels = $user->getAuthorisedViewLevels();
							$nodeaccess = explode(",", $pm["access"]);
							if (count(array_intersect($levels, $nodeaccess)) == 0)
							{
								$accessible = false;
							}

							// if access flag is 0 then members of this level are BLOCKED
							if ($pm["accessflag"]==0){
								$accessible = !$accessible;
							}
						}
						if (!$accessible){
							break;
						}

						// Version 1 API
						if ($pm["type"] == "jevrecaptcha"){
							if (Factory::getApplication()->input->getString("secretcaptcha") == md5(Factory::getApplication()->input->getString("recaptcha_response_field") . RsvpHelper::getTemplateSpecificParameter($rsvpdata, "recaptchaprivate",false) )){
								$x = 1;
							}
							// otherwise fall back in case browser didn't support javascript
							else {
								if (!defined("RECAPTCHA_API_SERVER"))	require_once(JPATH_SITE.'/plugins/jevents/jevrsvppro/rsvppro/recaptcha/recaptcha.php');
								$response = recaptcha_check_answer(RsvpHelper::getTemplateSpecificParameter($rsvpdata, "recaptchaprivate",false),JFactory::getApplication()->input->getString("REMOTE_ADDR","","server"), JFactory::getApplication()->input->getString("recaptcha_challenge_field"),JFactory::getApplication()->input->getString("recaptcha_response_field"),Factory::getApplication()->input->getString("recaptcha_response_field"));
								if (!$response || !$response->is_valid){

									echo "<script> alert('".Text::_("JEV_RECAPTCHA_ERROR",true)."'); window.history.go(-1); </script>\n";
									exit();
								}
							}
						}

						// Version 2 API
						else {
							// Did we set the secret captcha field correctly i.e. passed the javascript test
							if (Factory::getApplication()->input->getString("g-recaptcha-response") && Factory::getApplication()->input->getString("secretcaptcha") == md5(Factory::getApplication()->input->getString("g-recaptcha-response") . RsvpHelper::getTemplateSpecificParameter($rsvpdata, "recaptchaprivate",false) ) )
							{
								$x = 1;
							}
							else
							{
								echo "<script> alert('".Text::_("JEV_RECAPTCHA_ERROR",true)."'); window.history.go(-1); </script>\n";
								exit();
							}
						}
					}

					// Can this attendee use any coupons attached to the form record
					if ($pm["type"] == "jevrcoupon"){
						$fieldname = $pm["name"];
						// must trim the input
                        if (isset($_REQUEST["xmlfile"][$fieldname])){
                            $_REQUEST["xmlfile"][$fieldname]=trim($_REQUEST["xmlfile"][$fieldname]);
                            $_POST["xmlfile"][$fieldname]=trim($_POST["xmlfile"][$fieldname]);
                        }

						if (isset($pm["maxuses"]) && $pm["maxuses"] > 0)
						{
							$sql = "SELECT * FROM #__jev_rsvp_couponusage  where atd_id=" . intval($at_id);
							if (!$rsvpdata->allrepeats){
								$sql .= " AND rp_id=".intval($rp_id);
							}
							$db->setQuery($sql);
							$couponusage = $db->loadObject();
							if ($couponusage){
								$couponparams = json_decode($couponusage->params);
								// submitted coupon code
								$inparams = Factory::getApplication()->input->get("xmlfile",array(), 'ARRAY');
								if (!isset($inparams[$fieldname])){
									break;
								}
								$couponcode = trim($inparams[$fieldname]);
                                                                $maxuses = $pm["maxuses"];

								// is this a valid coupon code and could we be at the max use limit!
								if (isset($couponparams->$fieldname) && isset($couponparams->$fieldname->$couponcode) && intval($couponparams->$fieldname->$couponcode)+1 > $maxuses ){
									$canusecoupon = false;
									// make sure we are not already using this same coupon
									if ($attendee){
										$attendeeparams = json_decode($attendee->params);
										if (isset($attendeeparams->$fieldname) && trim($attendeeparams->$fieldname)==$couponcode){
											$canusecoupon	= true;
											//throwerror("Already using coupon");
										}
									}
									if (!$canusecoupon) {
										if (isset($_REQUEST["xmlfile"][$fieldname])){
											// Can't uyse JRequest setVar here because its an array!
											$_REQUEST["xmlfile"][$fieldname]="";
											$_POST["xmlfile"][$fieldname]="";
											// Important must also reset $params
											/*
											if (isset($params)){
												$jparams = json_decode($params);
												$decodeparams->$fieldname="";
												$params =  json_encode($decodeparams);
											}
											 */
										}
										// Load language
										$lang = Factory::getLanguage();
										$lang->load("plg_jevents_jevrsvppro", JPATH_ADMINISTRATOR);

										Factory::getApplication()->enqueueMessage(Text::_("RSVP_COUPON_ALREADY_USED"), "warning");
									}
								}
								//Text::

							}
						}
					}


				}
			}

			// Need the params for most of these scenarios
			$params = $this->getRsvpDataParams($rsvpdata);

			// I need the redirect link often
			$Itemid = Factory::getApplication()->input->getInt("Itemid");
			list ( $year, $month, $day ) = JEVHelper::getYMD();

			// Need the creation date for these records
			if (class_exists("JevDate")) {
				$date = JevDate::getDate("+0 seconds");
				$created = $date->toMySQL();
			}
			else {
				$date = Factory::getDate("+0 seconds");
				$created = $date->toSql();
			}

			Factory::getApplication()->triggerEvent( 'onSubmitRegistration' , array(&$attendee, $this->event, $rsvpdata));

			// Scenarios 1-4. No attendee Record
			if (!$attendee)
			{
				// 1. No attendee record - saying no
				// 2. No attendee record - saying maybe
				// 3. No attendee record - saying subject to approval


				if ($user->id == 0 && RsvpHelper::getTemplateSpecificParameter($rsvpdata, "attendemails", 0))
				{
					// in this case we have no email address to must set the variable
					$emailaddress = trim(strtolower(Factory::getApplication()->input->getString("jevattend_email")));
				}

				if ($jevattend == 0 || $jevattend == 2 || $jevattend == 3)
				{
					// Here we record the decision with no checks on waiting lists etc. but do require email confirmation


					$onWaitingList = false;
                                        $tempattendee = new stdClass();
                                        $tempattendee->attendstate = -1;
					$tempattendee->guestcount = 0;

					// add true to force count for new attendee record
                                        $this->countAttendeeContributionToAttendance($row, $rsvpdata, $tempattendee, true);

					if ($user->id == 0 && RsvpHelper::getTemplateSpecificParameter($rsvpdata, "attendemails", 0))
					{
						$confirmed = RsvpHelper::getTemplateSpecificParameter($rsvpdata, "requireconfirmation", 0)?0:1;
						$sql = "INSERT INTO #__jev_attendees SET at_id=" . $rsvpdata->id . ", confirmed=$confirmed, email_address=" . $db->Quote($emailaddress) . ", created=" . $db->Quote($created). ", modified=" . $db->Quote($created);
						$name = $emailaddress;
						$username = $emailaddress;
					}
					else
					{
						$sql = "INSERT INTO #__jev_attendees SET at_id=" . $rsvpdata->id . ", user_id=" . $user->id . ", created=" . $db->Quote($created). ", modified=" . $db->Quote($created);
						$name = $user->name;
						$username = $user->username;
					}
					if ($onWaitingList)
					{
						$sql .= " , waiting = 1";
					}
					if (!$rsvpdata->allrepeats)
					{
						$sql .= ", rp_id=" . $row->rp_id();
					}
					$sql .= ", params=" . $db->Quote($params);
					$sql .= ", atdcount=" . $row->newcnt;
					$sql .= ", guestcount=" . $guestcount;
					$sql .= ", attendstate=" . intval($jevattend);
					// And the locked template
					$sql .= ", lockedtemplate=" . intval($rsvpdata->template);
					$db->setQuery($sql);
					$db->execute();

					$insertid = $db->insertid();
					$db->setQuery("SELECT * from #__jev_attendees where id=" . intval($insertid));
					$attendee = $db->loadObject();
				} // 4. No attendee record - saying yes
				else if ($jevattend == 1)
				{
                                        $tempattendee = new stdClass();
                                        $tempattendee->attendstate = -1;
					$tempattendee->guestcount = 0;

                                        $this->countAttendeeContributionToAttendance($row, $rsvpdata, $tempattendee);

					if (RsvpHelper::getTemplateSpecificParameter($rsvpdata, "capacity", 0) && $rsvpdata->capacity > 0)
					{
						// If over capacity and waiting list then just ignore
						if ($attendeeCount + ($row->newcnt - $row->currentcnt) > $rsvpdata->capacity + $rsvpdata->waitingcapacity)
						//if ($attendeeCount >= $rsvpdata->capacity + $rsvpdata->waitingcapacity)
						{
							$link = $row->viewDetailLink($year, $month, $day, false, $Itemid);
							Factory::getApplication()->enqueueMessage(Text::_("RSVP_BOOKING_OVER_CAPACITY"),"warning");
							$mainframe->redirect($link);
						}

						// Should this be on the waiting list
						$onWaitingList = false;
						if ($attendeeCount + ($row->newcnt - $row->currentcnt) > intval($rsvpdata->capacity) && !$attendee->waiting && $row->newcnt>$row->currentcnt)
						//if ($attendeeCount >= intval($rsvpdata->capacity))
						{
							$onWaitingList = true;
						}
					}
					else
					{
						$onWaitingList = false;
					}

					if ($user->id == 0 && RsvpHelper::getTemplateSpecificParameter($rsvpdata, "attendemails", 0))
					{
						$confirmed = RsvpHelper::getTemplateSpecificParameter($rsvpdata, "requireconfirmation", 0) ? 0 : 1;
						$sql = "INSERT INTO #__jev_attendees SET at_id=" . $rsvpdata->id . ", confirmed=$confirmed, email_address=" . $db->Quote($emailaddress) . ", created=" . $db->Quote($created). ", modified=" . $db->Quote($created);
						$name = $emailaddress;
						$username = $emailaddress;
					}
					else
					{
						$sql = "INSERT INTO #__jev_attendees SET at_id=" . $rsvpdata->id . ", user_id=" . $user->id . ", created=" . $db->Quote($created). ", modified=" . $db->Quote($created);
						$name = $user->name;
						$username = $user->username;
					}
					if ($onWaitingList)
					{
						$sql .= " , waiting = 1";
					}
					if (!$rsvpdata->allrepeats)
					{
						$sql .= ", rp_id=" . $row->rp_id();
					}
					$sql .= ", params=" . $db->Quote($params);
					$sql .= ", attendstate=" . intval($jevattend);
					$sql .= ", atdcount=" . $row->newcnt;
					$sql .= ", notes=''";
					$sql .= ", attendnotes=''";
					$sql .= ", guestattend=''";
					$sql .= ", guestcount=" . $guestcount;
					// And the locked template
					$sql .= ", lockedtemplate=" . intval($rsvpdata->template);
					$db->setQuery($sql);
					$db->execute();

					$insertid = $db->insertid();
					$db->setQuery("SELECT * from #__jev_attendees where id=" . intval($insertid));
					$attendee = $db->loadObject();

					// auto remind attendees
					if (RsvpHelper::getTemplateSpecificParameter($rsvpdata, "autoremind", 0) == 1 && !$onWaitingList && ($rsvpdata->allowreminders || RsvpHelper::getTemplateSpecificParameter($rsvpdata, "forceautoremind", 0)))
					{
						// create reminder
						// NB email address must be in the request object
						Factory::getApplication()->input->set("jevremindemail", isset($emailaddress) ? $emailaddress : $attendee->email_address);
						$user = JEVHelper::getUser($attendee->user_id);
						$this->jevrreminders->remindUser($rsvpdata, $row, $user, isset($emailaddress) ? $emailaddress : $attendee->email_address);
						$mainframe->enqueueMessage(Text::_("JEV_REMINDER_CONFIRMED"));
					}

					// Record activity points
					$this->recordRsvpActivity("attendance.confirmed", $row);
				}

				// Make sure the counts are in sync!
				$this->countAttendees($rsvpdata->id);

				// Send notification/confirmation messages
				if ($user->id == 0 && RsvpHelper::getTemplateSpecificParameter($rsvpdata, "attendemails", 0))
				{
					$code = base64_encode($emailaddress . ":" . md5(RsvpHelper::getTemplateSpecificParameter($rsvpdata, "emailkey", "email key") . $emailaddress));
					$link = Route::_($row->viewDetailLink($year, $month, $day, false, $Itemid) . "&em=" . $code, false);

					if (RsvpHelper::getTemplateSpecificParameter($rsvpdata, "requireconfirmation", 0))
					{
						// send email with confirmation details
						$creator = JEVHelper::getUser($this->event->created_by());
						if ($creator)
						{
							// some sef addons have a but that adds the root already
							if (strpos($link, "/") !== 0 && strpos($link, 'http://') !== 0)
							{
								$link = "/" . $link;
							}

							$uri =  JURI::getInstance(JURI::base());
							$root = $uri->toString(array('scheme', 'host', 'port'));
							if (strpos($link, 'http://') !== 0){
								$link = $root . $link;
							}

							// confirm email message - passing special link through!
							$this->notifyUser($rsvpdata, $this->event, $user, $emailaddress, $username, $attendee,'cem', $onWaitingList,  false, "","",$link);
							//$this->notifyCreator($rsvpdata, $this->event, $emailaddress, $emailaddress, $attendee, false, $onWaitingList);

							/*
							$subject = Text::sprintf("JEV_CONFIRM_EMAIL_ATTEND_SUBJECT", $this->event->title());
							$message = Text::sprintf("JEV_CONFIRM_EMAIL_ATTEND", $link, $this->event->title(), $this->event->title());
							if ($rsvpdata->allowcancellation)
							{
								$message .= Text::_("JEV_ATTENDANCE_PENDING");
							}

							if (!$onWaitingList)
							{
								str_replace("{WAITINGMESSAGE}", "", $message);
							}
							else
							{
								str_replace("{WAITINGMESSAGE}", Text::_("JEV_WAITING_MESSAGE"), $message);
							}
							$this->helper->sendMail($creator->email, $creator->name, $emailaddress, $subject, $message, 1);
							 */
						}
						//reset the link to not have the 'code' in it!!
						$link = $row->viewDetailLink($year, $month, $day, false, $Itemid) . "&hideformpending=1";
						Factory::getApplication()->triggerEvent( 'onRegistrationStatusChange' , array(&$attendee, $this->event, $rsvpdata));

						Factory::getApplication()->enqueueMessage(Text::_("JEV_ATTENDANCE_PENDING"),"warning");
						$mainframe->redirect($link);
					}

					// reset attendance state
					$attendee->attendstate = $jevattend;
					// only send yes or subject to approval
					if ($jevattend == 1 || $jevattend == 3)
					{
						// Do not send notification if only sending cancelations!
						if (RsvpHelper::getTemplateSpecificParameter($rsvpdata, "notifycreator", 0) && RsvpHelper::getTemplateSpecificParameter($rsvpdata, "notifycreator", 0)!=3) {
							$this->notifyCreator($rsvpdata, $row, $name, $username, $attendee, false, $onWaitingList);
						}
						if (RsvpHelper::getTemplateSpecificParameter($rsvpdata, "notifyuser", 0)){
							$this->notifyUser($rsvpdata, $row, $user, $name, $username, $attendee,'ack', $onWaitingList);
						}
					}
					else if ($jevattend == 0 && RsvpHelper::getTemplateSpecificParameter($rsvpdata, "notifyadminnotattending", 0))
					{
						$this->notifyCreator($rsvpdata, $row, $name, $username, $attendee, 2 , $onWaitingList);
					}

					$this->postUpdateActions($rsvpdata, $row, $attendee,$onWaitingList, $rsvpparams);

					if ($onWaitingList && RsvpHelper::getTemplateSpecificParameter($rsvpdata, "waitingnopay",0) == 1)
					{
						Factory::getApplication()->triggerEvent( 'onRegistrationStatusChange' , array(&$attendee, $row, $rsvpdata));
						Factory::getApplication()->enqueueMessage(Text::_("JEV_ATTENDANCE_ON_WAITING_LIST"),"warning");
						$mainframe->redirect($link);
					}
					else
					{
						$message =  Text::_("JEV_ATTENDANCE_CONFIRMED");
						if ($jevattend == 3) {
							$message .="<br/>". Text::_("JEV_ATTENDING_PENDING_APPROVAL");
						}
						Factory::getApplication()->triggerEvent( 'onRegistrationStatusChange' , array(&$attendee, $row, $rsvpdata));
						if ($jevattend == 1 || $jevattend == 3) {
							$this->redirectToPaymentPage($attendee, $rsvpdata, $message);

							// special case where there is no balance outstanding due to a coupon being used etc.
							if (!RsvpHelper::getTemplateSpecificParameter($rsvpdata, "notifyuser", 0)){
								$feesAndBalances = isset($attendee->outstandingBalances) ? $attendee->outstandingBalances : false;
								if ($feesAndBalances && $feesAndBalances["feebalance"]<0.001 &&  $feesAndBalances["totalfee"]>0)
								{
									$this->notifyUser($rsvpdata, $row, $user, $name, $username, $attendee,'ack', $onWaitingList);
								}
							}
						}
						Factory::getApplication()->enqueueMessage($message);
						$mainframe->redirect($link);
					}
				}
				else
				{
					// reset attendance state
					$attendee->attendstate = $jevattend;
					// only send yes or subject to approval
					if ($jevattend == 1 || $jevattend == 3)
					{
						// Do not send notification if only sending cancelations!
						if (RsvpHelper::getTemplateSpecificParameter($rsvpdata, "notifycreator", 0) && RsvpHelper::getTemplateSpecificParameter($rsvpdata, "notifycreator", 0)!=3)
							$this->notifyCreator($rsvpdata, $row, $name, $username, $attendee, false, $onWaitingList);
						if (RsvpHelper::getTemplateSpecificParameter($rsvpdata, "notifyuser", 0))
							$this->notifyUser($rsvpdata, $row, $user, $name, $username, $attendee, 'ack', $onWaitingList);
					}
					else if ($jevattend == 0 && RsvpHelper::getTemplateSpecificParameter($rsvpdata, "notifyadminnotattending", 0))
					{
						$this->notifyCreator($rsvpdata, $row, $name, $username, $attendee, 2 , $onWaitingList);
					}

					$this->postUpdateActions($rsvpdata, $row, $attendee,$onWaitingList, $rsvpparams);

					$link = $row->viewDetailLink($year, $month, $day, false, $Itemid);
					$this->recordRsvpActivity("attendance.confirmed", $row);
					Factory::getApplication()->triggerEvent( 'onRegistrationStatusChange' , array(&$attendee, $row, $rsvpdata));

					if ($onWaitingList && RsvpHelper::getTemplateSpecificParameter($rsvpdata, "waitingnopay",0) == 1)
					{
						Factory::getApplication()->enqueueMessage(Text::_("JEV_ATTENDANCE_ON_WAITING_LIST"));
						$mainframe->redirect($link);
					}
					else
					{
						$message =  Text::_("JEV_ATTENDANCE_CONFIRMED");
						if ($jevattend == 3) {
							$message .="<br/>". Text::_("JEV_ATTENDING_PENDING_APPROVAL");
						}
						if ($jevattend == 1 || $jevattend == 3) {
							$this->redirectToPaymentPage($attendee, $rsvpdata, $message);

							// special case where there is no balance outstanding due to a coupon being used etc.
							if (!RsvpHelper::getTemplateSpecificParameter($rsvpdata, "notifyuser", 0)){
								$feesAndBalances = isset($attendee->outstandingBalances) ? $attendee->outstandingBalances : false;
								if ($feesAndBalances && $feesAndBalances["feebalance"]<0.001 &&  $feesAndBalances["totalfee"]>0)
								{
									$this->notifyUser($rsvpdata, $row, $user, $name, $username, $attendee,'ack', $onWaitingList);
								}
							}
						}
						Factory::getApplication()->enqueueMessage($message);
						$mainframe->redirect($link);
					}
				}
			} // Scenarios 5-20. Existing attendee record
			else
			{
                $onWaitingList = false;

				$this->countAttendeeContributionToAttendance($row, $rsvpdata, $attendee);

				if ($user->id == 0 && RsvpHelper::getTemplateSpecificParameter($rsvpdata, "attendemails", 0))
				{
					$code = base64_encode($emailaddress . ":" . md5(RsvpHelper::getTemplateSpecificParameter($rsvpdata, "emailkey", "email key") . $emailaddress));
					$link = Route::_($row->viewDetailLink($year, $month, $day, false, $Itemid) . "&em=" . $code);
				}
				else
				{
					$link = $row->viewDetailLink($year, $month, $day, false, $Itemid);
				}

				// 5. Existing attendee record was yes - now no
				// 6. Existing attendee record was yes - now maybe
				if (($attendee->attendstate == 1 || $attendee->attendstate == 4 ) && ($jevattend == 0 || $jevattend == 2))
				{

					// Only allow this if cancellation is allowed
					if ($rsvpdata->allowcancellation || $user->id == $row->created_by() || JEVHelper::isAdminUser($user) || JEVHelper::canDeleteEvent($row, $user))
					{
						// if anon user and email attendance is allowed then find accordingly
						if ($user->id == 0 && RsvpHelper::getTemplateSpecificParameter($rsvpdata, "attendemails", 0))
						{
							$name = $emailaddress;
							$username = $emailaddress;
						}
						else
						{
							$name = $user->name;
							$username = $user->username;
						}
						$sql = "UPDATE #__jev_attendees SET attendstate=" . intval($jevattend);
						$sql .= ", atdcount=" . $row->newcnt .", guestcount=" . $guestcount. ", modified=" . $db->Quote($created);
						// And the locked template
						$sql .= ", lockedtemplate=" . intval($rsvpdata->template);
						$sql .= ", params=" . $db->Quote($params) . ", waiting=0 WHERE id=" . $attendee->id;
						$db->setQuery($sql);
						$db->execute();

						// auto remind attendees
						if (RsvpHelper::getTemplateSpecificParameter($rsvpdata, "autoremind", 0) == 1 && $attendee && ($rsvpdata->allowreminders || RsvpHelper::getTemplateSpecificParameter($rsvpdata, "forceautoremind", 0)))
						{
							// cancel reminder
							$user = JEVHelper::getUser($attendee->user_id);
							$this->jevrreminders->unremindUser($rsvpdata, $row, $user, $attendee->email_address);
							$mainframe->enqueueMessage(Text::_("JEV_REMINDER_CANCELLED"),'notice');
						}

						// cancellation so $attendee is null
						// reset attendance state
						$attendee->attendstate = $jevattend;
						// Do not send notification if only sending notices of new and cancelled registrations
						if (RsvpHelper::getTemplateSpecificParameter($rsvpdata, "notifycreator", 0) && RsvpHelper::getTemplateSpecificParameter($rsvpdata, "notifycreator", 0)!=2)
						{
							$this->notifyCreator($rsvpdata, $row, $name, $username, $attendee, true);
						}
						// if specifically want notification of non-attendance then send one!
						else if ($jevattend == 0 && RsvpHelper::getTemplateSpecificParameter($rsvpdata, "notifyadminnotattending", 0))
						{
							$this->notifyCreator($rsvpdata, $row, $name, $username, $attendee, 2 , $onWaitingList);
						}

						if (RsvpHelper::getTemplateSpecificParameter($rsvpdata, "notifyuser", 0)) {
							$this->notifyUser($rsvpdata, $row, $user, $name, $username, $attendee, 'usercancel');
						}

						// Make sure the counts are in sync!
						$this->countAttendees($rsvpdata->id);

						$this->recordRsvpActivity("attendance.cancelled", $row);
						$this->postUpdateActions($rsvpdata, $row, $attendee,$onWaitingList, $rsvpparams);

						include_once(JPATH_ADMINISTRATOR . "/components/com_rsvppro/models/transactions.php");
						$feemodel = new TransactionsModelTransactions();
						$feemodel->getFeesPaid($attendee, false);

						Factory::getApplication()->triggerEvent( 'onRegistrationStatusChange' , array(&$attendee, $row, $rsvpdata));

						//$this->redirectToPaymentPage($attendee, $rsvpdata, Text::_("JEV_ATTENDANCE_CANCELLED"));
						Factory::getApplication()->enqueueMessage( Text::_("JEV_ATTENDANCE_CANCELLED"),'notice');
						$mainframe->redirect($link);
					}
				} // 7. Existing attendee record was yes - now subject to approval
				else if (($attendee->attendstate == 1 || $attendee->attendstate == 4 ) && $jevattend == 3)
				{
					// This should not arise so do nothing
					throw new Exception( "This should not be allowed 1", 403);
				} // 18. Existing attendee record was subject to approval - now yes
				else if ($attendee->attendstate == 3 && $jevattend == 1)
				{

					// setting must have changed to now allow this
					if ($rsvpdata->initialstate == 1)
					{
						// need to check the capacity on these and waiting list
						$onWaitingList = 0;
						if (RsvpHelper::getTemplateSpecificParameter($rsvpdata, "capacity", 0) && $rsvpdata->capacity > 0)
						{
							// If over capacity and waiting list then just ignore
							if ($attendeeCount >= $rsvpdata->capacity + $rsvpdata->waitingcapacity)
							{
								Factory::getApplication()->enqueueMessage(  Text::_("JEV_EVENT_FULL"), 'warning');
								$mainframe->redirect($link);
							}

							// Should this be on the waiting list
							$onWaitingList = 0;
							if ($attendeeCount >= intval($rsvpdata->capacity) && !$attendee->waiting)
							{
								$onWaitingList = 1;
								$sql = "UPDATE #__jev_attendees SET attendstate=" . intval($jevattend);
								$sql .=  ", atdcount=" . $row->newcnt . ", guestcount=" . $guestcount. ", modified=" . $db->Quote($created);
								// And the locked template
								$sql .= ", lockedtemplate=" . intval($rsvpdata->template);
								$sql .= ", params=" . $db->Quote($params) . ", waiting=" . $onWaitingList . " WHERE id=" . $attendee->id;

								$db->setQuery($sql);
								$db->execute();
								Factory::getApplication()->enqueueMessage(  Text::_("JEV_WILL_PUT_YOU_ON_WAITING_LIST"), 'warning');
								$mainframe->redirect($link);
							}
							// if already on waiting list and saying yes to attend then KEEP on the waiting list
							if ($jevattend == 1 && $attendee->waiting){
								$onWaitingList = 1;
							}

						}
						else
						{
							$onWaitingList = 0;
						}

						$sql = "UPDATE #__jev_attendees SET attendstate=" . intval($jevattend);
						$sql .=  ", atdcount=" . $row->newcnt . ", guestcount=" . $guestcount. ", modified=" . $db->Quote($created);
						// And the locked template
						$sql .= ", lockedtemplate=" . intval($rsvpdata->template);
						$sql .= ", params=" . $db->Quote($params) . ", waiting=" . $onWaitingList . " WHERE id=" . $attendee->id;

						$db->setQuery($sql);
						$db->execute();
						Factory::getApplication()->triggerEvent( 'onRegistrationStatusChange' , array(&$attendee, $row, $rsvpdata));

						if ($jevattend == 1 || $jevattend == 3) {
							$this->redirectToPaymentPage($attendee, $rsvpdata, Text::_("JEV_ATTENDANCE_UPDATED"));
						}
						Factory::getApplication()->enqueueMessage(  Text::_("JEV_ATTENDANCE_UPDATED"), 'info');
						$mainframe->redirect($link);
					}
					else
					{
						// This should not arise so do nothing
						throw new Exception( "This should not be allowed 2", 403);
					}
				}

				// 8. Existing attendee record was yes - now yes (params changed)
				// 10. Existing attendee record was no - now yes
				// 12. Existing attendee record was maybe - now yes
				else if (
						(($attendee->attendstate == 1 || $attendee->attendstate == 4 ) && $jevattend == 1)
						|| ($attendee->attendstate == 0 && $jevattend == 1)
						|| ($attendee->attendstate == 2 && $jevattend == 1))
				{
					// need to check the capacity on these and waiting list
					$onWaitingList = 0;
					if (RsvpHelper::getTemplateSpecificParameter($rsvpdata, "capacity", 0) && $rsvpdata->capacity > 0)
					{
						// If over capacity and waiting list then just ignore
						// remember to add the change in this registrations entry
						if ($attendeeCount + ($row->newcnt - $row->currentcnt) > $rsvpdata->capacity + $rsvpdata->waitingcapacity)
						{
							Factory::getApplication()->enqueueMessage(  Text::_("JEV_EVENT_FULL"), 'warning');
							$mainframe->redirect($link);
						}

						// Should this be on the waiting list?
						// remember to add the change in this registrations entry
						$onWaitingList = 0;
						if ($attendeeCount + ($row->newcnt - $row->currentcnt) > intval($rsvpdata->capacity) && !$attendee->waiting && $row->newcnt>$row->currentcnt)
						{
                                                        // 10. Existing attendee record was no - now yes
                                                        if ($attendee->attendstate == 0 && $jevattend == 1)
                                                        {
							  // DO WE WANT TO PUT SOMEONE ONTO THE WAITING LIST??
							  $onWaitingList = 1;
                                                          $mainframe->enqueueMessage(Text::_("JEV_ATTENDANCE_ON_WAITING_LIST"));
                                                        }
                                                        else {
	                                                        Factory::getApplication()->enqueueMessage(  Text::_("JEV_WILL_PUT_YOU_ON_WAITING_LIST"), 'warning');
                                                            $mainframe->redirect($link);
                                                        }
						}
						// if already on waiting list and saying yes to attend then KEEP on the waiting list
						if ($jevattend == 1 && $attendee->waiting){
							$onWaitingList = 1;
						}
					}
					else
					{
						$onWaitingList = 0;
					}

					$sql = "UPDATE #__jev_attendees SET attendstate=" . intval($jevattend);
					$sql .=  ", atdcount=" . $row->newcnt . ", guestcount=" . $guestcount. ", modified=" . $db->Quote($created);
					// And the locked template
					$sql .= ", lockedtemplate=" . intval($rsvpdata->template);
					$sql .= ", params=" . $db->Quote($params) . ", waiting=" . $onWaitingList . " WHERE id=" . $attendee->id;

					$db->setQuery($sql);
					$db->execute();

					// no->yes maybe->yes yes->yes (changed params) can trigger notification  message (but not if cancellation messages only
					$currentAttendState = $attendee->attendstate;
					if (RsvpHelper::getTemplateSpecificParameter($rsvpdata, "notifycreator", 0) && RsvpHelper::getTemplateSpecificParameter($rsvpdata, "notifycreator", 0)!=3 &&
							(($attendee->attendstate == 0 && $jevattend == 1) ||
							($attendee->attendstate == 2 && $jevattend == 1) ||
							 (($attendee->attendstate == 1 || $attendee->attendstate == 4 ) && $jevattend == 1)))
					{
						// reset attendance state
						$attendee->attendstate = $jevattend;
						if ($user->id == 0 && RsvpHelper::getTemplateSpecificParameter($rsvpdata, "attendemails", 0))
						{
							$emailaddress = $this->getEmailAddress();
							$username = $name = $emailaddress;
						}
						else
						{
							$name = $user->name;
							$username = $user->username;
						}
						$this->notifyCreator($rsvpdata, $row, $name, $username, $attendee, false);
					}

					// only notify user is attendstate has changed and not just the parameters changing
					if ($currentAttendState != $jevattend)
					{
						if (RsvpHelper::getTemplateSpecificParameter($rsvpdata, "notifyuser", 0)){
							if ($user->id == 0 && $this->params->get("attendemails", 0))
							{
								$emailaddress = $this->getEmailAddress();
								$username = $name = $emailaddress;
							}
							else
							{
								$name = $user->name;
								$username = $user->username;
							}
							// reset attendance state
							$attendee->attendstate = $jevattend;

							$this->notifyUser($rsvpdata, $row, $user, $name, $username, $attendee,'ack', $onWaitingList);
						}
					}

					$this->postUpdateActions($rsvpdata, $row, $attendee,$onWaitingList, $rsvpparams);
					Factory::getApplication()->triggerEvent( 'onRegistrationStatusChange' , array(&$attendee, $row, $rsvpdata));

					if ($jevattend == 1 || $jevattend == 3) {
						$this->redirectToPaymentPage($attendee, $rsvpdata, Text::_("JEV_ATTENDANCE_UPDATED"));
					}
					Factory::getApplication()->enqueueMessage(  Text::_("JEV_ATTENDANCE_UPDATED"), 'info');

					$mainframe->redirect($link);
				}

				// 9. Existing attendee record was no - now maybe
				// 11. Existing attendee record was no - now subject to approval
				// 13. Existing attendee record was maybe - now no
				// 14. Existing attendee record was maybe - now subject to approval
				// 15. Existing attendee record was maybe - now maybe  (params changed)
				// 16. Existing attendee record was subject to approval - now no
				// 17. Existing attendee record was subject to approval - now maybe
				// 19. Existing attendee record was subject to approval - now subject to approval  (params changed)
				else if (($attendee->attendstate == 0 && ($jevattend == 2 || $jevattend == 3)) || ($attendee->attendstate == 2 && ($jevattend == 0 || $jevattend == 3 || $jevattend == 2)) || ($attendee->attendstate == 3 && ($jevattend == 0 || $jevattend == 2 || $jevattend == 3)))
				{

					$sql = "UPDATE #__jev_attendees SET attendstate=" . intval($jevattend);
					$sql .=  ", atdcount=" . $row->newcnt . ", guestcount=" . $guestcount. ", modified=" . $db->Quote($created);
					// And the locked template
					$sql .= ", lockedtemplate=" . intval($rsvpdata->template);
					$sql .= ", params=" . $db->Quote($params) . ", waiting=0 WHERE id=" . $attendee->id;

					$db->setQuery($sql);
					$db->execute();

					// Make sure the counts are in sync!
					$this->countAttendees($rsvpdata->id);
					Factory::getApplication()->triggerEvent( 'onRegistrationStatusChange' , array(&$attendee, $row, $rsvpdata));

					// redirect to ensure not in tmpl=component mode
					if ($jevattend == 1 || $jevattend == 3) {
						$this->redirectToPaymentPage($attendee, $rsvpdata, Text::_("JEV_ATTENDANCE_UPDATED"));
					}
					Factory::getApplication()->enqueueMessage(  Text::_("JEV_ATTENDANCE_UPDATED"), 'info');
					$mainframe->redirect($link);
				}

				// 20. Existing attendee record was no - now no (params changed)
				else if ($attendee->attendstate == 0 && $jevattend ==0 )
				{

					$sql = "UPDATE #__jev_attendees SET attendstate=" . intval($jevattend);
					$sql .=  ", atdcount=" . $row->newcnt . ", guestcount=" . $guestcount. ", modified=" . $db->Quote($created);
					// And the locked template
					$sql .= ", lockedtemplate=" . intval($rsvpdata->template);
					$sql .= ", params=" . $db->Quote($params) . ", waiting=0 WHERE id=" . $attendee->id;

					$db->setQuery($sql);
					$db->execute();

					// Make sure the counts are in sync!
					$this->countAttendees($rsvpdata->id);
					Factory::getApplication()->triggerEvent( 'onRegistrationStatusChange' , array(&$attendee, $row, $rsvpdata));

					Factory::getApplication()->enqueueMessage(  Text::_("JEV_ATTENDANCE_UPDATED"), 'info');
					$mainframe->redirect($link);
				}

			}


		}
		return true;

	}

	public function listaction(){
		$user = Factory::getUser();

		// Check for request forgeries
		//Session::checkToken('request') or jexit( 'Invalid Token' );

		if ($user->id == 0 && !$this->params->get("attendemails", 0))
		{
			return false;
		}

		$db = Factory::getDBO();
		$at_id = Factory::getApplication()->input->getInt("at_id", -1);
		$sql = "SELECT * FROM #__jev_attendance WHERE id=" . $at_id;
		$db->setQuery($sql);
		$rsvpdata = $db->loadObject();
		if (!$rsvpdata)
			return false;

		$rp_id = Factory::getApplication()->input->getInt("rp_id", -1);
		$datamodel = new JEventsDataModel();
		$row = $datamodel->queryModel->listEventsById($rp_id, 1, "icaldb");
		if (!$row)
			return false;

		$this->event = $row;

		$cache =  Factory::getCache('com_jevents');
		$cache->clean();

		// Add reference to current row and rsvpdata to the registry so that we have access to these in the fields
		$registry = JevRegistry::getinstance("jevents");
		$registry->set("rsvpdata", $rsvpdata);
		$registry->set("event", $row);

		$db = Factory::getDBO();

		$this->countAttendees($rsvpdata->id);
		$guestcount = Factory::getApplication()->input->getInt("guestcount", 1);

		$sql = "SELECT atdcount FROM #__jev_attendeecount as a WHERE a.at_id=" . $rsvpdata->id;
		if (!$rsvpdata->allrepeats)
		{
			$sql .= " and a.rp_id=" . $row->rp_id();
		}
		$db->setQuery($sql);
		$attendeeCount = $db->loadResult();

		// Hygene trap
		$jevattendlist_id = Factory::getApplication()->input->getInt("jevattendlist_id", 0);
		$jevattendlist_id_approve = Factory::getApplication()->input->getInt("jevattendlist_id_approve", 0);
		if ($jevattendlist_id || $jevattendlist_id_approve)
		{

			// Is this an admin user cancelling an attendance record
			$this->adminCancelAttendance($rsvpdata, $row);

			// Is the admin approving an attendance record
			$this->adminApproveAttendance($rsvpdata, $row);
		}
	}

	private function templateHasFees($rsvpdata, $row){
		if ($rsvpdata->template != "")
		{
			$xmlfile = JevTemplateHelper::getTemplate($rsvpdata);

			if (is_int($xmlfile) || file_exists($xmlfile))
			{
				$rsvpparams = new JevRsvpParameter("", $xmlfile, $rsvpdata, $row);
				if (isset($rsvpparams->_rawtemplate) && isset($rsvpparams->_rawtemplate->withfees)){
					return $rsvpparams->_rawtemplate->withfees;
				}
			}
		}
		return false;

	}

	private function recordRsvpActivity($cmd, &$row)
	{
		if (!$this->jomsocial)
			return;
		$user = Factory::getUser();
		if ($user->id == 0)
			return;

		// Make sure not processed twice
		if (!isset($row->activitiesProcessed)){
			$row->activitiesProcessed = array();
		}
		if (isset($row->activitiesProcessed[$cmd])){
			return;
		}
		$row->activitiesProcessed[$cmd] = 1;

		require_once (JPATH_ROOT . '/' . 'components' . '/' . 'com_community' . '/' . 'defines.community.php');

		// Require the base controller
		require_once (COMMUNITY_COM_PATH . '/' . 'libraries' . '/' . 'error.php');
		require_once (COMMUNITY_COM_PATH . '/' . 'controllers' . '/' . 'controller.php');
		require_once (COMMUNITY_COM_PATH . '/' . 'libraries' . '/' . 'apps.php');
		require_once (COMMUNITY_COM_PATH . '/' . 'libraries' . '/' . 'core.php');
		require_once (COMMUNITY_COM_PATH . '/' . 'libraries' . '/' . 'template.php');
		require_once (COMMUNITY_COM_PATH . '/' . 'libraries' . '/' . 'userpoints.php');
		require_once (COMMUNITY_COM_PATH . '/' . 'views' . '/' . 'views.php');
		require_once (COMMUNITY_COM_PATH . '/' . 'helpers' . '/' . 'url.php');
		require_once (COMMUNITY_COM_PATH . '/' . 'helpers' . '/' . 'ajax.php');
		require_once (COMMUNITY_COM_PATH . '/' . 'helpers' . '/' . 'time.php');
		require_once (COMMUNITY_COM_PATH . '/' . 'helpers' . '/' . 'owner.php');
		require_once (COMMUNITY_COM_PATH . '/' . 'helpers' . '/' . 'azrul.php');
		require_once (COMMUNITY_COM_PATH . '/' . 'helpers' . '/' . 'string.php');

		if ($cmd == "attendance.cancelled")
		{
			CuserPoints::assignPoint('rsvppro.cancel');
		}
		else
		{
			CuserPoints::assignPoint('rsvppro.signup');
		}

		if (!$this->params->get("activitystream", 0))
			return;

		CFactory::load('libraries', 'activities');

		$mainframe = Factory::getApplication();
		$Itemid = Factory::getApplication()->input->getInt("Itemid");
		list ( $year, $month, $day ) = JEVHelper::getYMD();

		$link = Route::_($row->viewDetailLink($year, $month, $day, false, $Itemid));

		if (strpos($link, "/") !== 0)
		{
			$link = "/" . $link;
		}

		$uri =  JURI::getInstance(JURI::base());
		$root = $uri->toString(array('scheme', 'host', 'port'));

		$link = $root . $link;

		if ($cmd == "attendance.cancelled")
		{
			$title = Text::sprintf("JOMSOCIAL_ACTIVITY_CANCELLED", $link, $row->title());
		}
		else
		{
			$title = Text::sprintf("JOMSOCIAL_ACTIVITY_REGISTERED", $link, $row->title());
		}
		$act = new stdClass ();
		$act->cmd = $cmd;
		$act->actor = $user->id;
		$act->target = 0;
		$act->title = $title;
		$act->content = "";
		$act->app = "jevents";
		$act->cid = 0;

		$privateevent = isset($row->_privateevent) ? $row->_privateevent : 0;
		switch ($privateevent) {
			case 0 :
				// public event
				break;
			case 1 :
				// private event
				$act->access = 40;
				break;
			case 2 :
				// Jom social friends
				$act->access = 30;
				break;
			case 3 :
				// details private
				break;
			case 4 :
				// JomSocial Groups
				// No specific rule available for this
				$act->access = 30;
				break;
			case 5 :
			case 6 :
				// Group Jive or CB Connections
				break;
		}

		CActivityStream::add($act);

	}

	private function redirectToPaymentPage(&$attendee, $rsvpdata, $msg = "")
	{

		// Since we have just saved the attendee we must reload to get the new balance values correctly
		$db = Factory::getDBO();
		$sql = "SELECT * FROM #__jev_attendees WHERE id=" . $attendee->id;
		$db->setQuery($sql);
		$attendee = $db->loadObject();

		// ALSO redirect to payment page if necessary
		if (!isset($attendee->outstandingBalances))
		{
			// This MUST be called before renderToBasicArray to populate the balance fields - so we do it here to be safe
			if (isset($attendee->params))
			{
				$xmlfile = JevTemplateHelper::getTemplate($rsvpdata);

				if (is_int($xmlfile) || file_exists($xmlfile))
				{
					$params = new JevRsvpParameter($attendee->params, $xmlfile, $rsvpdata, null);
					$feesAndBalances = $params->outstandingBalance($attendee);
				}
			}
		}

		$feesAndBalances = isset($attendee->outstandingBalances) ? $attendee->outstandingBalances : false;
		if ($feesAndBalances && $params && $feesAndBalances["hasfees"])
		{
			$paymentoptionlistfield = "paymentoptionlist";
			$paramarray = $params->renderToBasicArray();
			// find the paymentoptionlist
			foreach ($paramarray as $paramfield => $param)
			{
				if ($param["type"] == "jevrpaymentoptionlist")
				{
					$paymentoptionlistfield = $param["name"];
				}
			}
			$gateway = $params->getValue($paymentoptionlistfield, 'xmlfile', "manual");
			$outstandingBalance = $feesAndBalances["feebalance"];
			$token = Session::getFormToken();;
			$code = "";
			$em = Factory::getApplication()->input->getString("em", "");
			if ($em != "")
			{
				$code = "&em=$em";
			}

			if ($outstandingBalance > 0)
			{
				// If we have a deposit that is required now then change the amount to be paid to match the deposit
				if ($feesAndBalances["deposit"]>0 && $feesAndBalances["feepaid"]==0){
					$amount = $feesAndBalances["deposit"];
				}
				else {
					$amount = $outstandingBalance;
				}

				$mainframe = Factory::getApplication();
				$Itemid = Factory::getApplication()->input->getInt("Itemid");
				$user = Factory::getUser();
				if ($user->id == 0 && $this->params->get("attendemails", 0) && $code == "")
				{
					$emailaddress = trim(strtolower(Factory::getApplication()->input->getString("jevattend_email")));
					$code = "&em=" .base64_encode($emailaddress . ":" . md5($this->params->get("emailkey", "email key") . $emailaddress));
                                    }
				$link = Route::_("index.php?option=com_rsvppro&task=accounts.paymentpage&gateway=$gateway&Itemid=$Itemid&amount=$amount&rsvpid=$rsvpdata->id&invoiceid=$attendee->id&$token=1$code", false);
				if ($msg == Text::_("JEV_ATTENDANCE_CONFIRMED")){
					$msg = Text::_("JEV_ATTENDANCE_RECORDED_OUTSTANDING_BALANCE");
				}
				Factory::getApplication()->enqueueMessage(  $msg, 'info');
				$mainframe->redirect($link);
			}
			else if ($outstandingBalance < 0)
			{

				// TODO request repayment
				return;
				$mainframe = Factory::getApplication();
				$link = Route::_("index.php?option=com_rsvppro&task=accounts.paymentpage&gateway=$gateway&Itemid=$Itemid&amount=$outstandingBalance&rsvpid=$rsvpdata->id&invoiceid=$attendee->id&$token=1$code", false);
				Factory::getApplication()->enqueueMessage(  $msg, 'info');
				$mainframe->redirect($link);
			}
		}

	}

	/*
	 * We cannot rely on __call magic function to pass variables by reference!
	 */
	private function notifyUser($rsvpdata, & $row, $user, $name, $username, $attendee = null, $messagetype = 'ack', $onWaitingList = false, $transaction = false, $subject = "", $message = "", $speciallink=false) {
		if (isset($this->event) && !isset($this->helper->event)){
			$this->helper->event = $this->event;
		}
		return $this->helper->notifyUser($rsvpdata,  $row, $user, $name, $username, $attendee, $messagetype, $onWaitingList, $transaction, $subject, $message, $speciallink);
	}

	private function notifyCreator($rsvpdata,& $row, $name, $username, $attendee = null, $cancellation = false, $onWaitingList = false) {
		if (isset($this->event) && !isset($this->helper->event)){
			$this->helper->event = $this->event;
		}
		return $this->helper->notifyCreator($rsvpdata, $row, $name, $username, $attendee, $cancellation, $onWaitingList) ;

	}

	/**
	 * Creates Joomla user from submitted form
	 *
	 */
	private function createUser($event, $rsvpdata){

		// Initialize new usertype setting
		jimport('joomla.application.component.helper');
		// Load Input for usage.
		$jinput = Factory::getApplication()->input;
		// Get ItemId
		$Itemid = $jinput->getInt('Itemid');
		$usersConfig = ComponentHelper::getParams( 'com_users' );
		$authorize	= Factory::getACL();
		$newUsertype = $usersConfig->get( 'new_usertype' );
		if (!$newUsertype) {
			// registered is 2 by default
			$newUsertype = 2;
		}

		$db = Factory::getDBO();

		// Make sure the username is valid and valid token presented etc.
		Session::checkToken('request') or jexit('Invalid Token');
		$username = Factory::getApplication()->input->getString("jevattend_username","");
		$email = Factory::getApplication()->input->getString("jevattend_email","");
		$password = Factory::getApplication()->input->getString("jevattend_password","");
		$password2 = Factory::getApplication()->input->getString("jevattend_password2","");

		if (!$username) {
			// throw error for invalid username
			$link = Route::_($event->viewDetailLink($event->yup(),$event->mup(), $event->dup(), false, $Itemid));
			Factory::getApplication()->enqueueMessage(  Text::_("JEV_MISSING_USERNAME"), 'warning');
			Factory::getApplication()->redirect($link);
			return false;
		}

		if (!$email) {
			// throw error for invalid email
			$link = Route::_($event->viewDetailLink($event->yup(),$event->mup(), $event->dup(), false, $Itemid));
			Factory::getApplication()->enqueueMessage(  Text::_("JEV_MISSING_EMAIL"), 'warning');
			Factory::getApplication()->redirect($link);
		}

		// missmatched passwords
		if (!$password || $password!=$password2){
			$link = Route::_($event->viewDetailLink($event->yup(),$event->mup(), $event->dup(), false, $Itemid));
			Factory::getApplication()->enqueueMessage(  Text::_("JEV_PASSWORDS_NOT_ENTERED_OR_DO_NOT_MATCH"), 'warning');
			Factory::getApplication()->redirect($link);
			return false;
		}


		$sql = "SELECT LOWER(username) as uname, ju.* FROM #__users as ju  WHERE LOWER(username) LIKE (".$db->Quote(strtolower(trim($username))).")";
		$db->setQuery($sql);
		$matchingUser = $db->loadObject();

		if ($matchingUser && !$matchingUser->block && !$matchingUser->activation){
			// throw error for username already in use
			$link = Route::_($event->viewDetailLink($event->yup(),$event->mup(), $event->dup(), false, $Itemid));
			Factory::getApplication()->enqueueMessage(  Text::_("JEV_USERNAME_ALREADY_IN_USE"), 'warning');
			Factory::getApplication()->redirect($link);
		}

		$sql = "SELECT LOWER(username) as uname, ju.* FROM #__users as ju  WHERE LOWER(email) LIKE (".$db->Quote(strtolower(trim($email))).")";
		$db->setQuery($sql);
		$matchingUser2 = $db->loadObject();

		if ($matchingUser2 && !$matchingUser2->block && !$matchingUser2->activation){
			// throw error for username already in use
			$link = Route::_($event->viewDetailLink($event->yup(),$event->mup(), $event->dup(), false, $Itemid));
			Factory::getApplication()->enqueueMessage(  Text::_("JEV_EMAIL_ALREADY_IN_USE"), 'warning');
			Factory::getApplication()->redirect($link);
		}

		$name = $username;
		$profiledata = array();
		if ($rsvpdata->template != "")
		{
			$xmlfile = JevTemplateHelper::getTemplate($rsvpdata);
			if (is_int($xmlfile) || file_exists($xmlfile))
			{
				$tempparams = new JevRsvpParameter("", $xmlfile, $rsvpdata, $event);
				$tempparams = $tempparams->renderToBasicArray( 'xmlfile', null);
				foreach ($tempparams as $param)
				{
					// is this a name field?  If so then use this in preference to the user's profile name
					if (isset($param["isname"]) && $param["isname"])
					{
						$fieldname = $param["name"];
						$inparams = Factory::getApplication()->input->get("xmlfile",array(), 'ARRAY');
						if (isset($inparams[$fieldname])){
							$name = $inparams[$fieldname];
							// name field might per attendee so pick out the first valid value
							if (is_array($name)) {
								if (isset($name[0]) && $name[0]!=""){
									$name = InputFilter::getInstance()->clean($name[0], "string");
								}
								else {
									$name = $username;
								}
							}
						}
					}

					if ($param["type"] == "jevrprofilefield")
					{
						$inparams = Factory::getApplication()->input->get("xmlfile",array(), 'ARRAY');
						if (isset($inparams[$param["name"]])){
							$profiledata[$param["fieldname"]] = InputFilter::getInstance()->clean($inparams[$param["name"]], 'string');
						}
					}
				}
			}
		}

		$joomlauser = new User();

		User::getInstance(0);

		$data = array();
		$data['id']=0;
		if ($matchingUser && $matchingUser->block && $matchingUser->activation ) {
			$data['id']=$matchingUser->id;
		}
		if ($matchingUser2 && $matchingUser2->block && $matchingUser2->activation ) {
			$data['id']=$matchingUser2->id;
		}

		$data['email'] = $email;
		$data['username'] = $username;
		$data['name'] =  $name;
		jimport("joomla.user.helper");
		$data['password'] = $password;
		$data['password2'] = $password;
		// defaut to registered group
		$data['groups'] = array(0=>$newUsertype);

		$data["profile"] = $profiledata;
		$joomlauser->bind($data);

		if (class_exists("JevDate"))
		{
			$datenow = JevDate::getDate();
			$joomlauser->set('registerDate', $datenow->toMySQL());
		}
		else {
			$datenow = Factory::getDate();
			$joomlauser->set('registerDate', $datenow->toSql());
		}
		$joomlauser->set('lastvisitDate',null);

		// If user activation is turned on, we need to set the activation information
		$useractivation = $usersConfig->get('useractivation');
		$sendpassword = $usersConfig->get('sendpassword');

		if ($useractivation == '1')
		{
			jimport('joomla.user.helper');
			$joomlauser->set('activation', md5( UserHelper::genRandomPassword()) );
			$joomlauser->set('block', '1');
		}

		// Load the users plugin group.
		PluginHelper::importPlugin('user');

		// If there was an error with registration, set the message and display form
		if ( !$joomlauser->save() )
		{
			throw new Exception('', Text::_( $joomlauser->getError()));
			return false;
		}

		// Compile the notification mail values.
		$data = $joomlauser->getProperties();
		$data['fromname']	= Factory::getConfig()->get('fromname');
		$data['mailfrom']	= Factory::getConfig()->get('mailfrom');
		$data['sitename']	= Factory::getConfig()->get('sitename');
		$data['siteurl']	= Uri::root();

		$lang = Factory::getLanguage();
		$lang->load("com_users", JPATH_SITE);

		// Handle account activation/confirmation emails.
		if ($useractivation == 2)
		{
			// Set the link to confirm the user email.
			$uri = JURI::getInstance();
			$base = $uri->toString(array('scheme', 'user', 'pass', 'host', 'port'));
			$data['activate'] = $base.Route::_('index.php?option=com_users&task=registration.activate&token='.$data['activation'], false);

			$emailSubject	= Text::sprintf(
				'COM_USERS_EMAIL_ACCOUNT_DETAILS',
				$data['name'],
				$data['sitename']
			);

			if ($sendpassword)
			{
				$emailBody = Text::sprintf(
					'COM_USERS_EMAIL_REGISTERED_WITH_ADMIN_ACTIVATION_BODY',
					$data['name'],
					$data['sitename'],
					$data['activate'],
					$data['siteurl'],
					$data['username'],
					$data['password_clear']
				);
			}
			else
			{
				$emailBody = Text::sprintf(
					'COM_USERS_EMAIL_REGISTERED_WITH_ADMIN_ACTIVATION_BODY_NOPW',
					$data['name'],
					$data['sitename'],
					$data['activate'],
					$data['siteurl'],
					$data['username']
				);
			}
		}
		elseif ($useractivation == 1)
		{
			// Set the link to activate the user account.
			$uri = JURI::getInstance();
			$base = $uri->toString(array('scheme', 'user', 'pass', 'host', 'port'));
			$data['activate'] = $base.Route::_('index.php?option=com_users&task=registration.activate&token='.$data['activation'], false);

			$emailSubject	= Text::sprintf(
				'COM_USERS_EMAIL_ACCOUNT_DETAILS',
				$data['name'],
				$data['sitename']
			);

			if ($sendpassword)
			{
				$emailBody = Text::sprintf(
					'COM_USERS_EMAIL_REGISTERED_WITH_ACTIVATION_BODY',
					$data['name'],
					$data['sitename'],
					$data['activate'],
					$data['siteurl'],
					$data['username'],
					$data['password_clear']
				);
			}
			else
			{
				$emailBody = Text::sprintf(
					'COM_USERS_EMAIL_REGISTERED_WITH_ACTIVATION_BODY_NOPW',
					$data['name'],
					$data['sitename'],
					$data['activate'],
					$data['siteurl'],
					$data['username']
				);
			}
		}
		else
		{

			$emailSubject	= Text::sprintf(
				'COM_USERS_EMAIL_ACCOUNT_DETAILS',
				$data['name'],
				$data['sitename']
			);

			$emailBody = Text::sprintf(
				'COM_USERS_EMAIL_REGISTERED_BODY',
				$data['name'],
				$data['sitename'],
				$data['siteurl'],
                $data['username'],
                $data['password_clear']
			);
		}

		// Send the registration email.
		try
		{
			$return = Factory::getMailer()->sendMail($data['mailfrom'], $data['fromname'], $data['email'], $emailSubject, $emailBody);
		}
		catch (Exception $e)
		{
			$return = false;
		}

		//Send Notification mail to administrators
		if (($usersConfig->get('useractivation') < 2) && ($usersConfig->get('mail_to_admin') == 1)) {
			$emailSubject = Text::sprintf(
				'COM_USERS_EMAIL_ACCOUNT_DETAILS',
				$data['name'],
				$data['sitename']
			);

			$emailBodyAdmin = Text::sprintf(
				'COM_USERS_EMAIL_REGISTERED_NOTIFICATION_TO_ADMIN_BODY',
				$data['name'],
				$data['username'],
				$data['siteurl']
			);

			// get all admin users
			$query = 'SELECT name, email, sendEmail' .
					' FROM #__users' .
					' WHERE sendEmail=1';

			$db->setQuery( $query );
			$rows = $db->loadObjectList();

			// Send mail to all superadministrators id
			foreach( $rows as $row )
			{
				try
				{
					$return = Factory::getMailer()->sendMail($data['mailfrom'], $data['fromname'], $row->email, $emailSubject, $emailBodyAdmin);
				}
				catch (Exception $e)
				{
					$return = false;
				}

				// Check for an error.
				if ($return !== true) {
					$this->setError(Text::_('COM_USERS_REGISTRATION_ACTIVATION_NOTIFY_SEND_MAIL_FAILED'));
					return false;
				}
			}
		}
		return $joomlauser->id;

	}

}
