<?php
/**
 * JEvents Component for Joomla 1.5.x
 *
 * @version     $Id$
 * @package     JEvents
 * @copyright   Copyright (C) 2008-2021 GWE Systems Ltd
 * @license     GNU/GPLv2, see http://www.gnu.org/licenses/gpl-2.0.html
 * @link        https://www.jevents.net
 */

defined( 'JPATH_BASE' ) or die( 'Direct Access to this location is not allowed.' );

use Joomla\CMS\Language\Text;
use Joomla\CMS\Component\ComponentHelper;
use Joomla\CMS\Pagination\Pagination;
use Joomla\CMS\Factory;
use Joomla\CMS\HTML\HTMLHelper;

include_once(RSVP_ADMINPATH."/controllers/sessions.php");

class FrontSessionsController extends AdminSessionsController   {

	function __construct($config = array())
	{
		parent::__construct($config);

		$this->registerTask( 'list',  'overview' );
		$this->registerDefaultTask("overview");

		// Load abstract "view" class
		$cfg = JEVConfig::getInstance();
		$theme = JEV_CommonFunctions::getJEventsViewName();
		JLoader::register('JEvents'.ucfirst($theme).'View',JEV_VIEWS."/$theme/abstract/abstract.php");

		// Load admin language file
		$lang = Factory::getLanguage();
		$lang->load(RSVP_COM_COMPONENT, JPATH_ADMINISTRATOR);
	}

	// frontend version is not interested invitations initially
	function overview( )
	{

		$user = Factory::getUser();

		// Fix attendee records for user who has registered in Joomla AFTER signing up for an event
		$db	= Factory::getDBO();
		if ($user->id>0 && $user->email!=""){
			$db->setQuery("Update #__jev_attendees  set user_id = ".$user->id."  where email_address = ". $db->Quote($user->email));
			$db->execute();
			$db->setQuery("Update #__jev_invitees set user_id = ".$user->id."  where email_address = ". $db->Quote($user->email));
			$db->execute();
			$db->setQuery("Update #__jev_reminders set user_id = ".$user->id."  where email_address = ". $db->Quote($user->email));
			$db->execute();
		}

		$jevparams = ComponentHelper::getParams('com_jevents');
                $rsvpparams = ComponentHelper::getParams(RSVP_COM_COMPONENT);

		// get the view
		$this->view = $this->getView("sessions","html");

		$mainframe = Factory::getApplication();
		$Itemid=Factory::getApplication()->input->getInt("Itemid");
		$option = Factory::getApplication()->input->getCmd("option");

		$search		= $mainframe->getUserStateFromRequest( "search{".RSVP_COM_COMPONENT."}".$Itemid, 'search', '' , 'string');
		$search = str_replace(array('"', '&quot'), ' ', $search);
		$search		= $db->escape( trim( strtolower( $search ) ) );

		$after		= $mainframe->getUserStateFromRequest( "after{".RSVP_COM_COMPONENT."}".$Itemid, 'after', '' );
		$after		= $db->escape( trim( strtolower( $after ) ) );
        if ($after){
            $inputdateformat = $jevparams->get("com_editdateformat", "d.m.Y");
            $after = DateTime::createFromFormat($inputdateformat, $after);
            $after = $after->format("Y-m-d");
        }

        $limit		= intval( $mainframe->getUserStateFromRequest( "sessions.viewlistlimit".$Itemid, 'limit', 15 ));

		$searchattendees = $mainframe->getUserStateFromRequest("atsearchattendees{" . RSVP_COM_COMPONENT . "}".$Itemid, 'searchattendees', '');
		$searchattendees = $db->escape(trim(strtolower($searchattendees)));

		$searchinvitees = $mainframe->getUserStateFromRequest("atsearchinvitees{" . RSVP_COM_COMPONENT . "}".$Itemid, 'searchinvitees', '');
		$searchinvitees = $db->escape(trim(strtolower($searchinvitees)));

		// Block showing ALL
		if ($limit==0 || $limit>100){
			$limit=100;
		}
		$limitstart = Factory::getApplication()->input->getInt('limitstart', 0 );
		$created_by	=  intval($mainframe->getUserStateFromRequest( "createdbyrsvp".$Itemid, 'created_by', 0 ));
		$whichsessions = $rsvpparams->get("whichsessions","ABD");
		$repeatingFilter = in_array($whichsessions, array("SBD","SO")) ? 1 : ($whichsessions === 'ABD' ? -1 : 0);
		$nonrepeating = intval($mainframe->getUserStateFromRequest( "repeatingrsvp".$Itemid, 'repeating', $repeatingFilter ));
		$newcats = $rsvpparams->get( "catidnew", false);

		if ( in_array($whichsessions, array("RO","SO"))) {
			$nonrepeating = $repeatingFilter;
		}
		$hidepast = intval($mainframe->getUserStateFromRequest( "hidepastrsvp".$Itemid, 'hidepast', 1 ));

		$catid = intval(Factory::getApplication()->getUserStateFromRequest("catidIcalEvents".$Itemid, 'catid', 0));
		$catidtop = $catid;

		$where = array();
		$join = array();

		if( $search ){
			$where[] = "LOWER(det.summary) LIKE '%$search%'";
		}

		if ($searchattendees) {
            if ($searchattendees=="*"){
                $where[] = "atdees.id IS NOT NULL ";
                //$where[] = "atdees.attendstate=1 ";
            }
            else {
                $searchallfields = $rsvpparams->get("searchallfields",0);
                if ($searchallfields ){
                    $where[] = "(atdees.params LIKE '%$searchattendees%' OR atdees.email_address LIKE '%$searchattendees%' OR jua.username LIKE '%$searchattendees%' OR jua.name LIKE '%$searchattendees%' OR jua.email LIKE '%$searchattendees%'  OR atdees.id = '$searchattendees' )";
                }
                else {
                    $where[] = "(atdees.email_address LIKE '%$searchattendees%' OR jua.username LIKE '%$searchattendees%' OR jua.name LIKE '%$searchattendees%' OR jua.email LIKE '%$searchattendees%'  OR atdees.id = '$searchattendees' )";
                }
            }
			$join [] = " #__users AS jua ON jua.id = atdees.user_id"	;
		}

		if ($searchinvitees) {
			$where[] = "(inv.email_address LIKE '%$searchinvitees%' OR jui.username LIKE '%$searchinvitees%' OR jui.email LIKE '%$searchinvitees%' )";
			$join [] = " #__users AS jui ON jui.id = atdees.user_id"	;
		}

		if ($jevparams->get("multicategory", 0)){
			$join[] = "\n #__jevents_catmap as catmap ON catmap.evid = rpt.eventid";
			$join[] = "\n #__categories AS catmapcat ON catmap.catid = catmapcat.id";
			$where[]= " catmapcat.access " . ' IN (' . JEVHelper::getAid($user) . ')' ;
			$needsgroup = true;
		}

		if ($newcats && is_array($newcats) && count($newcats) > 0){
			if ($jevparams->get("multicategory",0)){
				$where[] = "catmap.catid in ('".implode(",",$newcats)."')";
			}
			else {
				$where[] = "ev.catid in ('".implode(",",$newcats)."')";
			}
		}

		if ($created_by!==0 ) {
			$where[] = "ev.created_by=".$db->Quote($created_by);
		}
		else if ( $rsvpparams->get("personallist")==4) {
			$where[] = "ev.created_by=".$db->Quote($user->id);
		}

		$where[] = "ev.ev_id IS NOT NULL";

		$user = Factory::getUser();

		$jevparams =  ComponentHelper::getParams(JEV_COM_COMPONENT);

		if ($rsvpparams->get("personallist") && $user->id>0){
			// yes, maybe, pending, unpaid or overpaid
			$approved = "attendstate IN (1,2,3,4,5) AND ";
			if ($rsvpparams->get("personallist") ==2 || $rsvpparams->get("personallist") ==3){
				// yes or pending payment
				$approved = "attendstate IN (1,4) AND ";
			}

			// attended by someone / anyone at all
			if ($rsvpparams->get("personallist")==3){
					$where[] = "( (atd.allrepeats = 1 AND atd.id IN (SELECT distinct at_id FROM #__jev_attendees AS atdees WHERE $approved 1 )) OR "
					// for individual repeats
					 . " (atd.allrepeats = 1 AND rpt.rp_id IN (SELECT distinct rp_id FROM #__jev_attendees AS atdees WHERE $approved atdees.rp_id>0)) )";

			}
			else if ($rsvpparams->get("personallist")<3) {
				$where[] = "( (atd.allrepeats = 1 AND atd.id IN (SELECT at_id FROM #__jev_attendees AS atdees WHERE $approved (user_id=".$user->id." OR email_address = ". $db->Quote($user->email)." )) ) OR "
				// for individual repeats
				. " (atd.allrepeats = 1 AND rpt.rp_id IN (SELECT rp_id FROM #__jev_attendees AS atdees WHERE $approved (user_id=".$user->id." OR email_address = ". $db->Quote($user->email)." ) AND atdees.rp_id>0)) )";
			}
		}
		if ($rsvpparams->get("personallist") && $user->id==0){
			$where[] = '0';
		}

		$user = Factory::getUser();
		// if user is not logged in then restrict to sessions they can see
		if ($user->get('id') == 0 && !$rsvpparams->get("attendemails")){
			$where[] = " 0";
		}

		// by invitation only
		if ($user->get('id')==0 ){
			$where[] = " atd.allowregistration=1";
		}
		// admin users can see all sessions!
		else if (!JEVHelper::isAdminUser ()) {
			$where[] = " ((atd.allowregistration=1) OR (atd.allowregistration=2 AND (inv2.id IS NOT NULL OR ev.created_by=".$user->get('id').")))";
			$join[] = "#__jev_invitees as inv2 ON inv2.at_id=atd.id AND inv2.user_id=".$user->get('id');
		}

		if (!JEVHelper::isAdminUser($user)){
			// we will strip these out at display time
			//$where[] = "(atd.showattendees=1 OR ev.created_by=".intval($user->id).")";
		}

        if ($catid > 0)
        {
                if ($jevparams->get("multicategory",0) == 1){
                        $where[] = "catmap.catid='$catid'";
                }
                else {
                        $where[] = "ev.catid='$catid'";
                }
        }

        if ($after) {
            $where[] = "\n (rpt.startrepeat >".$db->quote($after)." OR atd.regclose>".$db->quote($after).")";
        }

		$filter_order		= $mainframe->getUserStateFromRequest( $option.'sess_filter_order',		'filter_order',		'startdate',	'cmd' );
		$filter_order_Dir	= $mainframe->getUserStateFromRequest( $option.'sess_filter_order_Dir',	'filter_order_Dir',	' ASC',		    'word' );

		// Force ASC filter and direction to valid values
		$filter_order	= in_array($filter_order , array('startdate',  'det.summary', 'capacity', 'atdcount', 'ev.catid', 'ev.created_by', 'invcount')) ? $filter_order : 'startdate';
		$filter_order_Dir	= in_array($filter_order_Dir , array('ASC',  'DESC')) ? $filter_order_Dir : 'ASC';

		$orderby 	= ' ORDER BY '.$filter_order.' '.$filter_order_Dir;

		// get the total number of records in three blocks
		if ($nonrepeating == -1)
		{
			if ($hidepast){
				if (class_exists("JevDate")) {
					$datenow = JevDate::getDate("-1 day");
					$datenowUnix = $db->quote($datenow->toUnix());
					$datenow = $db->quote($datenow->toMySQL());
				}
				else {
					$datenow = Factory::getDate("-1 day");
					$datenowUnix = $db->quote($datenow->toUnix());
					$datenow = $db->quote($datenow->toSql());
				}
				$where[] = "\n ((rpt.endrepeat>".$datenow." AND atd.allrepeats=0) OR "
					.  "((det.dtstart>".$datenowUnix." OR atd.regclose>".$datenow.") AND atd.allrepeats=1 AND ev.ev_id is not null))";
			}

			$where[] = "atd.allowregistration>0";
			// fix the access
			$where[] = "ev.access " .  ' IN (' . JEVHelper::getAid($user) . ')';
			$where[] = "(atd.sessionaccess=-1 OR atd.sessionaccess " . ' IN (' . JEVHelper::getAid($user) . ')' .")";

			$query = "SELECT count(distinct CASE atd.allrepeats WHEN 1 THEN -1 * ev.ev_id ELSE rpt.rp_id END)"
				. "\n FROM #__jevents_vevent AS ev "
				. "\n LEFT JOIN #__jevents_repetition as rpt ON rpt.eventid=ev.ev_id"
				. "\n LEFT JOIN #__jevents_vevdetail as det ON ev.detail_id=det.evdet_id"
				. "\n LEFT JOIN #__jev_attendance AS atd ON atd.ev_id = ev.ev_id"
				. "\n LEFT JOIN #__jev_invitees AS inv ON atd.id = inv.at_id  AND ((inv.rp_id = 0 AND atd.allrepeats = 1) OR (inv.rp_id = rpt.rp_id AND atd.allrepeats = 0)) "
				. "\n LEFT JOIN #__jev_attendeecount AS atdc ON atd.id = atdc.at_id AND ((atdc.rp_id = 0 AND atd.allrepeats = 1) OR (atdc.rp_id = rpt.rp_id AND atd.allrepeats = 0))"
				. "\n LEFT JOIN #__jev_attendees AS atdees ON atdees.at_id = atd.id AND ((atdees.rp_id = 0 AND atd.allrepeats = 1) OR (atdees.rp_id = rpt.rp_id AND atd.allrepeats = 0))"
				. ( count( $join) ? "\n LEFT JOIN  " . implode( ' LEFT JOIN ', $join) : '' )
				. ( count( $where ) ? "\n WHERE " . implode( ' AND ', $where ) : '' );
			$db->setQuery( $query);
			try {
				$total = $db->loadResult();
			}
			catch (RuntimeException $e)
			{
				$error = $e->getMessage();
				//echo $error;
				$total = 0;
			}

			if( $limit > $total ) {
				$limitstart = 0;
			}

			$query = "SELECT ev.ev_id as evd,
				CASE atd.allrepeats WHEN 1 THEN -1 * ev.ev_id ELSE rpt.rp_id END AS specialid,
				CASE atd.allrepeats WHEN 1 THEN 0 ELSE rpt.rp_id END AS repeat_id,
			    ev.state as evstate, det.*, atd.* ,atd.id as atd_id, atdc.atdcount, atdc.gucount,
				count(distinct inv.id) as invcount,
				sum(atdees.atdcount) as waitingcount,
				rpt.startrepeat as startdate"
				. "\n FROM #__jevents_vevent as ev "
				. "\n LEFT JOIN #__jevents_repetition as rpt ON rpt.eventid = ev.ev_id"
				. "\n LEFT JOIN #__jevents_exception as exc ON exc.rp_id = rpt.rp_id"
				. "\n LEFT JOIN #__jevents_vevdetail as det ON ev.detail_id = det.evdet_id"
				. "\n LEFT JOIN #__jev_attendance AS atd ON atd.ev_id = ev.ev_id"
				// Note we use the CASE defined repeat_id value here
				. "\n LEFT JOIN #__jev_invitees AS inv ON atd.id = inv.at_id  AND ((inv.rp_id = 0 AND atd.allrepeats = 1) OR (inv.rp_id = rpt.rp_id AND atd.allrepeats = 0)) "
				. "\n LEFT JOIN #__jev_attendeecount AS atdc ON atd.id = atdc.at_id AND  ((atdc.rp_id = 0 AND atd.allrepeats = 1) OR (atdc.rp_id = rpt.rp_id AND atd.allrepeats = 0))"
				. "\n LEFT JOIN #__jev_attendees AS atdees ON atdees.at_id = atd.id AND  ((atdees.rp_id = 0 AND atd.allrepeats = 1) OR (atdees.rp_id = rpt.rp_id AND atd.allrepeats = 0))"
				. ( count( $join) ? "\n LEFT JOIN  " . implode( ' LEFT JOIN ', $join) : '' )
				. ( count( $where ) ? "\n WHERE " . implode( ' AND ', $where ) : '' )
				. "\n GROUP BY specialid $orderby"
				//. "\n GROUP BY rpt.rp_id $orderby"
			;

			if ($limit>0){
				$query .= "\n LIMIT $limitstart, $limit";
			}
			$db->setQuery( $query );

			try {
				$rows = $db->loadObjectList();
			}
			catch (RuntimeException $e)
			{
				$error = $e->getMessage();
				echo $error;
				$rows = array();
			}
//			var_dump($rows);

			foreach ($rows as $key=>$val) {
				// Must refetch the waiting attendee count because its messed up when invitees are also present!
				$query = "SELECT sum(atdees.atdcount) as waitingcount "
					. "\n FROM #__jev_attendees AS atdees WHERE atdees.at_id = $val->atd_id AND atdees.rp_id=$val->repeat_id AND atdees.waiting=1";
				$db->setQuery( $query );

				try {
					$waitingcount = $db->loadResult();
				}
				catch (RuntimeException $e)
				{
					$error = $e->getMessage();
					echo $error;
					$waitingcount = 0;
				}

				if (!is_null($waitingcount)) {
					$rows[$key]->waitingcount = $waitingcount;
				}
				else {
					$rows[$key]->waitingcount = 0;
				}

				// set state variable to the event value not the event detail value
				$rows[$key]->state = $rows[$key]->evstate;

				// Find the first repeat
				if ($val->repeat_id > 0)
				{
					$vevent = $this->dataModel->queryModel->listEventsById( $val->repeat_id, false, "icaldb");
				}
				else
				{
					$vevent = $this->dataModel->queryModel->getEventById(  $val->ev_id, false, "icaldb");
				}
				// trap orphans!
				if (is_null($vevent)) {
					unset($rows[$key]);
					continue;
				}

				if ($val->repeat_id > 0)
				{
					$repeat = $vevent;
				}
				else
				{
					$repeat = $vevent->getFirstRepeat();
				}

				$rows[$key]->repeat = $repeat;
				$rows[$key]->starttime = $repeat->getUnixStartTime();
			}
			if ($total>count($rows)  && $limit>$total){
				$total = count($rows);
			}


		}
		// first the whole event sessions
		else if ($nonrepeating){
			if ($hidepast){
				if (class_exists("JevDate")) {
					$datenow = JevDate::getDate("-1 day");
				}
				else {
					$datenow = Factory::getDate("-1 day");
				}
				$where[] = "\n (det.dtstart>'".$datenow->toUnix()."' OR atd.regclose>".$db->quote($datenow->toSql()).")";
			}

			$where[] = "atd.allrepeats=1 AND atd.allowregistration > 0 ";
			$where[] = "ev.ev_id is not null ";
			// fix the access
			$where[] = "ev.access " . ' IN (' . JEVHelper::getAid($user) . ')' ;
			$where[] = "(atd.sessionaccess=-1 OR atd.sessionaccess " .  ' IN (' . JEVHelper::getAid($user) . ')' .")";

			$query = "SELECT count(distinct ev.ev_id)"
				. "\n FROM #__jevents_vevent AS ev "
				. "\n LEFT JOIN #__jevents_repetition as rpt ON rpt.eventid=ev.ev_id"
				. "\n LEFT JOIN #__jevents_vevdetail as det ON ev.detail_id=det.evdet_id"
				. "\n LEFT JOIN #__jev_attendance AS atd ON atd.ev_id = ev.ev_id"
				. "\n LEFT JOIN #__jev_invitees AS inv ON atd.id = inv.at_id  AND inv.rp_id=0"
				. "\n LEFT JOIN #__jev_attendees AS atdees ON atdees.at_id = atd.id AND atdees.rp_id=0"
				. "\n LEFT JOIN #__jev_attendeecount AS atdc ON atd.id = atdc.at_id AND atdc.rp_id=0"
				. ( count( $join) ? "\n LEFT JOIN  " . implode( ' LEFT JOIN ', $join) : '' )
				. ( count( $where ) ? "\n WHERE " . implode( ' AND ', $where ) : '' );
			$db->setQuery( $query);

			try {
				$total = $db->loadResult();
			}
			catch (RuntimeException $e)
			{
				$error = $e->getMessage();
				echo $error;
				$total = 0;
			}

			if( $limit > $total ) {
				$limitstart = 0;
			}

			// WE CAN'T GET INIVTEE COUNT and waiting count at the same time !!!
			$query = "SELECT ev.ev_id as evd, ev.state as evstate, det.*, atd.* ,atd.id as atd_id, atdc.atdcount, atdc.gucount,
				count(distinct inv.id) as invcount,
				sum(atdees.atdcount) as waitingcount, det.dtstart as startdate"
				. ", 0 as rp_id "
				. "\n FROM #__jevents_vevent as ev "
				. "\n LEFT JOIN #__jevents_repetition as rpt ON rpt.eventid=ev.ev_id"
				. "\n LEFT JOIN #__jevents_vevdetail as det ON ev.detail_id=det.evdet_id"
				. "\n LEFT JOIN #__jev_attendance AS atd ON atd.ev_id = ev.ev_id"
				. "\n LEFT JOIN #__jev_invitees AS inv ON atd.id = inv.at_id  AND inv.rp_id=0"
				. "\n LEFT JOIN #__jev_attendees AS atdees ON atdees.at_id = atd.id AND atdees.rp_id=0"
				. "\n LEFT JOIN #__jev_attendeecount AS atdc ON atd.id = atdc.at_id AND atdc.rp_id=0"
				. ( count( $join) ? "\n LEFT JOIN  " . implode( ' LEFT JOIN ', $join) : '' )
				. ( count( $where ) ? "\n WHERE " . implode( ' AND ', $where ) : '' )
				. "\n GROUP BY ev.ev_id $orderby"
			;

			if ($limit>0){
				$query .= "\n LIMIT $limitstart, $limit";
			}
			$db->setQuery( $query );

			try {
				$rows = $db->loadObjectList();
			}
			catch (RuntimeException $e)
			{
				$error = $e->getMessage();
				echo $error;
				$rows = array();
			}

			foreach ($rows as $key=>$val) {

				// Must refetch the waiting attendee count because its messed up when invitees are also present!
				//$query = "SELECT count(atdees.waiting) as waitingcount "
				$query = "SELECT sum(atdees.atdcount) as waitingcount "
					. "\n FROM #__jev_attendees AS atdees WHERE atdees.at_id = $val->atd_id AND atdees.rp_id=0 AND atdees.waiting=1"
				;
				$db->setQuery( $query );

				try {
					$waitingcount = $db->loadResult();
				}
				catch (RuntimeException $e)
				{
					$error = $e->getMessage();
					echo $error;
					$waitingcount = 0;
				}

				if (!is_null($waitingcount)) {
					$rows[$key]->waitingcount = $waitingcount;
				}
				else {
					$rows[$key]->waitingcount = 0;
				}

				// set state variable to the event value not the event detail value
				$rows[$key]->state = $rows[$key]->evstate;

				// Find the first repeat
				$vevent = $this->dataModel->queryModel->getEventById(  $val->ev_id, false, "icaldb");
				// trap orphans!
				if (is_null($vevent)) {
					unset($rows[$key]);
					continue;
				}
				$repeat = $vevent->getFirstRepeat();

				$rows[$key]->repeat = $repeat;
				$rows[$key]->starttime = $repeat->getUnixStartTime();
			}
			if ($total>count($rows) && $limit>$total){
				$total = count($rows);
			}
		}
		else {
			if ($hidepast){
				if (class_exists("JevDate")) {
					$datenow = JevDate::getDate("-1 day");
					$where[] = "\n rpt.endrepeat>'".$datenow->toMySQL()."'";
				}
				else {
					$datenow = Factory::getDate("-1 day");
					$where[] = "\n rpt.endrepeat>'".$datenow->toSql()."'";
				}
			}

			$where[] = "atd.allrepeats=0 AND atd.allowregistration>0";
			// fix the access
			$where[] = "ev.access " .  ' IN (' . JEVHelper::getAid($user) . ')';
			$where[] = "(atd.sessionaccess=-1 OR atd.sessionaccess " . ' IN (' . JEVHelper::getAid($user) . ')' .")";

			$query = "SELECT count(distinct rpt.rp_id)"
				. "\n FROM #__jevents_vevent AS ev "
				. "\n LEFT JOIN #__jevents_repetition as rpt ON rpt.eventid=ev.ev_id"
				. "\n LEFT JOIN #__jevents_vevdetail as det ON ev.detail_id=det.evdet_id"
				. "\n LEFT JOIN #__jev_attendance AS atd ON atd.ev_id = ev.ev_id"
				. "\n LEFT JOIN #__jev_invitees AS inv ON atd.id = inv.at_id  AND inv.rp_id=rpt.rp_id"
				. "\n LEFT JOIN #__jev_attendeecount AS atdc ON atd.id = atdc.at_id AND atdc.rp_id=rpt.rp_id"
				. "\n LEFT JOIN #__jev_attendees AS atdees ON atdees.at_id = atd.id AND atdees.rp_id=rpt.rp_id"
				. ( count( $join) ? "\n LEFT JOIN  " . implode( ' LEFT JOIN ', $join) : '' )
				. ( count( $where ) ? "\n WHERE " . implode( ' AND ', $where ) : '' );
			$db->setQuery( $query);
			try {
				$total = $db->loadResult();
			}
			catch (RuntimeException $e)
			{
				$error = $e->getMessage();
				//echo $error;
				$total = 0;
			}
			if( $limit > $total ) {
				$limitstart = 0;
			}

			$query = "SELECT rpt.rp_id,  ev.state as evstate, det.*, atd.* ,atd.id as atd_id, atdc.atdcount, atdc.gucount,
				count(distinct inv.id) as invcount,
				sum(atdees.atdcount) as waitingcount,
				rpt.startrepeat as startdate"
				. ", rpt.rp_id "
				. "\n FROM #__jevents_vevent as ev "
				. "\n LEFT JOIN #__jevents_repetition as rpt ON rpt.eventid=ev.ev_id"
				. "\n LEFT JOIN #__jevents_exception as exc ON exc.rp_id=rpt.rp_id"
				. "\n LEFT JOIN #__jevents_vevdetail as det ON ev.detail_id=det.evdet_id"
				. "\n LEFT JOIN #__jev_attendance AS atd ON atd.ev_id = ev.ev_id"
				. "\n LEFT JOIN #__jev_invitees AS inv ON atd.id = inv.at_id  AND inv.rp_id=rpt.rp_id"
				. "\n LEFT JOIN #__jev_attendeecount AS atdc ON atd.id = atdc.at_id AND atdc.rp_id=rpt.rp_id"
				. "\n LEFT JOIN #__jev_attendees AS atdees ON atdees.at_id = atd.id AND atdees.rp_id=rpt.rp_id"
				. ( count( $join) ? "\n LEFT JOIN  " . implode( ' LEFT JOIN ', $join) : '' )
				. ( count( $where ) ? "\n WHERE " . implode( ' AND ', $where ) : '' )
				. "\n GROUP BY rpt.rp_id $orderby"
			;

			if ($limit>0){
				$query .= "\n LIMIT $limitstart, $limit";
			}
			$db->setQuery( $query );

			try {
				$rows = $db->loadObjectList();
			}
			catch (RuntimeException $e)
			{
				$error = $e->getMessage();
			//	echo $error;
				$rows = array();
			}

			foreach ($rows as $key=>$val) {
				// Must refetch the waiting attendee count because its messed up when invitees are also present!
				$query = "SELECT sum(atdees.atdcount) as waitingcount "
					. "\n FROM #__jev_attendees AS atdees WHERE atdees.at_id = $val->atd_id AND atdees.rp_id=$val->rp_id AND atdees.waiting=1"
				;
				$db->setQuery( $query );

				try {
					$waitingcount = $db->loadResult();
				}
				catch (RuntimeException $e)
				{
					$error = $e->getMessage();
					echo $error;
					$waitingcount = 0;
				}

				if (!is_null($waitingcount)) {
					$rows[$key]->waitingcount = $waitingcount;
				}
				else {
					$rows[$key]->waitingcount = 0;
				}

				// set state variable to the event value not the event detail value
				$rows[$key]->state = $rows[$key]->evstate;

				// Find the first repeat
				$repeat = $this->dataModel->queryModel->listEventsById(  $val->rp_id, false, "icaldb");
				// trap orphans!
				if (is_null($repeat)) {
					unset($rows[$key]);
					continue;
				}

				$rows[$key]->repeat = $repeat;
				$rows[$key]->starttime = $repeat->getUnixStartTime();
			}
			if ($total>count($rows)  && $limit>$total){
				$total = count($rows);
			}
		}

		// reset for any missing orphan rows
		$rows = array_values($rows);

		// get list of creators
		$sql = "SELECT distinct u.id, u.* FROM #__jevents_vevent as jev LEFT JOIN #__users as u on u.id=jev.created_by";
		$db= Factory::getDBO();
		$db->setQuery( $sql );
		$users = $db->loadObjectList();
		$userOptions = array();
		$userOptions[] = HTMLHelper::_('select.option', 0, Text::_("JEV_EVENT_CREATOR") );
		foreach( $users as $user )
		{
			$userOptions[] = HTMLHelper::_('select.option', $user->id, $user->name. " ($user->username)" );
		}
		$userlist = HTMLHelper::_('select.genericlist', $userOptions, 'created_by', 'class="inputbox" size="1"  onchange="document.adminForm.submit();"', 'value', 'text', $created_by);

		if ($whichsessions=="SBD" || $whichsessions=="RBD"  || $whichsessions=="ABD" ) {
			$options = array();
			$options[] = HTMLHelper::_('select.option', '-1', Text::_('RSVP_ALL_EVENTS'));
			$options[] = HTMLHelper::_('select.option', '0', Text::_('RSVP_SEPARATE_REPEATS'));
			$options[] = HTMLHelper::_('select.option', '1', Text::_('RSVP_SINGLE_EVENTS'));
			$repeattypelist = HTMLHelper::_('select.genericlist', $options, 'repeating', 'class="inputbox" size="1" onchange="document.adminForm.submit();"', 'value', 'text', $nonrepeating );
		}
		else {
			$repeattypelist = false;
		}

		$options = array();
		$options[] = HTMLHelper::_('select.option', '0', Text::_('RSVP_NO'));
		$options[] = HTMLHelper::_('select.option', '1', Text::_('RSVP_YES'));
		$hidepast = HTMLHelper::_('select.genericlist', $options, 'hidepast', 'class="inputbox" size="1" onchange="document.adminForm.submit();"', 'value', 'text', $hidepast );

		// get list of categories
		$attribs = 'class="inputbox" size="1" onchange="document.adminForm.submit();"';
		$showUnpublishedCategories=false;
		$clist = JEventsHTML::buildCategorySelect($catid, $attribs, null, $showUnpublishedCategories, false, $catidtop, "catid");

		jimport('joomla.html.pagination');
		$pageNav = new Pagination( $total, $limitstart, $limit  );

		// Set the layout
		$this->view->setLayout('overview');

		$this->view->rows = $rows;
		$this->view->after = $after;
		$this->view->order = $filter_order;
		$this->view->orderdir = $filter_order_Dir;
		$this->view->userlist = $userlist;
		$this->view->catlist = $clist;
		$this->view->hidepast = $hidepast;
		$this->view->hidepast = $hidepast;
		$this->view->nonrepeating = $nonrepeating;
		$this->view->repeattypelist = $repeattypelist;
		$this->view->search = $search;
		$this->view->searchattendees = $searchattendees;
		$this->view->searchinvitees = $searchinvitees;
		$this->view->pageNav = $pageNav;

		$this->view->display();
	}

	function delete(){
		echo "Not implemented yet";return;
	}

}
