<?php

// no direct access
defined('_JEXEC') or die('Restricted access');

jimport('joomla.plugin.plugin');
jimport('joomla.log.log');

// Load QuickPay classes
//JLoader::registerNamespace('QuickPay\API', __DIR__ . '/');
//JLoader::registerNamespace('QuickPay', __DIR__ . '/');

// Load Omnipay autoloader
require JPATH_SITE . '/libraries/jev_omnipay/vendor/autoload.php';

// Use Money
use Joomla\Registry\Registry;
use Joomla\CMS\Language\Text;
use Joomla\CMS\Component\ComponentHelper;
use Joomla\CMS\Plugin\CMSPlugin;
use Joomla\CMS\Plugin\PluginHelper;
use Joomla\CMS\Router\Route;
use Joomla\CMS\Log\Log;
use Joomla\CMS\Uri\Uri;
use Joomla\CMS\Session\Session;
use Brick\Money\Money;

// Use Quickpay API
use Omnipay\Omnipay;

// Use Joomla Factory
use Joomla\CMS\Factory;


class plgRsvpproQuickpay extends CMSPlugin
{
	protected $autoloadLanguage = true;
		
	public function __construct(& $subject, $config)
	{

		// Enable Logging
		Log::addLogger(
			array(
				'text_file' => 'com_rsvppro.quickpay.log.php'
			),
			Log::ALL,
			array('com_rsvppro.quickpay')
		);

		parent::__construct($subject, $config);

	}


	/**
	 * Output sanitized order prefix: only alpha and - and maximum 10 characters long
	 *
	 * @return string
	 */

	private function _sanitizedPrefix($prefix){
		return substr(preg_replace('/[^-a-zA-Z0-9]/','',$prefix),0,10);
	}



	public function generatePaymentPage(&$html, $attendee, $rsvpdata, $event, &$transaction)
	{
		$plugin = PluginHelper::getPlugin("rsvppro", "quickpay");
		$params = new Registry($plugin->params);

		$jinput = Factory::getApplication()->input;

		$invoice = "-" . $jinput->getString("invoiceid", '') . '_' . $transaction->transaction_id;

		$detaillink = Route::_(Uri::root() . $event->viewDetailLink($event->yup(), $event->mup(), $event->dup(), false), false);

		$templateParams = RsvpHelper::getRawTemplateParams($rsvpdata);

		/* Test that there are outstanding fees so we don't double bill */
		if (isset($attendee->outstandingBalances) === false || round($attendee->outstandingBalances['feebalance'], 2) <= round(0, 2)) {
			/* There really is nothing owed */
			JLog::add("Transaction ".$transaction->transaction_id ." attempted payment in the amount of ".self::phpMoneyFormat($transaction->amount,  $templateParams)
					." for ". JFactory::getUser($attendee->user_id)->name . " when no balance is outstanding.",
					JLog::ERROR, 'com_rsvppro.quickpay');
			$app = JFactory::getApplication();
			$app->enqueueMessage(JText::_("RSVP_PRO_QUICKPAY_NO_OUTSTANDING_BALANCE"), 'message');
			$app->redirect($detaillink);
			return;
 		}


		$order_id = $event->yup() . $event->mup() .  $event->dup() . $this->_sanitizedPrefix( substr($event->title(), 0, 5) . $templateParams->get('qpa_order_prefix', '')) . $invoice;

		$comparams = ComponentHelper::getParams("com_rsvppro");
		$user      = JEVHelper::getUser($attendee->user_id);
		if ((int) $user->id === 0 && $comparams->get("attendemails", 0))
		{
			$name     = $attendee->email_address;
			$username = $attendee->email_address;
			$email = $attendee->email_address;
		}
		else
		{
			$name     = $user->name;
			$username = $user->username;
			$email = $user->email;
		}

		// Load the attendee helper
		include_once(JPATH_ADMINISTRATOR . "/components/com_rsvppro/libraries/attendeehelper.php");

		$helper = new RsvpAttendeeHelper($comparams);

		$currency   = $templateParams->get('Currency', "GBP");
		$money = Money::of($jinput->get('amount'), $currency);
		$start_date = JEventsHTML::getDateFormat($event->yup(), $event->mup(), $event->dup(), 0);
		$apikey = $templateParams->get('qpa_live_sk_apikey');

		$mode_val = (int) $templateParams->get('QuickpayMode');

		$defaultQuickpayPaymentLayout = "Invoiceid = {INVOICEID}<br/>Total Fees = {TOTALFEES}<br/>Fees Already Paid= {FEESPAID}<br/>Outstanding Balance = {BALANCE}<br/>";
		$html = $templateParams->get("qpa_template", $defaultQuickpayPaymentLayout);
		$html = $helper->parseMessage($html, $rsvpdata, $event, $name, $username, $attendee, false, false, $transaction);

		// setup transaction data
		$transaction->amount = (string)$money->getAmount();
		$transaction->currency = $currency;
		$transaction->attendee_id = $attendee->id;
		$transaction->gateway = 'quickpay';
		$transaction->gw_refundable = $templateParams->get('qpa_process_refunds', 0);
		$transaction->params = new stdClass();
		$transaction->params->apikey = $apikey;
		$transaction->params->detaillink = Route::_(Uri::root() . $event->viewDetailLink($event->yup(), $event->mup(), $event->dup(), false), false);
		$transaction->params->notify_payment = $templateParams->get('notifyqpapay', 0);
		$transaction->params->sk_apikey =  $templateParams->get('qpa_live_sk_apikey');

		$returnUrl = JURI::root() . "index.php?option=com_rsvppro&task=accounts.notify&gateway=quickpay&transID=" . $transaction->transaction_id . '&status=';

		$variables = [];
		if ($templateParams->get('qpa_variable_eventname') == 1) $variables[Text::_("RSVP_PRO_QUICKPAY_VARIABLES_EVENTNAME_LABEL")] = $event->title();
		if ($templateParams->get('qpa_variable_eventdate') == 1) $variables[Text::_("RSVP_PRO_QUICKPAY_VARIABLES_EVENTDATE_LABEL")] = $event->yup()."-".$event->mup()."-".$event->dup()." ".$event->hup().":".$event->minup().":".$event->sup();
		if ($attendee->user_id) {
			if ($templateParams->get('qpa_variable_attendee') == 1) $variables[Text::_("RSVP_PRO_QUICKPAY_VARIABLES_ATTENDEE_LABEL")] = Factory::getUser($attendee->user_id)->name;
			if ($templateParams->get('qpa_variable_email') == 1) $variables[Text::_("RSVP_PRO_QUICKPAY_VARIABLES_EMAIL_LABEL")] = Factory::getUser($attendee->user_id)->email;
		} else {
			if ($templateParams->get('qpa_variable_email') == 1) $variables[Text::_("RSVP_PRO_QUICKPAY_VARIABLES_EMAIL_LABEL")] = $attendee->email_address;
		}
		if ($templateParams->get('qpa_variable_sessionid') == 1) $variables[Text::_("RSVP_PRO_QUICKPAY_VARIABLES_SESSIONID_LABEL")] = $rsvpdata->id;

		// Quickpay Params
		$qpa = [
			"order_id" => $order_id,
			"transactionId" => $invoice,
			"apikey" => $apikey,
			"privatekey" => $templateParams->get('qpa_live_pk_apikey', ""),
			"amount" => (string)$money->getAmount(),
			"notifyUrl" => $returnUrl.'Completed',
			"cancelUrl" => $returnUrl.'Cancelled',
			"returnUrl" => $returnUrl.'Continue',
			"currency" => $currency,
			"detaillink" => $detaillink,
			"merchant" => (int)$templateParams->get('qpa_merchant_id', 0),
			"agreement" => (int)$templateParams->get('qpa_agreement_id', 0),
			"payment_methods" => $templateParams->get('qpa_payment_methods', "creditcard"),
			"description" => htmlspecialchars($event->title . " - " . $start_date),
			"item_name" => htmlspecialchars($event->title . " - " . $start_date),
			"auto_capture" => true,
			"variables" => $variables,
		];

		try
		{
			//Initialize client
			$gateway = Omnipay::create('Quickpay');
			$gateway->initialize($qpa);
			$response = $gateway->link($qpa)->send(); 

			if ($response->isSuccessful() !== true) {
				throw new \Exception(Text::_("RSVP_PRO_QUICKPAY_LINK_ERROR"));
			}

			$qpa["transactionReference"] = $response->getTransactionReference();
			$transaction->params->qpaData = $qpa;
			$transaction->params = json_encode($transaction->params);
			// Save the transaction data
			$transaction->store();

			//Autoredirect to paymentpage ?
			if($templateParams->get('autoredirect', 0) != 0) {
				$html .= "<script type='text/javascript'>function sendToQuickPay(){window.location.href = '".$response->getRedirectUrl()."'}; setTimeout('sendToQuickPay()',3000);</script>";
			}
			$html = str_replace("{INVOICEID}", $invoice, $html);
			$html = str_replace("{TOTALFEES}", self::phpMoneyFormat($attendee->outstandingBalances['totalfee'], $templateParams), $html);
			$html = str_replace("{FEESPAID}", self::phpMoneyFormat($attendee->outstandingBalances['feepaid'], $templateParams), $html);
			$html = str_replace("{BALANCE}", self::phpMoneyFormat($attendee->outstandingBalances['feebalance'], $templateParams), $html);
            $html = str_replace("{RAWBALANCE}", $attendee->outstandingBalances['feebalance'], $html);
			$html = str_replace("{DEPOSIT}",self::phpMoneyFormat($attendee->outstandingBalances['deposit'], $templateParams), $html);
			$html .= "<br/><form action='".$response->getRedirectUrl()."'><input type='submit' value='".JText::_('RSVP_PRO_QUICKPAY_PROCEED_TO_QUICKPAY')."'/></form>";
			JLog::add(
				'Quickpay payment page created for user ' . $user->name . ' for ' . self::phpMoneyFormat($transaction->amount, $templateParams)
				. ", Transaction ID: " . $transaction->transaction_id, JLog::INFO, 'com_rsvppro.quickpay'
			);
		} catch (\Exception $e)
		{
			$transError = sprintf(Text::_("RSVP_PRO_QUICKPAY_TRANSACTION_ERROR"), $e->getMessage());
			Factory::getApplication()->enqueueMessage($transError, 'error');
			Log::add($message . ", Transaction ID: " . $transaction->transaction_id, Log::ERROR, 'com_rsvppro.quickpay');
			Factory::getApplication()->redirect($qpa['detaillink']);
		}
	}

	static public function notifyRsvpGateway(&$args)
	{ 
		$input = Factory::getApplication()->input;
	
		$transID = $input->get("transID");
		$transaction = new rsvpTransaction();
		$transaction->load($transID);
		$transParams = json_decode($transaction->params);

		$qpa = $transParams->qpaData;

		$status = $input->get('status', 'unknown');

		// Send confirmation email setting, set false unless payment is successful.
		$send_cm = false;
		
		switch ($status) {
			case 'Pending':
				// pending sale flow
				$message = 'Transaction is Pending, reason: ' ; //TODO Missing reason for pending state
				$transaction->logdata = '|'.date("D M j G:i").':'.$message; // Any message the gateway his given us.
				Log::add($message . ", Transaction ID: " . $transaction->transaction_id, Log::INFO, 'com_rsvppro.quickpay');
				break;
			case 'Completed':
				// Ignore the completed state, we will verify in the continue
				//Log::add('Received Completed callback from omnipay' . ", Transaction ID: " . $transaction->transaction_id, Log::INFO, 'com_rsvppro.quickpay');
				return true;
			case 'Continue':
				// get the actual status
				//Initialize client
				$gateway = Omnipay::create('Quickpay');
				$gateway->initialize((array)$qpa);
				$response = $gateway->status(["transactionReference" => $qpa->transactionReference])->send();

				if ($response->isSuccessful() !== true) {
					$message = "There was an error, QuickPay responded with: " . $response->getMessage();  
					$transaction->logdata .= '|'.date("D M j G:i").':'.$message; // Any message the gateway his given us.
					$transError = sprintf(Text::_("RSVP_PRO_QUICKPAY_TRANSACTION_ERROR"), $response->getMessage());
					Factory::getApplication()->enqueueMessage($transError, 'error');
					Log::add($message . ", Transaction ID: " . $transaction->transaction_id, Log::ERROR, 'com_rsvppro.quickpay');
				} else {
					// normal successful flow
					$message = 'Payment was Successful.';
					Log::add($message . " Transaction ID: " . $transaction->transaction_id, Log::INFO, 'com_rsvppro.quickpay');
					$transaction->logdata  .= '|'.date("D M j G:i").':'.$message;
					$transaction->paymentstate = 1;
					$send_cm = true;
				}
				break;
			case 'Cancelled':
				// User has cancelled the payment on the Quickpay form
				$message = 'Transaction cancelled by user';
				Factory::getApplication()->enqueueMessage($message, 'error');
				$transaction->logdata = '|'.date("D M j G:i").':'.$message; // Any message the gateway his given us.
				Log::add($message . ", Transaction ID: " . $transaction->transaction_id, Log::INFO, 'com_rsvppro.quickpay');
				break;
		}

		// Upodate the transaction for any new payment state and log data
		$transaction->store();

		// Send Payment Confirmation
		if ($send_cm)
		{
			// fetch the session and the attendee
			$db = Factory::getDbo();
			$sql = "SELECT * FROM #__jev_attendees WHERE id=" . $transaction->attendee_id;
			$db->setQuery($sql);
			$attendee = $db->loadObject();

			$sql = "SELECT * FROM #__jev_attendance WHERE id=" . $attendee->at_id;
			$db->setQuery($sql);
			$rsvpdata = $db->loadObject();

			plgRsvpproQuickPay::notifyQuickPayPayment($transaction, $attendee, $rsvpdata, true);
		}

		Factory::getApplication()->redirect($qpa->detaillink);
	}


	private function notifyQuickPayRefund($transaction, $attendee, $rsvpdata)
	{ 
		$plugin = PluginHelper::getPlugin("rsvppro", "quickpay");
		$params = new Registry($plugin->params);

		$templateParams = RsvpHelper::getRawTemplateParams($rsvpdata);

		/* set the templateParams into the rsvpdata for the notifyUser */
		$rsvpdata->templateParams = $templateParams;
		// immediate notification
		if (($templateParams->get("notifyqparef", 1) == 1) && $transaction->gateway == "quickpay")
		{

			$comparams = ComponentHelper::getParams("com_rsvppro");
			include_once(JPATH_ADMINISTRATOR . "/components/com_rsvppro/libraries/attendeehelper.php");
			$helper = new RsvpAttendeeHelper($comparams);

			$rpid = $attendee->rp_id;

			$dataModel  = new JEventsDataModel();
			$queryModel = new JEventsDBModel($dataModel);

			// Find the first repeat
			$vevent = $dataModel->queryModel->getEventById($rsvpdata->ev_id, false, "icaldb", false);
			if ($rpid == 0)
			{
				$repeat = $vevent->getFirstRepeat();
			}
			else
			{
				list($year, $month, $day) = JEVHelper::getYMD();
				$repeatdata = $dataModel->getEventData(intval($rpid), "icaldb", $year, $month, $day);
				if ($repeatdata && isset($repeatdata["row"]))
					$repeat = $repeatdata["row"];
			}

			$user = JEVHelper::getUser($attendee->user_id);
			if ($user->id == 0 && $comparams->get("attendemails", 0))
			{
				$name     = $attendee->email_address;
				$username = $attendee->email_address;
			}
			else
			{
				$name     = $user->name;
				$username = $user->username;
			}

			$class      = "plgRsvppro" . ucfirst($transaction->gateway);
			$pluginpath = version_compare(JVERSION, "1.6.0", 'ge') ? JPATH_SITE . "/plugins/rsvppro/$transaction->gateway/" : JPATH_SITE . "/plugins/rsvppro/";
			JLoader::register($class, $pluginpath . $transaction->gateway . ".php");

			$helper->notifyUser($rsvpdata, $repeat, $user, $name, $username, $attendee, 'qparef', false, $transaction);
		}
	}


	protected function QuickPay_Message($text, $email)
	{
		plgRsvpproQuickPay::FixEmail($email);
		if ($email == '')
		{
			return;
		}
		$mail = Factory::getMailer();

		try {
		return $mail->sendMail($email, 'quickpay log', $email, 'QuickPay Message', $text, 0);
		}
		catch (Exception $e)
		{
			return false;
		}
	}

	protected function QuickPay_Error($text, $email)
	{
		plgRsvpproQuickPay::FixEmail($email);
		if ($email == '')
		{
			return;
		}
		$mail = Factory::getMailer();

		try {
		return $mail->sendMail($email, 'quickpay log', $email, 'QuickPay Error', $text, 0);
		}
		catch (Exception $e)
		{
			return false;
		}
	}

	protected static function FixEmail(& $email)
	{
		if ($email == '')
		{
			// get Jevents params and send messages to admin if necessary
			$params = ComponentHelper::getParams("com_jevents");
			$jevadmin = $params->get("jevadmin", -1);
			if ($jevadmin == -1)
			{
				return;
			}
			$jevadmin = JEVHelper::getUser($jevadmin);
			$email = $jevadmin->email;
		}
	}

	private static function notifyQuickPayPayment($transaction, $attendee, $rsvpdata, $aftertransation = false)
	{
		
		$templateParams = RsvpHelper::getRawTemplateParams($rsvpdata);

		// immediate notification
		if (((!$aftertransation && $templateParams->get("notifyqpapay", 1) == 2) || ($aftertransation && $templateParams->get("notifyqpapay", 1) == 1)) && $transaction->gateway == "quickpay")
		{

			$comparams = ComponentHelper::getParams("com_rsvppro");
			include_once(JPATH_ADMINISTRATOR . "/components/com_rsvppro/libraries/attendeehelper.php");
			$helper = new RsvpAttendeeHelper($comparams);

			$rpid = $attendee->rp_id;

			$dataModel = new JEventsDataModel();
			$queryModel = new JEventsDBModel($dataModel);

			// Find the first repeat
			$vevent = $dataModel->queryModel->getEventById($rsvpdata->ev_id, false, "icaldb", false);
			if ($rpid == 0)
			{
				$repeat = $vevent->getFirstRepeat();
			}
			else
			{
				list($year, $month, $day) = JEVHelper::getYMD();
				$repeatdata = $dataModel->getEventData(intval($rpid), "icaldb", $year, $month, $day);
				if ($repeatdata && isset($repeatdata["row"]))
					$repeat = $repeatdata["row"];
			}

			$user = JEVHelper::getUser($attendee->user_id);
			if ($user->id == 0 && $comparams->get("attendemails", 0))
			{
				$name = $attendee->email_address;
				$username = $attendee->email_address;
			}
			else
			{
				$name = $user->name;
				$username = $user->username;
			}

			$class = "plgRsvppro" . ucfirst($transaction->gateway);
			$pluginpath = version_compare(JVERSION, "1.6.0", 'ge') ? JPATH_SITE . "/plugins/rsvppro/$transaction->gateway/" : JPATH_SITE . "/plugins/rsvppro/";
			JLoader::register($class, $pluginpath . $transaction->gateway . ".php");

			$helper->notifyUser($rsvpdata, $repeat, $user, $name, $username, $attendee, 'qpapay', false, $transaction);
		}
	}
	

	public function phpMoneyFormat($amount, $template)
	{
		if (isset($template->Currency))
		{
			$digits = $template->CurrencyDigits;
			$symbol = $template->CurrencySymbol;
			$onLeft = (strcmp($template->CurrencyPlacement, 'left') == 0);
			$separator = $template->CurrencySeparator;
			$decimal = $template->CurrencyDecimal;
		}
		else
		{
			$eSessParams = ComponentHelper::getParams('com_rsvppro');

			$digits = $eSessParams->get("CurrencyDigits");
			$symbol = $eSessParams->get("CurrencySymbol");
			$onLeft = (strcmp($eSessParams->get("CurrencyPlacement"), 'left') == 0);
			$separator = $eSessParams->get("CurrencySeparator");
			$decimal = $eSessParams->get("CurrencyDecimal");
		}
		$formattedText = '';

		// negative amount
		if ($amount < 0)
			$formattedText .= '-';

		// currency symbol on the left
		if ($onLeft)
			$formattedText .= $symbol;

		// format with correct number of digits and separator
		if ($digits > 0)
		{
			$amount = RsvpHelper::ceil_dec(abs($amount), $digits, ".");
			$formattedText .= number_format(abs($amount), $digits, $decimal, $separator);
		}
		else
		{
			$formattedText = abs($amount);
		}

		// currency symbols on the right
		if (!$onLeft)
			$formattedText .= $symbol;


		return $formattedText;
	}

	public static function NotifyPayment($templateParams)
	{
		return $templateParams->get("notifyqpapay", 1);
	}

	public static function PaymentMessageType()
	{
		return "qpapay";
	}

	static public function transactionDetailLink($transaction)
	{
		return "";
	}
	
	public static function usePluginSettingsName() {
			return "qpauseglobalsettings";
	}

	
	//Set gateway name
	public function gatewayName() {
		return "quickpay";
	}
	
	
	public function onRegistrationStatusChange($attendee, $eventrepeat, $rsvpdata) {
		$gateway = null;
		
		/* Check that we have feedata, this only happens when a payment has been processed or the attendee has cancelled */
		if (!isset($attendee->_feedata) || empty($attendee->_feedata)) {
			return;
		}

		/* check that the new status is not attending and that the attendee was confirmed */
		if ($attendee->attendstate != 0 || $attendee->confirmed != 1) {

			return;
		}

		$plugin = PluginHelper::getPlugin("rsvppro", "quickpay");
		$params = new Registry($plugin->params);

		$templateParams = RsvpHelper::getRawTemplateParams($rsvpdata);
		
		/* Check if the templateParams allows for refunds */ 
		if ($templateParams->get('qpa_process_refunds', 0) == 0) {

			return;
		}
		
		/* Calculate any refund end dates */
		$refunds = new StdClass;
		$refunds->refundForPeriod = $templateParams->get('qpa_refund_for_period', 0);
		if ($refunds->refundForPeriod > 0) {
			$feedata = $attendee->_feedata[0];
			$refunds->refundForPeriod = strtotime($feedata->transaction_date) + ($refunds->refundForPeriod * 24 * 60 * 60);
		}
		$refunds->refundBefore = $templateParams->get('qpa_refund_before_period', '');
		if (strlen($refunds->refundBefore) > 0 ) {
			$refunds->refundBefore = strtotime($refunds->refundBefore);
		} else {
			$refunds->refundBefore = 0;
		}
		/* get the most restrictive date */
		$refundTo = 0;
		foreach ($refunds as $refundType) {
			if ($refundType == 0) continue;
			if ($refundTo == 0) $refundTo = $refundType;
			$refundTo = min($refundTo, $refundType);
		}
		
		$refundAmount = 0;

		foreach ($attendee->_feedata as $feedata) {

			$transaction = new rsvpTransaction();
			$transaction->load($feedata->transaction_id);
			if (!isset($transaction->transaction_id)) {
				continue;	// Probably should never happen
			}
			if (!isset($transaction->gateway) 										
				|| strlen($transaction->gateway) != strlen(self::gatewayName()) 
				|| strpos($transaction->gateway, self::gatewayName()) !== 0) {
					continue; // Not a quickpay transaction
			}
			if (!isset($transaction->gw_refundable) || $transaction->gw_refundable == 0) {
				continue;	// Not refundable
			}
			if (!isset($transaction->paymentstate) || $transaction->paymentstate != 1) {
				/* Don't really think we need to log these */
/*				$message = 'Attempt to refund an unsuccessful transaction.';
				$transaction->logdata .= '|'.date("D M j G:i").': '.$message; 
				$transaction->store(false, false);
				$message .= ", Transaction ID: ".$transaction->transaction_id;
				Log::add($message, Log::INFO, 'com_rsvppro.quickpay');
*/
				continue;	// Only refund successful transactions
			}
			

			/* Determine if we are beyond our refund period */
			if ($refundTo > 0 &&  time() > $refundTo) {

				$message = "Refund period has expired.";
				$transaction->logdata .= '|'.date("D M j G:i").': '.$message; 
				$transaction->store(false, false);
				$message .= ", Transaction ID: ".$transaction->transaction_id;
				Log::add($message, Log::INFO, 'com_rsvppro.quickpay');
				return; // beyond refund period
			}

			$transParams = json_decode($transaction->params);
			$qpa = $transParams->qpaData;

			//Initialize client
			$gateway = Omnipay::create('Quickpay');
			$gateway->initialize((array)$qpa);

			$response = $gateway->refund(["amount" => $transaction->amount, "transactionReference" => $qpa->transactionReference])->send();

			if ($response->isSuccessful()) {
				$transaction->paymentstate = 0;
				$transaction->logdata .= '|'.date("D M j G:i").':Transaction refunded';
				$transaction->store(false, false);
				plgRsvpproQuickPay::notifyQuickPayRefund($transaction, $attendee, $rsvpdata);
				Log::add(
					"Transaction ".$transaction->transaction_id ." refunded in the amount of ".self::phpMoneyFormat($transaction->amount,"")
					." for ". Factory::getUser($attendee->user_id)->name,
					Log::INFO, 'com_rsvppro.quickpay'
				);
			} else {
				// normal failure flow
				$message = 'There was a refund error, QuickPay responded with: ' . $response->getMessage();
				$errors = $response->getErrors();
				if (!empty($errors)) {
					foreach($errors as $item => $error) {
						$message .= ", " . $item . ":";
						if (is_array($error)) {
							$message .= implode('-', $error);
						} else {
							$message .= $error;
						}
					}
				}
				$transaction->logdata .= ''.date("D M j G:i").':|'.$message; // Any message the gateway his given us.
				$transaction->store(false, false);
				Factory::getApplication()->enqueueMessage($message, 'error');
				$message .= ", Transaction ID: ".$transaction->transaction_id;
				Log::add($message, Log::ERROR, 'com_rsvppro.quickpay');
				return false;
			}
		}	
	}
	

	public function activeGatewayClass(&$activeGatewayClass, $action = "notify")
	{
		$input = Factory::getApplication()->input;
		$gateway = $input->get("gateway");

		if ($gateway == "quickpay" || $gateway == "1" || strpos($gateway, "quickpay_") === 0)
		{
			$activeGatewayClass = __CLASS__;
		}
	}

	public function activeGateways(&$activeGatewayClasses)
	{
		$activeGatewayClasses[] = __CLASS__;
	}
	
}
