<?php

/**
 * copyright (C) 2009 - 2021 GWE Systems Ltd - All rights reserved
 */
// no direct access
defined('_JEXEC') or die('Restricted access');

use Joomla\Registry\Registry;
use Joomla\CMS\MVC\View\HtmlView;
use Joomla\CMS\Language\Text;
use Joomla\CMS\Component\ComponentHelper;
use Joomla\CMS\Plugin\PluginHelper;
use Joomla\CMS\Cache\Cache;
use Joomla\CMS\Factory;
use Joomla\CMS\HTML\HTMLHelper;
use Joomla\CMS\Filesystem\File;

JLoader::register('JevRsvpInvitees', JPATH_ADMINISTRATOR . "/components/com_rsvppro/libraries/jevrinvitees.php");
JLoader::register('JevRsvpReminders', JPATH_ADMINISTRATOR . "/components/com_rsvppro/libraries/jevrreminders.php");
JLoader::register('JevRsvpAttendees', JPATH_ADMINISTRATOR . "/components/com_rsvppro/libraries/jevrattendees.php");

// WARNING - REDIRECTS MUST NOT BE XHTML COMPLIANT LINKS


class JevRsvpDisplayAttendance
{

	private $params;
	private $jomsocial = false;
	private $cbuilder = false;
	private $groupjive = false;
	private $event = null;
	private $attendee = false;
	private $jevrinvitees;
	private $jevrreminders;
	private $jevrattendees;
	private $view;

	public function __construct($params)
	{
		$this->params = $params;

		jimport('joomla.filesystem.file');
		if (File::exists(JPATH_SITE . '/components/com_community/community.php'))
		{
			if (ComponentHelper::isEnabled("com_community"))
			{
				$this->jomsocial = true;
			}
		}
		if (File::exists(JPATH_SITE . '/components/com_comprofiler/comprofiler.php'))
		{
			if (ComponentHelper::isEnabled("com_comprofiler"))
			{
				$this->cbuilder = true;
				if (File::exists(JPATH_SITE . "/components/com_comprofiler/plugin/user/plug_cbgroupjive/cbgroupjive.php"))
				{
					$this->groupjive = true;
				}
			}
		}

		jimport('joomla.application.component.view');

		$theme           = JEV_CommonFunctions::getJEventsViewName();
		$this->_basepath = JPATH_SITE . "/plugins/jevents/jevrsvppro/rsvppro/";
		$this->view      = new HtmlView(array('base_path' => $this->_basepath, "template_path" => $this->_basepath . "tmpl/default", "name" => $theme));

		if (file_exists($this->_basepath . "tmpl/" . $params->get('frontendstyling', 'xxx')))
		{
			$this->view->addTemplatePath($this->_basepath . "tmpl/" . $params->get('frontendstyling', 'xxx'));
		}

		$this->view->addTemplatePath($this->_basepath . "tmpl/" . $theme);
		$this->view->addTemplatePath(JPATH_SITE . '/' . 'templates' . '/' . Factory::getApplication()->getTemplate() . '/' . 'html' . '/' . "plg_rsvppro" . '/' . "default");
		$this->view->addTemplatePath(JPATH_SITE . '/' . 'templates' . '/' . Factory::getApplication()->getTemplate() . '/' . 'html' . '/' . "plg_rsvppro" . '/' . $theme);

		$this->view->setLayout("attendee");

		$this->view->jomsocial = $this->jomsocial;
		$this->view->params    = $this->params;
	}

	public function displayCustomFields(&$row)
	{

		if ($row === null)
		{
			return false;
		}

		$this->event = $row;

		$this->view->row = $row;

		// I will need the correct start date for the event - dtstart is not reliable enough so add my own value here
		$this->fixDtStart($row);

		$db   = Factory::getDbo();
		$html = '';

		$eventid = (int) $row->ev_id();
		if ($eventid > 0)
		{
			// block recursive calls
			static $processedAlready = array();
			if (in_array($eventid, $processedAlready))
			{
				return;
			}
			$processedAlready[] = $eventid;

			$sql = "SELECT * FROM #__jev_attendance WHERE ev_id=" . $eventid;
			$db->setQuery($sql);
			$rsvpdata      = $db->loadObject();
			$row->rsvpdata = $rsvpdata;

			if ($rsvpdata)
			{
				$sql = "SELECT * FROM #__jev_attendance_translations WHERE atd_id=" . $rsvpdata->id . " AND language=" . $db->quote(Factory::getLanguage()->getTag());
				$db->setQuery($sql);
				$translatedRsvpdata = $db->loadObject();
				if ($translatedRsvpdata)
				{
					foreach (get_object_vars($translatedRsvpdata) as $k => $v)
					{
						if ($k != "id" && $k != "atd_id" and $k != "language" && $v != "")
						{
							$rsvpdata->$k = $v;
						}
					}
				}
				if ($rsvpdata->allowregistration)
				{

					// We want to be able to respond to specific situation e.g. a google conversion code so catch them here
					// REALLY klunky way of doing this !
					$messages = Factory::getApplication()->getMessageQueue();
					foreach ($messages as $message)
					{
						if ($message["message"] == Text::_("JEV_ATTENDANCE_CONFIRMED"))
						{
							Factory::getApplication()->input->set("rsvp_registered", 1);
						}
					}

					// If we have RSVP PRo data then need to block page caching too!!
					// Also make sure we disable Joomla caching on this page
					Factory::getApplication()->set("caching", 0);
					// Cache::getInstance('page', $options); doesn't give an instance its always a NEW copy
					$cache_plg = PluginHelper::getPlugin('system', 'cache');
					// Test needed for J4/J3.9 compatibility
					$dispatcher = method_exists(Factory::getApplication(), "getDispatcher") ? Factory::getApplication()->getDispatcher() : JEventDispatcher::getInstance();
					// Test needed for J4/J3.9 compatibility
					if (method_exists($dispatcher, 'get'))
					{
						// Joomla 3.9
						$observers = $dispatcher->get("_observers");
					}
					else
					{
						// Joomla 4
						$observers = Factory::getApplication()->get("_observers");
					}

					if ($observers && is_array($observers))
					{
						foreach ($observers as $observer)
						{
							if (is_object($observer) && get_class($observer) == "plgSystemCache")
							{
								$pagecache = @$observer->get("_cache");
								if ($pagecache)
								{
									$pagecache->setCaching(false);
								}
								break;
							}
						}
					}
					// Also disable JEvents Cache for this page
					$params = ComponentHelper::getParams(JEV_COM_COMPONENT);
					$params->set('com_cache', 0);
				}

				// Store details in registry - will need them for currency formatting etc.
				$registry = JevRegistry::getinstance("jevents");
				$registry->set("rsvpdata", $rsvpdata);
				$registry->set("event", $row);

				$this->processDirectAttendanceClick($row);

				$this->view->rsvpdata = $rsvpdata;

				$user = Factory::getUser();
				if (!$this->checkAccess($user, $rsvpdata, $row))
				{
					$row->attendancedata = isset($rsvpdata->sessionaccessmessage) ? $rsvpdata->sessionaccessmessage : "";

					return $row->attendancedata;
				}

				// if params allow attendance tracking by email then skip this check of being logged in
				if (!$this->params->get("attendemails", 0) && $user->id == 0 && $rsvpdata->allowregistration)
				{
					$uri                 = JURI::getInstance();
					$this->view->uri     = $uri;
					$html                = $this->view->loadTemplate("login");
					$row->rsvp_loginform = $html;

					$row->rsvp_attendanceform = $html;
					// $html .= $row->rsvp_attendanceform;
					$row->attendancedata = $html;

					// if we can show attendee list to anyone then don't return yet
					if ($user->id == 0 && !$this->params->get("showtoanon", 0))
					{
						// reset the dtstart value
						$this->fixDtStart($row, true);

						// This is normally set by the register/change/cancel form
						$sql = "SELECT atdcount FROM #__jev_attendeecount as a WHERE a.at_id=" . $rsvpdata->id;
						if (!$rsvpdata->allrepeats)
						{
							$sql .= " and a.rp_id=" . $row->rp_id();
						}
						$db->setQuery($sql);
						$row->attendeeCount = intval($db->loadResult());

						return $html;
					}
				}

				// Do we allow conditional sessions?
				if ($this->params->get("conditionsessions", 0) && $user->id > 0)
				{
					if (isset($rsvpdata->conditionsession) && trim($rsvpdata->conditionsession) !== "" && strpos($rsvpdata->conditionsession, "|") > 0)
					{
						$db = Factory::getDBO();

						list($csatd_id, $csrp_id) = explode("|", $rsvpdata->conditionsession);
						// is this user actually attending and fully paid up
						$query = "SELECT * FROM #__jev_attendees where attendstate=1 AND user_id=" . $user->id . " AND at_id = " . $csatd_id . " AND rp_id = " . $csrp_id;
						$db->setQuery($query);
						$csattendee = $db->loadObject();
						if (!$csattendee)
						{
							if ($csrp_id > 0)
							{
								$query = "SELECT CONCAT_WS( ' - ', det.summary, DATE_FORMAT(rpt.startrepeat , '%e %b %Y')) as title, ev.ev_id"
									. "\n FROM #__jevents_vevent as ev "
									. "\n LEFT JOIN #__jevents_repetition as rpt ON rpt.eventid=ev.ev_id"
									. "\n LEFT JOIN #__jevents_vevdetail as det ON rpt.eventdetail_id=det.evdet_id"
									. "\n LEFT JOIN #__jev_attendance AS atd ON atd.ev_id = ev.ev_id"
									. "\n WHERE atd.id = " . $csatd_id . " AND rpt.rp_id = " . $csrp_id;
								$db->setQuery($query);
							}
							else
							{
								$query = "SELECT det.summary as title, ev.ev_id "
									. "\n FROM #__jevents_vevent as ev "
									. "\n LEFT JOIN #__jevents_repetition as rpt ON rpt.eventid=ev.ev_id"
									. "\n LEFT JOIN #__jevents_vevdetail as det ON ev.detail_id=det.evdet_id"
									. "\n LEFT JOIN #__jev_attendance AS atd ON atd.ev_id = ev.ev_id"
									. "\n WHERE atd.id = " . $csatd_id;
								$db->setQuery($query);
							}
							$conditionsession = $db->loadObject();
							if ($conditionsession)
							{
								$conditionsessionlabel = $conditionsession->title;

								include_once(JPATH_SITE . "/components/com_jevents/jevents.defines.php");

								$datamodel = new JEventsDataModel();
								if ($csrp_id > 0)
								{
									$csevent = $datamodel->queryModel->listEventsById($csrp_id, true, "icaldb");
								}
								else
								{
									$csevent = $datamodel->queryModel->getEventById($conditionsession->ev_id, true, "icaldb");
								}
								if ($csevent)
								{
									$cslink                      = $csevent->viewDetailLink($csevent->yup(), $csevent->mup(), $csevent->dup(), true);
									$html                        .= Text::sprintf("RSVP_CONDITIONAL_REGISTRATION_SESSION_LINK", $cslink, $conditionsessionlabel) . "<Br/>";
									$rsvpdata->allowregistration = false;
								}
							}
						}

					}
				}

				// Fix attendee records for user who has registered in Joomla AFTER signing up for an event
				$db   = Factory::getDBO();
				$user = Factory::getUser();
				if ($user->id > 0 && $user->email != "")
				{
					$db->setQuery("Update #__jev_attendees  set user_id = " . $user->id . "  where email_address = " . $db->Quote($user->email));
					$x = $db->execute();
					$db->setQuery("Update #__jev_invitees set user_id = " . $user->id . "  where email_address = " . $db->Quote($user->email));
					$y = $db->execute();
					$db->setQuery("Update #__jev_reminders set user_id = " . $user->id . "  where email_address = " . $db->Quote($user->email));
					$z = $db->execute();
				}

				$this->jevrinvitees  = new JevRsvpInvitees($this->params, $this->jomsocial, $this->cbuilder, $this->groupjive);
				$this->jevrreminders = new JevRsvpReminders($this->params, $this->jomsocial);
				$this->jevrattendees = new JevRsvpAttendees($this->params, $this->jomsocial, $rsvpdata);
				$this->jevrattendees->setView($this->view);
				$this->jevrattendees->jevrinvitees = $this->jevrinvitees;

				$this->jevrinvitees->recordViewed($rsvpdata, $row);

				$this->jevrattendees->confirmAttendance($rsvpdata, $row);

				// Are we saving an attendance update
				//$this->recordAttendance($rsvpdata, $row);

				if (!$rsvpdata->allowregistration && !$rsvpdata->allowreminders && !$rsvpdata->invites)
				{
					$html                .= $this->showstatus();
					$row->attendancedata = $html;
					// reset the dtstart value
					$this->fixDtStart($row, true);

					return $html;
				}

				// NB can only invite if attendance is enabled
				//if (!$rsvpdata->allowregistration && !$rsvpdata->allowreminders) { $this->fixDtStart($row, true); return $this->showstatus();}

				$emailaddress = $this->jevrattendees->getEmailAddress("em2");
				if (!$emailaddress)
					$emailaddress = $this->jevrattendees->getEmailAddress("em");

				$this->view->emailaddress = $emailaddress;

				if ($rsvpdata->allowregistration)
				{
					HTMLHelper::stylesheet('components/com_rsvppro/assets/css/rsvpattend.css');
					/*
					  $row->rsvp_setFieldParameters = $this->setFieldParameters($row,$rsvpdata);
					  $html .= $row->rsvp_setFieldParameters;
					 */

					// clean up unpaid and unconfirmed attendees at this stage before offering attendance form
					$params         = ComponentHelper::getParams("com_rsvppro");
					$attendeehelper = new RsvpAttendeeHelper($params);
					$attendeehelper->cleanupUnpaidAttendees($rsvpdata->id, true);

					// Any status of attendance
					$this->attendee = $this->jevrattendees->isAttending($rsvpdata, $row, $emailaddress, true);
					$attending      = false;
					if ($this->attendee)
					{
						$attending = true;
					}
					$this->view->attending = $attending;
					// add link to attendee into the event
					$row->attendee = $this->attendee;
					// Store details in registry - will need them for currency formatting etc.
					$registry = JevRegistry::getinstance("jevents");
					$registry->set("attendee", $this->attendee);

					$this->jevrattendees->attendee = $this->attendee;

					// if attending (i.e. attendstate = 1 or 4 then calculate balances
					if ($attending)
					{
						$this->jevrattendees->calculateBalances($this->view, $rsvpdata, $row);
					}

					if (!isset($row->rsvp_attendanceform))
					{
						if (Factory::getApplication()->input->getCmd("task") != "accounts.paymentpage")
						{
							$row->rsvp_attendanceform = $this->jevrattendees->fetchAttendanceForm($row, $rsvpdata, $attending, $emailaddress);
						}
						else
						{
							$row->rsvp_attendanceform = "";
						}

						$html .= $row->rsvp_attendanceform;
					}
				}

				if (Factory::getApplication()->input->getCmd("task") != "accounts.paymentpage")
				{
					$row->rsvp_reminderform = $this->jevrreminders->reminderForm($row, $rsvpdata, $emailaddress);
					$html                   .= $row->rsvp_reminderform;

					$row->rsvp_messagarea = $this->showMessageArea($row, $rsvpdata);
					$html                 .= $row->rsvp_messagarea;

					$row->rsvp_fullattendees = $this->jevrattendees->showAttendees($row, $rsvpdata);
					$html                    .= $row->rsvp_fullattendees;

					$row->rsvp_createinvitations = $this->jevrinvitees->createInvitations($row, $rsvpdata);
					$html                        .= $row->rsvp_createinvitations;
				}
				else
				{
					$row->rsvp_reminderform      = "";
					$row->rsvp_messagarea        = "";
					$row->rsvp_fullattendees     = "";
					$row->rsvp_createinvitations = "";
				}
			}
			$html .= $this->showstatus();
		}

		$row->attendancedata = $html;
		// reset the dtstart value
		$this->fixDtStart($row, true);

		return $html;

	}

	protected function processDirectAttendanceClick($row)
	{

		$db    = Factory::getDbo();
		$app   = Factory::getApplication();
		$input = $app->input;

		$dadata   = Factory::getApplication()->input->getString("da", "");
		$dnadata  = Factory::getApplication()->input->getString("dna", "");
		$invrdata = Factory::getApplication()->input->getString("invr", "");

		if (!empty($dadata) || !empty($dnadata))
		{
			$directdata = $dadata ? @base64_decode($dadata) : @base64_decode($dnadata);
			if ($directdata && count(explode(":", $directdata)) == 2)
			{
				list($daemailaddress, $directcode) = explode(":", $directdata);
				$params       = ComponentHelper::getParams("com_rsvppro");
				$checkdacode  = base64_encode($daemailaddress . ":" . md5("directattend" . $params->get("emailkey", "email key") . $daemailaddress));
				$checkdnacode = base64_encode($daemailaddress . ":" . md5("directnoattend" . $params->get("emailkey", "email key") . $daemailaddress));
				if ($checkdacode == $dadata || $checkdnacode == $dnadata)
				{
					//echo $daemailaddress . ($dadata ? " is attending" : " is not attending");

					$registry = JevRegistry::getinstance("jevents");
					$rsvpdata = $registry->get("rsvpdata");
					if ($rsvpdata)
					{

						include_once(JPATH_ADMINISTRATOR . "/components/com_rsvppro/libraries/attendeehelper.php");

						// We see if registrations are open or cancellations are closed

						// Must use strtotime format for force JevDate to not just parse the date itself!!!
						$jnow = new JevDate("+1 second");
						$now  = $jnow->toUnix();

						// if attendance tracked for the event as a whole then must compare the time of the start of the event
						if ($rsvpdata->allrepeats)
						{
							$regclose = $rsvpdata->regclose == "0000-00-00 00:00:00" ? $row->dtstart() : strtotime($rsvpdata->regclose);
							$regopen  = $rsvpdata->regopen == "0000-00-00 00:00:00" ? strtotime("-1 year") : strtotime($rsvpdata->regopen);
							if ($now > $regclose)
							{
								Factory::getApplication()->enqueueMessage(Text::_("JEV_REGISTRATIONS_CLOSED"), 'message');

								return;

								// We have  special case where cancellations are allowed AFTER the registration close date
								/*
								if (!($this->attending  && $this->attendee->attendstate==1 && $this->canCancel))
								{
									return;
								}
								*/
							}
						}
						// otherwise the start of the repeat
						else
						{
							$regclose            = $rsvpdata->regclose == "0000-00-00 00:00:00" ? $row->dtstart() : strtotime($rsvpdata->regclose);
							$cancelclose         = $rsvpdata->cancelclose == "0000-00-00 00:00:00" ? $row->dtstart() : strtotime($rsvpdata->cancelclose);
							$regopen             = $rsvpdata->regopen == "0000-00-00 00:00:00" ? strtotime("-1 year") : strtotime($rsvpdata->regopen);
							$eventstart          = $row->dtstart();
							$repeatstart         = $row->getUnixStartTime();
							$adjustedregclose    = $regclose + ($repeatstart - $eventstart);
							$adjustedcancelclose = $cancelclose + ($repeatstart - $eventstart);
							$adjustedregopen     = $regopen + ($repeatstart - $eventstart);

							// Disable cancelations
							if ($now > $adjustedcancelclose)
							{
								$this->params->set('allowno', 0);
								$rsvpdata->allowcancellation = 0;
							}

							if ($now > $adjustedregclose)
							{
								Factory::getApplication()->enqueueMessage(Text::_("JEV_REGISTRATIONS_CLOSED"), 'message');

								return;
							}
						}


						$attendeehelper = new RsvpAttendeeHelper($params);
						$attendeehelper->countAttendees($rsvpdata->id);
						// Todo make this a count based on the template
						$guestcount = 1;

						$sql = "SELECT atdcount FROM #__jev_attendeecount as a WHERE a.at_id=" . $rsvpdata->id;
						if (!$rsvpdata->allrepeats)
						{
							$sql .= " and a.rp_id=" . $row->rp_id();
						}
						$db->setQuery($sql);
						$attendeeCount             = $db->loadResult();
						$attendeeCount             = (int) $attendeeCount;
						$tempattendee              = new stdClass();
						$tempattendee->attendstate = -1;
						$tempattendee->guestcount  = 0;

						// simulate guestcount of 1
						$input->set('guestcount', 1);
						$this->countAttendeeContributionToAttendance($row, $rsvpdata, $tempattendee, true);

						if ($this->params->get("capacity", 0) && $rsvpdata->capacity > 0)
						{
							// If over capacity and waiting list then just ignore
							if ($attendeeCount + ($row->newcnt - $row->currentcnt) > $rsvpdata->capacity + $rsvpdata->waitingcapacity)
								//if ($attendeeCount >= $rsvpdata->capacity + $rsvpdata->waitingcapacity)
							{
								$link = $row->viewDetailLink($row->yup(), $row->mup(), $row->dup(), false, $input->get('Itemid'));
								Factory::getApplication()->enqueueMessage(Text::_("RSVP_BOOKING_OVER_CAPACITY"));
								$app->redirect($link);
							}

							// Should this be on the waiting list
							$onWaitingList = false;
							if ($attendeeCount + ($row->newcnt - $row->currentcnt) > intval($rsvpdata->capacity) && $row->newcnt > $row->currentcnt)
								//if ($attendeeCount >= intval($rsvpdata->capacity))
							{
								$onWaitingList = true;
							}
						}
						else
						{
							$onWaitingList = false;
						}

						$rp_id         = $row->rp_id();
						$hasrepetition = $row->hasrepetition();

						// Find matching invitation
						$sql = "SELECT id , email_address as email , 0 as user_id from #__jev_invitees WHERE email_address = " . $db->Quote(strtolower($daemailaddress)) . " AND at_id=" . $rsvpdata->id;
						if (!$rsvpdata->allinvites && $hasrepetition)
						{
							$sql .= " AND rp_id=" . $rp_id;
						}
						$db->setQuery($sql);
						$invitee = $db->loadObject();
						if (!$invitee)
						{
							// May be a user based invitee
							$sql = "SELECT i.id , u.email , i.user_id from #__users as u "
								. " LEFT JOIN #__jev_invitees as i ON u.id=i.user_id "
								. " WHERE u.email = " . $db->Quote(strtolower($daemailaddress)) . " AND i.at_id=" . $rsvpdata->id;
							if (!$rsvpdata->allinvites && $hasrepetition)
							{
								$sql .= " AND rp_id=" . $rp_id;
							}
							$db->setQuery($sql);
							$invitee = $db->loadObject();
						}
						if ($invitee)
						{
							// First step marked as viewed
							$sql = "UPDATE #__jev_invitees set viewedevent=1  WHERE id =" . $invitee->id;
							$db->setQuery($sql);
							$db->execute();

							// Step 2 check  if we already have this attendee
							if ($invitee->user_id)
							{
								$sql = "SELECT * FROM #__jev_attendees WHERE at_id=" . $rsvpdata->id . " and user_id=" . $invitee->user_id;
							}
							else
							{
								$sql = "SELECT * FROM #__jev_attendees WHERE at_id=" . $rsvpdata->id . " and LOWER(email_address)=" . $db->Quote($invitee->email);
							}
							if (!$rsvpdata->allrepeats)
							{
								$sql .= " AND rp_id=" . $rp_id;
							}
							if ($onWaitingList)
							{
								// should not be looking for the same 'waiting state' since a reattempt to direct attend onto waiting list for existing attendee would then create a duplicate entry on the waiting list
								// $sql .= " AND waiting = 1";
							}

							$db->setQuery($sql);
							$testattendee = $db->loadObject();

							$waiting = ", waiting = " . ($onWaitingList ? "1" : "0");

							// 1 = attend, 3 = pending approval
							$dastate = $rsvpdata->initialstate ? 1 : 3;

							// Step 2 mark as attending
							if ($testattendee)
							{
								if (($testattendee->attendstate == 0 || $testattendee->attendstate == 2) && $dadata)
								{
									// if not attending already and now wanting to attend then set waiting if needed
									$waiting = ", waiting = " . ($onWaitingList ? "1" : "0");
								}
								else if ($testattendee->attendstate !== 0 && !$dadata)
								{
									// if attending already and now NOT wanting to attend then set waiting off
									$waiting = ", waiting = 0";
								}
								else
								{
									// otherwise leave waiting state as is
									$waiting = "";
								}
								$sql = "UPDATE #__jev_attendees SET attendstate = " . ($dadata ? $dastate : 0) . $waiting . " WHERE id=" . $testattendee->id;
							}
							else
							{
								// Need the creation date for these records
								if (class_exists("JevDate"))
								{
									$date    = JevDate::getDate("+0 seconds");
									$created = $date->toMySQL();
								}
								else
								{
									$date    = Factory::getDate("+0 seconds");
									$created = $date->toSql();
								}

								$sql = "INSERT INTO #__jev_attendees SET attendstate = " . ($dadata ? $dastate : 0) . ", at_id=" . $rsvpdata->id . ", user_id=" . $invitee->user_id . ", email_address=" . $db->Quote($invitee->email) . ", created=" . $db->Quote($created) . ", modified=" . $db->Quote($created);
								if (!$rsvpdata->allrepeats)
								{
									$sql .= ", rp_id=" . $rp_id;
								}
								$sql .= ", atdcount=" . $row->newcnt;
								$sql .= ", guestcount=" . 1;

								if (!$dadata)
								{
									// if not attending then don't set waiting list
									$waiting = "";
								}
								$sql .= $waiting;

							}
							$db->setQuery($sql);
							$db->execute();

							// Step x count attendee
							$attendeehelper->countAttendees($rsvpdata->id);

							// auto remind attendees
							if ($dadata && !$onWaitingList && empty($waiting))
							{
								if ($params->get("autoremind", 0) == 1 && ($rsvpdata->allowreminders || $params->get("forceautoremind", 0)))
								{
									// cancel reminder
									$user = JEVHelper::getUser($invitee->user_id);

									$attendeehelper = new RsvpAttendeeHelper($params);
									$attendeehelper->jevrreminders->remindUser($rsvpdata, $row, $user, $invitee->email);
								}
							}

							if ($onWaitingList && $dadata && !empty($waiting))
							{
								Factory::getApplication()->enqueueMessage(Text::_("JEV_ATTENDANCE_ON_WAITING_LIST"), 'message');
							}
							else
							{
								Factory::getApplication()->enqueueMessage($dadata ? Text::_('JEV_ATTENDANCE_RECORDED_MAY_NEED_TO_LOGIN') : Text::_('JEV_NOT_ATTENDANCE_RECORDED_MAY_NEED_TO_LOGIN'), 'notice');
							}
						}
					}
				}

			}
		}

		if (!empty($invrdata))
		{
			$decodeddata = @base64_decode($invrdata);
			if ($decodeddata && count(explode(":", $decodeddata)) == 3)
			{
				list($invremailaddress, $invrcode, $invrlinktext) = explode(":", $decodeddata);
				$params        = ComponentHelper::getParams("com_rsvppro");
				$checkinvrcode = base64_encode($invremailaddress . ":" . $invrcode . ":" . md5("invrlist" . $params->get("emailkey", "email key") . $invremailaddress));
				if ($checkinvrcode == $invrdata)
				{

					$registry = JevRegistry::getinstance("jevents");
					$rsvpdata = $registry->get("rsvpdata");
					if ($rsvpdata)
					{

						// Find matching invitation list
						$sql = "SELECT id , listname from #__jev_invitelist WHERE listname = " . $db->Quote($invrcode);
						$db->setQuery($sql);
						$invitelist = $db->loadObject();

						if ($invitelist)
						{

							$sql = "SELECT * FROM #__jev_invitelist_member WHERE list_id = " . $invitelist->id;
							$sql .= " AND ( email_address = " . $db->quote(strtolower($invremailaddress));
							$sql .= " OR  user_id = (SELECT id FROM #__users WHERE  email = " . $db->quote(strtolower($invremailaddress)) . "))";
							$db->setQuery($sql);
							$invitee = $db->loadObject();

							if ($invitee)
							{
								$sql = "DELETE FROM #__jev_invitelist_member WHERE list_id = " . $invitelist->id;
								$sql .= " AND id = " . $invitee->id;
								$db->setQuery($sql);
								if ($db->execute())
								{
									Factory::getApplication()->enqueueMessage(Text::_("JEV_ATTENDANCE_REMOVED_FROM_INVITATION_LIST"), 'message');
								}
							}
						}
					}
				}

			}
		}

	}

	public function displayCustomFieldsMultiRow(&$row, $rsvpdataArray = array())
	{
		// Skip in list of sessions view
		if (Factory::getApplication()->input->getCmd("task") == "sessions.list")
		{
			return;
		}

		$this->event = $row;

		$this->view->row = $row;

		// I will need the correct start date for the event - dtstart is not reliable enough so add my own value here
		$this->fixDtStart($row);

		$db      = Factory::getDBO();
		$html    = '';
		$eventid = intval($row->ev_id());
		if ($eventid > 0 && array_key_exists($eventid, $rsvpdataArray))
		{

			//$sql = "SELECT * FROM #__jev_attendance WHERE ev_id=" . $eventid;
			//$db->setQuery($sql);
			//$rsvpdata = $db->loadObject();
			//$row->rsvpdata = $rsvpdata;
			$row->rsvpdata = $rsvpdata = $rsvpdataArray[$eventid];
			if ($rsvpdata)
			{

				// Store details in registry - will need them for currency formatting etc.
				$registry = JevRegistry::getinstance("jevents");
				$registry->set("rsvpdata", $rsvpdata);
				$registry->set("event", $row);

				$this->view->rsvpdata = $rsvpdata;

				$user = Factory::getUser();
				if (!$this->checkAccess($user, $rsvpdata, $row))
				{
					$row->attendancedata = isset($rsvpdata->sessionaccessmessage) ? $rsvpdata->sessionaccessmessage : "";

					return $row->attendancedata;
				}

				$this->jevrinvitees  = new JevRsvpInvitees($this->params, $this->jomsocial, $this->cbuilder, $this->groupjive);
				$this->jevrreminders = new JevRsvpReminders($this->params, $this->jomsocial);
				$this->jevrattendees = new JevRsvpAttendees($this->params, $this->jomsocial, $rsvpdata);
				$this->jevrattendees->setView($this->view);
				$this->jevrattendees->jevrinvitees = $this->jevrinvitees;

				if ($rsvpdata->allowregistration)
				{

					$emailaddress = $this->jevrattendees->getEmailAddress("em2");
					if (!$emailaddress)
					{
						$emailaddress = $this->jevrattendees->getEmailAddress("em");
					}
					$this->attendee = $this->jevrattendees->isAttending($rsvpdata, $row, $emailaddress, true);
					$attending      = false;
					if ($this->attendee)
					{
						$attending = true;
					}
					$row->attendee         = $this->attendee;
					$this->view->attending = $attending;
					// for sites that want the registration form to appear on every event together
					/*
					if (!isset($row->rsvp_attendanceform))
					{
						if (Factory::getApplication()->input->getCmd("task")!="accounts.paymentpage"){
							$row->rsvp_attendanceform = $this->jevrattendees->fetchAttendanceForm($row, $rsvpdata, $attending, $emailaddress);
						}
						else {
							$row->rsvp_attendanceform = "";
						}

						$html .= $row->rsvp_attendanceform;
					}
					*/
				}

				if ($this->params->get("showattendeesinlists", 0))
				{
					$row->rsvp_fullattendees = $this->jevrattendees->showAttendees($row, $rsvpdata);
					$html                    .= $row->rsvp_fullattendees;
				}
			}
		}

		$row->attendancedata = $html;
		// reset the dtstart value
		$this->fixDtStart($row, true);

		return $html;

	}

	// These 3 functions are under development

	private function showMessageArea(&$row, $rsvpdata)
	{
		$html = "";
		$user = Factory::getUser();
		if ($user->id == 0)
			return $html;

		if (!$this->params->get("enablemessages", 0))
			return $html;

		if ($user->id != $row->created_by() && !JEVHelper::isAdminUser($user) && !JEVHelper::canDeleteEvent($row, $user))
			return $html;

		return "";

	}

	private function setFieldParameters($row, $rsvpdata)
	{
		$html = "";
		// special case for setting field parameters
		if (JEVHelper::canEditEvent($row) || JEVHelper::canPublishEvent($row))
		{
			$html = '<div class="cModule jevattendform"><h3><span>' . Text::_('JEV_SET_FIELD_PARAMETERS') . '</span></h3>';
			$html .= "xxx";
			$html .= "</div>";
		}

		return $html;

	}

	private function showstatus()
	{
		return "";
		$user = Factory::getUser();
		if ($this->params->get("showstatus", 0) && $user->id > 0)
		{
			HTMLHelper::script('plugins/jevents/jevrsvppro/rsvppro/rsvp.js');
			HTMLHelper::stylesheet('plugins/jevents/jevrsvppro/rsvppro/rsvp.css');
			$html = '
	<div class="button2-left" id="jevstatusbutton"  style="margin-right:10px;">
		<div class="blank">
			<a href="#' . Text::_('JEV_SHOW_MY_STATUS') . '" onclick="showJevStatus();return false;"  title="' . Text::_('JEV_SHOW_MY_STATUS') . '"  style="padding:0px 5px;text-decoration:none">' . Text::_('JEV_SHOW_MY_STATUS') . '</a>
		</div>
	</div>
	<div id="jevstatus" style="display:none;clear:both;">Show Me</div>	
	';

			return $html;
		}

		return "";

	}

	private function fixDtStart(&$row, $reset = false)
	{
		if (is_callable(array($row, "fixDtstart")))
		{
			if ($reset)
			{
				$row->dtstart($row->_olddtstart);
				$row->dtend($row->_olddtend);
				$row->_publish_up   = $row->_oldpu;
				$row->_publish_down = $row->_oldpd;
			}
			else
			{
				$row->_olddtstart = $row->dtstart();
				$row->_olddtend   = $row->dtend();
				$row->_oldpu      = $row->publish_up();
				$row->_oldpd      = $row->publish_down();
				$this->doFixDtstart($row);
				$row->_fixed_dtstart = $row->dtstart();
				$row->_fixed_dtend   = $row->dtend();
			}
		}
	}

	// Note that if the timezone gets changed after the event is created and before it is re-edited or compared to the registration date for close times etc
	// then it could be out.  This function fixes this
	function doFixDtstart(&$row)
	{

		// must only ever do this once!
		if (isset($row->dtfixed) && $row->dtfixed) return;

		$row->dtfixed = 1;

		$db = Factory::getDBO();

		// Now get the first repeat since dtstart may have been set in a different timezeone and since it is a unixdate it would then be wrong
		if (strtolower($row->freq()) == "none")
		{
			// no need to be this pedantic in RSVP Pro (more important when editing events) so skip this
			/*
			$repeat = $this->getFirstRepeat($row);
			$row->dtstart($repeat->getUnixStartTime());
			$row->dtend( $repeat->getUnixEndTime());
			 */
			return;
		}
		else
		{
			// get first repeat also checks if its an exception
			$repeat = $this->getFirstRepeat($row);
			// Is this repeat an exception?
			if (!isset($repeat->ex_id) || $repeat->ex_id == 0)
			{
				// set dtstart to match the first repeat
				$row->dtstart($repeat->getUnixStartTime());
				$row->dtend($repeat->getUnixEndTime());

				return;
			}
			else
			{
				// skip non time changing exceptions
				if (isset($repeat->_exception_type) && $repeat->_exception_type == 1 && $repeat->_exstartrepeat == $repeat->_exoldstartrepeat)
				{
					// set dtstart to match the first repeat
					$row->dtstart($repeat->getUnixStartTime());
					$row->dtend($repeat->getUnixEndTime());

					return;
				}
				// This is the scenario where the first repeat is an exception so check to see if we need to be worried
				$jregistry = JevRegistry::getinstance("jevents");
				// This is the server default timezone
				$jtimezone = $jregistry->get("jevents.timezone", false);
				if ($jtimezone)
				{
					// This is the JEvents set timezone
					$timezone = date_default_timezone_get();
					// Only worry if the JEvents  set timezone is different to the server timezone
					if ($timezone != $jtimezone)
					{
						// look for repeats that are not exceptions
						$repeat2 = $this->getFirstRepeat($row, false);
						// if we have none then use the first repeat and give a warning
						if (!$repeat2)
						{
							// set dtstart to match the first repeat
							$row->dtstart($repeat->getUnixStartTime());
							$row->dtend($repeat->getUnixEndTime());

							//Factory::getApplication()->enqueueMessage(Text::_('JEV_PLEASE_CHECK_START_AND_END_TIMES_FOR_THIS_EVENT'));
						}
						else
						{
							// Calculate the time adjustment (if any) then check against the non-exceptional repeat
							// Convert dtstart using system timezone to date
							date_default_timezone_set($jtimezone);
							$truestarttime = JevDate::strftime("%H:%M:%S", $row->dtstart());
							// if the system timezone version of dtstart is the same time as the first non-exceptional repeat
							// then we are safe to use this adjustment mechanism to dtstart.  We use the real "date" and convert
							// back into unix time using the  Jevents timezone
							if ($truestarttime == JevDate::strftime("%H:%M:%S", JevDate::mktime($repeat2->hup(), $repeat2->minup(), $repeat2->sup(), 0, 0, 0)))
							{
								$truedtstart = JevDate::strftime("%Y-%m-%d %H:%M:%S", $row->dtstart());
								$truedtend   = JevDate::strftime("%Y-%m-%d %H:%M:%S", $row->dtend());

								// switch timezone back to Jevents timezone
								date_default_timezone_set($timezone);
								$row->dtstart(JevDate::strtotime($truedtstart));
								$row->dtend(JevDate::strtotime($truedtend));
							}
							else
							{
								// In this scenario we have no idea what the time should be unfortunately - use the original event time/date from the first non-exception repeat
								$truedtstart = JevDate::strftime("%Y-%m-%d %H:%M:%S", $repeat2->dtstart());
								$truedtend   = JevDate::strftime("%Y-%m-%d %H:%M:%S", $repeat2->dtend());

								//Factory::getApplication()->enqueueMessage(Text::_('JEV_PLEASE_CHECK_START_AND_END_TIMES_FOR_THIS_EVENT'));

								// switch timezone back to Jevents timezone
								date_default_timezone_set($timezone);
								$row->dtstart(JevDate::strtotime($truedtstart));
								$row->dtend(JevDate::strtotime($truedtend));
							}


						}
					}
					else
					{
						// look for repeats that are not exceptions
						$repeat2 = $this->getFirstRepeat($row, false);
						if (!$repeat2)
						{
							// TODO should really try to find the original data!
							// set dtstart to match the first repeat
							$row->dtstart($repeat->getUnixStartTime());
							$row->dtend($repeat->getUnixEndTime());
						}
						else
						{
							// dtstart is from the event detail so we can use this
							$row->dtstart($repeat2->dtstart());
							$row->dtend($repeat2->dtstart());
						}
					}
				}
				else
				{
					// We would not need to do this for event editing but we MUST handle first event as an exception
					// look for repeats that are not exceptions
					$repeat2 = $this->getFirstRepeat($row, false);
					if (!$repeat2)
					{
						// TODO should really try to find the original data!
						// set dtstart to match the first repeat
						$row->dtstart($repeat->getUnixStartTime());
						$row->dtend($repeat->getUnixEndTime());
					}
					else
					{
						// dtstart is from the event detail so we can use this
						$row->dtstart($repeat2->dtstart());
						$row->dtend($repeat2->dtstart());
					}
				}

			}
		}

	}

	// Gets repeats for this event from databases
	private function getFirstRepeat($row, $allowexceptions = true)
	{

		static $firstrepeats = array();
		$key = $row->ev_id() . ($allowexceptions ? "ex" : "");
		if (isset($firstrepeats[$key]))
		{
			return $firstrepeats[$key];
		}
		$db    = Factory::getDBO();
		$query = "SELECT ev.*, rpt.*, rr.*, det.*, exc.ex_id, exc.startrepeat as exstartrepeat, exc.oldstartrepeat as exoldstartrepeat, exc.exception_type "
			. "\n , YEAR(rpt.startrepeat) as yup, MONTH(rpt.startrepeat ) as mup, DAYOFMONTH(rpt.startrepeat ) as dup"
			. "\n , YEAR(rpt.endrepeat  ) as ydn, MONTH(rpt.endrepeat   ) as mdn, DAYOFMONTH(rpt.endrepeat   ) as ddn"
			. "\n , HOUR(rpt.startrepeat) as hup, MINUTE(rpt.startrepeat ) as minup, SECOND(rpt.startrepeat ) as sup"
			. "\n , HOUR(rpt.endrepeat  ) as hdn, MINUTE(rpt.endrepeat   ) as mindn, SECOND(rpt.endrepeat   ) as sdn"
			. "\n FROM #__jevents_vevent as ev "
			. "\n LEFT JOIN #__jevents_repetition as rpt ON rpt.eventid = ev.ev_id"
			. "\n LEFT JOIN #__jevents_vevdetail as det ON det.evdet_id = rpt.eventdetail_id"
			. "\n LEFT JOIN #__jevents_rrule as rr ON rr.eventid = ev.ev_id"
			. "\n LEFT JOIN #__jevents_exception as exc ON exc.rp_id=rpt.rp_id"
			. "\n WHERE ev.ev_id = '" . $row->ev_id() . "' "
			. ((!$allowexceptions) ? "\n AND exc.rp_id IS NULL" : "")
			. "\n ORDER BY rpt.startrepeat asc LIMIT 1";

		$db->setQuery($query);
		$rows = $db->loadObjectList();

		$repeat = null;
		// iCal agid uses GUID or UUID as identifier
		if ($rows)
		{
			$repeat        = new jIcalEventRepeat($rows[0]);
			$repeat->ex_id = $rows[0]->ex_id;
		}
		$firstrepeats[$key] = $repeat;

		return $repeat;
	}

	private function checkAccess($user, $rsvpdata, $event)
	{
		if ($rsvpdata->sessionaccess >= 0)
		{
			$access = $rsvpdata->sessionaccess;
		}
		else
		{
			$access = $event->access();
		}
		if (version_compare(JVERSION, "1.6.0", 'ge'))
		{
			$access = explode(",", $access);

			return count(array_intersect($access, JEVHelper::getAid($user, 'array'))) > 0;
		}
		else
		{
			return $access <= $user->aid;
		}

	}

	protected function countAttendeeContributionToAttendance(&$row, $rsvpdata, $attendee, $newattendee = false)
	{
		// TAKE CARE $attendee could be a stdClass!

		// TODO bring this in line with full count attendees code!

		// get the existing contribution to the attendance count

		$unpaidtoo = false;
		if ($attendee->attendstate == 4)
		{
			$xmlfile = JevTemplateHelper::getTemplate($rsvpdata);
			if (is_int($xmlfile) && $xmlfile > 0)
			{
				if (RsvpHelper::getTemplateSpecificParameter($rsvpdata, "unpaidcapacity", 0))
				{
					$unpaidtoo = true;
				}
			}
		}

		if ($attendee->attendstate == 1 || ($attendee->attendstate == 4 && $unpaidtoo))
		{

			$attendeeparams  = new Registry($attendee->params);
			$row->currentcnt = $attendee->guestcount;
			unset($row->_reductionapplied);

			// experimental code for excluding primary attendee from the overall count
			if (RsvpHelper::getTemplateSpecificParameter($rsvpdata, "skipprimarycount", 0))
			{
				$row->currentcnt--;
			}

			// New parameterised fields
			if ($rsvpdata->template != "")
			{
				$xmlfile = JevTemplateHelper::getTemplate($rsvpdata);

				if (is_int($xmlfile) || file_exists($xmlfile))
				{
					$rsvpparams = new JevRsvpParameter($attendee->params, $xmlfile, $rsvpdata, $row);
					$params     = $rsvpparams->renderInputDataToBasicArray($attendee);
					foreach ($params as $param)
					{
						if (isset($param ["includeintotalcapacity"]) && $param ["includeintotalcapacity"] && isset($param ["nativevalue"]))
						{
							// multiple attendees
							if (is_array($param ["nativevalue"]))
							{
								for ($g = 0; $g <= $row->newcnt; $g++)
								{
									$row->currentcnt += intval($param ["nativevalue"][$g]);
									if (isset($param ["reducevaluefortotalcapacity"]) && intval($param ["nativevalue"][$g]) > 0 && !isset($row->_reductionapplied))
									{
										$row->currentcnt -= $param ["reducevaluefortotalcapacity"];
										if (!empty($param ["reducevaluefortotalcapacity"]))
										{
											// only count this as as a reduction applied IF the reduction has a value!
											$row->_reductionapplied = 1;
										}
									}
								}
							}
							else
							{
								$row->currentcnt += intval($param ["nativevalue"]);
								if (isset($param ["reducevaluefortotalcapacity"]) && intval($param ["nativevalue"]) > 0 && !isset($row->_reductionapplied))
								{
									$row->currentcnt -= $param ["reducevaluefortotalcapacity"];
									if (!empty($param ["reducevaluefortotalcapacity"]))
									{
										// only count this as as a reduction applied IF the reduction has a value!
										$row->_reductionapplied = 1;
									}
								}
							}
						}
					}
				}
			}
			else
			{
				$row->currentcnt = 1;
			}
		}
		else
		{
			$row->currentcnt = 0;
		}

		$jevattend = Factory::getApplication()->input->getInt("jevattend", 0);
		// Now the new ones
		if ($jevattend == 1 || $newattendee)
		{
			$attendeeparams = new Registry(Factory::getApplication()->input->get("xmlfile", array(), 'ARRAY'));

			unset($row->_reductionapplied);
			$row->newcnt = Factory::getApplication()->input->getInt("guestcount", 0);

			// experimental code for excluding primary attendee from the overall count
			if (RsvpHelper::getTemplateSpecificParameter($rsvpdata, "skipprimarycount", 0))
			{
				$row->newcnt--;
			}

			// New parameterised fields
			if ($rsvpdata->template != "")
			{
				$xmlfile = JevTemplateHelper::getTemplate($rsvpdata);

				if (is_int($xmlfile) || file_exists($xmlfile))
				{
					$rsvpparams = new JevRsvpParameter($attendeeparams, $xmlfile, $rsvpdata, $row);
					$params     = $rsvpparams->renderInputDataToBasicArray($attendee);
					if ($params)
					{
						foreach ($params as $param)
						{
							if (isset($param ["includeintotalcapacity"]) && $param ["includeintotalcapacity"] && isset($param ["nativevalue"]))
							{
								// multiple attendees
								if (is_array($param ["nativevalue"]))
								{
									for ($g = 0; $g <= $row->newcnt; $g++)
									{
										$row->newcnt += intval($param ["nativevalue"][$g]);
										if (isset($param ["reducevaluefortotalcapacity"]) && intval($param ["nativevalue"][$g]) > 0 && !isset($row->_reductionapplied))
										{
											$row->newcnt -= $param ["reducevaluefortotalcapacity"];
											if (!empty($param ["reducevaluefortotalcapacity"]))
											{
												// only count this as as a reduction applied IF the reduction has a value!
												$row->_reductionapplied = 1;
											}
										}
									}
								}
								else
								{
									$row->newcnt += intval($param ["nativevalue"]);
									if (isset($param ["reducevaluefortotalcapacity"]) && intval($param ["nativevalue"]) > 0 && !isset($row->_reductionapplied))
									{
										$row->newcnt -= $param ["reducevaluefortotalcapacity"];
										if (!empty($param ["reducevaluefortotalcapacity"]))
										{
											// only count this as as a reduction applied IF the reduction has a value!
											$row->_reductionapplied = 1;
										}
									}
								}
							}
						}
					}
				}
			}
			else
			{
				$row->newcnt = 1;
			}
		}
		else
		{
			$row->newcnt = 0;
		}
		unset($row->_reductionapplied);

	}

}
