<?php
// no direct access
defined('_JEXEC') or die('Restricted access');

jimport('joomla.plugin.plugin');
jimport('joomla.log.log');

// Load Omnipay autoloader
require JPATH_SITE . '/libraries/jev_omnipay/vendor/autoload.php';

// Use Omnipay
use Joomla\Registry\Registry;
use Joomla\CMS\Language\Text;
use Joomla\CMS\Component\ComponentHelper;
use Joomla\CMS\Plugin\CMSPlugin;
use Joomla\CMS\Plugin\PluginHelper;
use Joomla\CMS\Router\Route;
use Joomla\CMS\Log\Log;
use Joomla\CMS\Uri\Uri;
use Joomla\CMS\Factory;
use Omnipay\Omnipay;
// Use Money
use Brick\Money\Money;

class plgRsvpproStripe extends CMSPlugin
{
	protected $autoloadLanguage = true;

	public function __construct(& $subject, $config)
	{

		// Enable Logging
		Log::addLogger(
			array(
				'text_file' => 'com_rsvppro.stripe.log.php'
			),
			Log::ALL,
			array('com_rsvppro.stripe')
		);

		parent::__construct($subject, $config);

	}

	public function generatePaymentPage(&$html, $attendee, $rsvpdata, $event, &$transaction)
	{

		$plugin = PluginHelper::getPlugin("rsvppro", "stripe");
		$params = new Registry($plugin->params);

		$jinput = Factory::getApplication()->input;

		$invoice = $jinput->getString("invoiceid", '') . '_' . $transaction->transaction_id;

		$detaillink = Route::_(Uri::root() . $event->viewDetailLink($event->yup(), $event->mup(), $event->dup(), false), false);

		$templateParams = RsvpHelper::getRawTemplateParams($rsvpdata);

		/* Test that there are outstanding fees so we don't double bill */
		if (isset($attendee->outstandingBalances) === false || round($attendee->outstandingBalances['feebalance'], 2) <= round(0, 2)) {
			/* There really is nothing owed */
			Log::add("Transaction ".$transaction->transaction_id ." attempted payment in the amount of ".self::phpMoneyFormat($transaction->amount,  $templateParams)
					." for ". Factory::getUser($attendee->user_id)->name . " when no balance is outstanding.",
					Log::ERROR, 'com_rsvppro.stripe');
			$app = Factory::getApplication();
			$app->enqueueMessage(Text::_("RSVP_PRO_STRIPE_NO_OUTSTANDING_BALANCE"), 'message');
			$app->redirect($detaillink);
			return;
 		}

		$comparams = ComponentHelper::getParams("com_rsvppro");
		$user      = JEVHelper::getUser($attendee->user_id);
		if ((int) $user->id === 0 && RsvpHelper::getTemplateSpecificParameter($rsvpdata,  "attendemails", 0))
		{
			$name     = $attendee->email_address;
			$username = $attendee->email_address;
			$email = $attendee->email_address;
		}
		else
		{
			$name     = $user->name;
			$username = $user->username;
			$email = $user->email;
		}

		// Load the attendee helper
		include_once(JPATH_ADMINISTRATOR . "/components/com_rsvppro/libraries/attendeehelper.php");

		$helper = new RsvpAttendeeHelper($comparams);

		$defaultStripePaymentLayout = "Invoiceid = {INVOICEID}<br/>Total Fees = {TOTALFEES}<br/>Fees Already Paid= {FEESPAID}<br/>Outstanding Balance = {BALANCE}<br/><br/>Please proceed to Stripe ...<br/><br/>{FORM}";
		$html = $templateParams->get("str_template", $defaultStripePaymentLayout);
		$html = $helper->parseMessage($html, $rsvpdata, $event, $name, $username, $attendee, false, false, $transaction);

		$currency   = $templateParams->get('Currency', "GBP");
		$money = Money::of($jinput->get('amount'), $currency);
		$start_date = JEventsHTML::getDateFormat($event->yup(), $event->mup(), $event->dup(), 0);
		$apikey = $templateParams->get('stripeMode', 0) == 0 ? $templateParams->get('str_sandbox_pk_apikey') : $templateParams->get('str_live_pk_apikey');

		// setup transaction data
		$transaction->amount      = (string)$money->getAmount();
		$transaction->currency    = $currency;
		$transaction->attendee_id = $attendee->id;
		$transaction->gateway     = 'stripe';
		$transaction->gw_refundable = $templateParams->get('str_process_refunds', 0);
		$transaction->params        = new stdClass();
		$transaction->params->apikey = $apikey;
		$transaction->params->mode   = $templateParams->get('stripeMode', 0);
		// New stuff
		$transaction->params->detaillink = Route::_(Uri::root() . $event->viewDetailLink($event->yup(), $event->mup(), $event->dup(), false), false);
		$transaction->params->notify_payment = $templateParams->get('notifystrpay', 0);
		$transaction->params->collect_billing = $templateParams->get('str_billing_address', 0);
		$transaction->params->collect_shipping = $templateParams->get('str_shipping_address', 0);
		$transaction->params->sk_apikey = $templateParams->get('stripeMode', 0) == 0 ? $templateParams->get('str_sandbox_sk_apikey') : $templateParams->get('str_live_sk_apikey');
		$transaction->params->title = htmlspecialchars($event->_title);
		// end new stuff
		$transaction->params        = json_encode($transaction->params);
		// Save the transaction data
		$transaction->store();

		ob_start();
		?>

		<form action="<?php echo JURI::root() . "index.php?option=com_rsvppro&task=accounts.notify&gateway=stripe"; ?>" method="POST">
		  <script
		    src="https://checkout.stripe.com/checkout.js" class="stripe-button"
		    data-key="<?php echo $apikey; ?>"
		    data-amount="<?php echo $money->getMinorAmount(); ?>"
		    data-name="<?php echo $templateParams->get('str_brandName', ''); ?>"
		    data-description="<?php echo $event->title; ?>"
		    data-image="<?php echo $templateParams->get('str_logoImageUrl', 'https://stripe.com/img/documentation/checkout/marketplace.png'); ?>"
		    data-locale="<?php echo $templateParams->get('str_locale', 'auto'); ?>"
		    data-zip-code="<?php echo $templateParams->get('str_val_zipcode', 0) == 0 ? 'false' : 'true'; ?>"
		    data-billing-address="<?php echo $templateParams->get('str_val_address', 0) == 0 ? 'false' : 'true'; ?>"
		    data-currency="<?php echo $currency; ?>"
		    data-shipping-address="<?php echo $templateParams->get('str_shipping_address', 0) == 0 ? 'false' : 'true'; ?>"
		    data-email="<?php echo $email; ?>"
		    data-label="<?php echo $templateParams->get('str_label', ''); ?>"
		    data-allow-remember-me="<?php echo $templateParams->get('str_allow_rememberme', 0) == 0 ? 'false' : 'true'; ?>"
		    >
		  </script>
		  <input type="hidden" name="transaction_id" value="<?php echo $transaction->transaction_id; ?>" />
		</form>
		<?php
		$form = ob_get_clean();

		if (isset($attendee->outstandingBalances))
		{
			$html = str_replace("{INVOICEID}", $invoice, $html);
			$html = str_replace("{TOTALFEES}", self::phpMoneyFormat($attendee->outstandingBalances['totalfee'], $templateParams), $html);
			$html = str_replace("{FEESPAID}", self::phpMoneyFormat($attendee->outstandingBalances['feepaid'], $templateParams), $html);
			$html = str_replace("{BALANCE}", self::phpMoneyFormat($attendee->outstandingBalances['feebalance'], $templateParams), $html);
            $html = str_replace("{RAWBALANCE}", $attendee->outstandingBalances['feebalance'], $html);
			$html = str_replace("{DEPOSIT}",self::phpMoneyFormat($attendee->outstandingBalances['deposit'], $templateParams), $html);
		}

		// Make sure HTML includes the form (this is often forgotton!)
		if (strpos($html, "{FORM}") === false)
		{
			$html .= "<br/>{FORM}";
		}

		$html = str_replace("{FORM}", $form, $html);

		Log::add(
			'Stripe payment page created for user ' . $user->name . ' for ' . self::phpMoneyFormat($transaction->amount, $templateParams)
			. ", Transaction ID: " . $transaction->transaction_id, Log::INFO, 'com_rsvppro.stripe'
		);
	}

	static public function getAtdRsvpTrans($transaction_id) {
		$db  = Factory::getDbo();
		$transaction = new rsvpTransaction();
		$transaction->load($transaction_id);
		$transaction->params = json_decode($transaction->params);

		// fetch the session and the attendee
		$db  = Factory::getDbo();
		$sql = "SELECT * FROM #__jev_attendees WHERE id=" . $transaction->attendee_id;
		$db->setQuery($sql);
		$attendee = $db->loadObject();

		$sql = "SELECT * FROM #__jev_attendance WHERE id=" . $attendee->at_id;
		$db->setQuery($sql);
		$rsvpdata = $db->loadObject();

		return [$attendee, $rsvpdata, $transaction];

	}

	static public function  notifyRsvpGateway(&$args)
	{

		$app     = Factory::getApplication();
		$session = Factory::getSession();
		$jinput  = $app->input;

		$params = ComponentHelper::getParams(RSVP_COM_COMPONENT);

		// Get the Stripe Token from the request
		$token = $jinput->get('stripeToken');
		if (empty($token)) {
			$app->enqueueMessage(Text::_("RSVP_PRO_STRIPE_NO_TOKEN"), 'error');
			Log::add(Text::_("RSVP_PRO_STRIPE_NO_TOKEN"), Log::ERROR, 'com_rsvppro.stripe');
			return;
		}
		// Get our transaction ID from the request
		$rsvp_trans_id = $jinput->get('transaction_id');
		if (empty($rsvp_trans_id)) {
			$app->enqueueMessage(Text::_("RSVP_PRO_STRIPE_NO_TRANSACTION_ID"), 'error');
			Log::add(Text::_("RSVP_PRO_STRIPE_NO_TRANSACTION_ID"), Log::ERROR, 'com_rsvppro.stripe');
			return;
		}

		list($attendee, $rsvpdata, $transaction) = plgRsvpproStripe::getAtdRsvpTrans($rsvp_trans_id);
		// save the token in the params
		$transaction->params->token = $token;
		$transaction->params = json_encode($transaction->params);
		$transaction->store();
		$transaction->params = json_decode($transaction->params);

		/* Confirm there is actually an outstanding balance before we send to stripe */
		$xmlfile = JevTemplateHelper::getTemplate($rsvpdata);
		$rsvpParams = new JevRsvpParameter($attendee->params, $xmlfile, $rsvpdata, null);
		$feesAndBalances = $rsvpParams->outstandingBalance($attendee);
		if (round($feesAndBalances['feebalance'], 2) <= round(0, 2)) {
			/* There really is nothing owed */
			Log::add("Transaction ".$transaction->transaction_id ." attempted stripe notify in the amount of ".self::phpMoneyFormat($transaction->amount,  RsvpHelper::getTemplateParams($rsvpdata))
					." for ". Factory::getUser($attendee->user_id)->name . " when no balance is outstanding.",
					Log::ERROR, 'com_rsvppro.stripe');
			$app = Factory::getApplication();
			$app->enqueueMessage(Text::_("RSVP_PRO_STRIPE_NO_OUTSTANDING_BALANCE"), 'message');
			$app->redirect($transaction->params->detaillink);
			return;
		}

		$money = Money::of($transaction->amount, $transaction->currency);

		$gateway = Omnipay::create('Stripe\PaymentIntents');
		$gateway->initialize(array('apikey' => $transaction->params->sk_apikey));
		$dummyParams = new \Joomla\Registry\Registry();
		Log::add(
			"Stripe Gateway Purchase - Amount: " . self::phpMoneyFormat($transaction->amount, $dummyParams)
			. ", Currency: " . $transaction->currency . ", Token: " . $token . ", Transaction ID: " . $transaction->transaction_id, Log::INFO, 'com_rsvppro.stripe'
		);
		$response = $gateway->authorize([
			'amount' => (string)$money->getAmount(),
			'currency' => $transaction->currency,
			'description' => $transaction->params->title,
			'metadata' => array('token'=>$transaction->params->token),
			'token' => $token,
			'returnUrl' => JURI::root() . "index.php?option=com_rsvppro&task=accounts.confirm&gateway=stripe&transaction_id=".$transaction->transaction_id,
			'confirm' => true,
		])->send();

		// Send confirmation email setting, set false unless payment is successful.
		$send_cm = false;


	if ($response->isSuccessful())
		{
			// normal successful flow
			if ($response->requiresConfirmation()) {
				$response = $gateway->confirm([
					'paymentIntentReference' => $response->getPaymentIntentReference(),
					])->send();
			}
			$response = $gateway->capture([
				'paymentIntentReference' => $response->getPaymentIntentReference(),
				])->send();
			$message = 'Payment was Successful.';
			Log::add($message . " Transaction ID: " . $transaction->transaction_id, Log::INFO, 'com_rsvppro.stripe');
			$transaction->logdata      .= '|'.date("D M j G:i").':'.$message;
			$transaction->paymentstate = 1;
			if ($transaction->params->collect_billing || $transaction->params->collect_shipping) {
				$keep = [];
				if ($transaction->params->collect_billing) {
					$keep = array_merge($keep, ["stripeBillingName", "stripeBillingAddressCountry", "stripeBillingAddressZip", "stripeBillingAddressState",
							"stripeBillingAddressLine1", "stripeBillingAddressCity", "stripeBillingAddressCountryCode"]);
				}
				if ($transaction->params->collect_shipping) {
					$keep = array_merge($keep, ["stripeShippingName", "stripeShippingAddressCountry", "stripeShippingAddressZip", "stripeShippingAddressState",
							"stripeShippingAddressLine1", "stripeShippingAddressCity", "stripeShippingAddressCountryCode"]);
				}
				$transaction->params->addresses = array_intersect_key($jinput->getArray(), array_flip($keep));
			}
			$send_cm = true;
		}
		else if ($response->isRedirect())
		{ 
			// 3D Secure Authentication flow
			$transaction->params->paymentIntentReference = $response->getPaymentIntentReference();
			// Store updated transaction
			$transaction->params = json_encode($transaction->params);
			$transaction->store();
			$app->redirect($response->getRedirectUrl());
			return;
		}
		else
		{ 
			// normal failure flow
			$message = "There was an error, Stripe responded with: " . $response->getMessage();
			$transaction->logdata = '|'.date("D M j G:i").':'.$message; // Any message the gateway his given us.
			$transError = sprintf(Text::_("RSVP_PRO_STRIPE_TRANSACTION_ERROR"), $response->getMessage());
			$app->enqueueMessage($transError, 'error');
			Log::add($message . ", Transaction ID: " . $transaction->transaction_id, Log::ERROR, 'com_rsvppro.stripe');
		}

		$transaction->gw_transid = $response->getTransactionReference(); // Transaction Reference, for refunds etc.
		$detaillink = $transaction->params->detaillink;
		$notifyPayment = $transaction->params->notify_payment;

		// Store updated transaction
		$transaction->params = json_encode($transaction->params);
		$transaction->store();

		// Send Payment Confirmation
		if ($send_cm && $notifyPayment != 0)
		{
			// fetch the session and the attendee
			$db  = Factory::getDbo();
			$sql = "SELECT * FROM #__jev_attendees WHERE id=" . $transaction->attendee_id;
			$db->setQuery($sql);
			$attendee = $db->loadObject();
			
			$sql = "SELECT * FROM #__jev_attendance WHERE id=" . $attendee->at_id;
			$db->setQuery($sql);
			$rsvpdata = $db->loadObject();

			plgRsvpproStripe::notifyStripePayment($transaction, $attendee, $rsvpdata, 1);
			Log::add(
				"Payment confirmation sent to " . Factory::getUser($attendee->user_id)->name 
				. ", Transaction ID: " . $transaction->transaction_id, 
				Log::INFO, 'com_rsvppro.stripe'
			);
		}

		$app->redirect($detaillink);

	}

	static public function  confirmRsvpGateway(&$args)
	{
		$app     = Factory::getApplication();
		$session = Factory::getSession();
		$jinput  = $app->input;

		// Get our transaction ID from the request
		$rsvp_trans_id = $jinput->get('transaction_id');
		if (empty($rsvp_trans_id)) {
			$app->enqueueMessage(Text::_("RSVP_PRO_STRIPE_NO_TRANSACTION_ID"), 'error');
			Log::add(Text::_("RSVP_PRO_STRIPE_NO_TRANSACTION_ID"), Log::ERROR, 'com_rsvppro.stripe');
			return;
		}

		list($attendee, $rsvpdata, $transaction) = plgRsvpproStripe::getAtdRsvpTrans($rsvp_trans_id);

		$gateway = Omnipay::create('Stripe\PaymentIntents');
		$gateway->initialize(array('apikey' => $transaction->params->sk_apikey));
		$response = $gateway->confirm([
			'paymentIntentReference' => $transaction->params->paymentIntentReference,
			])->send();
		$detaillink = $transaction->params->detaillink;
		if ($response->isSuccessful()) {
			// normal successful flow
			$transaction->gw_transid = $response->getTransactionReference(); // Transaction Reference, for refunds etc.
			$notifyPayment = $transaction->params->notify_payment;

			$response = $gateway->capture([
				'paymentIntentReference' => $transaction->params->paymentIntentReference,
				])->send();
			$message = 'Payment was Successful.';
			Log::add($message . " Transaction ID: " . $transaction->transaction_id, Log::INFO, 'com_rsvppro.stripe');
			$transaction->logdata      .= '|'.date("D M j G:i").':'.$message;
			$transaction->paymentstate = 1;
			if ($transaction->params->collect_billing || $transaction->params->collect_shipping) {
				$keep = [];
				if ($transaction->params->collect_billing) {
					$keep = array_merge($keep, ["stripeBillingName", "stripeBillingAddressCountry", "stripeBillingAddressZip", "stripeBillingAddressState",
							"stripeBillingAddressLine1", "stripeBillingAddressCity", "stripeBillingAddressCountryCode"]);
				}
				if ($transaction->params->collect_shipping) {
					$keep = array_merge($keep, ["stripeShippingName", "stripeShippingAddressCountry", "stripeShippingAddressZip", "stripeShippingAddressState",
							"stripeShippingAddressLine1", "stripeShippingAddressCity", "stripeShippingAddressCountryCode"]);
				}
				$transaction->params->addresses = array_intersect_key($jinput->getArray(), array_flip($keep));
			}

			// Send Payment Confirmation
			if ($notifyPayment != 0)
			{
				// fetch the session and the attendee
				$db  = Factory::getDbo();
				$sql = "SELECT * FROM #__jev_attendees WHERE id=" . $transaction->attendee_id;
				$db->setQuery($sql);
				$attendee = $db->loadObject();
				
				$sql = "SELECT * FROM #__jev_attendance WHERE id=" . $attendee->at_id;
				$db->setQuery($sql);
				$rsvpdata = $db->loadObject();

				plgRsvpproStripe::notifyStripePayment($transaction, $attendee, $rsvpdata, 1);
				Log::add(
					"Payment confirmation sent to " . Factory::getUser($attendee->user_id)->name 
					. ", Transaction ID: " . $transaction->transaction_id, 
					Log::INFO, 'com_rsvppro.stripe'
				);
			}
			// Store updated transaction
			$transaction->params = json_encode($transaction->params);
			$transaction->store();
		} else 
		{
			// normal failure flow
			$message = "There was an error, Stripe responded with: " . $response->getMessage();
			$transaction->logdata = '|'.date("D M j G:i").':'.$message; // Any message the gateway his given us.
			$transError = sprintf(Text::_("RSVP_PRO_STRIPE_TRANSACTION_ERROR"), $response->getMessage());
			$app->enqueueMessage($transError, 'error');
			Log::add($message . ", Transaction ID: " . $transaction->transaction_id, Log::ERROR, 'com_rsvppro.stripe');
		}
		$app->redirect($detaillink);
	}

	static private function notifyStripePayment($transaction, $attendee, $rsvpdata, $aftertransation = false)
	{
		$plugin = PluginHelper::getPlugin("rsvppro", "stripe");
		$params = new Registry($plugin->params);

		$templateParams = RsvpHelper::getRawTemplateParams($rsvpdata);

		/* set the templateParams into the rsvpdata for the notifyUser */
		$rsvpdata->templateParams = $templateParams;

		// immediate notification
		if (((!$aftertransation && $templateParams->get("notifystrpay", 1) == 2) || ($aftertransation && $templateParams->get("notifystrpay", 1) == 1)) && $transaction->gateway == "stripe")
		{

			$comparams = ComponentHelper::getParams("com_rsvppro");
			include_once(JPATH_ADMINISTRATOR . "/components/com_rsvppro/libraries/attendeehelper.php");
			$helper = new RsvpAttendeeHelper($comparams);

			$rpid = $attendee->rp_id;

			$dataModel  = new JEventsDataModel();
			$queryModel = new JEventsDBModel($dataModel);

			// Find the first repeat
			$vevent = $dataModel->queryModel->getEventById($rsvpdata->ev_id, false, "icaldb", false);
			if ($rpid == 0)
			{
				$repeat = $vevent->getFirstRepeat();
			}
			else
			{
				list($year, $month, $day) = JEVHelper::getYMD();
				$repeatdata = $dataModel->getEventData(intval($rpid), "icaldb", $year, $month, $day);
				if ($repeatdata && isset($repeatdata["row"]))
					$repeat = $repeatdata["row"];
			}

			$user = JEVHelper::getUser($attendee->user_id);
			if ($user->id == 0 && RsvpHelper::getTemplateSpecificParameter($rsvpdata,  "attendemails", 0))
			{
				$name     = $attendee->email_address;
				$username = $attendee->email_address;
			}
			else
			{
				$name     = $user->name;
				$username = $user->username;
			}

			$class      = "plgRsvppro" . ucfirst($transaction->gateway);
			$pluginpath = version_compare(JVERSION, "1.6.0", 'ge') ? JPATH_SITE . "/plugins/rsvppro/$transaction->gateway/" : JPATH_SITE . "/plugins/rsvppro/";
			JLoader::register($class, $pluginpath . $transaction->gateway . ".php");

			$helper->notifyUser($rsvpdata, $repeat, $user, $name, $username, $attendee, 'strpay', false, $transaction);
		}
	}

	private function notifyStripeRefund($transaction, $attendee, $rsvpdata)
	{
		$templateParams = RsvpHelper::getRawTemplateParams($rsvpdata);
		/* set the templateParams into the rsvpdata for the notifyUser */
		$rsvpdata->templateParams = $templateParams;
		// immediate notification
		if (($templateParams->get("notifystrref", 1) == 1) && $transaction->gateway == "stripe")
		{

			$comparams = ComponentHelper::getParams("com_rsvppro");
			include_once(JPATH_ADMINISTRATOR . "/components/com_rsvppro/libraries/attendeehelper.php");
			$helper = new RsvpAttendeeHelper($comparams);

			$rpid = $attendee->rp_id;

			$dataModel  = new JEventsDataModel();
			$queryModel = new JEventsDBModel($dataModel);

			// Find the first repeat
			$vevent = $dataModel->queryModel->getEventById($rsvpdata->ev_id, false, "icaldb", false);
			if ($rpid == 0)
			{
				$repeat = $vevent->getFirstRepeat();
			}
			else
			{
				list($year, $month, $day) = JEVHelper::getYMD();
				$repeatdata = $dataModel->getEventData(intval($rpid), "icaldb", $year, $month, $day);
				if ($repeatdata && isset($repeatdata["row"]))
					$repeat = $repeatdata["row"];
			}

			$user = JEVHelper::getUser($attendee->user_id);
			if ($user->id == 0 && RsvpHelper::getTemplateSpecificParameter($rsvpdata,  "attendemails", 0))
			{
				$name     = $attendee->email_address;
				$username = $attendee->email_address;
			}
			else
			{
				$name     = $user->name;
				$username = $user->username;
			}

			$class      = "plgRsvppro" . ucfirst($transaction->gateway);
			$pluginpath = version_compare(JVERSION, "1.6.0", 'ge') ? JPATH_SITE . "/plugins/rsvppro/$transaction->gateway/" : JPATH_SITE . "/plugins/rsvppro/";
			JLoader::register($class, $pluginpath . $transaction->gateway . ".php");

			$helper->notifyUser($rsvpdata, $repeat, $user, $name, $username, $attendee, 'strref', false, $transaction);
		}
	}

	static public function phpMoneyFormat($amount, $templateParams)
	{
        $digits    = $templateParams->get('CurrencyDigits');
        $symbol    = $templateParams->get('CurrencySymbol');
        $onLeft    = (strcmp($templateParams->get('CurrencyPlacement'), 'left') == 0);
        $separator = $templateParams->get('CurrencySeparator');
        $decimal   = $templateParams->get('CurrencyDecimal');

		$formattedText = '';

		// negative amount
		if ($amount < 0) $formattedText .= '-';

		// currency symbol on the left
		if ($onLeft) $formattedText .= $symbol;

		// format with correct number of digits and separator
		if ($digits > 0)
		{
			$amount        = RsvpHelper::ceil_dec(abs($amount), $digits, ".");
			$formattedText .= number_format(abs($amount), $digits, $decimal, $separator);
		}
		else
		{
			$formattedText = abs($amount);
		}

		// currency symbols on the right
		if (!$onLeft) $formattedText .= $symbol;


		return $formattedText;
	}

	public static function NotifyPayment($templateParams)
	{
		return $templateParams->get("notifystrpay", 1);
	}

	public static function PaymentMessageType()
	{
		return "strpay";
	}

	static public function transactionDetailLink($transaction)
	{
		$pluginpath = 'plugins/jevents/jevrsvppro/rsvppro';
		$result = '';
		if (!empty($transaction->logdata)) {
			$cimg = '<img src="'.JURI::root().$pluginpath.'/assets/logs.png" style="height:16px;border:none;" alt="logs" />';
			$transaction->logdata = str_replace("|", "\r", $transaction->logdata);
			$result .= '<a href="#" title="'.$transaction->logdata.'">'.$cimg.'</a>';
		}
		return $result;						
	}
	public function activeGatewayClass(&$activeGatewayClass, $action = "notify")
	{
		$gateway = Factory::getApplication()->input->getString("gateway");

		if ($gateway == self::gatewayName() || $gateway == "1" || strpos($gateway, self::gatewayName()) === 0)
		{
			$activeGatewayClass = __CLASS__;
		}
	}

	public function activeGateways(&$activeGatewayClasses)
	{
		$activeGatewayClasses[] = __CLASS__;
	}

	public function gatewayName() {
		return "stripe";
	}

	public static function usePluginSettingsName() {
			return "struseglobalsettings";
	}

	public function onRegistrationStatusChange($attendee, $eventrepeat, $rsvpdata) {

		$gateway = null;

		/* Check that we have feedata, this only happens when a payment has been processed or the attendee has cancelled */
		if (!isset($attendee->_feedata) || empty($attendee->_feedata)) {
			return;
		}

		/* check that the new status is not attending and that the attendee was confirmed */
		if ($attendee->attendstate != 0 || $attendee->confirmed != 1) {
			return;
		}

		$templateParams = RsvpHelper::getRawTemplateParams($rsvpdata);

		/* Check if the templateParams allows for refunds */
		if ($templateParams->get('str_process_refunds', 0) == 0) {
			return;
		}

		/* Calculate any refund end dates */
		$refunds = new StdClass;
		$refunds->refundForPeriod = $templateParams->get('str_refund_for_period', 0);
		if ($refunds->refundForPeriod > 0) {
			$refunds->refundForPeriod = strtotime($transaction->transaction_date) + ($refunds->refundForPeriod * 24 * 60 * 60);
		}
		$refunds->refundBefore = $templateParams->get('str_refund_before_period', '');
		if (strlen($refunds->refundBefore) > 0 ) {
			$refunds->refundBefore = strtotime($refunds->refundBefore);
		} else {
			$refunds->refundBefore = 0;
		}
		/* get the most restrictive date */
		$refundTo = 0;
		foreach ($refunds as $refundType) {
			if ($refundType == 0) continue;
			if ($refundTo == 0) $refundTo = $refundType;
			$refundTo = min($refundTo, $refundType);
		}

		foreach ($attendee->_feedata as $feedata) {
			$transaction = new rsvpTransaction();
			$transaction->load($feedata->transaction_id);
			if (!isset($transaction->transaction_id)) {
				continue;	// Probably should never happen
			}
			if (!isset($transaction->gateway)
				|| strlen($transaction->gateway) != strlen(self::gatewayName())
				|| strpos($transaction->gateway, self::gatewayName()) !== 0) {
					continue; // Not a stripe transaction
			}
			if (!isset($transaction->gw_refundable) || $transaction->gw_refundable == 0) {
				continue;	// Not refundable
			}
			if (!isset($transaction->paymentstate) || $transaction->paymentstate != 1) {
				$message = 'Attempt to refund an unsuccessful transaction.';
				$transaction->logdata .= '|'.date("D M j G:i").': '.$message;
				$transaction->store(false, false);
				$message .= ", Transaction ID: ".$transaction->transaction_id;
				Log::add($message, Log::INFO, 'com_rsvppro.stripe');
				continue;	// Only refund successful transactions
			}
			/* Determine if we are beyond our refund period */
			if ($refundTo > 0 &&  time() > $refundTo) {
				$message = "Refund period has expired.";
				$transaction->logdata .= '|'.date("D M j G:i").': '.$message;
				$transaction->store(false, false);
				$message .= ", Transaction ID: ".$transaction->transaction_id;
				Log::add($message, Log::INFO, 'com_rsvppro.stripe');
				return; // beyond refund period
			}

			if (is_null($gateway)) {
				$gateway = Omnipay::create('Stripe');
				$apikey = $templateParams->get('stripeMode', 0) == 0 ? $templateParams->get('str_sandbox_sk_apikey') : $templateParams->get('str_live_sk_apikey');
				$gateway->initialize(array('apikey' => $apikey));
			}

            $money = Money::of($transaction->amount, $transaction->currency);
			$response = $gateway->refund([
					'amount' => (string)$money->getAmount(),
					'description' => $transaction->params->title,
					'transactionReference' => $feedata->gw_transid
				])->send();

			if ($response->isSuccessful()) {
				$transaction->paymentstate = 0;
				$transaction->logdata .= '|'.date("D M j G:i").':Transaction refunded';
				$transaction->store(false, false);
				plgRsvpproStripe::notifyStripeRefund($transaction, $attendee, $rsvpdata);
				Log::add(
					"Transaction ".$transaction->transaction_id ." refunded in the amount of ".self::phpMoneyFormat($transaction->amount, $templateParams)
					." for ". Factory::getUser($attendee->user_id)->name,
					Log::INFO, 'com_rsvppro.stripe'
				);
			} else {
				// normal failure flow
				$message = 'There was an error, Stripe responded with: ' . $response->getMessage();
				$transaction->logdata .= ''.date("D M j G:i").':|'.$message; // Any message the gateway his given us.
				$transaction->store(false, false);
				$transError = sprintf(Text::_("RSVP_PRO_STRIPE_TRANSACTION_ERROR"), $response->getMessage());
				Factory::getApplication()->enqueueMessage($transError, 'error');
				$message .= ", Transaction ID: ".$transaction->transaction_id;
				Log::add($message, Log::ERROR, 'com_rsvppro.stripe');
			}
		}
	}
}
